/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.konfigass;

public class KonfigBereichDaten implements Comparable<KonfigBereichDaten>, Cloneable
{
  private Boolean _bearbeitung = null;

  private Boolean _modelle = null;

  private Boolean _aktvieren = null;

  private Boolean _objekte = null;

  private String _pidAOE = null;

  private String _pid = null;  //  @jve:decl-index=0:

  private Long _datumAktivierung = null;

  private Short _versionAktivierung = null;

  private Short _versionAktuell = null;

  private Short _versionFreigabeZurUebernahme = null;

  private Short _versionFreigabeZurAktivierung = null;

  private Short _versionFreigabeZurAktivierungOhne = null;

  private String _name = null;
 

  public static enum EnumStatusKonfigBereich {
    IMPORT, FREIGABE, FEHLER
  }

  private EnumStatusKonfigBereich _statusKonfigBereich;

  private String _pfad = null;


  public KonfigBereichDaten(String pid, String name, String pidAOE, String pfad)
  {
    _pid = pid;
    _name  = name;
    _pidAOE = pidAOE;
    _pfad = pfad;
  }
  
  public KonfigBereichDaten(String pid, String name, Boolean bearbeitung, Boolean modelle, Boolean objekte, String pidAOE,
      Long datumAktivierung, Short versionAktivierung, Short versionAktuell, Short versionFreigabeZurUebernahme,
      Short versionFreigabeZurAktivierung, Short versionFreigabeZurAktivierungOhne,
      EnumStatusKonfigBereich statusKonfigBereich, String pfad)
  {
    _pid = pid;
    _name = name;
    _bearbeitung = bearbeitung;
    _modelle = modelle;
    _objekte = objekte;
    _pidAOE = pidAOE;
    _datumAktivierung = datumAktivierung;
    _versionAktivierung = versionAktivierung;
    _versionAktuell = versionAktuell;
    _versionFreigabeZurUebernahme = versionFreigabeZurUebernahme;
    _versionFreigabeZurAktivierung = versionFreigabeZurAktivierung;
    _versionFreigabeZurAktivierungOhne = versionFreigabeZurAktivierungOhne;
    _statusKonfigBereich = statusKonfigBereich;
    _pfad = pfad;
  }
  
  public KonfigBereichDaten(String pid, String name, Boolean bearbeitung, Boolean modelle, Boolean objekte, String pidAOE,
      Long datumAktivierung, Short versionAktivierung, Short versionAktuell, Short versionFreigabeZurUebernahme,
      Short versionFreigabeZurAktivierung, Short versionFreigabeZurAktivierungOhne,
      EnumStatusKonfigBereich statusKonfigBereich, String pfad, Boolean aktivieren)
  {
    _pid = pid;
    _name = name;
    _bearbeitung = bearbeitung;
    _modelle = modelle;
    _objekte = objekte;
    _pidAOE = pidAOE;
    _datumAktivierung = datumAktivierung;
    _versionAktivierung = versionAktivierung;
    _versionAktuell = versionAktuell;
    _versionFreigabeZurUebernahme = versionFreigabeZurUebernahme;
    _versionFreigabeZurAktivierung = versionFreigabeZurAktivierung;
    _versionFreigabeZurAktivierungOhne = versionFreigabeZurAktivierungOhne;
    _statusKonfigBereich = statusKonfigBereich;
    _pfad = pfad;
    _aktvieren = aktivieren;
  }

  public Boolean getBearbeitung()
  {
    return _bearbeitung;
  }

  public void setBearbeitung(Boolean bearbeitung)
  {
    _bearbeitung = bearbeitung;
  }

  public Boolean getModelle()
  {
    return _modelle;
  }

  public void setModelle(Boolean modelle)
  {
    _modelle = modelle;
  }

  public Boolean getObjekte()
  {
    return _objekte;
  }

  public void setObjekte(Boolean objekte)
  {
    _objekte = objekte;
  }

  public String getPidAOE()
  {
    return _pidAOE;
  }

  public void setPidAOE(String pidAOE)
  {
    _pidAOE = pidAOE;
  }

  public String getPid()
  {
    return _pid;
  }

  public void setPid(String pid)
  {
    _pid = pid;
  }

  public Long getDatumAktivierung()
  {
    return _datumAktivierung;
  }

  public void setDatumAktivierung(Long datumAktivierung)
  {
    _datumAktivierung = datumAktivierung;
  }

  public Short getVersionAktivierung()
  {
    return _versionAktivierung;
  }

  public void setVersionAktivierung(Short versionAktivierung)
  {
    _versionAktivierung = versionAktivierung;
  }

  public Short getVersionAktuell()
  {
    return _versionAktuell;
  }

  public void setVersionAktuell(Short versionAktuell)
  {
    _versionAktuell = versionAktuell;
  }

  public Short getVersionFreigabeZurUebernahme()
  {
    return _versionFreigabeZurUebernahme;
  }

  public void setVersionFreigabeZurUebernahme(Short versionFreigabeZurUebernahme)
  {
    _versionFreigabeZurUebernahme = versionFreigabeZurUebernahme;
  }

  public Short getVersionFreigabeZurAktivierung()
  {
    return _versionFreigabeZurAktivierung;
  }

  public void setVersionFreigabeZurAktivierung(Short versionFreigabeZurAktivierung)
  {
    _versionFreigabeZurAktivierung = versionFreigabeZurAktivierung;
  }

  public Short getVersionFreigabeZurAktivierungOhne()
  {
    return _versionFreigabeZurAktivierungOhne;
  }

  public void setVersionFreigabeZurAktivierungOhne(Short versionFreigabeZurAktivierungOhne)
  {
    _versionFreigabeZurAktivierungOhne = versionFreigabeZurAktivierungOhne;
  }

  public EnumStatusKonfigBereich getStatusKonfigBereich()
  {
    return _statusKonfigBereich;
  }

  public void setStatusKonfigBereich(EnumStatusKonfigBereich statusKonfigBereich)
  {
    _statusKonfigBereich = statusKonfigBereich;
  }

  public String getPfad()
  {
    return _pfad;
  }

  public void setPfad(String pfad)
  {
    _pfad = pfad;
  }
  
  public String getName()
  {
    return _name;
  }

  public void setName(String name)
  {
    _name = name;
  }
  
  @Override
  public String toString()
  {
    // TODO Auto-generated method stub
    if (_name != null){    
      return _name + " (" + _pid + ")";
    }
    
    return "";
  }
  
  
  @Override
  public boolean equals(Object obj)
  {
    // TODO Auto-generated method stub
    
    if (obj instanceof KonfigBereichDaten)
    {
      KonfigBereichDaten konfigBereichKapsel = (KonfigBereichDaten)obj;
      if (_pid != null && konfigBereichKapsel.getPid() != null){
        if (_pid.equals(konfigBereichKapsel.getPid()))
          return true;        
      }      
    }
    
    return false;
  }

  @Override
  protected Object clone() throws CloneNotSupportedException
  {
    // TODO Auto-generated method stub
    KonfigBereichDaten konfigBereichKapsel = new KonfigBereichDaten(_pid, _name, _bearbeitung, _modelle, _objekte, _pidAOE, _datumAktivierung, _versionAktivierung,
        _versionAktuell, _versionFreigabeZurUebernahme, _versionFreigabeZurAktivierung,
        _versionFreigabeZurAktivierungOhne, _statusKonfigBereich, _pfad);

    return konfigBereichKapsel;
  }

  @Override
  public int compareTo(KonfigBereichDaten arg0)
  {
    // TODO Auto-generated method stub
    if (_pid.equals(arg0._pid))
      return 1;
    
    return 0;
  }
  
  public Boolean getAktvieren()
  {
    return _aktvieren;
  }

  public void setAktvieren(Boolean aktvieren)
  {
    _aktvieren = aktvieren;
  }

}
