/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
 
package de.bsvrz.buv.plugin.konfigass;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringReader;
import java.net.URL;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;
import de.bsvrz.puk.config.main.managementfile.ManagementFile;

/**
 * Klasse zur Verwaltung von XML-Filtern, welche Listen von regulren Ausdrcken enthalten. Liefert Funktionen zum
 * Einlesen und Schreiben der entsprechenden XML-Strukturen.
 * 
 * @author HofmannJ
 * 
 */
public class XmlElementKonfigVerwaltung
{
  /**
   * Bezeichnung des Wurzelelementes der XML-Struktur
   */
  public static final String _xmlElementWurzel = "verwaltungsdaten";

  /**
   * Bezeichnung eines Filterelementes
   */
  public static final String _xmlElementKonfigVerantwortlicher = "konfigurationsverantwortlicher";

  /**
   * Bezeichnung eines Filterelementes
   */
  public static final String _xmlElementKonfigBereich = "konfigurationsbereich";

  /**
   * Regulrer Ausdruck innerhalb des Filterelementes
   */
  public static final String _xmlElementPid = "pid";

  /**
   * Bezeichnung des Attributes ID eines Filterelementes
   */
  public static final String _xmlAttId = "id";

  /**
   * Bezeichnung eines Filterelementes
   */
  public static final String _xmlAttBez = "bezeichnung";

  /**
   * JDOM Document, in dem die Verarbeitung erfolgt
   */
  private Document _doc = null;

  /**
   * JDOM Document, in dem die Verarbeitung erfolgt
   */
  private ConfigDataModel _config = null;

  /**
   * JDOM Document, in dem die Verarbeitung erfolgt
   */
  private final String _systemId = "verwaltungsdaten.dtd";

  /**
   * Wurzelelement
   */
  private Element _elementWurzel = null;

  public Element getElementWurzel()
  {
    return _elementWurzel;
  }

  public void setElementWurzel(Element elementWurzel)
  {
    _elementWurzel = elementWurzel;
  }

//  /**
//   * Konstruktor, erzeugt eine leere XML-Struktur.
//   */
//  public XmlElementKonfigVerwaltung()
//  {
//    _elementWurzel = new Element(_xmlElementWurzel);
//    _doc = new Document(_elementWurzel);
//  }

  public XmlElementKonfigVerwaltung(ConfigDataModel config)
  {
    // TODO Auto-generated constructor stub
    _config = config;
    _elementWurzel = new Element(_xmlElementWurzel);
    _doc = new Document(_elementWurzel);
//    new XmlElementKonfigVerwaltung();
  }

  /**
   * Liefert die zu dieser Struktur zugehrigen {@link ElementKonfig} nach ID sortiert zurck.
   * 
   * @return Zugehrige XML-Filter
   */
  public LinkedList<ElementKonfig> getElementKonfigBereich()
  {
    LinkedList<ElementKonfig> re = null;

    @SuppressWarnings("unchecked")
    List<Element> elements = _elementWurzel.getChildren(_xmlElementKonfigBereich);
    if ((elements != null) && (elements.size() > 0))
    {
      re = new LinkedList<ElementKonfig>();

      for (Element element : elements)
      {   
        ElementKonfig elementKonfig = new ElementKonfig(element);
        re.add(elementKonfig);
      }
    }

    /*
     * Sortieren nach ID
     */
    if ((re != null) && (re.size() > 1))
    {
      Collections.sort(re);
    }

    return re;
  }

  /**
   * Liefert die zu dieser Struktur zugehrigen {@link ElementKonfig} nach ID sortiert zurck.
   * 
   * @return Zugehrige XML-Filter
   */
  public ElementKonfig getElementKonfigVerantwortlicher()
  {
    ElementKonfig re = null;

    @SuppressWarnings("unchecked")
    List<Element> elements = _elementWurzel.getChildren(_xmlElementKonfigVerantwortlicher);

    if ((elements != null) && (elements.size() == 1))
    {
      re = new ElementKonfig(elements.get(0));
    }

    return re;
  }

  /**
   * Fgt ein {@link Element} vom Typ {@link ElementKonfig} zur XML-Struktur hinzu. Ist ein {@link Element} mit der
   * gleichen ID vorhanden, so wird dieses entfernt und durch das neue {@link Element} ersetzt.
   * 
   * @param elementKonfig
   */
  public void hinzufuegenElementKonfig(ElementKonfig elementKonfig)
  {
    Element addElement = elementKonfig.getElement();

    /*
     * Prfen ob das Element schon vorhanden ist & in Lschliste schreiben
     */
    LinkedList<Element> loeschListe = new LinkedList<Element>();
    @SuppressWarnings("unchecked")
    Iterator<Element> itPruefung = _elementWurzel.getChildren(_xmlElementKonfigBereich).iterator();
    while (itPruefung.hasNext())
    {
      Element pruefElement = itPruefung.next();
      // if (pruefElement.getAttributeValue(_xmlAttId).equals(
      // addElement.getAttributeValue(_xmlAttId)))
      {
        // loeschListe.add(pruefElement);
      }
    }

    /*
     * Elemente in der Lschliste vom Elternelement ablsen
     */
    // Iterator<Element> itLoeschen = loeschListe.iterator();
    // while (itLoeschen.hasNext())
    // {
    // Element loeschElement = itLoeschen.next();
    // loeschElement.detach();
    // }

    /*
     * Neues / gendertes Element anhngen
     */
    _elementWurzel.addContent(addElement);
  }

  public void entferneElementKonfig(ElementKonfig elementKonfig)
  {
    /*
     * Speichert zu lschende Elemente
     */
    LinkedList<Element> loeschListe = new LinkedList<Element>();

    /*
     * Suchen der Lschelemente
     */
    @SuppressWarnings("unchecked")
    Iterator<Element> itPruefen = _elementWurzel.getChildren(_xmlElementKonfigBereich).iterator();
    while (itPruefen.hasNext())
    {
      Element element = itPruefen.next();
      ElementKonfig ek = new ElementKonfig(element);
      if (ek.getBezeichnung() == elementKonfig.getBezeichnung())
      {
        loeschListe.add(element);
      }
    }

    /*
     * Lschen der Elemente aus der Lschliste
     */
    Iterator<Element> itLoeschen = loeschListe.iterator();
    while (itLoeschen.hasNext())
    {
      Element loeschElement = itLoeschen.next();
      boolean b = _elementWurzel.removeContent(loeschElement);
    }

  }

  // /**
  // * Bestimmt die nchste, freie ID fr Elemente vom Typ {@link ElementKonfig}.
  // *
  // * @return ID
  // */
  // public int bestimmeNaechsteFreieIdElementFilter()
  // {
  // int re = -1;
  //
  // @SuppressWarnings("unchecked")
  // List<Element> filter = _elementWurzel.getChildren(_xmlElementKonfigBereich);
  // if ((filter != null) && (filter.size() > 0))
  // {
  //
  // Iterator<Element> it = filter.iterator();
  // while (it.hasNext())
  // {
  // Element element = it.next();
  // ElementKonfig elementFilter = new ElementKonfig(element);
  // if (elementFilter.getId() > re)
  // {
  // re = elementFilter.getId();
  // }
  // }
  // }
  //
  // re = re + 1;
  //
  // return re;
  // }

  /**
   * Liest eine XML-Datei ein
   * 
   * @param dateiPfad
   *          Kompletter Pfad zur Datei
   * @throws IllegalArgumentException
   *           Wird geworfen, wenn die Datei nicht existiert
   */
  public void einlesenXmlAusDatei(String dateiPfad) throws IllegalArgumentException
  {
    SAXBuilder builder = new SAXBuilder();

    File file = new File(dateiPfad);
    if (file.exists())
    {
      try
      {
        builder.setValidation(false);

//        if (_config != null)
        {
          builder.setEntityResolver(new EntityResolver() {

            @Override
            public InputSource resolveEntity(String publicId, String systemId) throws SAXException, IOException
            {
              if ((systemId != null) && systemId.endsWith(_systemId))
              {
                URL url = ManagementFile.class.getResource(_systemId);
                
                if (url != null){
                  
                  String ef = url.toExternalForm();
                  
                  if (ef != null && ef.length() > 0){
                    return new InputSource(ef);   
                  }
                }
              }  
              MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.ERROR, "Fehler beim Auflsen der DTD-Datei:" + _systemId, null);
              MeldungenSpeicher.getInstanz().setStatus(status);
              MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(status, System.currentTimeMillis(), false);
              
              return null;
            }
          });

        }

        _doc = builder.build(file);
        _elementWurzel = _doc.getRootElement();
      }
      catch (JDOMException e)
      {
        e.printStackTrace();
      }
      catch (IOException e)
      {
        e.printStackTrace();
      }

    }
    else
    {
      throw new IllegalArgumentException("Die Datei " + dateiPfad + " existiert nicht!");
    }
  }

  /**
   * Liest die XML-Struktur aus einem String ein.
   * 
   * @param xmlString
   *          String der XML-Struktur, die eingelesen werden soll.
   */
  public void einlesenXmlAusString(String xmlString)
  {
    SAXBuilder builder = new SAXBuilder();
    builder.setValidation(false);
    try
    {
      StringReader reader = new StringReader(xmlString);
      _doc = builder.build(reader);
      _elementWurzel = _doc.getRootElement();

    }
    catch (JDOMException e)
    {
      System.err.println(getClass().getName() + " => JDOMException");
      e.printStackTrace();
    }
    catch (IOException e)
    {
      System.err.println(getClass().getName() + " => IOException");
      e.printStackTrace();
    }
  }

  /**
   * Liefert eine formatierte Ausgabe der erzeugten XML-Struktur. Setzt das Dirty-Flag auf <code>false</code>.
   * 
   * @return Formatierte XML-Struktur
   */
  public String ausgabeXmlFormatiert()
  {
    String returnString = null;
    if (_doc == null)
    {
      _doc = new Document(_elementWurzel);
    }

    Format prettyFormat = Format.getPrettyFormat();
    XMLOutputter out = new XMLOutputter(prettyFormat);
    Format f = out.getFormat();
//    f.setEncoding(XmlBezeichnerMenuSymbol.xmlCodierung);
    f.setEncoding("ISO-8859-1");
    out.setFormat(f);

    returnString = out.outputString(_doc);

    return returnString;
  }

  /**
   * Schreibt einen String in eine Datei
   * 
   * @param dateiPfad
   *          Kompletter Pfad zur Datei, in die geschrieben werden soll
   * @param schreibString
   *          String, der in die Datei geschrieben werden soll
   * @param ueberschreiben
   *          true == berschreiben der Datei, falls schon vorhanden
   * @throws IllegalArgumentException
   *           Wir geworfen, wenn die Datei existiert und nicht berschrieben werden kann
   */
  public void schreibeXMLStringInDatei(String dateiPfad, String schreibString, boolean ueberschreiben)
      throws IllegalArgumentException
  {
    File file = new File(dateiPfad);
    // File existiert, auf berschreiben prfen
    if (file.exists() && ueberschreiben)
    {
      try
      {
        PrintWriter pw = new PrintWriter(file);      
        pw.print(schreibString);
        pw.close();
      }
      catch (FileNotFoundException e)
      {
        e.printStackTrace();
      }
    }
    // File existiert nicht und kann einfach angelegt werden
    else if (!file.exists())
    {
      try
      {
        PrintWriter pw = new PrintWriter(file);
        pw.print(schreibString);
        pw.close();
      }
      catch (FileNotFoundException e)
      {
        e.printStackTrace();
      }
    }
    // File existiert und ueberschreiben == flase
    else
    {
      throw new IllegalArgumentException("Die Datei " + dateiPfad + " existiert und ueberschreiben == flase");
    }
  }

  public String toString()
  {
    String re = "===XmlFilterVerwaltung===\n";
    @SuppressWarnings("unchecked")
    Iterator<Element> it = _elementWurzel.getChildren(_xmlElementKonfigBereich).iterator();
    while (it.hasNext())
    {
      Element element = it.next();
      ElementKonfig fi = new ElementKonfig(element);
      re = re + "FI: " + fi.toString();
    }

    re = re + "=========================";
    return re;
  }

  // /**
  // * Kodiert einen String nach Base64
  // *
  // * @param string
  // * Zu kodierender String
  // * @return Base64-Kodierung des Strings. Liefert <code>null</code>, wenn
  // * der bergebene String <code>null</code> ist oder die Lnge == 0
  // * hat.
  // */
  // public String stringToBase64(String string)
  // {
  // String re = null;
  //
  // if ((string != null) && (string.length() > 0))
  // {
  // re = Base64.encodeBase64String(string.getBytes());
  // // re = new BASE64Encoder().encode(string.getBytes());
  // }
  //
  // return re;
  // }
  //
  // /**
  // * Decodiert einen Base64-kodierten String.
  // *
  // * @param base64
  // * Base64-kodierter String.
  // * @return Klartextreprsentation oder <code>null</code>, wenn
  // * <code>null</code> bergeben wurde oder eine {@link IOException}
  // * beim decodieren aufgetreten ist.
  // */
  // public String base64ToString(String base64)
  // {
  // String re = null;
  //
  // if (base64 != null)
  // {
  // byte[] decodeBuffer = Base64.decodeBase64(base64);
  // // try
  // // {
  // // byte[] decodeBuffer = new BASE64Decoder().decodeBuffer(base64);
  // // re = new String(decodeBuffer);
  // // } catch (IOException e)
  // // {
  // // e.printStackTrace();
  // // }
  // }
  //
  // return re;
  // }

  /**
   * Liefert eine tiefe Kopie der Instanz dieser Klasse
   */
  public XmlElementKonfigVerwaltung clone()
  {
    XmlElementKonfigVerwaltung clone = new XmlElementKonfigVerwaltung(_config);
    String xmlString = this.ausgabeXmlFormatiert();
    clone.einlesenXmlAusString(xmlString);
    return clone;
  }

}
