/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ObjectLookup;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;
import de.bsvrz.puk.config.main.dataview.VersionedView;

/**
 * Erzeugen von Instanzen vom Typ <code>ObjectLookup</code>
 * 
 * @author Sans
 *
 */

public class LookupFactory
{
  /**
   * Erzeugt eine Instanz von <code>ObjectLookup</code>, die fr alle aktuellen 
   * Konfigurationsbereiche als Version die jeweilige ModifiableVersion spezifiziert.
   * 
   * @param dataModel das Datenmodell der Konfiguration
   * @return Instanz vom Typ <code>ObjectLookup</code>, die fr alle aktuellen 
   * Konfigurationsbereiche als Version die jeweilige ModifiableVersion spezifiziert
   */
  public static ObjectLookup createLookupForModifiableVersion(ConfigDataModel dataModel)
  {
    Collection<ConfigurationArea> areas = dataModel.getAllConfigurationAreas().values();

    return createLookupForModifiableVersion(dataModel, areas);
  }  
  
  /**
   * Erzeugt eine Instanz von <code>ObjectLookup</code>, die fr die bergebenen 
   * Konfigurationsbereiche als Version die jeweilige ModifiableVersion spezifiziert.
   * 
   * @param dataModel das Datenmodell der Konfiguration
   * @param areas Sammlung der zu bercksichtigenden Konfigurationsbereiche
   * @return Instanz vom Typ <code>ObjectLookup</code>, die fr alle bergebenen 
   * Konfigurationsbereiche als Version die jeweilige ModifiableVersion spezifiziert
   */
  public static ObjectLookup createLookupForModifiableVersion(ConfigDataModel dataModel, 
      Collection<ConfigurationArea> areas)
  {
    Map<ConfigurationArea, Short> konfigVersionen = new HashMap<ConfigurationArea, Short>();
    
    for (ConfigurationArea kb: areas)
    {
      konfigVersionen.put(kb, kb.getModifiableVersion());
    }

    return new VersionedView(dataModel, konfigVersionen);
  }
  
}