/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis;

import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.ISaveablePart2;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.plugin.konfigass.IKonfigAssDatenModell;
import de.bsvrz.buv.plugin.konfigass.datenModellUpdateListener.DatenModellUpdateEvent;
import de.bsvrz.buv.plugin.konfigass.datenModellUpdateListener.DatenModellUpdateListener;
import de.bsvrz.buv.plugin.konfigass.datenModellUpdateListener.IDatenModellUpdateListener;
import de.bsvrz.buv.plugin.tkabasis.aktionen.AktionFactory.TKABasisAktion;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchie;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieManager;
import de.bsvrz.buv.plugin.tkabasis.tkaUpdateManagement.FindeObjektInHierarchie;
import de.bsvrz.buv.plugin.tkabasis.tkaUpdateManagement.FindePotentiellesElternObjekt;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;

/**
 * Nach Auswahl des Menpunkts "Betriebsmeldungen..." in der Rahmenapplikation muss sich der Dialog zur Anzeige der
 * Betriebsmeldungen ffnen.
 * 
 * <br>
 * <br>
 * Die Klasse erfllt die technische Anforderungen des Segments Bedienung und Visualisierung (BUV): <br>
 * <ul>
 * <li>TAnf-Vi-BW-C3-8</li>
 * </ul>
 * 
 * @version $Revision: 1.31 $ / $Date: 2011/06/01 14:25:00 $ / ($Author: Sans $)
 * @author HofmannJ
 * 
 */
public class TKABasisView extends ViewPart implements IKonfigAssDatenModell, IDatenModellUpdateListener,
  ISaveablePart2
{
  public TKABasisView() {
  }

  public static final String ID = "de.bsvrz.buv.plugin.tkabasis.tkaBasisView"; // @jve:decl-index=0:

  private Composite composite;

  private IMemento _memento = null;

  private String _secondViewId = null; // @jve:decl-index=0:

  private TKABasisComposite _tkaComposite;

  private Composite _parent;
  
  @Override
  public void init(IViewSite site, IMemento memento) throws PartInitException
  {
    _memento = memento;

    _secondViewId = site.getSecondaryId();

    setPartName(getPartName() + " " + _secondViewId);
    super.init(site, memento);
  }
  
  private void restoreState()
  {

    if (_memento == null)
    {
      return;
    }

    _secondViewId = _memento.getString("id");
  }

  @Override
  public void saveState(IMemento memento)
  {
    memento.putString("id", _secondViewId);
    super.saveState(memento);
  }


  
  @Override
  public void createPartControl(Composite parent)
  {
    _parent = parent;
    
    
    restoreState();
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessVerticalSpace = true;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 1;
    composite = new Composite(parent, SWT.NONE);
    
    composite.setLayout(new FillLayout());
       
    _tkaComposite = new TKABasisComposite(composite, SWT.NONE, _secondViewId);

    erzeugeDropDownMenue();
    
    DatenModellUpdateListener.getInstanz().addDatenModellUpdateListener(this);
    
  }
  
  
  public void erzeugeDropDownMenue()
  {
    IActionBars actionBars = getViewSite().getActionBars();
    IMenuManager dropDownMenu = actionBars.getMenuManager();
    if ( dropDownMenu.isEmpty() )
    {
      dropDownMenu.add(_tkaComposite.getAction(TKABasisAktion.VORGAENGER.getAktion().getAktionsID()));
      dropDownMenu.add(_tkaComposite.getAction(TKABasisAktion.NACHFOLGER.getAktion().getAktionsID()));
      dropDownMenu.add(_tkaComposite.getAction(TKABasisAktion.VORGAENGEREBENE.getAktion().getAktionsID()));
      dropDownMenu.add(_tkaComposite.getAction(TKABasisAktion.NACHFOLGEREBENE.getAktion().getAktionsID()));
    }
  }

  public Composite getComposite()
  {
    return composite;
  }

  public void setComposite(Composite composite)
  {
    this.composite = composite;
  }

  public TKABasisComposite getTkaComposite()
  {
    return _tkaComposite;
  }
    
  public void setTkaComposite(TKABasisComposite tkaComposite)
  {
    _tkaComposite = tkaComposite;
  }

//  @Override
//  public void setFocus()
//  {
//    // TODO Auto-generated method stub
//
//  }
  
  @Override
  public void setFocus()
  {
    //Fr die F1 Taste
    if (_parent != null)
      _parent.setFocus();

  }
  
  @Override
  public void dispose()
  {
//    super.dispose();
    IWorkbenchWindow dwindow = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
    IWorkbenchPage page = dwindow.getActivePage();
   
    if (page != null)
      page.hideView(this);
    
    DatenModellUpdateListener.getInstanz().removeDatenModellUpdateListener(this);
  }

  @Override
  public String holeBeschreibung()
  {
    return null;
  }

  @Override
  public String holeName()
  {
    return null;
  }

  @Override
  public String holePluginId()
  {
    return Activator.PLUGIN_ID;
  }


  @Override
  public void ladeKonfigDatenModell(String pfad)
  {  
    // TODO Auto-generated method stub
//    String meldung = Activator.PLUGIN_ID + "Zugriff auf das bisherige Datenmodell "
//    	+	"ist nicht mehr mglich, da ein neues Datenmodell geladen wird!";
//    
//    MessageBox msg = new MessageBox(PlatformUI.getWorkbench().getDisplay()
//        .getActiveShell(), SWT.OK | SWT.CANCEL | SWT.ICON_WARNING);
//    msg.setText("Neues Datenmodell wird geladen");
//    msg.setMessage(meldung);
//    msg.open();
    
  }
 
  private IModelInfo _modelInfo;
  
  public IModelInfo getModelInfo()
  {
    return _modelInfo;
  }

  public void setModelInfo(IModelInfo modelInfo)
  {
    _modelInfo = modelInfo;
  }
  
//  public static TKABasisView getView()
//  {
//    IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
//    IWorkbenchPart activePart = dw.getActivePage().getActivePart();
//    return (TKABasisView)activePart;
//  }
  
  public static TKABasisView getView(String secondViewId)
  {
    TKABasisView result = null;
    
    IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
    IWorkbenchPage page = dw.getActivePage();
    
    IViewReference ref = page.findViewReference(TKABasisView.ID, secondViewId);
   
    if (ref != null)
    {
      result = (TKABasisView)ref.getView(false);
    }

    return result;
  }

  @Override
  public IAction holeAktion()
  {
    return null;
  }
  
  public void datenModellUpdate(DatenModellUpdateEvent e)
  {
    final TKABasisComposite tkaComposite = getTkaComposite();
    
    Object src = e.getSource();
    
    if (src != tkaComposite && !(src instanceof DatenModellUpdateListener))
    {
      final List<SystemObject> listeAktualisiert = e.getListeSystemObjekte();
      final List<SystemObject> listeGeloescht = e.getListeSystemObjekteGeloescht();
      final List<SystemObject> listeNeu = e.getListeSystemObjekteNeu();
      
      PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() 
      {
        public void run()
        {
          IHierarchieManager hm = getModelInfo().getHierarchieManager();
          IHierarchie hierarchie = hm.getHierarchie();
          
          // TKA-Daten werden neu geladen, wenn
          // eines der aktualisierten Objekte in der Hierarchie vorhanden ist        
          boolean update = new FindeObjektInHierarchie(listeAktualisiert, 
              hierarchie).isEinSystemobjektGefunden();
          
          // oder eines der gelschten Objekte in der Hierarchie vorhanden ist
          update = update || new FindeObjektInHierarchie(listeGeloescht, 
              hierarchie).isEinSystemobjektGefunden();
          
          update = update || new FindePotentiellesElternObjekt(listeNeu, 
              hm).isEinPotentiellesElternGefunden();
          
          if (update)
          {
            ConfigDataModel dataModel = tkaComposite.getDataModel();
            tkaComposite.setConfigDataModel(null, null);
            tkaComposite.setConfigDataModel(dataModel, null);
          }
        }  
      });
    }  
  }
  
  
 
  
  //SetHelp fr die F1-Taste
  public void setHelp(String secondId, String contextId){

    if(_secondViewId.equals(secondId))
    {
      PlatformUI.getWorkbench().getHelpSystem().setHelp(_parent, contextId );
      //PlatformUI.getWorkbench().getHelpSystem().setHelp(_parent, contextId );
      //IWorkbenchHelpSystem help = PlatformUI.getWorkbench().getHelpSystem();
      //help.setHelp(_parent, contextId);        
    }
   
  }

//  @Override
//  public String holePfad()
//  {
//    // TODO Auto-generated method stub
//    return null;
//  }

  @Override
  public void setzeKonfigDatenModell(ConfigDataModel dataModel, String pfad)
  {
    // TODO Auto-generated method stub
    
  }

//  @Override
//  public String holeSecId()
//  {
//    // TODO Auto-generated method stub
//    return null;
//  }
  
  
  public void doSave(IProgressMonitor monitor)
  {
  }

  public void doSaveAs()
  {
  }

  public boolean isDirty()
  {
    return (_tkaComposite != null && _tkaComposite.getEditor() != null && _tkaComposite.getEditor().isDirty());
  }
  
  public boolean isSaveAsAllowed()
  {
    return false;
  }

  public boolean isSaveOnCloseNeeded()
  {
    return true;
  }
  
  public void firePropertyDirty()
  {
    firePropertyChange(PROP_DIRTY);
  }
  
  public int promptToSaveOnClose()
  {
    // Workaround wegen BeendenDlg
    if (PlatformUI.getWorkbench().isClosing())
    {
      return ISaveablePart2.NO;
    }
    
    int result = _tkaComposite.ausgabeWarnungBeiAenderung("TKA " + _secondViewId +
        " soll geschlossen werden");
    
    if (result == SWT.OK)
    {  
      return ISaveablePart2.NO;
    }
    else
    {
      return ISaveablePart2.CANCEL;
    }
  }
 
} // @jve:decl-index=0:visual-constraint="10,10,847,295"
