/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.datenbereich.datenbasis;

import java.util.LinkedList;
import java.util.List;

import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TableItem;

import de.bsvrz.buv.plugin.tkabasis.Activator;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * Liefert einen Dialog zur Identifiezierung von Daten
 * 
 * @author HofmannJ
 * 
 */
public class DatenidentifikationGtmOffline extends Dialog
{
  /**
   * Darstellung eines undefinierten SystemObjekts "_Undefiniert_" fr optionale Referenzen
   */
  private boolean _undefiniert = false;
  
  /**
	 * Liste der darzustellenden SystemObjektTypen
	 */
	private List<SystemObjectType> _types = null;

	/**
	 * Liste der SystemObjekte, wird fr die Rckgabe bentigt.
	 */
	private LinkedList<SystemObject> _systemObjects = new LinkedList<SystemObject>();

	/**
	 * Shell zur Darstellung
	 */
	private Shell _shell = null;

	/**
	 * Datendarstellung
	 */
	private GtmListeCompositeOffline _compGtmListe;

	/**
	 * Verbindung zum DaV. Wird mit Hilfe der Klasse {@link DaVVerbindung}
	 * bestimmt.
	 */
//	private ClientDavInterface _connection = null;

	/**
	 * Darstellungsart
	 */
	private int _darstellung = GtmListeCompositeOffline._darstlungSystemObjekteObjektTypenAttributGruppenAspekte;

	/**
	 * Dialogtitel
	 */
	private String _dialogTitel = "Datenidentifikation";

  private DataModel _dataModel;

	/**
	 * ffnet den Dialog zur Daten Identifikation
	 * 
	 * @param parent
	 *          Eltern-{@link Shell}, auf deren Basis die Darstellung erfolgen
	 *          soll.
	 * @param dialogTitel
	 *          Titel des des Dialoges. Bei <code>null</code> wird der
	 *          Default-Titel <code>Datenidentifikation</code> gesetzt.
	 */
	public DatenidentifikationGtmOffline(Shell parent, String dialogTitel, DataModel dataModel)
	{
		super(parent, 0);
		
		this._dataModel = dataModel;

		if (dialogTitel != null)
		{
			this._dialogTitel = dialogTitel;
		}
	}

	 /**
   * ffnet den Dialog zur Datenidentifikation.
   * 
   * @param types
   *          Liste von {@link SystemObjectType}s, die dargestellt werden soll.
   * @param darstellung
   *          Art der Darstellung. Die Darstellung ist whlbar ber
   *          {@link GtmListeCompositeOffline#_darstlungSystemObjekte},
   *          {@link GtmListeCompositeOffline#_darstlungSystemObjekteObjektTypen},
   *          {@link GtmListeCompositeOffline#_darstlungSystemObjekteObjektTypenAttributGruppen},
   *          {@link GtmListeCompositeOffline#_darstlungSystemObjekteObjektTypenAttributGruppenAspekte}
   *          sowie
   *          {@link GtmListeCompositeOffline#_darstlungSystemObjekteObjektTypenAttributGruppenAspekteSimVar}.
   * @param undefiniert
   *          Die Auswahl der SystemObjekte soll ein SystemObjekt "_Undefiniert_" enthalten.
   * @return Eine Liste der ausgewhlten {@link SystemObject}s. Wurden keine SystemObjekte
   *         selektiert, so ist die Liste leer. Wurde "_Undefiniert_" selektiert, so enthlt 
   *         die Liste ein undefiniertes SystemObjekt.
   */
  public LinkedList<SystemObject> oeffnen(List<SystemObjectType> types,
      int darstellung, boolean undefiniert)
  {
    this._undefiniert = undefiniert;
    return oeffnen(types, darstellung);
  }
  
	/**
	 * ffnet den Dialog zur Datenidentifikation.
	 * 
	 * @param types
	 *          Liste von {@link SystemObjectType}s, die dargestellt werden soll.
	 * @param darstellung
	 *          Art der Darstellung. Die Darstellung ist whlbar ber
	 *          {@link GtmListeCompositeOffline#_darstlungSystemObjekte},
	 *          {@link GtmListeCompositeOffline#_darstlungSystemObjekteObjektTypen},
	 *          {@link GtmListeCompositeOffline#_darstlungSystemObjekteObjektTypenAttributGruppen},
	 *          {@link GtmListeCompositeOffline#_darstlungSystemObjekteObjektTypenAttributGruppenAspekte}
	 *          sowie
	 *          {@link GtmListeCompositeOffline#_darstlungSystemObjekteObjektTypenAttributGruppenAspekteSimVar}.
	 * @return Eine Liste der ausgewhlten {@link SystemObject}s. Wurden keine
	 *         selektiert, so ist die Liste leer.
	 */
	public LinkedList<SystemObject> oeffnen(List<SystemObjectType> types,
			int darstellung)
	{
		this._types = types;
		return oeffnen(darstellung);
	}

	/**
	 * ffnet den Dialog zur Datenidentifikation.
	 * 
	 * @param darstellung
	 *          Art der Darstellung. Die Darstellung ist whlbar ber
	 *          {@link GtmListeCompositeOffline#_darstlungSystemObjekte},
	 *          {@link GtmListeCompositeOffline#_darstlungSystemObjekteObjektTypen},
	 *          {@link GtmListeCompositeOffline#_darstlungSystemObjekteObjektTypenAttributGruppen},
	 *          {@link GtmListeCompositeOffline#_darstlungSystemObjekteObjektTypenAttributGruppenAspekte}
	 *          sowie
	 *          {@link GtmListeCompositeOffline#_darstlungSystemObjekteObjektTypenAttributGruppenAspekteSimVar}.
	 * @return Eine Liste der ausgewhlten {@link SystemObject}s. Wurden keine
	 *         selektiert, so ist die Liste leer.
	 */
	public LinkedList<SystemObject> oeffnen(int darstellung)
	{
		this._darstellung = darstellung;

//		_connection = DaVVerbindung.getInstanz().getDaVVerbindung();

		_shell = new Shell(getParent(), SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL
				| SWT.RESIZE);
		_shell.setText(_dialogTitel);
		Image shellImage = Activator.getImageDescriptor("icons/find_16x16.png").createImage();
		_shell.setImage(shellImage);

		_shell.setSize(new Point(800, 600));
		_shell.setLayout(new GridLayout());

		erzeugeDarstellung();

		_shell.open();

		Display display = getParent().getDisplay();

		/*
		 * Positionieren des Dialogs bei der Position des Mauszeigers
		 */
//		int displayWidth = Display.getDefault().getBounds().width;
//		int displayHeight = Display.getDefault().getBounds().height;
	  int displayWidth = Display.getDefault().getClientArea().width;
    int displayHeight = Display.getDefault().getClientArea().height;

		Rectangle bounds = _shell.getBounds();
		Point cursorLocation = Display.getDefault().getCursorLocation();

		int y = cursorLocation.y;
		if ((cursorLocation.y + bounds.height) > displayHeight)
		{
			y = displayHeight - bounds.height;
			if (y < 0)
			{
				y = 0;
			}
		}

		int x = cursorLocation.x;
		if ((cursorLocation.x + bounds.width > displayWidth))
		{
			x = displayWidth - bounds.width;
			if (x < 0)
			{
				x = 0;
			}
		}

		Rectangle newBounds = new Rectangle(x, y, bounds.width, bounds.height);
		_shell.setBounds(newBounds);

		while (!_shell.isDisposed())
		{
			if (!display.readAndDispatch())
				display.sleep();
		}

		return _systemObjects;
	}

	private void erzeugeDarstellung()
	{
	  GtmDatenBasisOffline datenBasis = new GtmDatenBasisOffline(_dataModel, false, null);

		 

		/*
		 * SystemObjekte nach Typen bestimmen
		 */
		
		if (_types == null)
		{
//			 datenBasis.setAll();

//			DaVPreLader daVPreLader = DaVPreLader.getInstanz();
//			if (!daVPreLader.istLadenAbgeschlossen())
//			{
//				Warte warte = new Warte();
//				synchronized (warte)
//				{
//					try
//					{
//						warte.wait(10 * 60 * 1000);
//						datenBasis = warte.getGtmDatenBasis();
//					}
//					catch (InterruptedException e1)
//					{
//						e1.printStackTrace();
//					}
//				}
//			}
//			else
//			{
//				datenBasis = daVPreLader.getGtmDatenBasis();
//			}			
//			datenBasis = daVPreLader.getGtmDatenBasis();
		}
		else
		{
			datenBasis.setTypes(_types);
		}

		Composite compositeOben = new Composite(_shell, SWT.NONE);
		Composite compositeUnten = new Composite(_shell, SWT.NONE);
		
		_compGtmListe = new GtmListeCompositeOffline(compositeOben, SWT.NONE, datenBasis,
				_darstellung);
		
		
		_compGtmListe.getTvSystemObjekte().addDoubleClickListener(new IDoubleClickListener() {

      @Override
      public void doubleClick(DoubleClickEvent event)
      {
        _systemObjects = _compGtmListe.getSelektierteSystemObjekte();
        _shell.dispose();
      }
    });
		

		Button buttonUndef = null;
		if (_undefiniert)
		{
	    buttonUndef = new Button(compositeUnten, SWT.NONE);
	    buttonUndef.setText("Undefiniert");		
	    
	    buttonUndef.addSelectionListener(new SelectionAdapter()
	    {
	      public void widgetSelected(SelectionEvent e)
	      {
	        aktionUndef();
	      }
	    });
		}
		
		Button buttonOk = new Button(compositeUnten, SWT.NONE);
		buttonOk.setText("OK");

		Button buttonAbbrechen = new Button(compositeUnten, SWT.NONE);
		buttonAbbrechen.setText("Abbrechen");
	
		buttonOk.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				aktionOk();
			}
		});

		buttonAbbrechen.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				aktionAbbrechen();
			}
		});

		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = GridData.FILL;
		compositeOben.setLayout(new GridLayout());
		compositeOben.setLayoutData(gridData);

		RowLayout rowLayout = new RowLayout();
		rowLayout.pack = false;
		GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.END;
		gridData1.grabExcessHorizontalSpace = true;
		gridData1.verticalAlignment = GridData.CENTER;
		compositeUnten.setLayoutData(gridData1);
		compositeUnten.setLayout(rowLayout);

		GridData gd = new GridData();
		gd.grabExcessHorizontalSpace = true;
		gd.horizontalAlignment = GridData.FILL;
		gd.grabExcessVerticalSpace = true;
		gd.verticalAlignment = GridData.FILL;
		_compGtmListe.setLayoutData(gd);
	}

	 private void aktionUndef()
	  {
	    _systemObjects.clear();
	    _systemObjects.add(null);
	    _shell.dispose();
	  }
	
	private void aktionOk()
	{
		_systemObjects = _compGtmListe.getSelektierteSystemObjekte();
		_shell.dispose();
	}

	private void aktionAbbrechen()
	{
		_systemObjects.clear();
		_shell.dispose();
	}
}
