/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.datenbereich.editor;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.ScrollBar;

import de.bsvrz.buv.plugin.tkabasis.DataFactory;
import de.bsvrz.buv.plugin.tkabasis.LookupFactory;
import de.bsvrz.buv.plugin.tkabasis.TKABasisComposite;
import de.bsvrz.buv.plugin.tkabasis.TKABasisView;
import de.bsvrz.buv.plugin.tkabasis.aktionen.AktionFactory.TKABasisAktion;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.AtgInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HierarchieObjekt;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieObjektTyp;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.TypInfo;
import de.bsvrz.buv.plugin.tkabasis.zustaende.EndlicherAutomat;
import de.bsvrz.buv.plugin.tkabasis.zustaende.ZustandsFactory.Bearbeitungsmodus;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.MutableSet;
import de.bsvrz.dav.daf.main.config.ObjectLookup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;
import de.bsvrz.puk.config.configFile.datamodel.ConfigSystemObject;

/**
 * Die Klasse erweitert die Klasse {@link DatenBereichEditorKapsel} um Scroll-Funktionalitt und stellt Methoden zum
 * Setzen und Holen der zu bearbeitenden Daten zur Verfgung.
 * 
 * @author Pittner
 * 
 */
public class ScrolledDatenBereichEditorComposite extends Composite
{
  /**
   * Flag zum Debuggen
   */
  private static final boolean _debug = false;

  // Bereich fr den Composite basierten Editor
  private DataEditorComposite _dataEditorPanel = null;

  /**
   * Gibt an, ob die Felder des {@link ScrolledDatenBereichEditorComposite} editierbar sind, oder nicht.
   */
  private boolean _editable = false;

  /**
   * Kapsel mit den zu bearbeitenden Daten
   */
  private DatenBereichEditorKapsel _kapsel = null; // @jve:decl-index=0:

  private DataModel _dataModel = null;

  private String _secondId;

  private Canvas _scrolledArea;

  protected boolean _editorInhaltErzeugt = false;

  private boolean _virtuell = true;

  // virtuelles Anzeigefeld frs Scrollen
  private Rectangle _scrolledAnzeigefeld;

  // Scrollpunkt des sichtbaren Anzeigebereiches
  private Point _scrollPoint;

  // Scrollpunkt des virtuellen DatenEditors
  private Point _scrollPointVirtuellerEditor;

  private int _scrollVMaximum;

  private int _scrollHMaximum;

  /**
   * Lokaler Verweis
   */
  private ScrolledDatenBereichEditorComposite _this = null;

  /**
   * Konstruktor.
   * 
   * @param parent
   *          Basis-{@link Composite}, auf dessen Basis die Darstellung erfolgen soll
   * @param style
   *          SWT-Stil
   * @param connection
   *          Verbindung zum Dav
   */
  public ScrolledDatenBereichEditorComposite(Composite parent, int style, DataModel dataModel, String secId)
  {
    super(parent, style);

    if (dataModel == null)
      throw new IllegalArgumentException("DataModel ist null");

    this._dataModel = dataModel;
    this._secondId = secId;
    this._this = this;

    initialize();

    //Versuch damit der Standard-Resize den Editor nicht wieder auf die Position (5,5) platziert.
//    this.addListener(SWT.Resize, new Listener() {
//
//      @Override
//      public void handleEvent(Event event)
//      {
//        _this.layout(false);
//        // _this.layout(false, false);
//        event.doit = false;
//
//      }
//    });
  }

  private void initialize()
  {
    createCompositeDarstellung();

    setLayout(new FillLayout());
    Rectangle boundsParent = _this.getParent().getBounds();
    setSize(boundsParent.width + boundsParent.x, boundsParent.height + boundsParent.y);

    // Gre des Anzeigefeldes ermitteln bzw. auf ein greres Mass setzen
    Point scs = _scrolledArea.getSize();
    int weite = scs.x > 100? scs.x : 800;
    _scrolledAnzeigefeld = new Rectangle(0, 0, weite, 2000);
    _scrollPoint = new Point(0, 0);
    _scrollPointVirtuellerEditor = new Point(0, 0);
  }

  /**
   * This method initializes compositeDarstellung
   * 
   */
  private void createCompositeDarstellung()
  {
    _scrolledArea = new Canvas(this, SWT.H_SCROLL | SWT.V_SCROLL);
    _scrolledArea.setLayout(new GridLayout());

    _scrolledArea.getVerticalBar().setIncrement(20);
    _scrolledArea.getHorizontalBar().setIncrement(20);

    // damit die Scrollbalken nicht bei der Initialisierung kurzfristig aufflackern
    _scrolledArea.setVisible(false);

    // Scrollen
    final ScrollBar hBar = _scrolledArea.getHorizontalBar();
    if (hBar != null)
    {
      hBar.addListener(SWT.Selection, new Listener() {
        public void handleEvent(Event e)
        {
          if (_debug)
            System.out.println("\nScroll horizontal! Detail: " + e.detail + " Selection: " + hBar.getSelection());

          int hSelection = hBar.getSelection();
          int destX = -hSelection - _scrollPoint.x;
          Rectangle rect = _dataEditorPanel.getBounds();
          _scrolledArea.scroll(destX, 0, 0, 0, rect.width, rect.height, true);
          _scrollPoint.x = -hSelection;
        }
      });
    }

    final ScrollBar vBar = _scrolledArea.getVerticalBar();
    if (vBar != null)
    {
      vBar.addListener(SWT.Selection, new Listener() {
        @Override
        public void handleEvent(Event e)
        {
          if (_debug)
            System.out.println("\nScroll vertikal --> Anzeige erneuern!!! Detail: " + e.detail + " Selection: "
                + vBar.getSelection());

          scrollVertikal(vBar);
        }
      });
    }

    GridData gridDataEditor = new GridData();
    gridDataEditor.horizontalAlignment = GridData.FILL;
    gridDataEditor.grabExcessHorizontalSpace = true;

    _dataEditorPanel = new DataEditorComposite(_scrolledArea, SWT.NONE, _dataModel);
    _dataEditorPanel.setLayout(new GridLayout(1, false));
    _dataEditorPanel.setLayoutData(gridDataEditor);

    _scrolledArea.addListener(SWT.Resize, new Listener() {

      @Override
      public void handleEvent(Event event)
      {
        // if ( _virtuell )
        {
          // TODO Scroll beibehalten
//          System.out.println("\nResize vor resizeScrollBars:");
          // System.out.println("Selektion: " + _scrolledArea.getVerticalBar().getSelection());
          // System.out.println("_dataEditorPanel: " + _dataEditorPanel.getBounds());

          if (_virtuell && _scrolledArea.getVerticalBar().getSelection() != 0 && _scrollPointVirtuellerEditor.y != 0)
          {
            _scrolledArea.getVerticalBar().setSelection(0);
            scrollVertikal(_scrolledArea.getVerticalBar());
          }
          else
          {
            resetScrollPoint();
            _scrolledArea.getVerticalBar().setSelection(0);
          }


          resizeScrollBars();


          // System.out.println("Resize nach resizeScrollBars:");
          // System.out.println("Selektion: " + _scrolledArea.getVerticalBar().getSelection());
          // System.out.println("_dataEditorPanel: " + _dataEditorPanel.getBounds());
        }
        // else
        // {
        // funktioniert nicht
        // Point size = _scrolledArea.getSize();
        // _scrollHMaximum = size.x * 3 / 2;
        // _scrollVMaximum = size.y * 3 / 2;
        // resizeScrollBarsNichVirtuell();
        // }

      }
    });

  }

  public DataEditorComposite getDataEditorPanel()
  {
    return _dataEditorPanel;
  }

  /**
   * Setzt die {@link DatenBereichEditorKapsel} mit den Infos zu den zu bearbeitenden Daten ohne Anzeige.
   * 
   * @param kapsel
   *          {@link DatenBereichEditorKapsel}
   */
  protected void setKapsel(DatenBereichEditorKapsel kapsel)
  {
    this._kapsel = kapsel;

    if (_kapsel != null)
    {
      AttributeGroup atg = _dataModel.getAttributeGroup(_kapsel.getName());

      if (atg != null)
      {
        // Falls als Empfangs-Aspekt "asp.parameterDefault" angegeben ist, wird hier KEIN Datensatz erzeugt
        // => Panel erhlt den Zustand "keine Daten"
        boolean defaultParameter = kapsel.getPidAspEmpfang() != null
            && kapsel.getPidAspEmpfang().equals("asp.parameterDefault");

        if (!defaultParameter)
        {
          DatenBereichSOKapsel elternKapsel = _kapsel.getElternKapsel();
          if (elternKapsel != null)
          {
            SystemObject so = elternKapsel.getSystemObjekt();
            if (so != null)
            {
              ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion((ConfigDataModel)_dataModel);
              Data data = ((ConfigSystemObject)so).getConfigurationData(atg, lookup);
              
              HierarchieObjekt ho = elternKapsel.getHierarchieObjekt();

              if (!isNeu())
              {
                // bestehende Daten
                _kapsel.setDataEmpfangen(data);
                _dataEditorPanel.setData(data != null ? DataFactory.createModifiableCopy(data, lookup) : null);
              }
              else
              {
                // Neuanlage
                TKABasisView view = TKABasisView.getView(_secondId);
                if (view != null)
                {
                  IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();

                  TypInfo ti = hot.getTypInfo(so.getType().getPid());
                  AtgInfo ai = ti.getAtgInfo(kapsel.getName());

                  if (ai != null && ai.isOptional())
                  {
                    _kapsel.setDataEmpfangen(null);
                    _dataEditorPanel.setData(null);
                  }
                  else
                  {
                    data = view.getModelInfo().getBildungsRegeln().getDatensatz(ho, so, atg, data, false);
                    _kapsel.setDataEmpfangen(data);
                    _dataEditorPanel.setData(data != null ? DataFactory.createModifiableCopy(data, lookup) : null);
                  }
                }
              }
            }
          }
        }
        else
        {
          _kapsel.setDataEmpfangen(null);
          _dataEditorPanel.setData(null);
        }
      }

      DatenBereichSOKapsel elternKapsel = _kapsel.getElternKapsel();
      if (elternKapsel != null)
      {
        SystemObject so = elternKapsel.getSystemObjekt();
        if (so != null)
        {
          MutableSet menge = ((ConfigurationObject)so).getMutableSet(_kapsel.getName());

          if (menge != null)
          {
            // Falls als Empfangs-Aspekt "asp.parameterDefault" angegeben ist,
            // wird hier KEIN Datensatz erzeugt
            // => Panel erhlt den Zustand "keine Daten"
            boolean defaultParameter = kapsel.getPidAspEmpfang() != null
                && kapsel.getPidAspEmpfang().equals("asp.parameterDefault");

            if (!defaultParameter)
            {

              ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion((ConfigDataModel)_dataModel);
              Data data = null;
              HierarchieObjekt ho = elternKapsel.getHierarchieObjekt();
              AttributeGroup atgMenge = _dataModel.getAttributeGroup("atg.dynamischeMenge");

              if (!isNeu())
              {
                // bestehende Daten
                data = ((ConfigSystemObject)menge).getConfigurationData(atgMenge, lookup);
                _kapsel.setDataEmpfangen(data);
                _dataEditorPanel.setData(data != null ? DataFactory.createModifiableCopy(data, lookup) : null);
              }
              else
              {
                // Neuanlage
                TKABasisView view = TKABasisView.getView(_secondId);
                if (view != null)
                {
                  // if ( ai != null && ai.isOptional() )
                  // {
                  // _kapsel.setDataEmpfangen( null );
                  // _dataEditorPanel.setData( null );
                  // }
                  // else
                  {
                    data = view.getModelInfo().getBildungsRegeln().getDatensatz(ho, menge, atgMenge, data, false);
                    _kapsel.setDataEmpfangen(data);
                    _dataEditorPanel.setData(data != null ? DataFactory.createModifiableCopy(data, lookup) : null);
                  }
                }
              }

            }
            else
            {
              _kapsel.setDataEmpfangen(null);
              _dataEditorPanel.setData(null);
            }
          }
        }
      }

    }
  }

  protected boolean isEditorErzeugt()
  {
    return _editorInhaltErzeugt;
  }

  public void erzeugeEditorInhalt()
  {
    _editorInhaltErzeugt = true;

    _scrolledArea.setVisible(false);

    if (_virtuell)
      virtuelleEditorAnzeige();
    else
      _dataEditorPanel.createGroup(0, 0, null, false);

    this.layout();
    // geht auch ohne
    // _scrolledArea.layout();
    // resizeScrollBars();

    _scrolledArea.setVisible(true);

  }

  private void virtuelleEditorAnzeige()
  {
    if (_dataEditorPanel.getData() != null)
    {
      // Editor mit den Daten aufbauen
      int sel = _scrolledArea.getVerticalBar().getSelection();
      if (sel > 0)
        _scrolledArea.getVerticalBar().setSelection(0);

      _dataEditorPanel.createGroup(0, 0, _scrolledAnzeigefeld, _virtuell);

      int relHoehe = _dataEditorPanel.getBounds().height;
      if (_debug)
        System.out.println("DataEditor: " + _dataEditorPanel.getBounds() + " DataEditor-Hhe " + relHoehe
            + " < ScrolledAnzeigefeld-Hhe " + _scrolledAnzeigefeld.height + " ?");

      if (relHoehe < _scrolledAnzeigefeld.height)
      {
        resizeScrollBars(0, relHoehe + 30);
        setVirtuell(false);
      }
    }
    else
    {
      _dataEditorPanel.createGroup(0, 0, null, false);
    }

  }

  /**
   * ndert die Attribute der beiden Scrollbars.
   */
  void resizeScrollBarsNichVirtuell()
  {
//    System.out.println("resizeScrollBarsNichVirtuell:: _scrollVMaximum: " + _scrollVMaximum);

    Rectangle clientArea = _scrolledArea.getClientArea();
    ScrollBar bar = _scrolledArea.getHorizontalBar();
    if (bar != null)
    {
      bar.setMaximum(_scrollHMaximum);
      bar.setThumb(clientArea.width);
      bar.setPageIncrement(clientArea.width);
    }
    bar = _scrolledArea.getVerticalBar();
    if (bar != null)
    {
      bar.setMaximum(_scrollVMaximum);
      bar.setThumb(clientArea.height);
      bar.setPageIncrement(clientArea.height);
    }
  }

  /**
   * ndert die Attribute der beiden Scrollbars entsprechend den gesetzten Werten.
   */
  public void resizeScrollBars()
  {
    resizeScrollBars(_scrollHMaximum, _scrollVMaximum);
  }

  /**
   * ndert die Attribute der beiden Scrollbars unter Bercksichtigung der bergebenen Scroll-Werte.
   * 
   * @param scrollHMax
   *          spezielle ScrollWeite
   * @param scrollVMax
   *          spezielle ScrollHhe
   */
  public void resizeScrollBars(int scrollHMax, int scrollVMax)
  {
    Rectangle clientArea = _scrolledArea.getClientArea();

    ScrollBar vBar = _scrolledArea.getVerticalBar();
    if (vBar != null)
    {
      if (scrollVMax == 0)
        _scrollVMaximum = _dataEditorPanel.getBounds().height;
      // _scrollVMaximum = _dataEditorPanel.computeSize(SWT.DEFAULT, SWT.DEFAULT).y;
      else
        _scrollVMaximum = scrollVMax;

      if (_scrollVMaximum <= clientArea.height)
      {
        vBar.setVisible(false);
      }
      else
      {
        vBar.setVisible(true);
        vBar.setMaximum(_scrollVMaximum);
        vBar.setThumb(clientArea.height);
        vBar.setPageIncrement(clientArea.height);
      }

      _scrolledArea.layout();

      if (_debug)
        System.out.println("resizeScrollBars:: scrollVMax: " + scrollVMax + " <= clientArea.height: "
            + clientArea.height + " _scrolledArea: " + _scrolledArea.getBounds());

    }

    ScrollBar hBar = _scrolledArea.getHorizontalBar();
    if (hBar != null)
    {
      if (scrollHMax == 0)
        _scrollHMaximum = _dataEditorPanel.getBounds().width;
      // _scrollHMaximum = _dataEditorPanel.computeSize(SWT.DEFAULT, SWT.DEFAULT).x;
      else
        _scrollHMaximum = scrollHMax;

      if (_scrollHMaximum <= clientArea.width)
      {
        hBar.setVisible(false);
      }
      else
      {
        hBar.setVisible(true);
        hBar.setMaximum(_scrollHMaximum);
        hBar.setThumb(clientArea.width);
        hBar.setPageIncrement(clientArea.width);
      }

      // System.out.println("resizeScrollBars nach hBar:");
      // System.out.println("Selektion: " + _scrolledArea.getVerticalBar().getSelection());
      // System.out.println("_dataEditorPanel: " + _dataEditorPanel.getBounds());
    }
  }

  /**
   * Liefert das sichtbare Anzeigefeld des Scrollbereiches
   * 
   * @return Anzeigefeld
   */
  public Rectangle getScrolledAnzeigefeld()
  {
    return _scrolledAnzeigefeld;
  }

  public boolean isVirtuell()
  {
    return _virtuell;
  }

  public void setVirtuell(boolean virtuell)
  {
    if (_debug)
      System.out.println("Virtuell: " + virtuell);
    _virtuell = virtuell;
  }

  /**
   * Liefert die {@link DatenBereichEditorKapsel} mit den bearbeiteten Daten.
   * 
   * @return
   */
  protected DatenBereichEditorKapsel getKapsel()
  {
    return _kapsel;
  }

  /**
   * Fhrt die Synchronisierung zwischen den Daten des Editors und der Kapsel durch.
   */
  protected void synchronisiereBearbeitung()
  {
    if ((_kapsel != null) && (_dataEditorPanel != null))
    {
      _kapsel.setDataSenden(_dataEditorPanel.getData());
    }
  }

  protected void setEditable(boolean editable)
  {
    this._editable = editable;
    _dataEditorPanel.setEditable(editable);

  }

  protected boolean isEditable()
  {
    return _editable;
  }

  /**
   * 
   * @return <code>true</code>, wenn in der bergeordenten Instanz von TKABasisComposite der Zustand Neuanlage herrscht
   *         und dieser Zustand durch eine der Aktionen NEU oder NEUUO hervorgerufen wurde, <code>false</code> sonst
   */
  protected boolean isNeu()
  {
    TKABasisView view = TKABasisView.getView(_secondId);
    if (view != null)
    {
      TKABasisComposite composite = view.getTkaComposite();
      if (composite != null)
      {
        TKABasisAktion aktion = composite.getAktionDesLetztenUebergangs();
        if (aktion != null && (aktion.equals(TKABasisAktion.NEU) || aktion.equals(TKABasisAktion.NEUUO)))
        {
          EndlicherAutomat automat = composite.getZustandsmaschine();
          if (automat != null)
          {
            Bearbeitungsmodus bm = Bearbeitungsmodus.valueOf(automat.getMomentZustand().toString());
            return Bearbeitungsmodus.NEUANLAGE.equals(bm);
          }
        }
      }
    }
    return false;
  }

  public void resetScrollPoint()
  {
    _scrollPoint = new Point(0, 0);
    _scrollPointVirtuellerEditor = new Point(0, 0);
  }

  /**
   * Scroll in vertikaler Richtung mit virtuellem Scrollen
   * 
   * @param vBar  vertikale Scrollbar
   */
  private void scrollVertikal(final ScrollBar vBar)
  {
    int vSelection = vBar.getSelection();
    // Scroll-Differenz des sichtbaren Anzeigebereiches
    int destY = -vSelection - _scrollPoint.y;
    Rectangle rectEditor = _dataEditorPanel.getBounds();
    Rectangle clientArea = _scrolledArea.getClientArea();

    if (_debug)
      System.out.println("Virtuell! DestY: " + destY + " = - vSelection: " + vSelection + " - _scrollPoint.y: "
          + _scrollPoint.y + " Min: " + vBar.getMinimum() + " Max: " + vBar.getMaximum());

    VirtuellesElement virtuellElementVor = _dataEditorPanel.getVirtuellElementVor();
    
    if (_virtuell && virtuellElementVor != null)
    {
      // Differenz zwischen Selektion(+) und des Begins des Editorbereiches(-)
      int diffVirtY = vSelection + _scrollPointVirtuellerEditor.y;
      if (_debug)
        System.out.println("diffVirtY: " + diffVirtY + " = _scrollPoint.y: " + _scrollPoint.y
            + " + _scrollPointVirtuellerEditor.y: " + _scrollPointVirtuellerEditor.y + " + destY: " + destY);

      Composite parent = virtuellElementVor.getParent();
      int clientBegin = virtuellElementVor.getY() + parent.getBounds().y + parent.getChildren()[1].getBounds().y;

      if (destY < 0 && ((rectEditor.height - diffVirtY) <= (clientArea.height + 5)))
      {
        // Falls das Ende des ClientArea-Bereichs das Ende des DataEditorPanels fast erreicht hat

        if (_debug)
          System.out.println("\nScrollDown: destY: " + destY + "; rectDEP.height: " + rectEditor.height
              + " - diffVirtY: " + diffVirtY + " <= clientArea.height: " + clientArea.height + " + 5");

        // bestehende Daten im neuen Anzeigebereich anzeigen
        int virtDiff = _dataEditorPanel.scrollEditor(destY, _scrolledAnzeigefeld);

        if (_debug)
        {
          System.out.println("virtDiff: " + virtDiff);
          System.out.println("_dataEditorPanel: " + _dataEditorPanel.getBounds() + " _scrolledArea:_"
              + _scrolledArea.getBounds());
        }

        // neu erstellten DatenEditorPanel entsprechend der virtuellen Differenz hoch scrollen
        _scrollPointVirtuellerEditor.y -= virtDiff;
        destY += virtDiff;

        if (_debug)
          System.out.println("DestY: " + destY);
      }
      else if (destY > 0 && (diffVirtY <= clientBegin))
      {
        // Falls der Begin des ClientArea-Bereichs das erste Array-Element des Anzeigebereiches fast erreicht hat

        if (_debug)
          System.out.println("\nScrollUp: destY: " + destY + "; diffVirtY: " + diffVirtY + " <= clientBegin: "
              + clientBegin);

        // bestehende Daten im neuen Anzeigebereich anzeigen
        int virtDiff = _dataEditorPanel.scrollEditor(destY, _scrolledAnzeigefeld);

        if (_debug)
          System.out.println("virtDiff: " + virtDiff);

        // neu erstellten DatenEditorPanel entsprechend der virtuellen Differenz runter scrollen
        _scrollPointVirtuellerEditor.y -= virtDiff;
        destY += virtDiff;

        if (_debug)
          System.out.println("DestY: " + destY);
      }
    }

    _scrolledArea.scroll(0, destY, 0, 0, rectEditor.width, rectEditor.height, true);
    _scrollPoint.y = -vSelection;

  }

}
