/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.navigator;

import java.util.Arrays;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ColumnViewerToolTipSupport;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.ui.IDecoratorManager;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.plugin.tkabasis.Activator;
import de.bsvrz.buv.plugin.tkabasis.TKABasisComposite;
import de.bsvrz.buv.plugin.tkabasis.TKABasisView;
import de.bsvrz.buv.plugin.tkabasis.aktionen.MyAction;
import de.bsvrz.buv.plugin.tkabasis.aktionen.AktionFactory.TKABasisAktion;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HierarchieObjekt;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchie;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieManager;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieObjektTyp;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieBeobachter;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.KBHierarchie;
import de.bsvrz.buv.plugin.tkabasis.navigator.navigatorModell.AbstractNavigatorElement;
import de.bsvrz.buv.plugin.tkabasis.navigator.navigatorModell.HONavigatorElement;
import de.bsvrz.buv.plugin.tkabasis.navigator.navigatorModell.INavigatorElement;
import de.bsvrz.buv.plugin.tkabasis.navigator.navigatorModell.KBNavigatorElement;
import de.bsvrz.buv.plugin.tkabasis.navigator.navigatorModell.INavigatorElement.STATUS;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Klasse zur Darstellung der Navigatorbereichs mit
 * - Einstellungen
 * - Baumdarstellung 
 * - Suchbereich nach Ebenen, Suchetext, Filtertext
 * - Drag and Drop
 * - Erzeugen neuer Navigatorelemente
 * - Sysnchronisierung mit der Hierarchie
 * - Update bei Aktualisierung der Hierarchie
 * 
 * @author Krause
 *
 */
public class NavigatorComposite extends Composite
{

  /**
   * Flag zum Debuggen
   */
  protected static final boolean _debug = false;
  
  private Composite _composite;
  private Composite _composite1;
  private Group _group;
  
  private Spinner _spinner;
  private Combo _combo;
  private Text _text;
  
  private TreeViewer _treeViewer;
  private int _aktEbene = 1;
  private int _maxEbene = 9;
  private KBHierarchie _hierarchie;
 
  private boolean _startItemGefunden = false;
  private MenuManager _kontextMenuMgr;
  
  // Regulren Ausdruck als Pattern fr den Vergleich setzen
  private Pattern _patFilter;
  
  // aktuelle Sortierrichtung
//  private int _sortRichtung;
  
  private String _secondViewId = "";
  private NavigatorDragListener _navigatorDragListener;
  private NavigatorDropListener _navigatorDropListener;

  public enum DNDModus {
    COPY, COPY_DEEP, MOVE
  };
  
  /**
   * Konstruktor zur Darstellung des Navigators
   * 
   * @param parent  Elternteil, in dem der Navigator angezeigt werden soll 
   * @param style   SWT-Style, in dem der Navigatorangezeigt werden soll
   * @param secondViewId secondViewId der zugehrigen View
   */
  public NavigatorComposite(Composite parent, int style, String secondViewId)
  {
    super(parent, style);
    _composite = this;
    _secondViewId = secondViewId;
    
    GridLayout gridLayout6 = new GridLayout();
    gridLayout6.numColumns = 1;
    gridLayout6.marginHeight = 0;
    gridLayout6.marginWidth = 0;
    GridData gridData6 = new GridData();
    gridData6.horizontalAlignment = GridData.FILL;
    gridData6.grabExcessHorizontalSpace = true;
    gridData6.grabExcessVerticalSpace = true;
    gridData6.verticalAlignment = GridData.FILL;
    _composite.setLayout(gridLayout6);
    _composite.setLayoutData(gridData6);
    
    //Einstellungen
    createEinstellungenGroup1();
    createEinstellungenGroup2();
    createEinstellungenGroup3();
    createEinstellungenGroup4();
    //Baumdarstellung
    createBaumdarstellungComposite();
    
  }
  
  /**
   * Darstellung der Gruppe Ebenen
   * 
   */
  private void createEinstellungenGroup1()
  {
    GridData gridData15 = new GridData();
    gridData15.horizontalAlignment = GridData.FILL;
    gridData15.verticalAlignment = GridData.CENTER;
    GridLayout gridLayout53 = new GridLayout();
    gridLayout53.numColumns = 4;
    Group group23 = new Group(_composite, SWT.NONE);
    group23.setText("Ebenen");
    group23.setLayoutData(gridData15);
    group23.setLayout(gridLayout53);
    Button button152 = new Button(group23, SWT.NONE);
    ImageDescriptor id = Activator.getImageDescriptor("icons/collapse.PNG");
    button152.setImage(id.createImage());
    button152.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        _treeViewer.collapseAll();
      }
    });
    Button button162 = new Button(group23, SWT.NONE);
    id = Activator.getImageDescriptor("icons/expand.png");
    button162.setImage(id.createImage());
    button162.addMouseListener(new MouseAdapter() {        
      @Override
      public void mouseDoubleClick(MouseEvent e)
      {
        _treeViewer.expandAll(); 
      }
      @Override
      public void mouseDown(MouseEvent e)
      {
        int level = _spinner.getSelection();
        _treeViewer.expandToLevel(level);          
      }    
    });
    Label label = new Label(group23, SWT.NONE);
    label.setText("Ebenen einblenden");
    _spinner = new Spinner(group23, SWT.BORDER);
    _spinner.setMinimum(1);
    _spinner.setMaximum(_maxEbene);
    _spinner.pack();
    _spinner.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        _aktEbene = _spinner.getSelection();
        _treeViewer.getTree().setRedraw( false );
        _treeViewer.collapseAll();
        _treeViewer.expandToLevel(_aktEbene);
        _treeViewer.getTree().setRedraw( true );
      }
    });
  
  }

  /**
   * Darstellung der Gruppe Sortierung
   * 
   */
  private void createEinstellungenGroup2()
  {
    GridLayout gridLayout5 = new GridLayout();
    gridLayout5.numColumns = 2;
    GridData gridData6 = new GridData();
    gridData6.horizontalAlignment = GridData.FILL;
    gridData6.grabExcessHorizontalSpace = false;
    gridData6.verticalAlignment = GridData.CENTER;
    Group group2 = new Group(_composite, SWT.NONE);
    group2.setLayoutData(gridData6);
    group2.setLayout(gridLayout5);
    group2.setText("Sortierung");
    Button button15 = new Button(group2, SWT.NONE);
    ImageDescriptor id = Activator.getImageDescriptor("icons/pfeil_runter.png");
    button15.setImage(id.createImage());
    button15.addSelectionListener(new SelectionAdapter() {
      
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        aktionSortierung(SWT.UP);
      }
    });
    
    Button button16 = new Button(group2, SWT.NONE);
    id = Activator.getImageDescriptor("icons/pfeil_hoch.png");
    button16.setImage(id.createImage());
    button16.addSelectionListener(new SelectionAdapter() {
      
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        aktionSortierung(SWT.DOWN);
      }
    });
  }


  /**
   * Darstellung der Gruppe Filter
   * 
   */
  private void createEinstellungenGroup3()
  {
    GridData gridData9 = new GridData();
    gridData9.horizontalAlignment = GridData.FILL;
    gridData9.verticalAlignment = GridData.CENTER;
    GridLayout gridLayout51 = new GridLayout();
    _group = new Group(_composite, SWT.NONE);
    _group.setText("Filter");
    createCombo();
    _group.setLayoutData(gridData9);
    _group.setLayout(gridLayout51);
  }

  /**
   * Darstellung der Combobox zum Filter
   * 
   */
  private void createCombo()
  {
    GridData gridData26 = new GridData();
    gridData26.horizontalAlignment = GridData.FILL;
    gridData26.grabExcessHorizontalSpace = true;
    gridData26.verticalAlignment = GridData.CENTER;
    _combo = new Combo(_group, SWT.NONE);
    _combo.setLayoutData(gridData26);
    _combo.addSelectionListener(new SelectionAdapter() {
      
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        String regAusdruck = _combo.getText();
        aktionFilter( regAusdruck );
      }
    });
    _combo.addKeyListener(new KeyAdapter() {
      
      @Override
      public void keyReleased(KeyEvent e)
      {
        String regAusdruck = _combo.getText();
        aktionFilter( regAusdruck );
        
        //echten regulren Ausdruck bei Enter als Item anzeigen
        if ( e.keyCode == SWT.CR && _patFilter != null )
        {
          List<String> listItems = new LinkedList<String>( Arrays.asList(_combo.getItems()) );
          if ( !listItems.contains(regAusdruck) )
          {
            listItems.add(0, regAusdruck);
            List<String> listComboItems = null;
            if ( listItems.size() > 20 )
              listComboItems = listItems.subList(0, 20);
            else
              listComboItems = listItems;
            
            _combo.setItems(listComboItems.toArray(new String[]{}));
            _combo.setText(regAusdruck);
          }
        }
        else if ( e.keyCode == SWT.CR && _patFilter == null )
        {
          MessageBox msg = new MessageBox( PlatformUI.getWorkbench().getDisplay().getActiveShell(),
              SWT.OK | SWT.ICON_INFORMATION );
          msg.setText( "Achtung" );
          msg.setMessage( "Bitte einen regulren Ausdruck eingeben!" );
          msg.open();
        }
        
      }
    });
  }

  /**
   * Darstellung der Gruppe Suchen
   * 
   */
  private void createEinstellungenGroup4()
  {
    GridData gridData28 = new GridData();
    gridData28.horizontalAlignment = GridData.FILL;
    gridData28.grabExcessHorizontalSpace = true;
    gridData28.verticalAlignment = GridData.CENTER;
    GridData gridData27 = new GridData();
    gridData27.horizontalAlignment = GridData.FILL;
    gridData27.grabExcessHorizontalSpace = false;
    gridData27.verticalAlignment = GridData.CENTER;
    GridLayout gridLayout52 = new GridLayout();
    gridLayout52.numColumns = 3;
    Group group22 = new Group(_composite, SWT.NONE);
    group22.setText("Suchen");
    group22.setLayoutData(gridData27);
    group22.setLayout(gridLayout52);
    _text = new Text(group22, SWT.BORDER);
    _text.setLayoutData(gridData28);
    _text.addKeyListener(new KeyAdapter() {     
      @Override
      public void keyReleased(KeyEvent e)
      {
        aktionSuchen(_treeViewer.getTree().getItems(), null, _text.getText(), SWT.UP);
      }
    });

    Button button151 = new Button(group22, SWT.NONE);
    ImageDescriptor id = Activator.getImageDescriptor("icons/pfeil_runter.png");
    button151.setImage(id.createImage());
    button151.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        TreeItem treeItem = null;
        if ( _treeViewer.getTree().getSelection().length > 0 )
        {
          treeItem = _treeViewer.getTree().getSelection()[0];
        }
        aktionSuchen(_treeViewer.getTree().getItems(), treeItem, _text.getText(), SWT.UP);
      }
    });
    Button button161 = new Button(group22, SWT.NONE);
    id = Activator.getImageDescriptor("icons/pfeil_hoch.png");
    button161.setImage(id.createImage());
    button161.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        TreeItem treeItem = null;
        if ( _treeViewer.getTree().getSelection().length > 0 )
        {
          treeItem = _treeViewer.getTree().getSelection()[0];
        }
        aktionSuchen(_treeViewer.getTree().getItems(), treeItem, _text.getText(), SWT.DOWN);
      }
    });
  }
   
  /**
   * Baumdarstellung
   * 
   */
  private void createBaumdarstellungComposite()
  {
    FillLayout fillLayout = new FillLayout();
    fillLayout.type = org.eclipse.swt.SWT.VERTICAL;
    GridData gridData29 = new GridData();
    gridData29.horizontalAlignment = GridData.FILL;
    gridData29.grabExcessHorizontalSpace = true;
    gridData29.grabExcessVerticalSpace = true;
    gridData29.verticalAlignment = GridData.FILL;
    _composite1 = new Composite(_composite, SWT.NONE);
    _composite1.setLayoutData(gridData29);
    _composite1.setLayout(fillLayout);
   
    // Erzeuge Navigator-Baum
    erzeugeBaum();

    // Erzeuge Sorter, Toolbar, Kontextmens.
    aktionSortierung(SWT.UP);
//    erzeugeToolbar();
    erzeugeKontextMenue();

  }

  /**
   * Erzeugung der Baumdarstellung
   * 
   */
  private void erzeugeBaum()
  {
//    System.out.println("=====>>>>>> Baum neu erzeugen!!!");
    
    // zur Performance Verbesserung eventuell VIRTUAL verwenden
    // m_treeViewer = new TreeViewer( parent, SWT.VIRTUAL );
    _treeViewer = new TreeViewer(_composite1);
    //Drag & Drop
    int operations = DND.DROP_COPY | DND.DROP_MOVE;
    Transfer[] transferTypes = new Transfer[]{TextTransfer.getInstance()};
    _navigatorDragListener = new NavigatorDragListener(_treeViewer);
    _treeViewer.addDragSupport(operations, transferTypes,_navigatorDragListener);
    _navigatorDropListener = new NavigatorDropListener(_treeViewer);
    _treeViewer.addDropSupport(operations, transferTypes, _navigatorDropListener);
    
    ColumnViewerToolTipSupport.enableFor(_treeViewer);
    _treeViewer.getTree().addKeyListener(new KeyAdapter() {
      
      @Override
      public void keyPressed(KeyEvent e)
      {
//        System.out.println("Key pressed: " + e.keyCode + " StateMask: " + e.stateMask);
        
        if ( (e.stateMask == SWT.SHIFT) && (e.keyCode == SWT.CONTROL)
          || (e.keyCode == SWT.SHIFT) && (e.stateMask == SWT.CONTROL) )
          _navigatorDropListener.setShiftStrgPressed(true);      
      }
      
      @Override
      public void keyReleased(KeyEvent e)
      {
//        System.out.println("Key released: " + e.keyCode + " StateMask: " + e.stateMask);
        
        if ( (e.stateMask & SWT.SHIFT & SWT.CONTROL) != 0 
            || e.keyCode == SWT.SHIFT || e.keyCode == SWT.CONTROL )
        _navigatorDropListener.setShiftStrgPressed(false);   
      }
    });
    //Provider
    _treeViewer.setContentProvider(new NavigatorContentProvider());
    IDecoratorManager decoratorManager = PlatformUI.getWorkbench().getDecoratorManager();
    NavigatorLabelProvider navigatorLabelProvider = new NavigatorLabelProvider();
    NavigatorDecoratingStyledCellLabelProvider navigatorDecorator = new NavigatorDecoratingStyledCellLabelProvider(navigatorLabelProvider, decoratorManager.getLabelDecorator(), null);
    _treeViewer.setLabelProvider(navigatorDecorator);
    
    //Nachfolgendes funktioniert nicht
//    NavigatorLabelProviderDecorator nd = NavigatorLabelProviderDecorator.getNavigatorDecorator();
//    System.out.println("navigatorDecorator: " + nd);
//    NavigatorLabelProviderDecorator navigatorDecorator = new NavigatorLabelProviderDecorator();
//    _treeViewer.setLabelProvider(navigatorDecorator);
//    nd = NavigatorLabelProviderDecorator.getNavigatorDecorator();
//    System.out.println("navigatorDecorator: " + nd);
    //    navigatorDecorator.refesh();
    
    _treeViewer.setUseHashlookup(true);  
    //Listener fr Einfachklicks   
    _treeViewer.addSelectionChangedListener(new ISelectionChangedListener() {

      @Override
      public void selectionChanged(SelectionChangedEvent event)
      {
        IStructuredSelection selection = (IStructuredSelection)event.getSelection();
        Object firstElement = selection.getFirstElement();

        if (firstElement != null)
        {
          if (firstElement instanceof INavigatorElement)
          {

            TKABasisView view = TKABasisView.getView(_secondViewId);
            TKABasisComposite tkaComposite = view.getTkaComposite();
            Action action = (Action)tkaComposite.getActionRegistry().getAction(TKABasisAktion.SELEKTIEREN.getAktion().getAktionsID());
            action.run();

          }
        }
      }
    });
    //Listener fr Doppelklicks
    _treeViewer.addDoubleClickListener(new IDoubleClickListener() {
      
      @Override
      public void doubleClick(DoubleClickEvent event)
      {
        IStructuredSelection selection = (IStructuredSelection) _treeViewer.getSelection();
        Object firstElement = selection.getFirstElement();

        if (firstElement != null)
        {
          if (firstElement instanceof HONavigatorElement)
          {
            HONavigatorElement ne = (HONavigatorElement)firstElement;
            if (ne.isAenderbar())
            {
              // entsprechende Action der View ausfhren
              IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
              IWorkbenchPart activePart = dw.getActivePage().getActivePart();
              TKABasisComposite tkaComposite = ((TKABasisView)activePart).getTkaComposite();
              Action action = (Action)tkaComposite.getActionRegistry().getAction(
                  TKABasisAktion.BEARBEITEN.getAktion().getAktionsID());
              action.run();
            }
          }
        }               
      }
    });      
  }

  /**
   * Liefert das KontextMenue zum Navigator
   */
  private void erzeugeKontextMenue()
  {  
    _kontextMenuMgr = new MenuManager("#NavigatorMenu");
    
    //die Aktionen des Kontextmens werden im aufrufenden Objekt gesetzt und ausgefhrt
//    _kontextMenuMgr.add(createAction("Neu"));
//    _kontextMenuMgr.add(createAction("Neu (untergeordnetes Objekt)"));
//    _kontextMenuMgr.add(createAction("Kopieren (nur Objekt)"));
//    _kontextMenuMgr.add(createAction("Kopieren (mit untergeordneten Objekten)"));
//    _kontextMenuMgr.add(createAction("Verschieben"));
//    _kontextMenuMgr.add(createAction("Lschen"));
//    _kontextMenuMgr.add(createAction("Darstellen"));
//    _kontextMenuMgr.add(createAction("Bearbeiten"));
    
    // Men erzeugen - Viewer wird dabei dem Men bekannt gemacht
    Menu menu = _kontextMenuMgr.createContextMenu( _treeViewer.getControl() );
    // Das Men dem Baum bekannt machen
    _treeViewer.getControl().setMenu( menu );
    
  }
  
//  /**
//   * Diese Methode prft die Mehrfachselektion der NavigatorElemente und fllt das
//   * Kontextmen mit den entsprechenden {@link Action}s.
//   * 
//   * @param manager
//   *          Der {@link IMenuManager}, dessen Kontextmen gefllt werden soll.
//   */
//  private void fuelleKontextMenu( IMenuManager manager )
//  {
//    IStructuredSelection selection = (IStructuredSelection) _treeViewer.getSelection();
//    if ( selection.isEmpty() )
//      return;
//
//    // Prfung, ob alle ausgewhlten Elemente vom gleichen Typ(Kategorie) sind
//    AbstractNavigatorElement erstesElement = (AbstractNavigatorElement) selection.getFirstElement();
////    KATEGORIE kategorie = erstesElement.getKategorie();
////    for (Iterator iterator = selection.iterator(); iterator.hasNext();)
////    {
////      NavigatorElement selElement = (NavigatorElement) iterator.next();
////
////      if ( selElement.getKategorie() != kategorie )
////      {
////        //Die Kategorien sind nicht gleich ==> kein Kontextmen
////        return;
////      }
////    }// for
//
//    //wird eventuell spter mal bentigt:
//    // KontextMenu zum ausgewhlten Element erstellen
//    // an Hand der Kategorie der ausgewhlten Navigatorelemente
//
//    // // und der Kategorie des Elternelements (z.B. beim KonfigBereich
//    // // notwendig)
//    // NavigatorElementMitStati parent = (NavigatorElementMitStati)
//    // firstElement.getParent();
//    // KontextMenuInhalt kontextMenu = new KontextMenuInhalt(
//    // firstElement.getKategorie(), parent.getKategorie(), null );
//
//    // Erstellen des darzustellenden Kontextmens
//    Action action = null;
//    boolean defaultElement = false;
//    manager.add( action );
//    m_listeKinder.add( action );
//    
//    //DefaultAktion setzen
//    if ( defaultElement && m_defaultKind == null )  
//    {
//      m_defaultKind = action;
//      m_defaultKind.setChecked( true );
//      
//      if (m_elternMenu != null && m_elternMenu.getDefaultChild() == null)
//      {
//        m_elternMenu.setDefaultChild( m_defaultKind );
//      }
//    }
//    
//    // Kontextmenu anzeigen
//    LinkedList<Object> listeKontextMenu = kontextMenu.getKontextListe();
//
//    for (Iterator iterator = listeKontextMenu.iterator(); iterator.hasNext();)
//    {
//      Object object = (Object) iterator.next();
//      if ( object instanceof Action )
//      {
//        Action action = (Action) object;
//        manager.add( action );
//      }
//      if ( object instanceof MenuManager || object instanceof Separator )
//      {
//        IContributionItem ici = (IContributionItem) object;
//        manager.add( ici );
//      }
//    }// for
//
//  }
  
  /**
   * Kollabiert die aktuelle Selektion im Baum
   * 
   */
//  private void aktionCollapseAktuelleSelektion()
//  {
//    if ( _treeViewer != null )
//    {
//      LinkedList<INavigatorElement> expandedElem = new LinkedList<INavigatorElement>();
//
//      ISelection selection = _treeViewer.getSelection();
//      if ( selection instanceof StructuredSelection )
//      {
//        StructuredSelection ss = (StructuredSelection) selection;
//
//        Iterator itExpElem = ss.iterator();
//        while (itExpElem.hasNext())
//        {
//          Object obj = itExpElem.next();
//
//          if ( obj instanceof INavigatorElement )
//          {
//            INavigatorElement elem = (INavigatorElement) obj;
//            if ( elem.hasChildren() )
//            {
//              if ( !expandedElem.contains( elem ) )
//              {
//                expandedElem.add( elem );
//              }
//            }
//            else if ( elem.hasParent() )
//            {
//              INavigatorElement parentElem = (INavigatorElement) elem.getParent();
//              if ( !expandedElem.contains( parentElem ) )
//              {
//                expandedElem.add( parentElem );
//              }
//            }
//          }
//        }
//      }
//
//      if ( expandedElem.size() > 0 )
//      {
//        _treeViewer.getTree().setRedraw( false );
//        Iterator<INavigatorElement> it = expandedElem.iterator();
//        while (it.hasNext())
//        {
//          INavigatorElement elem = it.next();
//          _treeViewer.collapseToLevel( elem, AbstractTreeViewer.ALL_LEVELS );
//        }
//        _treeViewer.getTree().setRedraw( true );
//      }
//
//    }
//  }
  
  /**
   * Setzt die Sortierung des Baums entsprechend auf- oder abwrts
   * unter Bercksichtigung des gegebenen Comparators
   * 
   * @param up  Richtung der Sortierung (SWT.UP/SWT.DOWN)
   */
  private void aktionSortierung(int up)
  {
//    if ( up == SWT.UP )
//      _sortRichtung = 1;
//    else
//      _sortRichtung = -1;
    
    if (up == SWT.UP)
    {
      _treeViewer.setSorter(new ViewerSorter() {
        public int compare(Viewer viewer, Object e1, Object e2)
        {
          if (e1 instanceof KBNavigatorElement && _hierarchie != null)
          {
            ConfigurationArea kb1 = ((KBNavigatorElement)e1).getKonfigurationsBereich();
            ConfigurationArea kb2 = ((KBNavigatorElement)e2).getKonfigurationsBereich();
            return _hierarchie.comparatorSOAufsteigend().compare(kb1, kb2);
          }
          else if (e1 instanceof HONavigatorElement && _hierarchie != null)
          {
            HierarchieObjekt ho1 = ((HONavigatorElement)e1).getHierarchieObjekt();
            HierarchieObjekt ho2 = ((HONavigatorElement)e2).getHierarchieObjekt();
            return _hierarchie.comparatorHOAufsteigend().compare(ho1, ho2);
          }
          else
            return super.compare(viewer, e1, e2);
        }
      });
    }
    else
    {
      _treeViewer.setSorter(new ViewerSorter() {
        public int compare(Viewer viewer, Object e1, Object e2)
        {
          if (e1 instanceof KBNavigatorElement && _hierarchie != null)
          {
            ConfigurationArea kb1 = ((KBNavigatorElement)e1).getKonfigurationsBereich();
            ConfigurationArea kb2 = ((KBNavigatorElement)e2).getKonfigurationsBereich();
            return _hierarchie.comparatorSOAbsteigend().compare(kb1, kb2);
          }
          else if (e1 instanceof HONavigatorElement && _hierarchie != null)
          {
            HierarchieObjekt ho1 = ((HONavigatorElement)e1).getHierarchieObjekt();
            HierarchieObjekt ho2 = ((HONavigatorElement)e2).getHierarchieObjekt();
            return _hierarchie.comparatorHOAbsteigend().compare(ho1, ho2);
          }
          else
            return super.compare(viewer, e1, e2) * (-1);

        }
      });
    }
  }
 
  /**
   * Setzt den Filter des Baums auf den angegebenen regulren Ausdruck
   * - wobei aus Performance-Grnden die contains-Methode mit verwendet wird 
   * 
   * @param regAusdruck   regulrer Ausdruck
   */
  private void aktionFilter(String regAusdruck)
  {
    // ViewFilter setzen
    final ViewerFilter textFilter = new ViewerFilter() {

      @Override
      public boolean select(Viewer viewer, Object parentElement, Object element)
      {
        INavigatorElement iNe = (INavigatorElement)element;
        // String als Matcher fr den Vergleich setzen und Ergebnis abfragen
        Matcher mat = _patFilter.matcher(iNe.getName());

        if (iNe.getName().contains(_patFilter.pattern()) || mat.find())
          return true;
        else if (element instanceof INavigatorElement)
        {
          Object[] children = ((INavigatorElement)element).getChildren();
          for (int i = 0; i < children.length; i++)
          {
            if (select(viewer, element, children[i]))
              return true;
          }
        }
        return false;
      }
    };

    _patFilter = null;
    try
    {
      _patFilter = Pattern.compile(regAusdruck);
    }
    catch (PatternSyntaxException pse)
    {}
    catch (Exception e)
    {
      e.printStackTrace();
    }
    
    //Liegt ein regulrer Ausdruck vor?
    boolean isRegAusdruck = false;
    if (_patFilter != null)
      isRegAusdruck = true;
    
    if (isRegAusdruck)
    {
      _treeViewer.getTree().setRedraw( false );
      //alten Filter lschen
      if (_treeViewer.getFilters().length > 0)
        _treeViewer.removeFilter(_treeViewer.getFilters()[0]);

      //neuen Filter einfgen und Ebenen entsprechend expandieren
      if (regAusdruck.length() > 0)
      {
        _treeViewer.addFilter(textFilter);
        _treeViewer.expandToLevel(_aktEbene);
      }
      _treeViewer.getTree().setRedraw( true );
    }
    
  }
  
  /**
   * Suchen nach dem ersten Vorkommen des regulren Ausdruckes in den TreeItems ab dem StartItem
   * aufwrts oder abwrts
   * 
   * @param items         TreeItems, die durchsucht werden sollen
   * @param startItem     StartItems, ab dem gesucht werden soll
   * @param regAusdruck   regulrer Ausdruck
   * @param up            Richtung der Suche
   * @return
   */
  private void aktionSuchen(TreeItem[] items, TreeItem startItem, String regAusdruck, int up)
  {
    //Regulren Ausdruck als Pattern fr den Vergleich setzen
    Pattern pat = null;
    try
    {
      pat = Pattern.compile(regAusdruck);
    }
    catch (PatternSyntaxException pse)
    {}
    catch (Exception e)
    {
      e.printStackTrace();
    }

    //regulrer Ausdruck vorhanden
    if ( pat != null )
    {
      _startItemGefunden = false;    
      sucheRegAusdruck(items, startItem, pat, up);
    }

  }

  /**
   * Suchen nach dem ersten Vorkommen des Patterns des regulren Ausdruckes in den TreeItems ab dem StartItem
   * aufwrts oder abwrts
   * 
   * @param ti            TreeItems, die durchsucht werden sollen 
   * @param startItem     StartItems, ab dem gesucht werden soll
   * @param pat           Pattern eines regulren Ausdrucks
   * @param up            Richtung der Suche          
   * @return
   */
  private boolean sucheRegAusdruck(TreeItem[] items, TreeItem startItem, Pattern pat, int up)
  {
    if ( up == SWT.UP )
    {
      for (int i = 0; i < items.length; i++)
      {     
        TreeItem ti = items[i];
        
        TreeItem parent = ti.getParentItem();
        if (parent != null && !parent.getExpanded())
        {
          continue;
        }
        
        if ( _startItemGefunden )
        {
          Matcher mat = pat.matcher(ti.getText());
          if ( ti.getText().contains(pat.pattern()) || mat.find() )
          {
            _treeViewer.getTree().setSelection(ti);
            _treeViewer.setSelection(_treeViewer.getSelection(), true);
  
            return true;
          }
        }
        else if ( startItem == null || items[i].equals(startItem) )
            _startItemGefunden = true;

        TreeItem[] tiItems = ti.getItems();
        if (tiItems != null)
        {
          if (sucheRegAusdruck(tiItems, startItem, pat, up))
            return true;
        }
      }
    }
    else
    {
      for (int i = items.length-1; i >= 0; i--)
      {     
        TreeItem ti = items[i];
        
        TreeItem parent = ti.getParentItem();
        if (parent != null && !parent.getExpanded())
        {
          continue;
        }
        
        TreeItem[] tiItems = ti.getItems();
        if (tiItems != null)
        {
          if (sucheRegAusdruck(tiItems, startItem, pat, up))
            return true;
        }
        
        if ( _startItemGefunden )
        {
          Matcher mat = pat.matcher(ti.getText());
          if ( ti.getText().contains(pat.pattern()) || mat.find() )
          {
            _treeViewer.getTree().setSelection(ti);
            _treeViewer.setSelection(_treeViewer.getSelection(), true);
  
            return true;
          }
        }
        else if ( startItem == null || ti.equals(startItem) )
            _startItemGefunden = true;
      }
    }    
    
    return false;
  }
  
  private TreeItem getNextItem(TreeItem[] items, TreeItem item, int relIndex)
  {
    
    for (int i = 0; i < items.length; i++)
    {
      TreeItem ti = (TreeItem) items[i];         
                
      if (ti == item)
      {                    
         int nextIndex = i + relIndex;
         if ( nextIndex < 0 )
           nextIndex = 0;
         else if (nextIndex > items.length-1 )
           nextIndex = items.length-1;

         return items[nextIndex];
      
      }
      
      TreeItem[] tiItems = ti.getItems();
      if (tiItems != null)
      {
        TreeItem nextItem = getNextItem(tiItems, item, relIndex);
        if (nextItem != null)
          return nextItem;
      }
    }
    return null;
     
  }
  
  /**
   * Rekursive Suche und Selektion nach einem Baum-Element
   * ber den Namen des Baum-Elements
   * 
   * @param items   Baum-Elemente, die durchsucht werden sollen
   * @param parent  Elternteil der Baum-Elemente oder null
   * @param item    zu selektierendes Baum-Element oder null
   * @param itemText  Name des Baum-Elements, das gesucht werden soll
   * @return gefunden
   */
  private boolean sucheSelektiereItem(TreeItem[] items, TreeItem parent, TreeItem item, String itemText)
  {
      
    if (item != null)
    {
      _treeViewer.getTree().setSelection(item);
      _treeViewer.setSelection(_treeViewer.getSelection(), true);
      if (parent == null)
        parent = item.getParentItem();
      _treeViewer.expandToLevel(parent, 1);
      return true; 
    }

    boolean gefunden = false;
    
    // Suche nach dem ersten Vorkommen des Namens im Baum
    for (TreeItem ti : items)
    {               
      if (ti.getText().startsWith(itemText))
      {
        gefunden = sucheSelektiereItem(items, parent, ti, itemText);
        if (gefunden)
          break;
      }
      
      if (ti != null && !ti.isDisposed()){
        TreeItem[] tiItems = ti.getItems();
        if (tiItems != null)
        {
          gefunden = sucheSelektiereItem(tiItems, ti, null, itemText);
          if (gefunden)
            break;
        }        
      }
    }  
    
    return gefunden;
     
  }

  /**
   * Hierarchie setzen
   * 
   * @param hierachie
   */
  public void setHierarchie(KBHierarchie hierachie)
  {
    _hierarchie = hierachie;
    if ( _hierarchie != null )
    {
      setMaxEbene( _hierarchie.getAnzahlEbenen() );
      
      _hierarchie.beobachterAnmelden(new IHierarchieBeobachter(){

        @Override
        public void hierarchieAktualisiert(EVENT event, ConfigurationArea kb, Object vater, HierarchieObjekt ho)
        {       
          if (_debug)
            System.out.println("NavigatorComposite:setHierarchie:: event: " + event 
                + "\n\tKB: " + kb.getPid() + " Vater: " + vater + "/" + vater.hashCode() + " HO: " + ho + "/" + ho.hashCode());
          
          AbstractNavigatorElement root = (AbstractNavigatorElement)_treeViewer.getInput();
          
          AbstractNavigatorElement kbNe = sucheItem(root, kb, false);
          AbstractNavigatorElement parentNe = kbNe;
          if (kb != vater)
            parentNe = sucheItem(kbNe, vater, true);
          
          if (parentNe == null)
            return;
          
          switch (event)
          {
            case OBJEKT_NEU:
              erzeugeNeuesHONavigatorElement(parentNe, ho, false);
              refreshNavigator(parentNe);
              break;
            case OBJEKT_ENTFERNT:
              entferneNavigatorElement(parentNe, ho);
              refreshNavigator(parentNe);
              break;
            case OBJEKT_VERSCHOBEN:
              ConfigurationArea neuerKb = _hierarchie.getKonfigurationsBereich(ho);
              if (neuerKb != kb)
                kbNe = sucheItem(root, neuerKb, false);
              Object neuerVater = _hierarchie.getUebergeordnetesObjekt(ho);
              AbstractNavigatorElement neuerParentNe = kbNe;
              if (neuerKb != neuerVater)
                neuerParentNe = sucheItem(kbNe, neuerVater, true);
              AbstractNavigatorElement hoNe = sucheItem(parentNe, ho, false);
              
              verschiebeNavigatorElement(parentNe, neuerParentNe, hoNe);
              refreshNavigator(neuerParentNe);
              refreshNavigator(parentNe);
            case OBJEKT_AKTUALISIERT:
              refreshNavigator(parentNe);
            default:
              break;
          }
        }
        
      });
    }
  }

  /**
   * Maximale Ebenen setzen
   * 
   * @param max maximale Ebene
   */
  private void setMaxEbene(int max)
  {
    _maxEbene = max;
    _spinner.setMaximum(_maxEbene);
  }
  
  /**
   * Baum-Input setzen
   * 
   * @param input
   */
  public synchronized void setNavigatorInput(INavigatorElement input)
  {
//    System.out.println("=======>>>>>>>>>>>>> Navigator-Input aktualisieren!");
    
    _treeViewer.setInput(input);
    if ( _hierarchie != null )
      setMaxEbene( _hierarchie.getAnzahlEbenen() );
    else{
      setMaxEbene( 1 );      
    }
    refreshNavigator();
  }
   
  /**
   * Aktion im Kontextmen setzen
   * 
   * @param action  Aktion die im Kontextmen erscheinen soll
   */
  public void addtoKontextMenue(Action action)
  {
    if ( action != null )
      _kontextMenuMgr.add(action);
  }

  /**
   * Drag-Aktion fr den entsprechenden DND-Modus setzen
   *
   * @param operation DND-Operations-Modus (COPY, COPY_DEEP, MOVE)
   * @param action    Drag-Aktion, die bei der entsprechende Operation ausgefhrt werden soll
   */
  public void setDragAction(DNDModus operation, Action action)
  {
    if ( action != null )
      _navigatorDropListener.setDragAction(operation, action);
  }
  
  /**
   * Drop-Aktion setzen
   *
   * @param action    Drop-Aktion, die ausgefhrt werden soll
   */
  public void setDropAction(MyAction action)
  {
    if ( action != null )
      _navigatorDropListener.setDropAction(action);
  }  
  
  /**
   * Liefert die Selektion im Navigator-Baum 
   * 
   * @return
   */
  public ISelection getNavigatorSelection()
  {
    return _treeViewer.getSelection();
  }

  /**
   * Suchen eines {@link HONavigatorElement}-Baum-Elementes ber sein Objekt bei den Kindern des Elternteils
   * 
   * @param parent  {@link INavigatorElement}-Elternteil
   * @param objekt  {@link Object}-Element
   * @param rekursiv rekursive Suche auch bei den Kindern der Kinder gewnscht
   * @return gesuchtes {@link HONavigatorElement}-Element 
   */
  public HONavigatorElement sucheItem(INavigatorElement parent, Object objekt, boolean rekursiv)
  {
    AbstractNavigatorElement absNe = sucheItem((AbstractNavigatorElement)parent, objekt, rekursiv);
    if (absNe instanceof HONavigatorElement)
      return (HONavigatorElement) absNe;
    else
      return null;
  }
  
  /**
   * Suchen eines Baum-Elementes ber sein Objekt bei den Kindern des Elternteils
   * 
   * @param parent  {@link AbstractNavigatorElement}-Elternteil
   * @param objekt  {@link Object}-Element
   * @param rekursiv rekursive Suche auch bei den Kindern der Kinder gewnscht
   * @return gesuchtes {@link AbstractNavigatorElement}-Element 
   */
  public AbstractNavigatorElement sucheItem(AbstractNavigatorElement parent, Object objekt, boolean rekursiv)
  {
    AbstractNavigatorElement ne = null;
    
    if (parent != null && objekt != null && parent.hasChildren())
    {
      Object[] children = parent.getChildren();
      for (Object child : children)
      {
        AbstractNavigatorElement childNe = (AbstractNavigatorElement)child;
        if (childNe.getObjekt() == objekt)
        {
          ne = childNe;
        }
        else if (rekursiv)
        {
          ne = sucheItem(childNe, objekt, rekursiv);
        }
        if (ne != null)
          break;
      }
    }

    return ne;
  }
  
  /**
   * Selektieren eines Baum-Elementes
   * 
   * @param element  Navigator-Element
   */
  public void selektiereItem(INavigatorElement element)
  {
    if (element != null)
    {
      _treeViewer.setSelection(new StructuredSelection(element), true);
      Object parentNE = element.getParent();
      _treeViewer.expandToLevel(parentNE, 1);     
    }
  }

  /**
   * Suchen und Selektieren eines Baum-Elementes ber sein Objekt - 
   * rekursive Suche bei den Kindern des bergebenen Elternteils
   * 
   * @param parent  {@link INavigatorElement}-Elternteil
   * @param objekt  Objekt des Baum-Elementes
   */
  public void sucheSelektiereItem(INavigatorElement parent, Object objekt)
  {
    if (parent instanceof AbstractNavigatorElement)
    {
      AbstractNavigatorElement ne = sucheItem((AbstractNavigatorElement)parent, objekt, true);
      selektiereItem(ne);
    }
  }
  
  /**
   * Suchen und Selektieren eines Baum-Elementes
   * 
   * @param itemText  Name des Baum-Elementes
   */
  public void sucheSelektiereItem(String itemText)
  {
    sucheSelektiereItem(_treeViewer.getTree().getItems(), null, null, itemText);
  }

  /**
   * Prfen,ob ein Vorgnger auf der gleichen Ebene zum aktuellen selektierten Baum-Element 
   * unter Beachtung der aktuellen Sortierung existiert.
   *
   * @return Vorgnger existiert/existiert nicht
   */
  public boolean hasVorgaenger()
  {
    TreeItem[] selection = _treeViewer.getTree().getSelection();
    if (selection.length == 0)
      return false;

    TreeItem parentItem = selection[0].getParentItem();
    return getElement(parentItem.getItems(), selection[0], -1) == null ? false : true;
    
//    IStructuredSelection selection = (IStructuredSelection)getNavigatorSelection();
//    INavigatorElement firstElement = (INavigatorElement)selection.getFirstElement();  
//    
//    return getElement(firstElement, -1) == null ? false : true;
  }

  /**
   * Prfen,ob ein Vorgnger auf der gleichen Ebene zum aktuellen selektierten Baum-Element 
   * unter Beachtung der aktuellen Sortierung existiert.
   *
   * @return Vorgnger existiert/existiert nicht
   */
  public boolean hasNachfolger()
  {
    TreeItem[] selection = _treeViewer.getTree().getSelection();
    if (selection.length == 0)
      return false;

    TreeItem parentItem = selection[0].getParentItem();
    return getElement(parentItem.getItems(), selection[0], 1) == null ? false : true;
    
//    IStructuredSelection selection = (IStructuredSelection)getNavigatorSelection();
//    INavigatorElement firstElement = (INavigatorElement)selection.getFirstElement();  
//    
//    return getElement(firstElement, 1) == null ? false : true;

  }
  
  /**
   * Ermitteln und Selektieren des Vorgngers eines Baum-Elementes
   * auf der gleichen Ebene zum aktuellen selektierten Element
   * unter Beachtung der aktuellen Sortierung.
   *
   * @return Vorgnger-Baum-Element
   */
  public INavigatorElement getVorgaenger()
  {
    TreeItem[] selection = _treeViewer.getTree().getSelection();
    if (selection.length == 0)
      return null;

    TreeItem parentItem = selection[0].getParentItem();
    return getElement(parentItem.getItems(), selection[0], -1);
    
//    IStructuredSelection selection = (IStructuredSelection)getNavigatorSelection();
//    INavigatorElement firstElement = (INavigatorElement)selection.getFirstElement();  
//    
//    return getElement(firstElement, -1);
  }

  /**
   * Ermitteln und Selektieren des Nachfolgers eines Baum-Elementes
   * auf der gleichen Ebene zum aktuellen selektierten Element
   * unter Beachtung der aktuellen Sortierung.
   *
   * @return Nachfolger-Baum-Element
   */
  public INavigatorElement getNachfolger()
  {
    TreeItem[] selection = _treeViewer.getTree().getSelection();
    if (selection.length == 0)
      return null;

    TreeItem parentItem = selection[0].getParentItem();
    return getElement(parentItem.getItems(), selection[0], 1);
    
//    IStructuredSelection selection = (IStructuredSelection)getNavigatorSelection();
//    INavigatorElement firstElement = (INavigatorElement)selection.getFirstElement();  
//    
//    return getElement(firstElement, 1);
  }
  
  /**
   * Ermitteln und Selektieren eines Baum-Elementes
   * auf der gleichen Ebene in Relation zu einem Start-Element
   * unter Beachtung der aktuellen Sortierung.
   * Vorgnger (relIndex = -1), Nachfolger (relIndex = 1)
   * 
   * @param items      Baum-Elemente, die durchsucht werden sollen
   * @param startItem  Start-Element 
   * @param relIndex   relativer Index 
   * @return relatives Baum-Element - null, falls Keines existiert
   */
  private INavigatorElement getElement(TreeItem[] items, TreeItem startItem, int relIndex)
  {
    TreeItem nextItem = getNextItem(items, startItem, relIndex);
    INavigatorElement hoNe = null;

    if ( nextItem == null )
    {
      // auf Grund einer Pid-nderung kann die Suche im Tree negativ verlaufen, da der Text im Tree noch nicht aktualisiert ist
      // ==> dann doch Suche in den NavigatorElementen
      // da dann der Bearbeitungsstatus nicht auf Darstellung steht, ist dies jedoch berflssig
//      hoNe = getElementNe(ne, relIndex);      
    }
    else if ( nextItem.getData() instanceof INavigatorElement )
    {
      hoNe = (INavigatorElement)(nextItem.getData());
    }
    
    return nextItem == startItem ? null : hoNe ;
  }
  
//  /**
//   * Ermitteln und Selektieren eines Baum-Elementes
//   * auf der gleichen Ebene in Relation zu einem Start-Element
//   * unter Beachtung der aktuellen Sortierung.
//   * Vorgnger (relIndex = -1), Nachfolger (relIndex = 1)
//   * 
//   * @param ne         Start-Element 
//   * @param relIndex   relativer Index 
//   * @return relatives Baum-Element
//   */
//  public INavigatorElement getElementNe(INavigatorElement ne, int relIndex)
//  {
//    INavigatorElement parentNe = (INavigatorElement)ne.getParent();
//        
//    List<Object> children = Arrays.asList( parentNe.getChildren());
//    
//    int nextIndex = children.indexOf(ne) + relIndex * _sortRichtung;
//    if ( nextIndex < 0 )
//      nextIndex = 0;
//    else if (nextIndex > children.size() )
//      nextIndex = children.size()-1;
//      
//    return (INavigatorElement) children.get( nextIndex );
//    
//  }  
  
  /**
   * Eine Baumebene des Navigators zum NavigatorElement ausklappen
   * 
   * @param ne  Element des Navigators
   */
  public void expandNavigator(INavigatorElement ne)
  {
    if (!_treeViewer.getExpandedState(ne))
    {
      _treeViewer.setExpandedState(ne, true);
      _treeViewer.refresh(ne, false);
    }
    else
      _treeViewer.refresh(ne, false);
  }

  /**
   * Navigator aktualisieren
   */
  public void refreshNavigator()
  {
    _treeViewer.refresh();
  }
  
  /**
   * Navigator zum NavigatorElement aktualisieren
   * 
   * @param ne  Element des Navigators
   */
  public void refreshNavigator(INavigatorElement ne)
  {
    if ( ne != null )
      _treeViewer.refresh(ne);
    else
      _treeViewer.refresh();
  }
  
  /**
   * Baumstruktur initialisieren
   * 
   * @param parentElement
   * @param parentData
   */
  public void initialisiereBaumStruktur(Object parentElement, Object parentData)
  {
//    if (_debug)
//      System.out.println("=========>>>>>>>> Navigator-Baum initialisieren zu: " + ((INavigatorElement)parentElement).getName());
    
    if (_hierarchie != null){
      
      if (parentData == null) // Anfang mit Baumwurzel => Konfigurationsbereiche
      {
        for (ConfigurationArea kb : _hierarchie.getKonfigurationsBereiche())
        {
          INavigatorElement element = new KBNavigatorElement(parentElement, kb.getPid(), null, kb);
          element.setImagePath("KB");
          
          //Status fr nderung erlaubt ermitteln
          if ( HierarchieObjekt.isSystemObjektAenderbar(kb) )
            element.setStatus(STATUS.aenderbar);
          else
            element.setStatus(STATUS.nichtAenderbar);

          initialisiereBaumStruktur(element, kb);
        }
      }
      else
      {
        for (HierarchieObjekt ho : _hierarchie.getUntergeordneteObjekte(parentData))
        {
          
          String hoTyp = ho.getHierarchieObjektTyp().getSymbolName();
          
          INavigatorElement element = new HONavigatorElement(parentElement, ho.getSystemObjekte().get(0).getPid(), null,
              ho);
          element.setImagePath(hoTyp);
          
          setStatus(element, ho);
          
          initialisiereBaumStruktur(element, ho);
        }
      }
    }
    
  }
  
  /**
   * Setzt den Status des Navigatorelements entsprechend den Rechten von dessen Hierarchieobjekts
   * 
   * @param element
   * @param ho
   */
  public void setStatus(INavigatorElement element, HierarchieObjekt ho)
  {
    // Status fr nderung erlaubt ermitteln
    // nderungen fr HOT bzw. bergeordnetes Objekt erlaubt?
    IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
    TKABasisView view = TKABasisView.getView(_secondViewId);
    IHierarchieManager hierarchieManager = view.getModelInfo().getHierarchieManager();
    Object ueObj = hierarchieManager.getHierarchie().getUebergeordnetesObjekt(ho);
    if (ueObj instanceof HierarchieObjekt)
    {
      HierarchieObjekt ueHo = (HierarchieObjekt)ueObj;
      if (!HierarchieObjekt.isSystemObjektAenderbar(ueHo.getSystemObjekt(0)))
      {
        // Berechtigung fr jegliche nderungen zum Parent nicht vorhanden
        element.setStatus(STATUS.nichtAenderbar); 
        return;
      }      
    }
    else if (ueObj instanceof ConfigurationArea)
    {
      ConfigurationArea ueKb = (ConfigurationArea)ueObj;
      if (!HierarchieObjekt.isSystemObjektAenderbar(ueKb))
      {
        // Berechtigung fr jegliche nderungen zum Parent nicht vorhanden
        element.setStatus(STATUS.nichtAenderbar); 
        return;
      }  
    }

    if (hot.isSystemObjektNichtAenderbar())
    {
      // Berechtigung fr teilweise nderungen (Verschieben usw.) vorhanden, keine direkte nderung
      element.setStatus(STATUS.teilweiseAenderbar);      
    }

    STATUS status = element.getStatus();
       
    // Berechtigung fr nderungen der SO vorhanden?
    Object[] objs = (ho.getSystemObjekte()).toArray();
    for (int i = 0; i < objs.length; i++)
    {
      SystemObject so = (SystemObject)objs[i];
      if (HierarchieObjekt.isSystemObjektAenderbar(so))
      {
        if (status == null || status == STATUS.unbekannt)
          status = STATUS.aenderbar;
        else if (status == STATUS.nichtAenderbar)
          status = STATUS.teilweiseAenderbar;
      }
      else
      {
        if (status == null || status != STATUS.nichtAenderbar)
          status = STATUS.teilweiseAenderbar;
      }

      if (element.getStatus() == STATUS.teilweiseAenderbar)
        break;        
    }
    element.setStatus(status);
  }
  
  /**
   * Erzeugt ein {@link HONavigatorElement} zum {@link HierarchieObjekt}
   * 
   * @param parentNe  {@link INavigatorElement}-Elternteil
   * @param neu       neues {@link HierarchieObjekt}
   * @param rekursiv  rekursive Neuanlage auch zu den Kindern gewnscht
   * @return  erzeugtes neues {@link HONavigatorElement}
   */
  private HONavigatorElement erzeugeNeuesHONavigatorElement(INavigatorElement parentNe, HierarchieObjekt neu,
      boolean rekursiv)
  {

    if (neu != null)
    {
      String typ = neu.getHierarchieObjektTyp().getSymbolName();

      HONavigatorElement neuesHoNe = new HONavigatorElement(parentNe, neu.getSystemObjekte().get(0).getPid(), null, neu);
      neuesHoNe.setImagePath(typ);
      // nderungen fr HOT und Systemobjekt erlaubt?
      setStatus(neuesHoNe, neu);
      
      // folgendes wurde ersetzt durch setStatus
//      IHierarchieObjektTyp hot = neu.getHierarchieObjektTyp();     
//      SystemObject so = neu.getSystemObjekt(0);
//      if (hot.isSystemObjektNichtAenderbar())
//        neuesHoNe.setStatus(STATUS.teilweiseAenderbar);
//      else if (HierarchieObjekt.isSystemObjektAenderbar(so))
//        neuesHoNe.setStatus(STATUS.aenderbar);
//      else
//        neuesHoNe.setStatus(STATUS.nichtAenderbar);
      
//      this.expandNavigator(parentNe);

      if (rekursiv)
      {  
        TKABasisView view = TKABasisView.getView(_secondViewId);
        IHierarchieManager hm = view.getModelInfo().getHierarchieManager();
        IHierarchie hierarchie = hm.getHierarchie();
  
        for (HierarchieObjekt kind : hierarchie.getUntergeordneteObjekte(neu))
        {
          erzeugeNeuesHONavigatorElement(neuesHoNe, kind, true);
        }
      }  

      return neuesHoNe;
    }
    return null;
  }

  public void entferneNavigatorElement(INavigatorElement parent, Object objekt)
  {
    Object[] children = parent.getChildren();
    for (Object child : children)
    {
      AbstractNavigatorElement childNe = (AbstractNavigatorElement)child;
      if (childNe.getObjekt() == objekt)
      {
        parent.remove(childNe);
//        System.out.println("ChildNe gelscht: " + childNe.getName() );
        break;
      }
    }
  }
  
  public void entferneNavigatorElement(INavigatorElement ne)
  {
    INavigatorElement parent = (INavigatorElement)ne.getParent();
    parent.remove(ne);
  }
  

  private void verschiebeNavigatorElement(AbstractNavigatorElement parentNe, AbstractNavigatorElement neuerParentNe,
      AbstractNavigatorElement hoNe)
  {
    parentNe.remove(hoNe);
    neuerParentNe.add(hoNe);
  }
  
  @Deprecated
  public HONavigatorElement synchronisiereMitHierarchie(INavigatorElement parentEl, HierarchieObjekt selHO,
      boolean rekursiv)
  {
//    System.out.println("TKABasisComposite.synchronisiereMitHierarchie() zu Parent: " + parentEl.getName() + " HO: " + selHO);
    
    HONavigatorElement result = null;
    Object objekt = null;
    
    if (parentEl instanceof KBNavigatorElement)
    {
      objekt = ((KBNavigatorElement)parentEl).getKonfigurationsBereich();
    }
    else if (parentEl instanceof HONavigatorElement)
    {
      objekt = ((HONavigatorElement)parentEl).getHierarchieObjekt();
    }
    
    if (objekt != null)
    {
      TKABasisView view = TKABasisView.getView(_secondViewId);
      IHierarchieManager hm = view.getModelInfo().getHierarchieManager();
      IHierarchie hierarchie = hm.getHierarchie();
      
      Set<HierarchieObjekt> mengeHO = new HashSet<HierarchieObjekt>(); 
      
//      List<HierarchieObjekt> uos = hierarchie.getUntergeordneteObjekte(objekt);
//      
//      for (HierarchieObjekt ho : uos)
//      {
//        System.out.println("untergeordnetesHO: " +  ho);
//      }
      
      
      mengeHO.addAll(hierarchie.getUntergeordneteObjekte(objekt));
      
      List<Object> listEl = new LinkedList<Object>();
      listEl.addAll(Arrays.asList(parentEl.getChildren()));
      
      for (Object item: listEl)
      {
        if (item instanceof HONavigatorElement)
        {
          HONavigatorElement kindEl = (HONavigatorElement)item;
          HierarchieObjekt ho = kindEl.getHierarchieObjekt();
          if (mengeHO.contains(ho))
          {
            if (selHO != null && selHO.equals(ho))
            {
              result = kindEl;
            }
            mengeHO.remove(ho);
          }
          else
          {
            // Fr dieses Navigator-Element existiert kein HO mehr in der Hierarchie
            
//            System.out.println("remove NE: " + kindEl.getName());
            
            parentEl.remove(kindEl);
          }
        }
      }
     
      // HOs, die jetzt noch in mengeHO enthalten sind, sind noch nicht als
      // Navigator-Elemente im Navigator
      for (HierarchieObjekt ho: mengeHO)
      {
        String typ = ho.getHierarchieObjektTyp().getSymbolName();

        HONavigatorElement kindEl = new HONavigatorElement(parentEl, ho.getSystemObjekte().get(0).getPid(), null, ho);
        kindEl.setImagePath(typ);
        this.setStatus(kindEl, ho);
        
        if (selHO != null && selHO.equals(ho))
        {
          result = kindEl;
        }
      }
      
      if (rekursiv)
      {  
        for (Object kind: parentEl.getChildren())
        {
          if (kind instanceof INavigatorElement)
          {
            HONavigatorElement ret = synchronisiereMitHierarchie((INavigatorElement)kind, selHO, rekursiv);
            if (ret != null)
            {
              result = ret;
            }
          }
        }
      }
    }
    
    return result;
  }
  
  
}
