/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.navigator;

import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.dnd.DragSourceAdapter;
import org.eclipse.swt.dnd.DragSourceEvent;

import de.bsvrz.buv.plugin.tkabasis.navigator.navigatorModell.HONavigatorElement;

/**
 * Liefert den {@link DragSourceAdapter} fr die Elemente des TreeViewers im
 * {@link NavigatorComposite}.
 * 
 * @author Krause
 *
 */
public class NavigatorDragListener extends DragSourceAdapter
{
  /**
   * Flag zum Debuggen
   */
  protected static final boolean _debug = false;
  
  /**
   * Viewer, auf dem gearbeitet wird.
   */
  private final TreeViewer _viewer;
  
  
  public NavigatorDragListener(TreeViewer viewer)
  {
    this._viewer = viewer;
  }
  
  /**
   * Wird whrend des Beenden der Drag-Vorgangs aufgerufen.
   */
  @Override
  public void dragFinished(DragSourceEvent event)
  {
    if (_debug)
      System.out.println("Finished Drag! EventDetail: " + event.detail);
  }

  /**
   * Liefert die per dragging selektierten Elemente des TreeViewers.
   */
  @Override
  public void dragSetData(DragSourceEvent event)
  {  
    //event.data wird beim Drop nicht weiter verwendet, 
    //muss jedoch wegen des transferTypes einen Text enthalten
    event.data = " ";
      
  }

  /**
   * Wird whrend des Starts der Drag-Vorgangs aufgerufen
   * und beinhaltet fr den Drag nicht selektierbare Elemente
   */
  @Override
  public void dragStart(DragSourceEvent event)
  {
    if (_debug)
      System.out.println("\nStart Drag! EventDetail: " + event.detail);

    IStructuredSelection selection = (IStructuredSelection)_viewer.getSelection();
    Object[] selArray = selection.toArray();
    for (int i = 0; i < selArray.length; i++)
    {
      Object selObj = selArray[i];
      // kein Drag fr nicht nderbare HONavigatorElement oder KBNavigatorElement
      if (selObj instanceof HONavigatorElement)
      {
        HONavigatorElement ne = (HONavigatorElement)selObj;
        if (ne.isVerschiebbar())
          event.doit = true;
        else
          event.doit = false;
      }
      else
        event.doit = false;
    }
  }
  
}
