/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.navigator.navigatorModell;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Image;

import de.bsvrz.buv.plugin.tkabasis.Activator;

/**
 * Fabrik zur Erstellung der bentigten Bilder, damit diese
 * nicht mehrfach erstellt werden.
 * 
 * @author Krause
 *
 */
public class ImageFactory
{ 
  /**
   * Singleton-Instanz
   */
  private static ImageFactory _instanz = null;

  /*
   * Hashmap, die die Bilder beinhaltet
   */
  private Map<String, Image> _imageRegistry;

  /**
   * Liefert die Singletoninstanz.
   * 
   * @return Instanz dieser Klasse.
   */
  public static ImageFactory getInstanz()
  {
    if (_instanz == null)
    {
      _instanz = new ImageFactory();
    }
    return _instanz;
  }

  /**
   * Privater Konstruktor
   */
  private ImageFactory()
  {
    _imageRegistry = new HashMap<String, Image>();
    loadImages();
  }

  /**
   * Fgt die Bilder der Registry hinzu
   * 
   */
  private void loadImages()
  {
    ImageDescriptor id = Activator.getImageDescriptor("icons/items/Default_unb.png");
    Image imageDefault = id.createImage(); 
    _imageRegistry.put("icons/items/Default_unb.png", imageDefault);
    
//    id = Activator.getImageDescriptor("icons/items/KB_unb.png");
//    _imageRegistry.put("icons/items/KB_unb.png", id.createImage());
//
//    id = Activator.getImageDescriptor("icons/items/MQ_unb.png");
//    _imageRegistry.put("icons/items/MQ_unb.png", id.createImage());
//    
//    id = Activator.getImageDescriptor("icons/items/FS_unb.png");
//    _imageRegistry.put("icons/items/FS_unb.png", id.createImage());
    
  }

/**
 * Liefert das Bild zum gegebenen Bildpfad.
 * 
 * @param imageFilePath Pfad zum Bild
 * @return
 */
  public Image getImage(String imageFilePath)
  {

    Image image = _imageRegistry.get(imageFilePath);

    if ( image == null )
    {
      setImage(imageFilePath);
      image = _imageRegistry.get(imageFilePath);
    }

    return image;

  }

  /**
   * Registrieren eines Bildes im Register, falls vorhanden,
   * ansonsten wird ein Default-bild eingestellt.
   * 
   * @param imageFilePath Pfad zum Bild
   */
  private void setImage(String imageFilePath)
  {
    ImageDescriptor id = Activator.getImageDescriptor(imageFilePath);
    Image image = null;
    if ( id != null )
      image = id.createImage();
    else
      image = _imageRegistry.get("icons/items/Default_unb.png");
    _imageRegistry.put(imageFilePath, image);
  }

}
