/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.pruefungen;

import org.eclipse.core.runtime.IStatus;

import de.bsvrz.buv.plugin.tkabasis.hierarchie.AtgInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HODaten;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HierarchieObjekt;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchie;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Schnittstelle fr die Prfungen der Plausibilitt, Konstistenz und von Attributwerten
 * 
 * @author Sans
 *
 */
public interface IPruefungen
{
  /**
   * Bestimmt die Hierarchie, die fr die Prfungen benutzt werden kann
   * 
   * @return die Hierarchie, die fr die Prfungen benutzt werden kann
   */
  public IHierarchie getHierarchie();
  
  /**
   * Setzt die Hierarchie, die fr die Prfungen benutzt werden kann
   * @param hierachie die Hierarchie, die fr die Prfungen benutzt werden kann
   */
  public void setHierarchie(IHierarchie hierachie);
  
 
  /**
   * Prft die Daten, die fr ein Hierarchieobjekt bernommen werden sollen
   * 
   * @param ho das Hierarchieobjekt, fr das Daten bernommen werden sollen
   * @param daten die zu bernehmenden Daten
   * @return eine Instanz vom Typ IStatus, die den Status der Prfung und alle 
   * Informationen enthlt
   */
  public IStatus pruefeHierarchieObjekt(HierarchieObjekt ho, HODaten daten);
  
  /**
   * Prft die Daten einer Attributgruppe, die fr eine Systemobjekt eines
   * Hierarchieobjekts bernommen werden sollen
   * 
   * @param ho das Hierarchieobhekt, zu dem das bergebene Systemobjekt gehrt
   * @param so das Systemobjekt, zu dem ATG-Daten bernommen werden sollen
   * @param atgInfo Info ber die ATG der zu bernehmenden Daten
   * @param daten Datensatz der Daten, die geprft werden sollen
   * @return eine Instanz vom Typ IStatus, die aussagt, ob es zulssig ist, die 
   * bergebenen ATG-Daten fr das Systemobjekt zu bernehmen.
   */
  public IStatus pruefeDaten(HierarchieObjekt ho, SystemObject so, AtgInfo atgInfo, Data daten);
}
