/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.regeln;

import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchWizard;

import de.bsvrz.buv.plugin.tkabasis.hierarchie.HierarchieObjekt;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieObjektTyp;

/**
 * Wizard-Klasse, die einen Wizard fr den Eingabedialog von Werten fr die Bildung
 * von PID und Namen von Systemobjekten gemss Bildungsregeln implementiert
 */
public class BildungsRegelnAssistent extends Wizard implements INewWizard
{
  /**
   * Wizard Seite zur Eingabe von Werten
   */
	private BildungsRegelnEingabe _bildungsRegelnStartSeite;	

	/**
	 * Erzeugt den Wizard mit Eingabedialog fr Werte, aus denen mit den Bildungsregeln
	 * PID und Name von Systemobjekten gebildet werden knnen
	 * 
	 * @param hierarchieObjektTyp der Hierarchieobjekttyp des Hierarchieobjekts, fr das
	 * PID und Name mit den Bildungsregeln gebildet werden sollen
	 * @param modell die BildungsRegelnMitKB-Instanz, die mit den Eingabewerten versorft werde soll
	 * @param uebergeordnetesObjekt das in der Hierarchie bergeordnete Objekt
   * @param zuKopierendesHO Falls der Assistent zum Kopieren von HO benutzt wird, kann hier 
   * das zu kopierende Objekt angegeben werden - dessen Daten dann zustzlich angezeigt werden
   * @param werteVerwenden gibt an, ob die Felder mit im Modell vorhandenen Werten initialisiert
   * werden
	 */
	public BildungsRegelnAssistent(IHierarchieObjektTyp hierarchieObjektTyp, BildungsRegelnMitKB modell,
	    Object uebergeordnetesObjekt, HierarchieObjekt zuKopierendesHO, boolean werteVerwenden) 
	{
		super();
		setWindowTitle("Eingabe von Daten");
		
		_bildungsRegelnStartSeite = new BildungsRegelnEingabe(hierarchieObjektTyp, modell, uebergeordnetesObjekt,
		    zuKopierendesHO, werteVerwenden);
		
		 addPage(_bildungsRegelnStartSeite);
	}
	
	/**
   * Erzeugt den Wizard mit Eingabedialog fr Werte, aus denen mit den Bildungsregeln
   * PID und Name von Systemobjekten gebildet werden knnen
   * 
   * @param hierarchieObjektTyp der Hierarchieobjekttyp des Hierarchieobjekts, fr das
   * PID und Name mit den Bildungsregeln gebildet werden sollen
   * @param modell die BildungsRegelnMitKB-Instanz, die mit den Eingabewerten versorft werde soll
   * @param uebergeordnetesObjekt das in der Hierarchie bergeordnete Objekt
   * @param zuKopierendesHO Falls der Assistent zum Kopieren von HO benutzt wird, kann hier 
   * das zu kopierende Objekt angegeben werden - dessen Daten dann zustzlich angezeigt werden
   */
  public BildungsRegelnAssistent(IHierarchieObjektTyp hierarchieObjektTyp, BildungsRegelnMitKB modell,
      Object uebergeordnetesObjekt, HierarchieObjekt zuKopierendesHO) 
  {
    this(hierarchieObjektTyp, modell, uebergeordnetesObjekt, zuKopierendesHO, false);
  }
	
	/**
	 * @see IWorkbenchWizard#init(IWorkbench, IStructuredSelection)
	 */
	public void init(IWorkbench workbench, IStructuredSelection selection) 
	{
  }

	/**
	 * Die Daten des Bildungsregel-Assistenten werden in sein Datenmodell, also in die
	 * IBildungsregel-Instanz bernommen
	 * 
	 * @return <code>true</code>, wenn die Daten bernommen werden konnten, <code>false</code>
	 * sonst
	 */
	public boolean performFinish() 
	{
	  if (canFinish())
	  {  
	    _bildungsRegelnStartSeite.schreibeDatenInModell();
	    return true;
	  }
	  else
	  {
	    return false;
	  }
	}
	
}
