/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.regeln;

import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;

import org.eclipse.jface.dialogs.IDialogPage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.VerifyEvent;
import org.eclipse.swt.events.VerifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;

import de.bsvrz.buv.plugin.tkabasis.hierarchie.HierarchieObjekt;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchie;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieObjektTyp;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;

/**
 * Wizard Seite des Bildungsregel-Assistenten
 * 
 * @author Sans
 *
 */
public class BildungsRegelnEingabe extends WizardPage implements Listener, BildungsRegelnBeobachter
{
  
  /**
   * Der Hierarchieobjekttyp des Hierarchieobjekts, fr das PID und Name mit den Bildungsregeln 
   * gebildet werden sollen
   */
  private IHierarchieObjektTyp _hierarchieObjektTyp;
  
  /**
   * IBildungsregel-Instanz - diese fungiert als Datenmodell fr den Bildungsregel-Assistenten
   */
  private BildungsRegelnMitKB _modell;
  
  /**
   * Das in der Hierarchie bergeordnete Objekt
   */
  private Object _uebergeordnetesObjekt;
  
  /**
   * Zu kopierendes Hieraechieobjekt
   */
  private HierarchieObjekt _zuKopierendesHO;
        
  /**
   * Zuordnung der Controls dieser Wizard-Seite zu den Eingabefeldern
   */
  private Map<Control, String> _controls = new HashMap<Control, String>();
  
  /**
   * Menge der Eingabefelder, die Pflichtfelder sind
   */
  private Set<String> _pflichtFelder = new HashSet<String>();
  
  /**
   * Menge der auf nderungen von Eingabefeldern angemeldenten Instanzen vom Typ 
   * BildungsRegelnBeobachter
   */
  private Set<BildungsRegelnBeobachter> _beobachter = new LinkedHashSet<BildungsRegelnBeobachter>();
  
  /**
   * Flag, das angibt ob beim Erzeugen des Vorbelegungsdialogs die Felder mit bereits im Modell
   * vorhandenen Feldwerten vorbelegt wird
   */
  private boolean _werteVerwenden;
  
  /**
   * Erzeugt die Wizard Seite fr den Bildungsregel-Assistenten
   * 
   * @param hierarchieObjektTyp hotNeuesObjekt der Hierarchieobjekttyp des Hierarchieobjekts, fr das
   * PID und Name mit den Bildungsregeln gebildet werden sollen
   * @param modell BildungsregelnMitKB-Instanz - diese fungiert als Datenmodell fr den Bildungsregel-Assistenten
   * @param uebergeordentesObjekt das in der Hierarchie bergeordnete Objekt
   * @param zuKopierendesHO Falls der Sialog zum Kopieren von HO benutzt wird, kann hier 
   * das zu kopierende Objekt angegeben werden - dessen Daten dann zustzlich angezeigt werden
   * @param werteVerwenden gibt an, ob die Felder mit im Modell vorhandenen Werten initialisiert
   * werden
   */
  public BildungsRegelnEingabe(IHierarchieObjektTyp hierarchieObjektTyp, BildungsRegelnMitKB modell,
      Object uebergeordentesObjekt, HierarchieObjekt zuKopierendesHO, boolean werteVerwenden)
  {
    super("BildungsRegelnEingabe");
    setTitle("Erforderliche Daten zum Anlegen eines Objekts vom Typ " + hierarchieObjektTyp.getName());
    setDescription("Bildung PID: " + modell.getBildungsRegeln().getPidRegelText(hierarchieObjektTyp)
        + "\nBildung Name: " + modell.getBildungsRegeln().getNameRegelText(hierarchieObjektTyp)); 
       
    _hierarchieObjektTyp = hierarchieObjektTyp;
    _modell = modell;
    _uebergeordnetesObjekt = uebergeordentesObjekt;
    _zuKopierendesHO = zuKopierendesHO;
    _werteVerwenden = werteVerwenden;
  }
    
  /**
   * Erszeugt die Wizard Seite fr den Bildungsregel-Assistenten
   * 
   * @param hierarchieObjektTyp hotNeuesObjekt der Hierarchieobjekttyp des Hierarchieobjekts, fr das
   * PID und Name mit den Bildungsregeln gebildet werden sollen
   * @param modell BildungsregelnMitKB-Instanz - diese fungiert als Datenmodell fr den Bildungsregel-Assistenten
   * @param uebergeordentesObjekt das in der Hierarchie bergeordnete Objekt
   * @param zuKopierendesHO Falls der Sialog zum Kopieren von HO benutzt wird, kann hier 
   * das zu kopierende Objekt angegeben werden - dessen Daten dann zustzlich angezeigt werden

   */
  public BildungsRegelnEingabe(IHierarchieObjektTyp hierarchieObjektTyp, BildungsRegelnMitKB modell,
      Object uebergeordentesObjekt, HierarchieObjekt zuKopierendesHO)
  {
    this(hierarchieObjektTyp, modell, uebergeordentesObjekt, zuKopierendesHO, false);
  }

  /**
   * @see IDialogPage#createControl(Composite)
   */
  public void createControl(Composite parent)
  {
    Composite composite = new Composite(parent, SWT.NULL);
    
    // create the desired layout for this wizard page
    GridLayout gl = new GridLayout();
    gl.numColumns = 2;
    composite.setLayout(gl);
    
    if (_zuKopierendesHO != null)
    {
      initialsiereInfoKopie(composite);
    }  
        
    initialisiereKBs(composite);
    
    // Zwischenraum zum Datenbereich
    Composite filler = new Composite(composite, SWT.NONE);
    GridData gdFiller = new GridData();
    gdFiller.horizontalAlignment = GridData.FILL;
    gdFiller.grabExcessHorizontalSpace = true;
    gdFiller.verticalAlignment = GridData.CENTER;
    gdFiller.horizontalSpan = 2;
    gdFiller.heightHint = 10;
    filler.setLayoutData(gdFiller);    
    
    GridData gd = new GridData();
    gd.horizontalAlignment = GridData.FILL;
    gd.grabExcessHorizontalSpace = true;
    gd.verticalAlignment = GridData.CENTER;
    
    boolean focusSet = false;
    IBildungsRegeln bildungsRegeln = _modell.getBildungsRegeln();
    
    for (String feldName: bildungsRegeln.getEingabeFelder(_hierarchieObjektTyp))
    {
      Label label = new Label(composite, SWT.NONE);
      label.setText(feldName);
      
      HierarchieObjekt verbundenesHO = null;
      if (_uebergeordnetesObjekt instanceof HierarchieObjekt)
      {
        verbundenesHO = (HierarchieObjekt)_uebergeordnetesObjekt;
      }
      EingabeVorbelegung vorbelegung = bildungsRegeln.getEingabeVorbelegung(_hierarchieObjektTyp, 
          feldName, verbundenesHO);
      
      String wert = bildungsRegeln.getWert(_hierarchieObjektTyp, feldName);
      
      if (vorbelegung.getListe().length > 0)
      {
        Combo control = new Combo(composite, vorbelegung.isEditierbar() ? SWT.BORDER : SWT.BORDER | SWT.READ_ONLY);
        _controls.put(control, feldName);
        control.setLayoutData(gd);
        control.setItems(vorbelegung.getListe());
        control.setText(vorbelegung.getWert());
        
        if (_werteVerwenden && wert != null && !wert.isEmpty())
        {
          control.setText(wert);
        }
        
        control.addListener(SWT.Modify, this);
        
        if (!focusSet)
        {
          control.setFocus();
          focusSet = true;
        }
      }
      else
      {
        Text control = new Text(composite, vorbelegung.isEditierbar() ? SWT.BORDER : SWT.BORDER | SWT.READ_ONLY);
        _controls.put(control, feldName);
        control.setLayoutData(gd);
        control.setText(vorbelegung.getWert());
        
        if (_werteVerwenden && wert != null && !wert.isEmpty())
        {
          control.setText(wert);
        }
        
        control.addListener(SWT.Modify, this);
        
        if (!focusSet)
        {
          control.setFocus();
          focusSet = true;
        }
        
        if (vorbelegung.isNummer())
        {
          final Text text = control;
          final long min = vorbelegung.getMinNummer();
          final long max = vorbelegung.getMaxNummer();
          
          text.addVerifyListener(new VerifyListener() 
          {
            public void verifyText(VerifyEvent e) 
            {
              if (!Character.isDigit(e.character))
              {  
                if (!Character.isISOControl(e.character)) 
                {
                  e.doit = false;
                }
              }  
              else
              {
                String eingabe = text.getText();
                eingabe += e.character;
                if (!eingabe.isEmpty())
                {
                  try
                  {
                    long nummer = Long.parseLong(eingabe); 
                    if (nummer > max || nummer < min)
                    {
                      e.doit = false;
                    }
                  }
                  catch(NumberFormatException ex)
                  {
                    e.doit = false;
                  }
                }
              }
            }
          });
        }
      }
      
      if (!vorbelegung.isLeerErlaubt())
      {
        _pflichtFelder.add(feldName);
      }
    }
    
    bildungsRegeln.beobachterAnmelden(this);
    beobachterAnmelden(bildungsRegeln);
    
    setPageComplete(isVollstaendig());

    // set the composite as the control for this page
    setControl(composite);
    
  }

  /**
   * @see Listener#handleEvent(Event)
   */
  public void handleEvent(Event event)
  {
    String feldName = _controls.get(event.widget);
    
    if (feldName != null)
    {
      if (event.widget instanceof Text)
      {
        alleBeobachterInformieren(feldName, ((Text)event.widget).getText());
      }
      else if (event.widget instanceof Combo)
      {
        alleBeobachterInformieren(feldName, ((Combo)event.widget).getText());
      }
    }  
    
    setPageComplete(isVollstaendig());   
  }

  /**
   * Bestimmt, ob die Eingaben des Bildungsregel-Assistenten vollstndig sind, d. h.
   * ob die Finish-Schaltflche aktivierbar ist
   * 
   * @return <code>true</code> wenn die Eingaben vollstndig sind, <code>false</code> sonst
   */
  public boolean isVollstaendig()
  {
    boolean complete = true;
    
    for (Control control: _controls.keySet())
    {
      String text = BildungsRegelnTools.LEERER_WERT;
      
      String feldName = _controls.get(control);
      if (!_pflichtFelder.contains(feldName))
      {
        continue;
      }
      
      if (control instanceof Text)
      {
        text = ((Text)control).getText();
      }
      else if (control instanceof Combo)
      {
        text = ((Combo)control).getText();
      }
      
      if (text.trim().isEmpty())
      {
        complete = false;
        break;
      }
    }
    
    return complete;
  }
 
  @Override
  public void dispose()
  {
    super.dispose();
    IBildungsRegeln bildungsRegeln = _modell.getBildungsRegeln();
    beobachterAbmelden(bildungsRegeln);
    bildungsRegeln.beobachterAbmelden(this);
  }
  
  /**
   * @see BildungsRegelnBeobachter#wertGeaendert(String, String)
   */
  public void wertGeaendert(String feldName, String wert)
  {
    Control control = null;
    
    for (Control lauf: _controls.keySet())
    {
      String feld = _controls.get(lauf);
      if (feld != null && feld.equals(feldName))
      {
        control = lauf;
        break;
      }
    }
    
    if (control instanceof Text)
    {
      ((Text)control).setText(wert);
    }
    else if (control instanceof Combo)
    {
      ((Combo)control).setText(wert);
    }
  }
  
  /**
   * Schreibt die aktuellen Eingabedaten des Bildungsregel-Assistenten in das
   * Datenmodell, also in die IBildungsregeln-Instanz
   */
  public void schreibeDatenInModell()
  {
    for (Control control: _controls.keySet())
    {  
      String feldName = _controls.get(control);
      
      if (feldName != null)
      {
        if (control instanceof Text)
        {
          _modell.getBildungsRegeln().setWert(_hierarchieObjektTyp, feldName, ((Text)control).getText());
        }
        else if (control instanceof Combo)
        {
          _modell.getBildungsRegeln().setWert(_hierarchieObjektTyp, feldName, ((Combo)control).getText());
        }
      }  
    }  
  }
  
  /**
   * Meldet eine BildungsRegelnBeobachter-Instanz auf nderungen an einem Eingabefeld an
   * 
   * @param beobachter anzumeldende BildungsRegelnBeobachter-Instanz
   */  
  public void beobachterAnmelden(BildungsRegelnBeobachter beobachter)
  {
    _beobachter.add(beobachter);
  }
      
  /**
  * Meldet eine angemeldete BildungsRegelnBeobachter-Instanz ab
  * 
  * @param beobachter abzumeldende BildungsRegelnBeobachter-Instanz
  */
  public void beobachterAbmelden(BildungsRegelnBeobachter beobachter)
  {
    _beobachter.remove(beobachter);
  }
  
  /**
   * Informiert alle angemeldeten BildungsRegelnBeobachter-Instanzen ber eine
   * nderung in einem Eingabefeld
   * 
   * @param feldName der Name des Eingabefeldes
   * @param wert der genderte Wert des Eingabefeldes
   */
  public void alleBeobachterInformieren(String feldName, String wert)
  {
    for (BildungsRegelnBeobachter beobachter: _beobachter)
    {
      beobachter.wertGeaendert(feldName, wert);
    }
  }
  
  
  /**
   * Erzeugt und initialisiert die Controls, die Auskunft ber das zu kopierende Objekt 
   * geben, wenn der Vorbelegungsdialog zum Kopieren verwendet wird
   * 
   * @param composite das Composite, auf dem die Controls liegen
   */
  private void initialsiereInfoKopie(Composite composite)
  {
    Group grp = new Group(composite, SWT.NONE);
    grp.setText("Zu kopierendes Objekt");
    GridData gdGrp = new GridData();
    gdGrp.horizontalAlignment = GridData.FILL;
    gdGrp.grabExcessHorizontalSpace = true;
    gdGrp.verticalAlignment = GridData.CENTER;
    gdGrp.horizontalSpan = 2;
    grp.setLayoutData(gdGrp);
    
    GridLayout glGrp = new GridLayout();
    glGrp.numColumns = 2;
    grp.setLayout(glGrp);
    
    GridData gdTxt = new GridData();
    gdTxt.horizontalAlignment = GridData.FILL;
    gdTxt.grabExcessHorizontalSpace = true;
    gdTxt.verticalAlignment = GridData.CENTER;
    
    Label labelPid = new Label(grp, SWT.NONE);
    labelPid.setText("Pid:");
    
    Text textPid = new Text(grp, SWT.BORDER | SWT.READ_ONLY);
    textPid.setLayoutData(gdTxt);
    textPid.setText(_zuKopierendesHO.getSystemObjekt(0).getPid());
                
    Label labelName = new Label(grp, SWT.NONE);
    labelName.setText("Name:");
    
    Text textName = new Text(grp, SWT.BORDER | SWT.READ_ONLY);
    textName.setLayoutData(gdTxt);
    textName.setText(_zuKopierendesHO.getSystemObjekt(0).getName());
  }
  
  /**
   * Initialisiert die auswhlbaren Konfigurationsbereiche in der bergebenen Combobox
   * 
   * @param kbCombo die Combobox, in deren Liste die KB angezeigt werden
   */
  private void initialisiereKBs(Composite composite)
  {
    // Groupbox Konfigurationsbereich
    Group grpKB = new Group(composite, SWT.NONE);
    grpKB.setText("Konfigurationsbereich");
    GridData gdGrpKB = new GridData();
    gdGrpKB.horizontalAlignment = GridData.FILL;
    gdGrpKB.grabExcessHorizontalSpace = true;
    gdGrpKB.verticalAlignment = GridData.CENTER;
    gdGrpKB.horizontalSpan = 2;
    grpKB.setLayoutData(gdGrpKB);
    
    // Groupbox Layout
    GridLayout glGrp = new GridLayout();
    glGrp.numColumns = 2;
    grpKB.setLayout(glGrp);
    
    // Label KB
    Label labelKB = new Label(grpKB, SWT.NONE);
    labelKB.setText("KB-Pid:");
    
    // Combobox KB
    final Combo comboKB = new Combo(grpKB, SWT.BORDER | SWT.READ_ONLY);
    GridData gdComboKB = new GridData();
    gdComboKB.horizontalAlignment = GridData.FILL;
    gdComboKB.grabExcessHorizontalSpace = true;
    gdComboKB.verticalAlignment = GridData.CENTER;
    comboKB.setLayoutData(gdComboKB);
    
    IHierarchie hierarchie = _modell.getBildungsRegeln().getHierarchie();
    final ConfigDataModel dataModel = hierarchie.getDataModel();
   
    for (ConfigurationArea kb: dataModel.getAllConfigurationAreas().values())
    {
      if (kb.getConfigurationAuthority() == dataModel.getConfigurationAuthority())
      {
        comboKB.add(kb.getPid());
      }
    }
    
//    ConfigurationArea kb = hierarchie.getKonfigurationsBereich(_uebergeordnetesObjekt);
//    if (kb != null)
//    {
//      comboKB.setText(kb.getPid());
//      _modell.setKB(kb);
//    }
        
    if (_uebergeordnetesObjekt instanceof HierarchieObjekt)
    {
      // Selektiere - wenn mglich - kb des bergeordneten Objekts
      ConfigurationArea kb = ((HierarchieObjekt)_uebergeordnetesObjekt).getSystemObjekt(0).getConfigurationArea();
      if (!Arrays.asList(comboKB.getItems()).contains(kb.getPid()))
      {
        // Falls kb des bergeordneten Objekts nicht in Combo-Liste (etwa, weil keine Rechte vorliegen)
        // selektiere Wurzel-KB der Hierarchie
        kb = hierarchie.getKonfigurationsBereich(_uebergeordnetesObjekt);
      }
      if (kb != null)
      {
        comboKB.setText(kb.getPid());
        _modell.setKB(kb);
      }

      comboKB.addListener(SWT.Modify, new Listener()
      {
        public void handleEvent(Event event)
        {
          if (event.widget instanceof Combo)
          {
            String pid = ((Combo)event.widget).getText();
            
            // Findet keine neuen KB:
            // _kb = dataModel.getConfigurationArea(pid);
            
            ConfigurationArea kb = null;
            
            for (ConfigurationArea lauf: dataModel.getAllConfigurationAreas().values())
            {
              if (lauf.getPid().equals(pid))
              {
                kb = lauf;
                break;
              }
            }
            _modell.setKB(kb);
          }
        }
      });
    }
    else
    {
      // Selektiere Wurzel-KB der Hierarchie
      ConfigurationArea kb = hierarchie.getKonfigurationsBereich(_uebergeordnetesObjekt);
      if (kb != null)
      {
        comboKB.setText(kb.getPid());
        _modell.setKB(kb);
      }
      comboKB.setEnabled(false);
    }
    
  }

}
