/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.regeln;

import org.eclipse.jface.wizard.IWizard;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.plugin.tkabasis.hierarchie.HierarchieObjekt;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieObjektTyp;

/**
 * Konstanten und Hilfsmethoden zur Untersttzung der Anendung der Bildungsregeln
 * 
 * @author Sans
 *
 */
public class BildungsRegelnTools
{
  /**
   * Leere Eingabevorbelegung
   */
  public static final EingabeVorbelegung LEERE_VORBELEGUNG = new EingabeVorbelegung(null, true);
    
  /**
   * Leere Textliste
   */
  public static final String[] LEERE_TEXT_LISTE = new String[0];
  
  /**
   * Leerer Wert
   */
  public static final String LEERER_WERT = "";
  
  
  /**
   * Es werden alle deutschen Umlaute entsprechend ersetzt. Alle Zeichen ausser [a-zA-Z0-9_.] 
   * werden entfernt und ggf. der folgende Buchstabe in den entsprechenden Grossbuchstaben umgewandelt.
   * Der erste Buchstabe des resultierenden Textes wird in einen Kleinbuchstaben umgewandelt.
   *  
   * @param text der zu bearbeitende Text
   * @param keineKleinGrossWandlung Flag, das angibt, ob die Umwandlung in Gross-/Kleinbuchstaben (nur diese) unterbleiben soll
   * @param keineUmlautWandlung Flag, das angibt, ob die Umwandlung der Umlaute (nur diese) unterbleiben soll
   * @return den entsprechend konvertierten Text
   */
  public static String getPidText(String text, boolean keineKleinGrossWandlung, boolean keineUmlautWandlung)
  {
    StringBuffer buffer = new StringBuffer();
    boolean toUpper = false;  
    
    for (char c: text.toCharArray())
    {
      boolean isKeinUmlaut = false;
    
      if (!keineUmlautWandlung)
      {  
        StringBuffer gewandelt = new StringBuffer();;
        
        switch (c)
        {
          case '':
            gewandelt.append("ae");
            break;
          
          case '':
            gewandelt.append("Ae");
            break;
            
          case '':
            gewandelt.append("oe");
            break;
          
          case '':
            gewandelt.append("Oe");
            break;
            
          case '':
            gewandelt.append("ue");
            break;  
          
          case '':  
            gewandelt.append("Ue");
            break;
            
          case '':
            gewandelt.append("ss");
            break;
            
          default:
            isKeinUmlaut = true;
            break;
        }
        
        if (gewandelt.length() > 0)
        {
          if (toUpper)
          {
            // Anfang von 'gewandelt' auf jeden Fall gross
            char a = gewandelt.charAt(0);
            if (Character.isLowerCase(a))
            {
              gewandelt.setCharAt(0, Character.toUpperCase(a));
            }
          }
          buffer.append(gewandelt);
          toUpper = false;
        }
      }
      else
      {
        isKeinUmlaut = true;
      }
      
      if (isKeinUmlaut)
      {  
        if (Character.isLetterOrDigit(c) || c == '.'|| c == '_')
        {
          buffer.append(toUpper ? Character.toUpperCase(c) : c);
          toUpper = false;
        }
        else
        {
          toUpper = !keineKleinGrossWandlung;
        }
      }  
    }
    
    if (!keineKleinGrossWandlung && buffer.length() > 0)
    {
      // Erster Buchstabe klein
      buffer.setCharAt(0, Character.toLowerCase(buffer.charAt(0)));
    }
    
    return buffer.toString();
  }

   
  /**
   * ffnet den Bildungsregel-Assistenten
   * 
   * @param modell BildungsregelnMitKB-Instanz
   * @param hotNeuesObjekt der Hierarchieobjekttyp des Hierarchieobjekts, fr das
   * PID und Name mit den Bildungsregeln gebildet werden sollen
   * @param uebergeordnetesObjekt das in der Hierarchie bergeordnete Objekt
   * @param zuKopierendesHO Falls der Assistent zum Kopieren von HO benutzt wird, kann hier 
   * das zu kopierende Objekt angegeebne werden - dessen Daten dann zustzlich angezeigt werden
   * @param werteVerwenden gibt an, ob die Felder mit im Modell vorhandenen Werten initialisiert
   * werden

   * @return <code>Dialog.OK</code> oder <code>Dialog.CANCEL</code>
   */
  public static int oeffneAssistent(BildungsRegelnMitKB modell, IHierarchieObjektTyp hotNeuesObjekt, 
      Object uebergeordnetesObjekt, HierarchieObjekt zuKopierendesHO, boolean werteVerwenden)
  {
    if (modell == null)
    {
      throw new IllegalArgumentException("Es muss eine gltige Instanz fr die Bildungsregeln bergeben werden");
    }
    if (hotNeuesObjekt == null)
    {
      throw new IllegalArgumentException("Es muss eine gltige Instanz fr den Hierarchieobjekttyp bergeben werden");
    }

    IWizard assistent = new BildungsRegelnAssistent(hotNeuesObjekt, modell, uebergeordnetesObjekt, zuKopierendesHO, werteVerwenden);
    WizardDialog dialog = new WizardDialog(PlatformUI.getWorkbench().getDisplay().getActiveShell(), assistent);
    dialog.create();
    
    return dialog.open();
  } 
  
  /**
   * ffnet den Bildungsregel-Assistenten
   * 
   * @param modell BildungsregelnMitKB-Instanz
   * @param hotNeuesObjekt der Hierarchieobjekttyp des Hierarchieobjekts, fr das
   * PID und Name mit den Bildungsregeln gebildet werden sollen
   * @param uebergeordnetesObjekt das in der Hierarchie bergeordnete Objekt
   * @param zuKopierendesHO Falls der Assistent zum Kopieren von HO benutzt wird, kann hier 
   * das zu kopierende Objekt angegeebne werden - dessen Daten dann zustzlich angezeigt werden
   *
   * @return <code>Dialog.OK</code> oder <code>Dialog.CANCEL</code>
   */
  public static int oeffneAssistent(BildungsRegelnMitKB modell, IHierarchieObjektTyp hotNeuesObjekt, 
      Object uebergeordnetesObjekt, HierarchieObjekt zuKopierendesHO)
  {
    return oeffneAssistent(modell, hotNeuesObjekt, uebergeordnetesObjekt, zuKopierendesHO, false);
  } 
  
  /**
   * ffnet den Bildungsregel-Assistenten
   * 
   * @param modell BildungsregelnMitKB-Instanz
   * @param hotNeuesObjekt der Hierarchieobjekttyp des Hierarchieobjekts, fr das
   * PID und Name mit den Bildungsregeln gebildet werden sollen
   * @param uebergeordnetesObjekt das in der Hierarchie bergeordnete Objekt
   * 
   * @return <code>Dialog.OK</code> oder <code>Dialog.CANCEL</code>
   */
  public static int oeffneAssistent(BildungsRegelnMitKB modell, IHierarchieObjektTyp hotNeuesObjekt, 
      Object uebergeordnetesObjekt)
  {
    return oeffneAssistent(modell, hotNeuesObjekt, uebergeordnetesObjekt, null, false);
  } 
}
