/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.tkaUpdateManagement;

import java.util.List;
import java.util.Vector;

import de.bsvrz.buv.plugin.tkabasis.hierarchie.HierarchieObjekt;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchie;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;

/**
 * Rekursives Iterieren aller Hierarchieobjekte einer Hierarchie
 * 
 * @author Sans
 *
 */
public abstract class HierarchieIterierer
{
  /**
   * Die zugrundeliegende Hierarchie
   */
  private IHierarchie _hierarchie;
  
  /**
   * Erzeugt eine neue Instanz fr eine zugrundeliegende Hierarchie
   * 
   * @param hierarchie die zugrundeliegende Hierarchie
   */
  public HierarchieIterierer(IHierarchie hierarchie)
  {
    _hierarchie = hierarchie;
  }
  
  /**
   * Die Methode iteriere() wird fr alle HierarchieObjekte der Hierarchie rekursiv 
   * aufgerufen
   */
  protected final void alleObjekteIterieren()
  {
    List<ConfigurationArea> liste = new Vector<ConfigurationArea>(_hierarchie.getDataModel().getAllConfigurationAreas().values());
    for (ConfigurationArea kb: liste)
    {
      if (!kinderIterieren(kb))
      {
        break;
      }
    }
  }
  
  /**
   * Die Methode iteriere() wird das bergebene Objekt aufgerufen,
   * danach erfolgen rekursive Aufrufe mit den ggf. vorhandenen Kindern
   * des bergebenen Objekts
   *  
   * @param objekt das zu aktualisierende Objekt
   * @return true, wenn die Iteration weiter durchgefhrt werden soll, false wenn sie abgebrochen werden soll
   */
  private boolean kinderIterieren(Object objekt)
  {
    if (!iteration(objekt))
    {
      return false;
    }
    
    List<HierarchieObjekt> liste = new Vector<HierarchieObjekt>(_hierarchie.getUntergeordneteObjekte(objekt));
    for (HierarchieObjekt kind: liste)
    {
      if (!kinderIterieren(kind))
      {
        return false;
      }
    }
      
    return true;
  }
  
  /**
   * Diese Methode wird fr jedes Objekt in der der zugrundeliegenden Hierarchie aufgerufen
   * 
   * @param objekt das zu aktualisierende Objekt
   * @return true, wenn die Iteration weiter durchgefhrt werden soll, false wenn sie abgebrochen werden soll
   */
  protected abstract boolean iteration(Object objekt);
}
