/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.tkaUpdateManagement;

import java.util.List;
import java.util.Vector;

import de.bsvrz.buv.plugin.tkabasis.hierarchie.HierarchieObjekt;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieBeobachter;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Sammeln genderter Daten bei Aktionen fr eine TKA
 * 
 * @author Sans
 *
 */
public class TKAUpdate implements IHierarchieBeobachter
{
  /**
   * Liste der genderten Systemobjekte
   */
  private List<SystemObject> _listeAktualisiert = new Vector<SystemObject>();
  
  /**
   * Liste der gelschten Systemobjekte
   */
  private List<SystemObject> _listeGeloescht = new Vector<SystemObject>();
  
  /**
   * Liste der neuen Systemobjekte
   */
  private List<SystemObject> _listeNeu = new Vector<SystemObject>();
  
    
  /**
   * Bestimmt die Liste der genderten Systemobjekte
   * 
   * @return die Liste der genderten Systemobjekte
   */
  public List<SystemObject> getListeAktualisiert()
  {
    return _listeAktualisiert;
  }
  
  /**
   * Bestimmt die Liste der gelschten Systemobjekte
   * 
   * @return die Liste der gelschten Systemobjekte
   */
  public List<SystemObject> getListeGeloescht()
  {
    return _listeGeloescht;
  }
  
  /**
   * Bestimmt die Liste der neuen Systemobjekte
   * 
   * @return die Liste der neuen Systemobjekte
   */
  public List<SystemObject> getListeNeu()
  {
    return _listeNeu;
  }
  
  public void hierarchieAktualisiert(EVENT event, ConfigurationArea kb, Object vater, HierarchieObjekt ho)
  {
    for (SystemObject so: ho.getSystemObjekte())
    {
      switch (event)
      {
        case OBJEKT_AKTUALISIERT:
        case OBJEKT_VERSCHOBEN:
          if (!_listeAktualisiert.contains(so))
          {
            _listeAktualisiert.add(so);
          }
          break;
        
        case OBJEKT_ENTFERNT:
          if (!ho.getHierarchieObjektTyp().isSystemObjektVorhanden())
          {
            if (!_listeGeloescht.contains(so))
            {
              _listeGeloescht.add(so);
            }
          }
          break;
        
        case OBJEKT_NEU:
          if (!ho.getHierarchieObjektTyp().isSystemObjektVorhanden())
          {
            if (!_listeNeu.contains(so))
            {
              _listeNeu.add(so);
            }
          }
          break;
      }
    }
  }
    
}
