/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.zustaende;

import java.util.Hashtable;


/**
 * The FSM states. The default FSM State simply stores a hash table with transitions
 * along with a state name.
 */    
/**
 * Die Klasse kapselt den Zustand einer Zustandsmaschine 
 *
 * @version $Revision: 1.4 $ / $Date: 2011/01/17 13:30:41 $ / ($Author: Sans $)
 *
 * @author Dambach-Werke GmbH
 * @author Timo Pittner
 *
 */
public class Zustand {
   
    /**
     * Der Name der Zustands
     */
    protected String _name;
    
    /**
     * Speichert die Uebergaenge 
     */
    protected Hashtable<Integer, Zustand> _uebergangHash = new Hashtable<Integer, Zustand>(); 
            
    /**
     * Konstruktor, mit Name des Zustands
     * 
     * @param name
     */
    public Zustand(String name) {
        _name = name;
    }
     
    
    public void setName(String name) {
        _name = name;
    }
   
        
    /**
     * Fgt eine Uebergang zum Zustand hinzu
     * 
     * @param uebergang
     *          Index des Uebergangs
     * @param endZustand
     *          der Endzustand
     */
    public void addUebergang(int uebergang, Zustand endZustand) {
        _uebergangHash.put(new Integer(uebergang), endZustand);            
        
    }
            
    /**
     * Realisiert den Uebergang anhand des uebergebenen Index. Wenn der uebergebene Zustand 
     * nicht definert ist, wird null zurueckgegeben
     * 
     * @param uebergang
     *          Index des Uebergangs
     * @return
     *          der Endzustand
     */
    public Zustand uebergang(int uebergang) {
        return (Zustand)_uebergangHash.get(new Integer(uebergang));
    }
    
     
    public String toString() {
        return _name;
    }

   
    
}