/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkamq;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;
import java.util.Vector;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;

import de.bsvrz.buv.plugin.tkabasis.Activator;
import de.bsvrz.buv.plugin.tkabasis.LookupFactory;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.AtgInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HODaten;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HierarchieObjekt;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IAttributInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchie;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.SystemObjektDaten;
import de.bsvrz.buv.plugin.tkabasis.pruefungen.IPruefungen;
import de.bsvrz.buv.plugin.tkabasis.pruefungen.PruefungenTools;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.dav.daf.main.config.NonMutableSet;
import de.bsvrz.dav.daf.main.config.ObjectLookup;
import de.bsvrz.dav.daf.main.config.ObjectTimeSpecification;
import de.bsvrz.dav.daf.main.config.StringAttributeType;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;
import de.bsvrz.puk.config.configFile.datamodel.ConfigSystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Prfungen von Attributwerten fr den TKA Verkehr MQ
 * 
 * @author Sans
 *
 */
public class PruefungenMQ implements IPruefungen
{
	/**
   * Debug-Logger fr Logging-Ausgaben
   */
  private final static Debug _debug = Debug.getLogger();

  /**
   * Die Hierarchie, die fr die Prfungen benutzt werden kann
   */
	private IHierarchie _hierarchie;
	
	 /**
   * Bestimmt die Hierarchie, die fr die Prfungen benutzt werden kann
   * 
   * @return die Hierarchie, die fr die Prfungen benutzt werden kann
   */
	public IHierarchie getHierarchie()
	{
		return _hierarchie;
	}
  
	/**
   * Setzt die Hierarchie, die fr die Prfungen benutzt werden kann
   * @param hierachie die Hierarchie, die fr die Prfungen benutzt werden kann
   */
	public void setHierarchie(IHierarchie hierarchie)
	{
		_hierarchie = hierarchie;
	}
	
	public IStatus pruefeHierarchieObjekt(HierarchieObjekt ho, HODaten daten)
	{
		MultiStatus result = new MultiStatus(Activator.PLUGIN_ID, Status.OK, 
				"Prfe Hierarchieobjekt " + ho, null);
	  		
		for (SystemObject so: ho.getSystemObjekte())
  	{
			 SystemObjektDaten soDaten = daten.getSystemObjektDaten(so);
			 if (soDaten != null)
			 {
				 MultiStatus objektStatus;
				 if (ho.getSystemObjekte().size() > 1)
				 {
					 objektStatus = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe Systemobjekt " + 
							 so.getPid() + " vom Typ " + so.getType().getPid(), null);
				 }
				 else
				 {
					 objektStatus = result;
				 }
				 
				 // objektStatus.add(pruefePid(so, soDaten.getPid()));
				 objektStatus.add(new PruefungenTools(getHierarchie().getDataModel()).pruefePid(soDaten.getPid(), so));	
				 
				 if (so.isOfType("typ.messQuerschnittVirtuell"))
				 {
					 objektStatus.add(pruefeAtgMessquerschnittVirtuell(soDaten));
				 }
				 else if (so.isOfType("typ.fahrStreifen"))
				 {
					 objektStatus.add(pruefeZuordnungFahrstreifen(ho, so, soDaten));
				 }
				 
				 if (objektStatus != result)
				 {
					 result.add(objektStatus);
				 }
			 }	
  	}
		
		return result;
	}
	
	public IStatus pruefeDaten(HierarchieObjekt ho, SystemObject so, AtgInfo atgInfo, Data daten)
	{
		MultiStatus result = new MultiStatus(Activator.PLUGIN_ID,	Status.OK, 
  			"Prfe Daten der ATG " + atgInfo.getAtg(), null);
		
		 if (so.isOfType("typ.messQuerschnitt"))
		 {
				if ("atg.messQuerschnittAllgemein".equals(atgInfo.getAtg()) ||
						"atg.punktKoordinaten".equals(atgInfo.getAtg()) ||
						"atg.punktLiegtAufLinienObjekt".equals(atgInfo.getAtg()))
				{
					pruefeDatenDefinition(atgInfo, daten , result);
				}
		 }
		 else if (so.isOfType("typ.messQuerschnittVirtuell"))
		 {
			 if ("atg.messQuerschnittVirtuellStandard".equals(atgInfo.getAtg()))
			 {
				 pruefeDatenAtgMessQuerschnittVirtuellStandard(atgInfo, daten, result);
			 }
			 else if ("atg.messQuerschnittAllgemein".equals(atgInfo.getAtg()) ||
					    	"atg.punktKoordinaten".equals(atgInfo.getAtg()) ||
					    	"atg.punktLiegtAufLinienObjekt".equals(atgInfo.getAtg()) ||
					    	"atg.messQuerschnittVirtuell".equals(atgInfo.getAtg()) ||
								"atg.messQuerschnittVirtuellVLage".equals(atgInfo.getAtg()))
			 {
				 pruefeDatenDefinition(atgInfo, daten , result);
			 }
		 }
		 else if (so.isOfType("typ.messQuerschnittHandZhlung"))
		 {
			 if ("atg.messQuerschnittAllgemein".equals(atgInfo.getAtg()) ||
						"atg.punktKoordinaten".equals(atgInfo.getAtg()) ||
						"atg.punktLiegtAufLinienObjekt".equals(atgInfo.getAtg()))
				{
					pruefeDatenDefinition(atgInfo, daten , result);
				}
		 }
		 else if (so.isOfType("typ.straenAbschnitt"))
		 {
			 if ("atg.straenAbschnitt".equals(atgInfo.getAtg()))
			 {	 
				 pruefeDatenAtgStrassenAbschnitt(atgInfo, daten, result);
			 }
		 }	
		 else if (so.isOfType("typ.fahrStreifen"))
		 {
			 if ("atg.fahrStreifen".equals(atgInfo.getAtg()))
			 {	
				 pruefeDatenAtgFahrStreifen(ho, so, atgInfo, daten, result);
			 }
		 }
		
		return result;
	}
	
	
  IStatus pruefeFahrStreifen(HierarchieObjekt ho, SystemObject so)
  {
  	return null;
  }
  
    
  /**
   * Prft einen Attributwert auf zulssigkeit
   * 
   * @param ho Hierarchieobhekt, dessen Daten geprft werden sollen
   * @param so Systemobjekt, gehrt zu ho
   * @param atg ATG, die das zu prfende Attribut enthlt, gehrt zu so
   * @param attInfo Info ber das zu prfenden Attribut
   * @param daten Datensatz der Daten, die geprft werden sollen
   * @return eine Instanz vom Typ IStatus, die aussagt, ob es zulssig ist, die 
   * bergebenen Daten als Attributwert zu bernehmen, oder <code>null</code>, wenn
   * keine Prfung fr die bergebenen Parameter implementiert ist
   */
//  public IStatus pruefeDaten(HierarchieObjekt ho, SystemObject so, AttributeGroup atg,
//      IAttributInfo attInfo, Data daten)
//  {
////  	MultiStatus	result = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe Wert fr Attribut " + 
////  			attInfo.getName() +	": \"" + daten.getItem(attInfo.getName()).asTextValue().getValueText() + "\"", null); 
//  	MultiStatus	result = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe Wert fr Attribut " +
//  			attInfo.getName(), null);
//  	pruefeDaten(ho, so, atg, attInfo, daten, result);
//
//  	return result;
//  }
	
//  private void pruefeDaten(HierarchieObjekt ho, SystemObject so, AttributeGroup atg, IAttributInfo attInfo, 
//  		Data daten ,MultiStatus status)
//	{
//  	// Prfe Zuordnung Systemobjekt -> HO
//  	if (!ho.getSystemObjekte().contains(so))
//  	{
//  		throw new IllegalStateException("Systemobjekt " + so + " ist nicht in Hierarchieobjekt " +
//  				ho + " enthalten");
//  	}
//  	
//		if (so.isOfType("typ.messQuerschnitt") ||
//				so.isOfType("typ.messQuerschnittHandZhlung") ||
//				so.isOfType("typ.messQuerschnittVirtuell"))
//		{
//			if ("atg.messQuerschnittAllgemein".equals(atg.getPid()) ||
//					"atg.punktKoordinaten".equals(atg.getPid()) ||
//					"atg.punktLiegtAufLinienObjekt".equals(atg.getPid()))
//			{
//				if (!pruefeDatenDefinition(attInfo, daten , status))
//				{
//					return;
//				}
//			}
//		}
//		
//		if (so.isOfType("typ.messQuerschnittVirtuell"))
//		{
//			if ("atg.messQuerschnittVirtuell".equals(atg.getPid()) ||
//					"atg.messQuerschnittVirtuellVLage".equals(atg.getPid()))
//			// atg.messQuerschnittVirtuellStandard wird bereits in pruefeHierarchieobjekt geprft	
//				
//			{
//				if (!pruefeDatenDefinition(attInfo, daten , status))
//				{
//					return;
//				}
//			}
//		}
//		
//		// typ.straenAbschnitt wird bereits in pruefeHierarchieobjekt geprft
//		
//		// typ.fahrStreifen wird bereits in pruefeHierarchieobjekt geprft	
//		
//	}
    
  private boolean pruefeDatenDefinition(IAttributInfo attInfo, Data daten, MultiStatus status)
//<<<<<<< PruefungenMQ.java
//  {
//
//=======
  {	  
//>>>>>>> 1.11
		Data item = daten.getItem(attInfo.getName());
		if (!item.isDefined())
		{
			status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Wert ist nicht korrekt definiert"));  
			return false;
		}
		else
		{
			status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, "Prfen Daten-Definition: OK"));
		}
		
		// Plichtfeld-Prfung:
		if (item.isPlain())
		{	
			if (attInfo.isPflichtFeld())
			{	
				String wert = item.asTextValue().getText();
				if ("".equals(wert) || (!(item.getAttributeType() instanceof StringAttributeType) && 
						"undefiniert".equals(wert) || "_Undefiniert_".equals(wert)))
				{
					status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Wert fr Pflichtfeld nicht angegeben"));  
					return false;
				}
				else
				{
					status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, "Prfen Pflichtfeld-Eigenschaft: OK"));
				}
			}	
		}		
  	return true;
  }
  
  private void pruefeDatenDefinition(AtgInfo atgInfo, Data daten, MultiStatus status)
  {
  	for (String attribut: atgInfo.getAttribute())
  	{
  		MultiStatus	meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe Wert fr Attribut " +
    			attribut, null);
  		IAttributInfo attInfo = atgInfo.getAttributInfo(attribut);
  		pruefeDatenDefinition(attInfo, daten,  meldung);
  		status.add(meldung);
  	}
  }
  
  /**
   * Prft, ob eine PID fr ein Systemobjekt vergeben werden kann
   * 
   * @param so das Systemobjekt, fr das die PID vergeben werden soll
   * @param pid die PID, die vergeben werden soll
   * @return eine Instanz vom Typ IStatus, die aussagt, ob es zulssig ist, die 
   * PID fr das Systemobjekt zu vergeben, oder <code>null</code>, wenn keine
   * Prfung implementiert ist
   * 
   * @deprecated
   */
  private IStatus pruefePid(SystemObject so, String pid)
  {
  	MultiStatus result = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe PID " + pid, null);
  	pruefePid(so, pid, null, result);
  	return result;
  }
  
  /**
   * @deprecated
   */
  private void pruefePid(SystemObject so, String pid, ConfigurationArea kb, MultiStatus status)
  {
    if (kb != null)
    {
      for (SystemObject lauf: kb.getCurrentObjects())
      {
        if ((lauf instanceof ConfigurationObject &&
        		!(((ConfigurationObject)lauf).getNotValidSince() >= kb.getModifiableVersion())) ||
        		(lauf instanceof DynamicObject &&
            		!(((DynamicObject)lauf).getNotValidSince() >= kb.getModifiableVersion()))        )
        {	
          // Objekt wird in zuknftiger Version nicht ungltig
        	pruefePid(lauf, so, pid, status);
        }	
      }
      for (SystemObject lauf: kb.getNewObjects())
      {
      	pruefePid(lauf, so, pid, status);
      }
    }
    else
    {
      for (ConfigurationArea lauf: _hierarchie.getDataModel().getAllConfigurationAreas().values())
      {
        pruefePid(so, pid, lauf, status);
      }
    }
  }
  	
  /**
   * @deprecated
   */
  private void pruefePid(SystemObject gefundenesObjekt,	
  		SystemObject vorhandenesObjekt, String pid, MultiStatus status)
  {
    if (pid.equals(gefundenesObjekt.getPid()))
    {
      if (!vorhandenesObjekt.equals(gefundenesObjekt))
      {
        int severity = vorhandenesObjekt.getConfigurationArea().equals(gefundenesObjekt.getConfigurationArea())?
            Status.ERROR : Status.WARNING;
        
        status.add(new Status(severity, Activator.PLUGIN_ID, "PID " + pid +
          " wird bereits verwendet von Objekt vom Typ " + gefundenesObjekt.getType().getPid() +
          " im KB " + gefundenesObjekt.getConfigurationArea().getPid()));  
	    }
	  }
	}
  
  IStatus pruefeAtgMessquerschnittVirtuell(SystemObjektDaten soDaten)
  {
  	int anzahl = 0;
  	
  	for (String atg: soDaten.getAtg())
  	{
  		if ("atg.messQuerschnittVirtuell".equals(atg) ||
  				"atg.messQuerschnittVirtuellStandard".equals(atg) ||
  				"atg.messQuerschnittVirtuellVLage".equals(atg))
  		{
  			anzahl++;
  			if (anzahl > 1)
  			{
  				break;
  			}
  		}
  	}

  	if (anzahl > 1)
		{
			return new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Es darf nur max. EINE der " +
					"Attributgruppen atg.messQuerschnittVirtuell, atg.messQuerschnittVirtuellStandard, " +
					"atg.messQuerschnittVirtuellVLage existieren");
		}
  	else
  	{
  		return new Status(IStatus.OK, Activator.PLUGIN_ID, 
  				"Prfen Attributgruppen MessquerschnittVirtuell: OK");
  	}
  }
  
  private void pruefeDatenAtgMessQuerschnittVirtuellStandard(AtgInfo atgInfo, Data data, MultiStatus status)
  {
  	int anzReferenzen = 5;
  	Set<String> referenzen = new HashSet<String>();
  	String wertLage = null;
  	Data lage = data.getItem("Lage");
  	if (lage.isDefined())
  	{
  		wertLage = lage.asTextValue().getText();
  	}
  	boolean fehlerLage = false;
  	
  	ConfigDataModel dataModel = _hierarchie.getDataModel();
  	ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel); 
  	
  	for (String attribut: atgInfo.getAttribute())
  	{
  		MultiStatus	meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe Wert fr Attribut " +
    			attribut, null);
  		IAttributInfo attInfo = atgInfo.getAttributInfo(attribut);
  		if (!pruefeDatenDefinition(attInfo, data, meldung))
			{
  			if ("MessQuerschnittVor".equals(attribut) ||
  					"MessQuerschnittMitte".equals(attribut) ||
  					"MessQuerschnittNach".equals(attribut) ||
  					"MessQuerschnittAusfahrt".equals(attribut) ||
  					"MessQuerschnittEinfahrt".equals(attribut))

  			{
  				anzReferenzen -= 1;
  			}
			}
  		
  		else
  		{	
  			if ("MessQuerschnittVor".equals(attribut) ||
					"MessQuerschnittMitte".equals(attribut) ||
					"MessQuerschnittNach".equals(attribut) ||
					"MessQuerschnittAusfahrt".equals(attribut) ||
					"MessQuerschnittEinfahrt".equals(attribut))
				{
					String pid = data.getItem(attribut).asReferenceValue().getSystemObjectPid();
					if (pid == null || pid.isEmpty())
					{
						anzReferenzen -= 1;
					}
					else
					{	
						referenzen.add(pid);
						if (wertLage != null)
						{
							if (attribut.endsWith(wertLage))
							{
								fehlerLage = true;
							}
						}
					}	
				}
  		}
  		status.add(meldung);
  	}
  	
  	if (anzReferenzen > 1)
  	{
  		if (referenzen.size() < anzReferenzen)
  		{
  			status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Die Attribute " + 
  					"MessQuerschnittXXX mssen auf unterschiedliche Messquerschnitte referenzieren"));
  			
  		}
    	else
    	{
    		status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, 
    				"Prfen Referenzen MessQuerschnittXXX: OK"));
    	}
  	}
  	
  	if (wertLage != null && anzReferenzen > 0)
  	{
  		if (fehlerLage)
  		{
  			status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Wegen Attributwert Lage \"" +	wertLage + 
  					"\" darf in MessQuerschnitt" + wertLage + " keine Referenz angegeben werden"));
  		}
    	else
    	{
    		status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, 
    				"Prfen Attribut Lage gegen Messquerschnitt-Typen: OK"));
    	}
  	}
  	
  }
  
  private void pruefeDatenAtgStrassenAbschnitt(AtgInfo atgInfo, Data data, MultiStatus status)
  {
  	SystemObject vonMq = null;
  	SystemObject bisMq = null;
  	
  	for (String attribut: atgInfo.getAttribute())
  	{
  		MultiStatus	meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe Wert fr Attribut " +
    			attribut, null);
  		IAttributInfo attInfo = atgInfo.getAttributInfo(attribut);
  		if (pruefeDatenDefinition(attInfo, data , meldung))
			{
  			if ("vonMessQuerschnitt".equals(attribut))
    		{
    			vonMq = data.getItem(attribut).asReferenceValue().getSystemObject(); 
    		}
    		if ("bisMessQuerschnitt".equals(attribut))
    		{
    			bisMq = data.getItem(attribut).asReferenceValue().getSystemObject(); 
    		}
			}
  		
  		status.add(meldung);
  	}
  	
  	if (vonMq != null && bisMq != null)
  	{
  		if (vonMq.equals(bisMq))
  		{
  			status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
				"Die begrenzenden Messquerschnitte mssen unterschiedlich sein"));
  		}
  		else
  		{
  			status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, 
				"Prfen begrenzende Messquerschnitte: OK"));
  		}
  	}
  	
  }
  
  private IStatus pruefeZuordnungFahrstreifen(HierarchieObjekt ho, SystemObject so, SystemObjektDaten soDaten)
  {
   	Object vater = _hierarchie.getUebergeordnetesObjekt(ho);
  	MultiStatus result = null;
   	
  	if (vater instanceof HierarchieObjekt && 
  			((HierarchieObjekt)vater).getHierarchieObjektTyp().equals(HOT_MQ.MQ))
  	{
  		SystemObject mq = ((HierarchieObjekt)vater).getSystemObjekt(0);
  		NonMutableSet menge = ((ConfigurationObject)mq).getNonMutableSet("FahrStreifen");
  		if (menge != null)
  		{
  			if (menge.getElementsInModifiableVersion().contains(so))
  			{
  				ConfigDataModel dataModel = _hierarchie.getDataModel();
    	  	    	  	
  				for (ConfigurationArea kb: dataModel.getAllConfigurationAreas().values())
  				{
  					SystemObjectType[] typSpez = new SystemObjectType[] { dataModel.getType("typ.messQuerschnitt") };
  					Collection<SystemObject> mqListe = new Vector<SystemObject>();
  					mqListe.addAll(kb.getObjects(Arrays.asList(typSpez), ObjectTimeSpecification.valid()));
  					
  					for (SystemObject lauf: kb.getNewObjects())
  					{
							if (lauf.isOfType("typ.messQuerschnitt"))
							{
								mqListe.add(lauf);
							}
  					}
  					
  					for (SystemObject lauf: mqListe)
  					{
  						if (!lauf.equals(mq))
  						{	
	  						ConfigurationObject ko = (ConfigurationObject)lauf;
	  						
	  						if (!(ko.getNotValidSince() >= kb.getModifiableVersion()))
	  		        // Objekt wird in zuknftiger Version nicht ungltig
			          {
									NonMutableSet set = ((ConfigurationObject)lauf).getNonMutableSet("FahrStreifen");
									if (set != null)
									{
										if (set.getElementsInModifiableVersion().contains(so))
										{
											if (result == null)
											{
												result = new MultiStatus(Activator.PLUGIN_ID, Status.OK, 
														"Der FS " + so.getPid() + " darf nicht mehreren MQ zugeordnet sein", null);
											}
											result.add( new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
													"Der FS " + so.getPid() + " ist zustzlich dem MQ " + lauf.getPid() + " zugeordnet"));
										}
									}
			          }
  						}	
  					}
  				}
  			}
  		}
  		else
  		{
  			_debug.warning("Menge Fahrstreifen am Objekt " + mq.getPid() + " nicht vorhanden");
//  			throw new IllegalStateException("Menge Fahrstreifen am Objekt " + mq.getPid() + " nicht vorhanden");
  		}
  	}
  	else
  	{
  		throw new IllegalStateException("bergeordnetes Objekt von " + ho + " vom Typ " +
  				HierarchieObjekt.class + " vom HOT " + HOT_MQ.MQ + " erwartet");
  		
  	}
  	  	
  	if (result == null)
  	{
  		return new Status(IStatus.OK, Activator.PLUGIN_ID, 
  				"Prfen Zuordnung Fahrstreifen zum Messquerschnitt: OK");
		}
  	else
  	{
  		return result;
  	}
  }
  
  private void pruefeDatenAtgFahrStreifen(HierarchieObjekt ho, SystemObject so, 
  		AtgInfo atgInfo, Data daten, MultiStatus status)
  {

  	for (String attribut: atgInfo.getAttribute())
  	{
  		MultiStatus	meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe Wert fr Attribut " +
    			attribut, null);
  		IAttributInfo attInfo = atgInfo.getAttributInfo(attribut);
  		if (pruefeDatenDefinition(attInfo, daten , meldung))
			{
  			if ("Lage".equals(attribut))
    		{
  				meldung.add(pruefeLageFahrStreifen(ho, so, daten));
    		}
    		else if ("Typ".equals(attribut))
    		{
    			meldung.add(pruefeTypFahrStreifen(ho, so, daten)); 
    		}
    		else if ("ErsatzFahrStreifen".equals(attribut))
    		{
    			meldung.add(pruefeErsatzFahrStreifenFahrStreifen(ho, so, daten)); 
    		}
			}
  		status.add(meldung);
  	}
  	
  }
  
  /**
   * Es wird geprft ob der Wert des Attributs Lage der atg.fahrStreifen nur einmal
   * fr den  bergeordneten MQ vorhanden ist
   */
  private IStatus pruefeLageFahrStreifen(HierarchieObjekt ho, SystemObject so, Data daten)
  {
  	boolean fehler = false;
  	String lage = daten.getItem("Lage").asTextValue().getText();
   	Object vater = _hierarchie.getUebergeordnetesObjekt(ho);
   	
  	if (vater instanceof HierarchieObjekt && 
  			((HierarchieObjekt)vater).getHierarchieObjektTyp().equals(HOT_MQ.MQ))
  	{
  		SystemObject mq = ((HierarchieObjekt)vater).getSystemObjekt(0);
  		NonMutableSet menge = ((ConfigurationObject)mq).getNonMutableSet("FahrStreifen");
  		if (menge != null)
  		{
  			ConfigDataModel dataModel = _hierarchie.getDataModel();
  	  	ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel); 
  			
  			for (SystemObject fs: menge.getElementsInModifiableVersion())
  			{
  				if (fs == null)
  				{
  					continue;
  				}
  				Data data = null;
  				if (!fs.equals(so))
  				{
  					AttributeGroup atg = dataModel.getAttributeGroup("atg.fahrStreifen");
  					data = ((ConfigSystemObject)fs).getConfigurationData(atg, lookup);
  				}
  				if (data != null)
  				{
  					fehler = lage.equals(data.getItem("Lage").asTextValue().getText());
  					if (fehler)
  					{
  						// Wert des Attributs Lage in atg.fahrStreifen kommt noch in anderen FS 
  						// des bergeordneten MQ vor 
  						break;
  					}	
   				}
  			}
  		}
  		else
  		{
  			_debug.warning("Menge Fahrstreifen am Objekt " + mq.getPid() + " nicht vorhanden");
//  			throw new IllegalStateException("Menge Fahrstreifen am Objekt " + mq.getPid() + " nicht vorhanden");
  		}
  	}
  	else
  	{
  		throw new IllegalStateException("bergeordnetes Objekt von " + ho + " vom Typ " +
  				HierarchieObjekt.class + " vom HOT " + HOT_MQ.MQ + " erwartet");
  	}
  	
  	if (fehler)
		{
			return new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
					"Der Wert des Attributs Lage \"" + lage + "\" darf nicht in mehreren FS eines MQ vorkommen");
		}
  	else
  	{
  		return new Status(IStatus.OK, Activator.PLUGIN_ID, 
  				"Prfen Wert des Attributs Lage: OK");
		}
  }  
  

  
  /**
   * Wenn der Typ in der atg.fahrStreifen HauptFahrStreifen ist, wird geprft ob
   * noch ein ander FS des bergeordneten MQ auch den Typ HauptFahrStreifen in
   * atg.fahrStreifen spezifiziert
   */
  private IStatus pruefeTypFahrStreifen(HierarchieObjekt ho, SystemObject so, Data daten)
  {
  	String typ = daten.getItem("Typ").asTextValue().getText();
  	boolean fehler = false;
  	
  	if ("HauptFahrStreifen".equals(typ))
  	{	
	   	Object vater = _hierarchie.getUebergeordnetesObjekt(ho);
	  	if (vater instanceof HierarchieObjekt && 
	  			((HierarchieObjekt)vater).getHierarchieObjektTyp().equals(HOT_MQ.MQ))
	  	{
	  		SystemObject mq = ((HierarchieObjekt)vater).getSystemObjekt(0);
	  		NonMutableSet menge = ((ConfigurationObject)mq).getNonMutableSet("FahrStreifen");
	  		if (menge != null)
	  		{
	  			ConfigDataModel dataModel = _hierarchie.getDataModel();
	  	  	ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel); 
	  			
	  			for (SystemObject fs: menge.getElementsInModifiableVersion())
	  			{
	  				if (fs == null)
	  				{
	  					continue;
	  				}
	  				Data data = null;
	  				if (!fs.equals(so))
	  				{
	  					AttributeGroup atg = dataModel.getAttributeGroup("atg.fahrStreifen");
	  					data = ((ConfigSystemObject)fs).getConfigurationData(atg, lookup);
	  				}
	  				if (data != null)
	  				{
	  					fehler = "HauptFahrStreifen".equals(data.getItem("Typ").asTextValue().getText());
	  					if (fehler)
	  					{
	  						// Typ HauptFahrStreifen in atg.fahrStreifen kommt noch in anderen FS 
	  						// des bergeordneten MQ vor 
	  						break;
	  					}	
	   				}
	  			}
	  		}
	  		else
	  		{
	  			_debug.warning("Menge Fahrstreifen am Objekt " + mq.getPid() + " nicht vorhanden");
//	  			throw new IllegalStateException("Menge Fahrstreifen am Objekt " + mq.getPid() + " nicht vorhanden");
	  		}
	  	}
	  	else
	  	{
	  		throw new IllegalStateException("bergeordnetes Objekt von " + ho + " vom Typ " +
	  				HierarchieObjekt.class + " vom HOT " + HOT_MQ.MQ + " erwartet");
	  	}
  	}
  	if (fehler)
		{
			return new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
					"Der Wert des Attributs Typ \"HauptFahrStreifen\" darf nicht in mehreren FS eines MQ vorkommen");
		}
  	else
  	{
  		return new Status(IStatus.OK, Activator.PLUGIN_ID, 
  				"Prfen Wert des Attributs Typ: OK");
		}
  }
  
  /**
   * Es wird geprft, ob der FS in der atg.fahrStreifen im Attribut ErsartFahrStreifen auf sich selbst
   * verweist
   */
  private IStatus pruefeErsatzFahrStreifenFahrStreifen(HierarchieObjekt ho, SystemObject so, 
  		Data daten)
  {
  	SystemObject ersatzFs = daten.getItem("ErsatzFahrStreifen").asReferenceValue().getSystemObject();
  	  	
  	if (ersatzFs != null && ersatzFs.equals(so))
		{
			return new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
			    "Der Fahrstreifen darf ber das Attribut ErsatzFahrStreifen nicht auf sich selbst verweisen");
		}
  	else
  	{
  		return new Status(IStatus.OK, Activator.PLUGIN_ID, 
  				"Prfen des Fahrstreifen bezgl. Attribut ErsatzFahrStreifen: OK");
  	}	
  }
  
}
  