/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkamqgrp;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

import de.bsvrz.buv.plugin.tkabasis.DataFactory;
import de.bsvrz.buv.plugin.tkabasis.LookupFactory;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HierarchieObjekt;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchie;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieObjektTyp;
import de.bsvrz.buv.plugin.tkabasis.regeln.BildungsRegelnBeobachter;
import de.bsvrz.buv.plugin.tkabasis.regeln.BildungsRegelnTools;
import de.bsvrz.buv.plugin.tkabasis.regeln.EingabeVorbelegung;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.ObjectLookup;
import de.bsvrz.dav.daf.main.config.ObjectTimeSpecification;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;

public class BildungsRegelnBWMQGrp extends BildungsRegelnAllgemeinMQGrp
{
  protected Map<String, EingabeVorbelegung> _vorbelegungen = new HashMap<String, EingabeVorbelegung>();

  protected Set<BildungsRegelnBeobachter> _beobachter = new LinkedHashSet<BildungsRegelnBeobachter>();

  protected void initVorbelegungen()
  {
    _vorbelegungen.put("Strae", new EingabeVorbelegung(null, false));
    _vorbelegungen.put("BezeichnungVonMS", new EingabeVorbelegung(null, false));
    _vorbelegungen.put("BezeichnungBisMS", new EingabeVorbelegung(null, false));
    _vorbelegungen.put("vonKnotennummer", new EingabeVorbelegung(null, false));
    _vorbelegungen.put("bisKnotennummer", new EingabeVorbelegung(null, false));
    _vorbelegungen.put("Fahrtrichtung", new EingabeVorbelegung(new String[] { "N", "O", "S", "W" }));
    _vorbelegungen.put("Fernziel", new EingabeVorbelegung(null, false));

    _vorbelegungen.put("PrflingPID", new EingabeVorbelegung(new String[0], false, false));
  }

  @Override
  protected void initPraefixe()
  {
    // super.initPraefixe();
    _praefixe.put("typ.messStellenGruppe", "msg");
    _praefixe.put("typ.messStelle", "ms");
  }

  @Override
  protected void initFelder()
  {
    // super.initFelder();
    _felder.put(HOT_MQGrp.MessStellenGruppe, new String[] { "Strae", "BezeichnungVonMS", "BezeichnungBisMS",
        "vonKnotennummer", "bisKnotennummer", "Fahrtrichtung", "Fernziel" });
    _felder.put(HOT_MQGrp.MessStelle, new String[] { "PrflingPID" });
  }

  public BildungsRegelnBWMQGrp()
  {
    super();

    initVorbelegungen();
  }

  public EingabeVorbelegung getEingabeVorbelegung(IHierarchieObjektTyp hotNeuesObjekt, String eingabeFeld,
      HierarchieObjekt verbundenesHO)
  {

    if (hotNeuesObjekt == HOT_MQGrp.MessStellenGruppe)
    {
      if (Arrays.asList(_felder.get(hotNeuesObjekt)).contains(eingabeFeld))
      {
        String text = null;
        EingabeVorbelegung result = _vorbelegungen.get(eingabeFeld);

// Keine Vererbung der Feldinhalte  		        
//        if (Arrays.asList(_felder.get(HOT_MQGrp.MessStellenGruppe)).contains(eingabeFeld))
//        {
//          Map<String, String> werte = Collections.emptyMap();
//
//          if (verbundenesHO != null)
//          {
//            werte = _historie.get(verbundenesHO.getSystemObjekte().get(0).getPid());
//          }
//
//          if (werte != null)
//          {
//            text = werte.get(eingabeFeld);
//          }
//        }

        if (text != null && text.length() > 0)
        {
          String[] liste = result.getListe();
          boolean leerErlaubt = result.isLeerErlaubt();
          return liste.length > 0 ? new EingabeVorbelegung(text, liste, leerErlaubt) : new EingabeVorbelegung(text,
              leerErlaubt);
        }
        else
        {
          return result;
        }
      }
    }
    else if (hotNeuesObjekt == HOT_MQGrp.MessStelle)
    {
      List<String> list = new ArrayList<String>();

      IHierarchie hierarchie = getHierarchie();

      if (hierarchie != null)
      {
        ConfigDataModel dataModel = hierarchie.getDataModel();
        if (dataModel != null)
        {
          SystemObjectType typ = dataModel.getType("typ.messQuerschnitt");
          if (typ != null)
          {
            Collection<SystemObjectType> typeSpez = new Vector<SystemObjectType>(1);
            typeSpez.add(typ);

            Map<String, ConfigurationArea> areas = dataModel.getAllConfigurationAreas();
            for (ConfigurationArea kb : areas.values())
            {
              for (SystemObject so : kb.getObjects(typeSpez, ObjectTimeSpecification.valid()))
              {
              	ConfigurationObject ko = (ConfigurationObject)so;
    			    	if (!(ko.getNotValidSince() >= ko.getConfigurationArea().getModifiableVersion())){
    			    	// Objekt wird in zuknftiger Version nicht ungltig	
                  if (so.getPid() != null && so.getPid().length() > 0){
                    list.add(so.getPid());
                  }  
                }
              }

              for (SystemObject so : kb.getNewObjects())
              {
                if (so.isOfType(typ))
                {
                  if (so.getPid() != null && so.getPid().length() > 0){
                    list.add(so.getPid());
                  }
                }
              }
            }
          }
        }
      }
      String[] ss = list.toArray(new String[list.size()]);        
      Arrays.sort(ss);
      _vorbelegungen.put("PrflingPID", new EingabeVorbelegung(ss, false, false));

      if (Arrays.asList(_felder.get(hotNeuesObjekt)).contains(eingabeFeld))
      {
        String text = null;
        EingabeVorbelegung result = _vorbelegungen.get(eingabeFeld);

        if (Arrays.asList(_felder.get(HOT_MQGrp.MessStelle)).contains(eingabeFeld))
        {
          Map<String, String> werte = Collections.emptyMap();

          if (verbundenesHO != null)
          {
            werte = _historie.get(verbundenesHO.getSystemObjekte().get(0).getPid());
          }

          if (werte != null)
          {
            text = werte.get(eingabeFeld);
          }
        }

        if (text != null && text.length() > 0)
        {
          String[] liste = result.getListe();
          boolean leerErlaubt = result.isLeerErlaubt();
          return liste.length > 0 ? new EingabeVorbelegung(text, liste, leerErlaubt) : new EingabeVorbelegung(text,
              leerErlaubt);
        }
        else
        {
          return result;
        }
      }
    }

    return BildungsRegelnTools.LEERE_VORBELEGUNG;
  }

  @Override
  public boolean setWerte(IHierarchieObjektTyp hotNeuesObjekt, HierarchieObjekt uebergeordnetesHO,
      HierarchieObjekt vorlageHO)
  {
//    boolean result = false;
//
//    if (HOT_MQGrp.MQPruefling.equals(hotNeuesObjekt))
//    {
//      if (uebergeordnetesHO != null)
//      {
//        EingabeVorbelegung belegung = getEingabeVorbelegung(hotNeuesObjekt, "PrflingPID", uebergeordnetesHO);
//        String wert = belegung.getWert();
//        
//        System.out.println("wert: " + wert);
//        
//
////        Map<IHierarchieObjektTyp, Integer> typMap = new HashMap<IHierarchieObjektTyp, Integer>();
////
////        for (HierarchieObjekt ho : getHierarchie().getUntergeordneteObjekte(uebergeordnetesHO))
////        {
////          IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
////          Integer anzahl = typMap.get(hot);
////          typMap.put(hot, anzahl == null ? 1 : anzahl.intValue() + 1);
////        }
////        Integer suffix = typMap.get(hotNeuesObjekt);
////        // if (suffix != null)
////        // {
////        // wert += (suffix.intValue() + 1);
////        // }
////        //        
////        // setWert(hotNeuesObjekt, "Bezeichnung", wert);
////        //
////        // if (suffix == null)
////        // {
////        // result = true;
////        // }
////
////        if (suffix == null)
//        {
//          setWert(hotNeuesObjekt, "Prfling", wert);
//          result = true;
//        }
//      }
//    }

    return super.setWerte(hotNeuesObjekt, uebergeordnetesHO, vorlageHO);
  }

  @Override
  public String getPidRegelText(IHierarchieObjektTyp hot)
  {
    String result;

    if (hot.equals(HOT_MQGrp.MessStellenGruppe))
    {
      result = getPraefix(hot.getTypen().get(0)) + ".Strae" + ".vonKnotennummer" + "-bisKnotennummer"
          + ".Fahrtrichtung";
    }
    else if (hot.equals(HOT_MQGrp.MessStelle))
    {
      result = getPraefix(hot.getTypen().get(0)) + ".PrflingPIDModifiziert";
    }
    else
    {
      result = super.getPidRegelText(hot);
    }

    return result;
  }

  @Override
  public String getNameRegelText(IHierarchieObjektTyp hot)
  {
    String result;

    if (hot.equals(HOT_MQGrp.MessStellenGruppe))
    {
      result = "Messstellengruppe" + " Strae" + " MS" + " BezeichnungVonMS" + "bis MS" + " BezeichnungBisMS" + " FR"
          + " Fernziel";
    }
    else if (hot.equals(HOT_MQGrp.MessStelle))
    {
      result = "MS NameDesPrflingsModifiziert";
    }
    else
    {
      result = super.getNameRegelText(hot);
    }

    return result;
  }

  @Override
  public String[] getPids(IHierarchieObjektTyp hotNeuesObjekt, HierarchieObjekt uebergeordnetesHO)
  {

    if (hotNeuesObjekt.equals(HOT_MQGrp.MessStellenGruppe) || hotNeuesObjekt.equals(HOT_MQGrp.MessStelle))
    {
      String text = "";
      if (hotNeuesObjekt.equals(HOT_MQGrp.MessStellenGruppe))
      {
        String text1 = "";
        text1 += getWert(hotNeuesObjekt, "Strae");
        text1 += ".";       
        text1 += getWert(hotNeuesObjekt, "vonKnotennummer");

        String s1 = BildungsRegelnTools.getPidText(text1, true, true);

        String text2 = "";
        text2 += getWert(hotNeuesObjekt, "bisKnotennummer");
        text2 += ".";
        text2 += getWert(hotNeuesObjekt, "Fahrtrichtung");
        
        String s2 = BildungsRegelnTools.getPidText(text2, true, true);
        
        text += s1;
        text += "-";
        text += s2;

      }
      else if (hotNeuesObjekt.equals(HOT_MQGrp.MessStelle))
      {       
        String s1 = getWert(hotNeuesObjekt, "PrflingPID");

        if (s1.startsWith("mq."))
        {
          String modifiziert = s1.replaceFirst("mq.", "");
          text += modifiziert;
        }
        else
        {
          String[] pids = super.getPids(hotNeuesObjekt, uebergeordnetesHO);
          text += pids[0];
        }

      }

      String[] result = new String[1];

      String pidTypNeuesObjekt = hotNeuesObjekt.getTypen().get(0);

      StringBuffer buffer = new StringBuffer();

      buffer.append(getPraefix(pidTypNeuesObjekt));
      buffer.append('.');

      if (hotNeuesObjekt.equals(HOT_MQGrp.MessStellenGruppe))
        buffer.append(text);
      else
        buffer.append(BildungsRegelnTools.getPidText(text, true, true));

      String pid = buffer.toString();
      Map<String, String> werte = _feldWerte.get(hotNeuesObjekt);
      if (werte != null && !werte.isEmpty())
      {
        _historie.put(pid, new HashMap<String, String>(werte));
      }

      result[0] = pid;

      return result;
    }
    else
    {
      return super.getPids(hotNeuesObjekt, uebergeordnetesHO);
    }

  }

  public String getName(SystemObject objekt)
  {
    String pid = objekt.getPid();

    Map<String, String> werte = _historie.get(pid);

    if (werte != null)
    {
      SystemObjectType typ = objekt.getType();
      StringBuffer buffer = new StringBuffer();

      if (typ.getPid().equals("typ.messStellenGruppe"))
      {
        buffer.append("Messstellengruppe ");
        buffer.append(werte.get("Strae"));

        buffer.append(" MS ");
        buffer.append(werte.get("BezeichnungVonMS"));

        buffer.append(" bis MS ");
        buffer.append(werte.get("BezeichnungBisMS"));

        buffer.append(" FR ");
        buffer.append(werte.get("Fernziel"));

        return buffer.toString();
      }
      else if (typ.getPid().equals("typ.messStelle"))
      {
        String s1 = super.getName(objekt);        
        
        if (s1.startsWith("ms."))
        {
          String pidMq = s1.replaceFirst("ms.", "mq.");
          IHierarchie hierarchie = super.getHierarchie();

          if (hierarchie != null)
          {
            ConfigDataModel dataModel = hierarchie.getDataModel();
            if (dataModel != null)
            {              
              Collection<SystemObjectType> typeSpez = new Vector<SystemObjectType>(1);
              SystemObjectType type2 = dataModel.getType("typ.messQuerschnitt");
              typeSpez.add(type2);

              SystemObject mq = null;
              
              Map<String, ConfigurationArea> areas = dataModel.getAllConfigurationAreas();
              for (ConfigurationArea kb : areas.values())
              {
                for (SystemObject so : kb.getObjects(typeSpez, ObjectTimeSpecification.valid()))
                {
                  if (so.getPid().equals(pidMq)){
                     mq = so;
                     break;
                  }
                  
                }
                
                for (SystemObject so : kb.getNewObjects())
                {
                  if (so.getPid().equals(pidMq))
                  {
                    mq = so;
                    break;
                  }
                }
              }
              
              if (mq != null){
                String name = mq.getName();
                
                if (name.startsWith("MQ"))
                {
                  String modifiziert = name.replaceFirst("MQ", "MS");
                  buffer.append(modifiziert);
                }else
                {                                    
                  System.out.println();
                  buffer.append(super.getName(objekt));
                }
                
              }else
              {                  
                buffer.append(super.getName(objekt));
              }
              
            }
          }
          
        }
     
        return buffer.toString();
      }
    }

    // return BildungsRegelnTools.LEERER_WERT;
    return super.getName(objekt);
  }

  @Override
  public Data getDatensatz(HierarchieObjekt ho, SystemObject objekt, AttributeGroup atg, Data daten,
      boolean nurPflichtFelder)
  {
    
//    if (objekt.getType().getPid().equals("typ.messQuerschnitt") ||
//        objekt.getType().getPid().equals("typ.messQuerschnittVirtuell") ||
//        objekt.getType().getPid().equals("typ.messQuerschnittHandZhlung"))
//    {
//      if (!objekt.getType().getAttributeGroups().contains(atg))
//      {
//        throw new IllegalArgumentException("Attributgruppe " + atg.getName() +
//            " kann nicht mit Systemobjekt " + objekt.getPid() + "verwendet werden "); 
//      }
//      
//      Data data;
//      if (daten == null)
//      {
//        data = DataFactory.createData(atg);
//        // Default und undefiniert Werte setzen
//        data.setToDefault();
//      }
//      else
//      {
//        ObjectLookup lookUp = LookupFactory.createLookupForModifiableVersion(getHierarchie().getDataModel());
//        data = DataFactory.createModifiableCopy(daten, lookUp);
//      }
//      
//      if (atg.getPid().equals("atg.messQuerschnittAllgemein"))
//      {
//        data.getItem("Typ").asTextValue().setText("HauptFahrbahn");
//        if (atg.getPid().equals("atg.messQuerschnittAllgemein"))
//        {
//          data.getItem("ErsatzMessQuerschnitt").asReferenceValue().setSystemObject(null);
//        } 
//        
//        return data;
//      }
//    }
  
    
    if (objekt.getType().getPid().equals("typ.messStelle")){
      if (!objekt.getType().getAttributeGroups().contains(atg))
      {
        throw new IllegalArgumentException("Attributgruppe " + atg.getName() +
            " kann nicht mit Systemobjekt " + objekt.getPid() + "verwendet werden "); 
      }

      ObjectLookup lookUp = LookupFactory.createLookupForModifiableVersion(getHierarchie().getDataModel());
      Data data = null;
      if (daten == null)
      {
        data = DataFactory.createData(atg);
        // Default und undefiniert Werte setzen
        data.setToDefault();
      }
      else
      {
        data = DataFactory.createModifiableCopy(daten, lookUp);
      }
      
      if (atg.getPid().equals("atg.messStelle"))
      {
        
//        String[] strings = _felder.get("PrflingPID");
        
        
        String wert = getWert(ho.getHierarchieObjektTyp(), "PrflingPID");
//          HierarchieObjekt ueo = (HierarchieObjekt)getHierarchie().getUebergeordnetesObjekt(ho);
//          EingabeVorbelegung belegung = getEingabeVorbelegung(ho.getHierarchieObjektTyp(), "PrflingPID", ueo);
//          
//          System.out.println("belegung: " + belegung.getWert());
        
        if (wert != null && wert.length() > 0){             
            data.getItem("Prfling").asReferenceValue().setSystemObjectPid(wert, lookUp);
        }
      }
      
      return data;      
    }
    
//    if (HOT_MQGrp.MQPruefling.equals(hotNeuesObjekt))
//    {
//      if (uebergeordnetesHO != null)
//      {
//        EingabeVorbelegung belegung = getEingabeVorbelegung(hotNeuesObjekt, "PrflingPID", uebergeordnetesHO);
//        String wert = belegung.getWert();
//        
//        System.out.println("wert: " + wert);
//        
//
////        Map<IHierarchieObjektTyp, Integer> typMap = new HashMap<IHierarchieObjektTyp, Integer>();
////
////        for (HierarchieObjekt ho : getHierarchie().getUntergeordneteObjekte(uebergeordnetesHO))
////        {
////          IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
////          Integer anzahl = typMap.get(hot);
////          typMap.put(hot, anzahl == null ? 1 : anzahl.intValue() + 1);
////        }
////        Integer suffix = typMap.get(hotNeuesObjekt);
////        // if (suffix != null)
////        // {
////        // wert += (suffix.intValue() + 1);
////        // }
////        //        
////        // setWert(hotNeuesObjekt, "Bezeichnung", wert);
////        //
////        // if (suffix == null)
////        // {
////        // result = true;
////        // }
////
////        if (suffix == null)
//        {
//          setWert(hotNeuesObjekt, "Prfling", wert);
//          result = true;
//        }
//      }
//    }
    
    return super.getDatensatz(ho, objekt, atg, daten, nurPflichtFelder);
  }

  @Override
  public void beobachterAnmelden(BildungsRegelnBeobachter beobachter)
  {
    // _beobachter.add(beobachter);
  }

  @Override
  public void beobachterAbmelden(BildungsRegelnBeobachter beobachter)
  {
    // _beobachter.remove(beobachter);
  }

  @Override
  public void alleBeobachterInformieren(String feldName, String wert)
  {
    for (BildungsRegelnBeobachter beobachter : _beobachter)
    {
      beobachter.wertGeaendert(feldName, wert);
    }
  }

  @Override
  public void wertGeaendert(String feldName, String wert)
  {
    // if (feldName.equals("Typ"))
    // {
    // if (wert.equals("HFB"))
    // {
    // alleBeobachterInformieren("TypName", "");
    // }
    // else if (wert.equals("NFB"))
    // {
    // alleBeobachterInformieren("TypName", "NFB");
    // }
    // else if (wert.equals("Aus"))
    // {
    // alleBeobachterInformieren("TypName", "Ausfahrt");
    // }
    // else if (wert.equals("Ein"))
    // {
    // alleBeobachterInformieren("TypName", "Einfahrt");
    // }
    // }
  }

}
