/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkamqgrp;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;

import de.bsvrz.buv.plugin.tkabasis.Activator;
import de.bsvrz.buv.plugin.tkabasis.DataFactory;
import de.bsvrz.buv.plugin.tkabasis.LookupFactory;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.AtgInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HODaten;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HOHierarchie;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HOMitStatus;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HierarchieObjekt;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchie;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieManager;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieObjektTyp;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.MengeInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.SystemObjektDaten;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.TypInfo;
import de.bsvrz.buv.plugin.tkabasis.pruefungen.IPruefungen;
import de.bsvrz.buv.plugin.tkabasis.pruefungen.LoeschenChecker;
import de.bsvrz.buv.plugin.tkabasis.pruefungen.PruefungenTools;
import de.bsvrz.buv.plugin.tkabasis.regeln.IBildungsRegeln;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.ReferenceArray;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.AttributeGroupUsage;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationChangeException;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.ConfigurationObjectType;
import de.bsvrz.dav.daf.main.config.NonMutableSet;
import de.bsvrz.dav.daf.main.config.ObjectLookup;
import de.bsvrz.dav.daf.main.config.ObjectSet;
import de.bsvrz.dav.daf.main.config.ObjectSetType;
import de.bsvrz.dav.daf.main.config.ObjectTimeSpecification;
import de.bsvrz.dav.daf.main.config.ReferenceType;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.puk.config.configFile.datamodel.ConfigConfigurationObject;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;
import de.bsvrz.puk.config.configFile.datamodel.ConfigSystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

public class HierachieManagerMQGrp implements IHierarchieManager
{
  /**
   * Debug-Logger fr Logging-Ausgaben
   */
  private final static Debug _debug = Debug.getLogger();

  // private static IHierarchieObjektTyp[] EBENE0 = new IHierarchieObjektTyp[] {HOT_MQ.KB};

  private static IHierarchieObjektTyp[] EBENE1 = new IHierarchieObjektTyp[] { HOT_MQGrp.MessStellenGruppe };

  private static IHierarchieObjektTyp[] EBENE2 = new IHierarchieObjektTyp[] { HOT_MQGrp.MessStelle };

  private static IHierarchieObjektTyp[] EBENE3 = new IHierarchieObjektTyp[] { HOT_MQGrp.MQPruefling,
      HOT_MQGrp.MQZufahrt, HOT_MQGrp.MQAbfahrt };
  
  private static IHierarchieObjektTyp[] SORTIER_FOLGE_1 = new IHierarchieObjektTyp[] { HOT_MQGrp.MessStellenGruppe };

  private static IHierarchieObjektTyp[] SORTIER_FOLGE_2 = new IHierarchieObjektTyp[] { HOT_MQGrp.MessStelle };

  private static IHierarchieObjektTyp[] SORTIER_FOLGE_3 = new IHierarchieObjektTyp[] { HOT_MQGrp.MQPruefling,
      HOT_MQGrp.MQZufahrt, HOT_MQGrp.MQAbfahrt };

  private IHierarchie _hierarchie;

  public IHierarchie getHierarchie()
  {
    return _hierarchie;
  }

  public void setHierarchie(IHierarchie hierarchie)
  {
    _hierarchie = hierarchie;
  }

  public void bestimmeUntergeordneteHOs(Object objekt)
  {
    ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(_hierarchie.getDataModel());

    bestimmeUntergeordneteHOs(objekt, lookup);
  }

  private void bestimmeUntergeordneteHOs(Object objekt, ObjectLookup lookup)
  {
    if (objekt instanceof ConfigurationArea)
    {
      ConfigurationArea kb = (ConfigurationArea)objekt;

      for (IHierarchieObjektTyp hot : EBENE1)
      {
      	String typPid = hot.getTypen().get(0);
			  SystemObjectType  typ = _hierarchie.getDataModel().getType(typPid);
			  
			  if (typ == null)
			  {
			  	throw new RuntimeException("Typ " + typPid + " wurde im aktuellen Datemmodell nicht gefunden");
			  }
        
        Collection<SystemObjectType> typeSpez = new Vector<SystemObjectType>(1);
        typeSpez.add(typ);

        for (SystemObject so : kb.getObjects(typeSpez, ObjectTimeSpecification.valid()))
        {
          ConfigurationObject ko = (ConfigurationObject)so;
          if (!(ko.getNotValidSince() >= ko.getConfigurationArea().getModifiableVersion()))
          // Objekt wird in zuknftiger Version nicht ungltig
          {
            HierarchieObjekt ho = new HierarchieObjekt(hot, new SystemObject[] { so }, EBENE1);
            _hierarchie.objektHinzufuegen(kb, ho);
            bestimmeUntergeordneteHOs(ho);
          }
        }

        for (SystemObject so : kb.getNewObjects())
        {
          if (so.isOfType(typ))
          {
            HierarchieObjekt ho = new HierarchieObjekt(hot, new SystemObject[] { so }, EBENE1);
            _hierarchie.objektHinzufuegen(kb, ho);
            bestimmeUntergeordneteHOs(ho);
          }
        }
      }
    }
    else if (objekt instanceof HierarchieObjekt)
    {
      HierarchieObjekt ho = (HierarchieObjekt)objekt;

      if (ho.getHierarchieObjektTyp() == HOT_MQGrp.MessStelle)
      {

        ConfigurationObject co = (ConfigurationObject)ho.getSystemObjekte().get(0);

        NonMutableSet zufahrten = co.getNonMutableSet("Zufahrten");
        if (zufahrten != null)
        {
          for (SystemObject so : zufahrten.getElementsInModifiableVersion())
          {
            if (so != null)
            {
              if (so.isOfType("typ.messQuerschnitt"))
              {
                IHierarchieObjektTyp hot = HOT_MQGrp.MQZufahrt;
                HierarchieObjekt kind = new HierarchieObjekt(hot, new SystemObject[] { so }, getTypSortierFolge(hot));
                _hierarchie.objektHinzufuegen(ho, kind);
                // bestimmeUntergeordneteHOs(kind);
              }
            }
          }
        }
        else
        {
          // Log Fehler: Menge fehlt
          _debug.warning("Menge Zufahrten nicht gefunden fr Objekt " + co.getPid());
        }

        NonMutableSet abfahrten = co.getNonMutableSet("Abfahrten");
        if (abfahrten != null)
        {
          for (SystemObject so : abfahrten.getElementsInModifiableVersion())
          {
            if (so != null)
            {
              if (so.isOfType("typ.messQuerschnitt"))
              {
                IHierarchieObjektTyp hot = HOT_MQGrp.MQAbfahrt;
                HierarchieObjekt kind = new HierarchieObjekt(hot, new SystemObject[] { so }, getTypSortierFolge(hot));
                _hierarchie.objektHinzufuegen(ho, kind);
                // bestimmeUntergeordneteHOs(kind);
              }              
            }
          }
        }
        else
        {
          // Log Fehler: Menge fehlt
          _debug.warning("Menge Zufahrten nicht gefunden fr Objekt " + co.getPid());
        }

        ConfigDataModel dataModel = _hierarchie.getDataModel();
        AttributeGroup atg = dataModel.getAttributeGroup("atg.messStelle");

        Data data = ((ConfigConfigurationObject)co).getConfigurationData(atg, lookup);

        if (data != null)
        {
          SystemObject ref = data.getReferenceValue("Prfling").getSystemObject();
          if (ref != null)
          {
            IHierarchieObjektTyp hot = HOT_MQGrp.MQPruefling;
            HierarchieObjekt kind = new HierarchieObjekt(hot, new SystemObject[] { ref }, getTypSortierFolge(hot));
            _hierarchie.objektHinzufuegen(ho, kind);
            // bestimmeUntergeordneteHOs(kind);
          }
        }
      }
      else if (ho.getHierarchieObjektTyp() == HOT_MQGrp.MessStellenGruppe)
      {
        ConfigurationObject co = (ConfigurationObject)ho.getSystemObjekte().get(0);

        ConfigDataModel dataModel = _hierarchie.getDataModel();
        AttributeGroup atg = dataModel.getAttributeGroup("atg.messStellenGruppe");

        Data data = ((ConfigConfigurationObject)co).getConfigurationData(atg, lookup);

        if (data != null)
        {
          SystemObject[] sos = data.getReferenceArray("MessStellen").getSystemObjectArray();

          if (sos != null && sos.length > 0)
          {
            for (int i = 0; i < sos.length; i++)
            {
              SystemObject r = sos[i];
              if (r.isOfType("typ.messStelle"))
              {
                IHierarchieObjektTyp hot = HOT_MQGrp.MessStelle;
                HierarchieObjekt kind = new HierarchieObjekt(hot, new SystemObject[] { r }, getTypSortierFolge(hot));
                _hierarchie.objektHinzufuegen(ho, kind);
                bestimmeUntergeordneteHOs(kind);
              }
            }
          }

        }
      }
    }
  }

  public IHierarchieObjektTyp[] getUntergeordneteHOTs(Object objekt)
  {

    if (objekt instanceof ConfigurationArea)
    {
      return EBENE1;
    }
    else if (objekt instanceof HierarchieObjekt)
    {
      HierarchieObjekt ho = (HierarchieObjekt)objekt;

      IHierarchieObjektTyp typ = ho.getHierarchieObjektTyp();

      if (typ.equals(HOT_MQGrp.MessStellenGruppe))
      {
        return EBENE2;
      }
      else if (typ.equals(HOT_MQGrp.MessStelle))
      {
        return EBENE3;
      }
    }
    return null;
  }

  private List<ObjectSet> erzeugeMengen(IHierarchieObjektTyp hot, SystemObjectType typ, ConfigurationArea kb,
      ConfigDataModel dataModel, MultiStatus status)
  {
    List<ObjectSet> result = new Vector<ObjectSet>();

    for (String mengenName : hot.getTypInfo(typ.getPid()).getMengen())
    {
      MengeInfo info = hot.getTypInfo(typ.getPid()).getMengeInfo(mengenName);

      SystemObjectType mengenTyp = dataModel.getType(info.getTyp());
      ObjectSet menge;
      try
      {
        menge = (ObjectSet)kb.createConfigurationObject((ConfigurationObjectType)mengenTyp, "", mengenName, null);

        status.merge(new Status(Status.OK, Activator.PLUGIN_ID, "Menge " + mengenName + " vom Typ "
            + mengenTyp.getPid() + " erzeugt"));

      }
      catch (ConfigurationChangeException e)
      {
        status.merge(new Status(Status.ERROR, Activator.PLUGIN_ID, "Fehler beim Erzeugen der Menge " + mengenName
            + " vom Typ " + mengenTyp.getName() + " erzeugt", e));
        return result;
      }

      result.add(menge);
    }

    return result;
  }

//  private List<ObjectSet> erzeugeMengenFuerTyp(IHierarchieObjektTyp hot, String typ, ConfigurationArea kb,
//      MultiStatus status, List<SystemObject> undoListe)
//  {
//    List<ObjectSet> result = new Vector<ObjectSet>();
//
//    for (String mengenName : hot.getTypInfo(typ).getMengen())
//    {
//      MengeInfo info = hot.getTypInfo(typ).getMengeInfo(mengenName);
//      String mengenTyp = info.getTyp();
//      ObjectSetType setTyp = kb.getDataModel().getObjectSetType(mengenTyp);
//      ObjectSet menge;
//      try
//      {
//        menge = (ObjectSet)kb.createConfigurationObject(setTyp, "", mengenName, null);
//
//        undoListe.add(0, menge);
//        status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Menge " + mengenName + " vom Typ " + setTyp.getPid()
//            + " erzeugt"));
//      }
//      catch (ConfigurationChangeException e)
//      {
//        status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Fehler beim Erzeugen der Menge " + mengenName
//            + " vom Typ " + setTyp.getName() + " erzeugt", e));
//        return result;
//      }
//
//      result.add(menge);
//    }
//
//    return result;
//  }

  private SystemObject erzeugeSystemObjekt(String pid, ConfigurationObjectType typ, ConfigurationArea kb,
      List<ObjectSet> mengen, MultiStatus status)
  {
    SystemObject result = null;

    try
    {
      result = kb.createConfigurationObject(typ, pid, "", mengen);

      if (mengen != null && !mengen.isEmpty())
      {
        MultiStatus unterStatus = new MultiStatus(Activator.PLUGIN_ID, IStatus.OK, "Systemobjekt " + pid + " vom Typ "
            + typ.getPid() + " erzeugt", null);
        status.add(unterStatus);

        for (ObjectSet set : mengen)
        {
          unterStatus.add(new Status(Status.OK, Activator.PLUGIN_ID, "Menge " + set.getName() + " dem Systemobjekt "
              + pid + " vom Typ " + typ.getPid() + " zugewiesen"));
        }
      }
      else
      {
        status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Systemobjekt " + pid + " vom Typ " + typ.getPid()
            + " erzeugt"));
      }
    }
    catch (ConfigurationChangeException e)
    {
      status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Systemobjekt " + pid + " vom Typ " + typ.getPid()
          + " konnte nicht erzeugt werden", e));
    }

    return result;
  }

  private ObjectSet erzeugeBeliebigeMenge(String pidTyp, String name, ConfigurationObject ko, MultiStatus status)
  {
    // TODO Auto-generated method stub
    ObjectSet result = null;

    ObjectSetType setTyp = _hierarchie.getDataModel().getObjectSetType(pidTyp);
    try
    {
      result = (ObjectSet)ko.getConfigurationArea().createConfigurationObject(setTyp, null, name, null);
      ko.addSet(result);

      status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Menge " + name + " vom Typ " + setTyp.getPid()
          + " fr Systemobjekt " + ko.getPid() + " erzeugt"));

    }
    catch (ConfigurationChangeException e)
    {
      status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Menge " + name + " vom Typ " + setTyp.getPid()
          + " konnte fr das Systemobjekt " + ko.getPid() + " nicht erstellt werden", e));
    }

    return result;
  }

  private void behandleMengenZuordnung(Object vater, SystemObject so, IHierarchieObjektTyp hot, MultiStatus status)
  {
    if (vater instanceof HierarchieObjekt)
    {
      HierarchieObjekt ho = (HierarchieObjekt)vater;

      SystemObject vaterSo = ho.getSystemObjekte().get(0);

      ObjectSet menge = null;
      
      ConfigDataModel dataModel = _hierarchie.getDataModel();

      if (vaterSo.isOfType("typ.messStellenGruppe"))
      {
        if (so.isOfType("typ.messStelle"))
        {          
          AttributeGroup atg = dataModel.getAttributeGroup("atg.messStellenGruppe");

          ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(_hierarchie.getDataModel());
          Data data = ((ConfigConfigurationObject)vaterSo).getConfigurationData(atg, lookup);

          if (data == null)
          {
            data = DataFactory.createData(atg);
            // Default und undefiniert Werte setzen
            data.setToDefault();
          }

          ReferenceArray ra = data.getReferenceArray("MessStellen");
          int len = ra.getLength();
          ra.setLength(len + 1);
          ra.getReferenceValue(len).setSystemObject(so);

          try
          {
          	if (!HierarchieObjekt.isAttributgruppeAenderbar(vaterSo, atg))
            {  
              vaterSo = behandleSystemObjektAenderung(ho, (ConfigurationObject)vaterSo, vaterSo.getPid());
              ho.setSystemObjekt(0, vaterSo);
            }  
          	
            vaterSo.setConfigurationData(atg, data);
            status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Systemobjekt " + vaterSo.getPid() + " als Referenz "
                + so.getPid() + " gesetzt"));
          }
          catch (ConfigurationChangeException e)
          {
            status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Systemobjekt " + vaterSo
                + " konnt nicht als Refrenz " + so.getPid() + " gesetzt werden", e));
          }
        }
      }
      else if (vaterSo.isOfType("typ.messStelle"))
      {
        if (so.isOfType("typ.messQuerschnitt"))
        {
          List<SystemObject> list = new ArrayList<SystemObject>();

          NonMutableSet set = null;

          if (!hot.equals(HOT_MQGrp.MQZufahrt))
          {
            set = ((ConfigurationObject)vaterSo).getNonMutableSet("Zufahrten");

            if (set != null)
            {
              list.addAll(set.getElementsInModifiableVersion());
            }
          }

          if (!hot.equals(HOT_MQGrp.MQAbfahrt))
          {
            set = ((ConfigurationObject)vaterSo).getNonMutableSet("Abfahrten");

            if (set != null)
            {
              list.addAll(set.getElementsInModifiableVersion());
            }
          }

          if (!hot.equals(HOT_MQGrp.MQPruefling))
          {

            AttributeGroup atg = dataModel.getAttributeGroup("atg.messStelle");
            ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(_hierarchie.getDataModel());
            Data data = ((ConfigConfigurationObject)vaterSo).getConfigurationData(atg, lookup);
            if (data != null)
            {
              SystemObject pr = data.getReferenceValue("Prfling").getSystemObject();
              if (pr != null)
                list.add(pr);
            }
          }

          if (list.contains(so))
          {
            status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Systemobjekt " + so.getPid()
                + " konnte nicht als Referenz in " + vaterSo.getPid()
                + " gesetzt werden, weil alle der Messstelle zugeordneten MQs eindeutig sein mssen."));
            return;
          }

          if (hot.equals(HOT_MQGrp.MQAbfahrt))
          {

            menge = ((ConfigurationObject)vaterSo).getNonMutableSet("Abfahrten");

            if (menge == null)
            {
              menge = erzeugeBeliebigeMenge("menge.abfahrten", "Abfahrten", 
              		(ConfigurationObject)vaterSo, status);
            }

            if (menge == null || status.matches(IStatus.ERROR | IStatus.CANCEL))
            {
              return;
            }

            try
            {
              menge.add(so);

              status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Systemobjekt " + so.getPid()
                  + " wird der Menge Abfahrten des Systemobjekts " + vaterSo.getPid() + " hinzugefgt"));
            }
            catch (ConfigurationChangeException e)
            {
              status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Systemobjekt " + so.getPid()
                  + " konnte nicht der Menge Abfahrten des Systemobjekts " + vaterSo.getPid() + " hinzugefgt werden",
                  e));
            }
          }
          else if (hot.equals(HOT_MQGrp.MQZufahrt))
          {
            menge = ((ConfigurationObject)vaterSo).getNonMutableSet("Zufahrten");

            if (menge == null)
            {
              menge = erzeugeBeliebigeMenge("menge.zufahrten", "Zufahrten", 
              		(ConfigurationObject)vaterSo, status);
            }

            if (menge == null || status.matches(IStatus.ERROR | IStatus.CANCEL))
            {
              return;
            }

            try
            {
              menge.add(so);

              status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Systemobjekt " + so.getPid()
                  + " wird der Menge Zufahrten des Systemobjekts " + vaterSo.getPid() + " hinzugefgt"));
            }
            catch (ConfigurationChangeException e)
            {
              status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Systemobjekt " + so.getPid()
                  + " konnte nicht der Menge Zufahrten des Systemobjekts " + vaterSo.getPid() + " hinzugefgt werden",
                  e));
            }
          }
          else if (hot.equals(HOT_MQGrp.MQPruefling))
          {
            AttributeGroup atg = dataModel.getAttributeGroup("atg.messStelle");

            ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(_hierarchie.getDataModel());
            Data data = ((ConfigConfigurationObject)vaterSo).getConfigurationData(atg, lookup);

            if (data == null)
            {
              data = DataFactory.createData(atg);
              // Default und undefiniert Werte setzen
              data.setToDefault();
            }
            else
            {
              SystemObject sx = data.getReferenceValue("Prfling").getSystemObject();
              List<HierarchieObjekt> uos = getHierarchie().getUntergeordneteObjekte(vater);

              HierarchieObjekt del = null;
              for (HierarchieObjekt hox : uos)
              {
                SystemObject kind = hox.getSystemObjekt(0);
                if (kind.equals(sx))
                {
                  del = hox;
                }
              }
              if (del != null)
              {
                getHierarchie().objektEntfernen(vater, del);
              }

            }

            data.getReferenceValue("Prfling").setSystemObject(so);

            try
            {
            	if (!HierarchieObjekt.isAttributgruppeAenderbar(vaterSo, atg))
              {  
                vaterSo = behandleSystemObjektAenderung(ho, (ConfigurationObject)vaterSo, vaterSo.getPid());
                ho.setSystemObjekt(0, vaterSo);
              }  
            	
              vaterSo.setConfigurationData(atg, data);
              status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Systemobjekt " + vaterSo.getPid()
                  + " als Referenz " + so.getPid() + " gesetzt"));
            }
            catch (ConfigurationChangeException e)
            {
              status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Systemobjekt " + vaterSo
                  + " konnt nicht als Referenz " + so.getPid() + " gesetzt werden", e));
            }
          }
        }
      }
    }
  }
   
  public HOMitStatus erzeugeNeuesHO(ConfigurationArea kb, Object vater, String pid[], IHierarchieObjektTyp hot, 
  		boolean pruefePid)
  {
    ConfigDataModel dataModel = _hierarchie.getDataModel();

    ConfigurationObjectType typ = (ConfigurationObjectType)dataModel.getType(hot.getTypen().get(0));
    if (typ == null)
    {
      throw new IllegalArgumentException("Fr " + hot.getTypen().get(0)
          + " kann kein Typ im Datenmodell bestimmt werden");
    }

    IHierarchieObjektTyp[] ebene = getUntergeordneteHOTs(vater);

    if (!Arrays.asList(ebene).contains(hot))
    {
      throw new IllegalArgumentException("Unzulssiger HOT " + hot + " fr bergeordnetes Objekt " + vater);
    }

    // EbeneInfo info = new EbeneInfo(vater, hot);
    MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Erzeuge neues Hierarchieobjekt vom Typ "
        + hot, null);

     HierarchieObjekt vaterHO = null;

    if (vater instanceof HierarchieObjekt)
    {
      vaterHO = (HierarchieObjekt)vater;
    }
    else if (!(vater instanceof ConfigurationArea))
    {
      throw new IllegalArgumentException("bergebenes Objekt " + vater + " ist von unzulssigem Typ");
    }

    HierarchieObjekt ho = null;

    if (!hot.isSystemObjektVorhanden())
    {
      if (pruefePid)
      {
        status.add(new PruefungenTools(dataModel).pruefePid(pid[0]));
      }

      if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
      {
        List<ObjectSet> mengen = erzeugeMengen(hot, typ, kb, dataModel, status);

        if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
        {
          SystemObject so = erzeugeSystemObjekt(pid[0], typ, kb, mengen, status);

          if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
          {
            if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
            {

              // if (hot.equals(HOT_AQ.Zeichen) || hot.equals(HOT_AQ.Ligaturen) || hot.equals(HOT_AQ.WzgInhaltGrafik)
              // || hot.equals(HOT_AQ.WechselText) || hot.equals(HOT_AQ.WvzInhalt) || hot.equals(HOT_AQ.Zeichensatz)
              // || hot.equals(HOT_AQ.Anzeige) || hot.equals(HOT_AQ.AnzeigeQuerschnittSub))
              {
                behandleMengenZuordnung(vater, so, hot, status);
              }
              if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
              {
                ho = new HierarchieObjekt(hot, new SystemObject[] { so }, getTypSortierFolge(hot));
                getHierarchie().objektHinzufuegen(vater, ho);
              }
            }
          }
        }
      }
    }
    else
    {
      SystemObject so = dataModel.getObject(pid[0]);

      if (so == null)
      {
        Collection<ConfigurationArea> areas = dataModel.getAllConfigurationAreas().values();
        for (ConfigurationArea ca : areas)
        {
          Collection<SystemObject> elements = ca.getNewObjects();
          for (SystemObject soNeu : elements)
          {
            if (pid[0].equals(soNeu.getPid()))
            {
              so = soNeu;
            }
          }
        }
      }

      if (so != null)
      {
        behandleMengenZuordnung(vater, so, hot, status);
        if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
        {
          ho = new HierarchieObjekt(hot, new SystemObject[] { so }, getTypSortierFolge(hot));
          getHierarchie().objektHinzufuegen(vater, ho);
        }
      }
    }

    return new HOMitStatus(ho, status);
  }

  public HOMitStatus kopiereHO(HierarchieObjekt ho, ConfigurationArea kb, Object neuerVater, String neuePid[], boolean rekursiv,
      boolean isWurzel)
  {
    return kopiereHO(ho, kb, neuerVater, neuePid, rekursiv, true, true, isWurzel);
  }

  private HOMitStatus kopiereHO(HierarchieObjekt ho, ConfigurationArea kb, Object neuerVater, String neuePid[], boolean rekursiv,
      boolean anwendenBR, boolean pruefePid, boolean isWurzel)
  {
    // if (neuePid == null && !(neuerVater instanceof HierarchieObjekt))
    // {
    // throw new IllegalArgumentException("Wenn fr die PID null bergeben wird"
    // + ", muss das neue bergeordnete Objekt vom Typ " + HierarchieObjekt.class + " sein");
    // }

    MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Kopiere Hierarchieobjekt " + ho, null);

    String[] pid = neuePid;

    if (pid == null)
    {
      pid = new String[ho.getSystemObjekte().size()];

      for (int i = 0; i < pid.length; i += 1)
      {
        pid[i] = ho.getSystemObjekt(i).getPid();
      }

      // IBildungsRegeln bildungsRegeln = DatenSpeicher.getInstanz().getBildungsRegeln();
      // bildungsRegeln.setWerte(ho.getHierarchieObjektTyp(), (HierarchieObjekt)neuerVater, ho);
      //
      // if (!ho.getHierarchieObjektTyp().isSystemObjektVorhanden())
      // {
      //
      // pid = bildungsRegeln.getPids(ho.getHierarchieObjektTyp(), (HierarchieObjekt)neuerVater);
      //
      // }
      // else
      // {
      // pid = new String[1];
      // pid[0] = ho.getSystemObjekt(0).getPid();
      // }
    }

    HOMitStatus result = erzeugeNeuesHO(kb, neuerVater, pid, ho.getHierarchieObjektTyp(), pruefePid);
    HierarchieObjekt neu = result.getHO();
    status.add(result.getStatus());

    if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
    {
      status.merge(kopiereDaten(ho, neu, anwendenBR, anwendenBR));
    }

    if (!status.matches(IStatus.ERROR | IStatus.CANCEL) && rekursiv)
    {
      for (HierarchieObjekt lauf : _hierarchie.getUntergeordneteObjekte(ho))
      {
        MultiStatus meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK,
            "Kopiere untergeordnetes Hierarchieobjekt " + lauf + " zu " + ho, null);
        result = kopiereHO(lauf, kb, neu, null, true, anwendenBR, pruefePid, false);
        meldung.merge(result.getStatus());
        status.add(meldung);
        if (status.matches(IStatus.ERROR | IStatus.CANCEL))
        {
          break;
        }
      }
    }

    return new HOMitStatus(neu, status);
  }

  private MultiStatus kopiereDaten(HierarchieObjekt srcHO, HierarchieObjekt destHO, boolean bildungsRegelnFuerName,
      boolean bildungsRegelnFuerAtg)
  {
    if (!srcHO.getHierarchieObjektTyp().equals(destHO.getHierarchieObjektTyp()))
    {
      throw new IllegalArgumentException(
          "Beide bergebenen Hierarchieobjekte mssen vom gleichen Hierarchieobjekttyp sein");
    }

    int index = 0;
    MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Kopiere Daten von " + srcHO + " nach "
        + destHO, null);
    IBildungsRegeln br = DatenSpeicher.getInstanz().getBildungsRegeln();
    ConfigDataModel dataModel = _hierarchie.getDataModel();

    ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);

    for (SystemObject src : srcHO.getSystemObjekte())
    {
      SystemObject dest = destHO.getSystemObjekte().get(index);
      String name;
      if (bildungsRegelnFuerName)
      {
        name = br.getName(dest);
      }
      else
      {
        name = src.getName();
      }

      if (!destHO.getHierarchieObjektTyp().isSystemObjektVorhanden())
      {
        try
        {

          dest.setName(name);
          status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Setze Name \"" + name + "\" fr Systemobjekt "
              + dest.getPid()));
        }
        catch (ConfigurationChangeException e)
        {
          status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Name \"" + name + "\" fr Systemobjekt "
              + dest.getPid() + " konnte nicht gesetzt werden", e));
          return status;
        }

        for (AttributeGroupUsage atgu : src.getUsedAttributeGroupUsages())
        {
          Data dataSrc = ((ConfigSystemObject)src).getConfigurationData(atgu, lookup);
          if (dataSrc != null)
          {
            try
            {
              Data dataDest = DataFactory.createModifiableCopy(dataSrc, lookup);
              if (bildungsRegelnFuerAtg)
              {
                dataDest = br.getDatensatz(destHO, dest, atgu.getAttributeGroup(), dataDest, true);
              }
              dest.setConfigurationData(atgu, dataDest);
              
              if (atgu.getAttributeGroup().getPid().equals("atg.messStelle")){

                List<HierarchieObjekt> uos = _hierarchie.getUntergeordneteObjekte(destHO);
                
                ArrayList<HierarchieObjekt> delList = new ArrayList<HierarchieObjekt>();
                
                for (HierarchieObjekt ho2 : uos)
                {
                  delList.add(ho2);
                }
                
                for (HierarchieObjekt ho2 : delList)
                {
                  _hierarchie.objektEntfernen(destHO, ho2);                    
                }
                
                bestimmeUntergeordneteHOs(destHO);	                  
              }
              
              
              status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Kopiere Daten der ATG-Verwendung "
                  + atgu.getAttributeGroup().getPid() + "/" + atgu.getAspect().getPid() + " fr Systemobjekt "
                  + dest.getPid()));
            }
            catch (ConfigurationChangeException e)
            {
              status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Daten der ATG-Verwendung "
                  + atgu.getAttributeGroup().getPid() + "/" + atgu.getAspect().getPid() + " fr Systemobjekt "
                  + dest.getPid() + " konnten nicht kopiert werden", e));
              return status;
            }
          }
        }

      }

      index++;
    }

    return status;
  }

  public HOMitStatus verschiebeHO(HierarchieObjekt ho, Object neuerVater)
  {
    Object alterVater = _hierarchie.getUebergeordnetesObjekt(ho);
    ConfigurationArea srcKB = _hierarchie.getKonfigurationsBereich(alterVater);
    Assert.isNotNull(srcKB, "Der Konfigurationsbereich zu " + alterVater + " darf nicht null sein");
    ConfigurationArea destKB = _hierarchie.getKonfigurationsBereich(neuerVater);
    Assert.isNotNull(destKB, "Der Konfigurationsbereich zu " + neuerVater + " darf nicht null sein");

    MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Verschiebe Hierarchieobjekt " + ho, null);
    
    HierarchieObjekt neu = null;

    if (srcKB.equals(destKB)) // HO wird im gleichen kb verschoben
    {
      if (neuerVater.equals(alterVater))
      {
        status.add(new Status(Status.INFO, Activator.PLUGIN_ID, "Objekt " + ho
            + " kann nicht in sich selbst verschoben werden"));
      }
      else
      {
        behandleVerschieben(ho, neuerVater, status);
        
        if (!status.matches(IStatus.ERROR | IStatus.CANCEL ))
  			{
  				neu = ho; // HO verschoben
  			}
      }
    }
    else
    // HO wird in einen anderen kb verschoben
    {
    	HOHierarchie hoHierarchie = new HOHierarchie(ho, _hierarchie);
   	  
   	  status.add(loescheHO(ho));
   	  
   	  if (!status.matches( IStatus.ERROR | IStatus.CANCEL ))
   	  {
   	    HOMitStatus result = kopiereHORekursiv(hoHierarchie.getWurzel(), destKB, neuerVater,
   	        hoHierarchie, true);
   	    status.add( result.getStatus() );
   	    neu = result.getHO();
   	  }
    }
    return new HOMitStatus( neu, status );
  }
  
  private HOMitStatus kopiereHORekursiv(HierarchieObjekt ho, ConfigurationArea kb, Object vater,
      HOHierarchie hierarchie, boolean isWurzel)
  {
    MultiStatus meldung;
    
    if (ho == hierarchie.getWurzel())
    {  
      meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Kopiere Hierarchieobjekt " + ho, null);
    }
    else
    {
      meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Kopiere untergeordnetes Hierarchieobjekt " + 
          ho + " zu " + vater, null);
    }
    
    HOMitStatus result = kopiereHO(ho, kb, vater, null, false, false, false, isWurzel);
    meldung.merge( result.getStatus() );
    HierarchieObjekt neu = result.getHO();

    if (neu != null)
    {  
      for (HierarchieObjekt kind: hierarchie.getUntergeordneteObjekte( ho ))
      {
        result = kopiereHORekursiv(kind, kb, neu, hierarchie, false);
        meldung.add( result.getStatus() );
      }
    }
    
    return new HOMitStatus(neu, meldung);
  }

  private void behandleVerschieben(HierarchieObjekt ho, Object neuerVater, MultiStatus status)
  {
    IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
    Object alterVater = _hierarchie.getUebergeordnetesObjekt(ho);

    if (hot.equals(HOT_MQGrp.MessStelle))
    {
      // MessStellenGruppe
      Assert.isLegal(alterVater instanceof HierarchieObjekt
          && ((HierarchieObjekt)alterVater).getHierarchieObjektTyp().equals(HOT_MQGrp.MessStellenGruppe));
      Assert.isLegal(neuerVater instanceof HierarchieObjekt
          && ((HierarchieObjekt)neuerVater).getHierarchieObjektTyp().equals(HOT_MQGrp.MessStellenGruppe));

      behandleAtgVerschieben(ho, (HierarchieObjekt)alterVater, (HierarchieObjekt)neuerVater, "atg.messStellenGruppe",
          "MessStellen", true, status);
    }
    else if (hot.equals(HOT_MQGrp.MQPruefling))
    {
      // MessStelle
      Assert.isLegal(alterVater instanceof HierarchieObjekt
          && ((HierarchieObjekt)alterVater).getHierarchieObjektTyp().equals(HOT_MQGrp.MessStelle));
      Assert.isLegal(neuerVater instanceof HierarchieObjekt
          && ((HierarchieObjekt)neuerVater).getHierarchieObjektTyp().equals(HOT_MQGrp.MessStelle));

      behandleAtgVerschieben(ho, (HierarchieObjekt)alterVater, (HierarchieObjekt)neuerVater, "atg.messStelle",
          "Prfling", false, status);

    }
    else if (hot.equals(HOT_MQGrp.MQZufahrt))
    {
      // MessStelle
      Assert.isLegal(alterVater instanceof HierarchieObjekt
          && ((HierarchieObjekt)alterVater).getHierarchieObjektTyp().equals(HOT_MQGrp.MessStelle));
      Assert.isLegal(neuerVater instanceof HierarchieObjekt
          && ((HierarchieObjekt)neuerVater).getHierarchieObjektTyp().equals(HOT_MQGrp.MessStelle));

      behandleMengenVerschieben(ho, (HierarchieObjekt)alterVater, (HierarchieObjekt)neuerVater, "menge.zufahrten",
          "Zufahrten", 0, status);
    }
    else if (hot.equals(HOT_MQGrp.MQAbfahrt))
    {
      // MessStelle
      Assert.isLegal(alterVater instanceof HierarchieObjekt
          && ((HierarchieObjekt)alterVater).getHierarchieObjektTyp().equals(HOT_MQGrp.MessStelle));
      Assert.isLegal(neuerVater instanceof HierarchieObjekt
          && ((HierarchieObjekt)neuerVater).getHierarchieObjektTyp().equals(HOT_MQGrp.MessStelle));

      behandleMengenVerschieben(ho, (HierarchieObjekt)alterVater, (HierarchieObjekt)neuerVater, "menge.abfahrten",
          "Abfahrten", 0, status);
    }
  }

  private ConfigurationArea bestimmeKbAusObjekt(Object neuerVater)
  {
    ConfigurationArea kb = null;
    HierarchieObjekt vaterHO = null;

    if (neuerVater instanceof ConfigurationArea)
    {
      kb = (ConfigurationArea)neuerVater;
    }
    else if (neuerVater instanceof HierarchieObjekt)
    {
      vaterHO = (HierarchieObjekt)neuerVater;
      Object objekt = _hierarchie.getUebergeordnetesObjekt(vaterHO);

      while (objekt instanceof HierarchieObjekt)
      {
        objekt = _hierarchie.getUebergeordnetesObjekt((HierarchieObjekt)objekt);
      }
      if (!(objekt instanceof ConfigurationArea))
      {
        throw new IllegalStateException("Konfigurationsbereich zum HO " + vaterHO + " nicht gefunden ");
      }
      kb = (ConfigurationArea)objekt;

    }
    else
    {
      throw new IllegalArgumentException("bergebenes Objekt " + neuerVater + " ist von unzulssigem Typ");
    }

    return kb;
  }
 
  public IStatus pruefeLoeschen(HierarchieObjekt ho)
  {
    return new Status(Status.ERROR, Activator.PLUGIN_ID, Status.OK, "NICHT IMPLEMENTIERT", null);
  }

  /*
   * Status status = new Status(Status.WARNING, Activator.PLUGIN_ID, Status.WARNING, newRecord, null);
   * 
   * //Status[] children = new Status[1]; //children[0] = child; // //MultiStatus status = new
   * MultiStatus(Activator.PLUGIN_ID, Status.OK, children, "Aktion wurde erfolgreich beendet", null);
   * 
   * Activator.getDefault().getLog().log(status);
   */

  public IStatus loescheHO(HierarchieObjekt ho)
  {
    MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Lsche Hierarchieobjekt " + ho, null);

    loescheHO(ho, status);

    return status;
    // Activator.getDefault().getLog().log(status);
  }

  protected void loescheHO(HierarchieObjekt ho, MultiStatus status)
  {
    IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();

    if (!Arrays.asList(EBENE1).contains(hot) && !Arrays.asList(EBENE2).contains(hot)
        && !Arrays.asList(EBENE3).contains(hot))
    {
      throw new IllegalArgumentException("Unzulssiger HOT " + hot + " fr das Lschen in der Hierarchie Verkehr MQ");
    }

    Object vater = getHierarchie().getUebergeordnetesObjekt(ho);
    if (vater == null)
    {
      throw new IllegalArgumentException("Kein bergeordnetes Objekt fr " + ho + " gefunden  in der Hierarchie MQGrp");
    }

    while (!getHierarchie().getUntergeordneteObjekte(ho).isEmpty())
    {
      HierarchieObjekt objekt = getHierarchie().getUntergeordneteObjekte(ho).get(0);
      MultiStatus unterStatus = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Lsche Hierarchieobjekt " + objekt,
          null);

      loescheHO(objekt, unterStatus);

      status.add(unterStatus);

      if (unterStatus.matches(IStatus.CANCEL | IStatus.ERROR))
      {
        // break;
        return;
      }
    }

    ConfigurationObject ko = (ConfigurationObject)ho.getSystemObjekte().get(0);

    if (hot.equals(HOT_MQGrp.MessStelle))
    {
      // Lsche Menge Zeichen
      ObjectSet menge = ko.getNonMutableSet("Zufahrten");
      if (menge != null)
      {
        try
        {
          // ko.removeSet(menge);
          menge.invalidate();
          status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Menge Zufahrten des Objekts " + ko.getPid()
              + " vom Typ " + ko.getType().getName() + " gelscht."));

        }
        catch (ConfigurationChangeException e)
        {
          status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Menge Zufahrten des Objekts " + ko.getPid()
              + " vom Typ " + ko.getType().getName() + " konnte nicht gelscht werden.", e));
          return;
        }
      }

      // Lsche Menge Ligaturen
      menge = ko.getNonMutableSet("Abfahrten");
      if (menge != null)
      {
        try
        {
          // ko.removeSet(menge);
          menge.invalidate();
          status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Menge Abfahrten des Objekts " + ko.getPid()
              + " vom Typ " + ko.getType().getName() + " gelscht."));

        }
        catch (ConfigurationChangeException e)
        {
          status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Menge Abfahrten des Objekts " + ko.getPid()
              + " vom Typ " + ko.getType().getName() + " konnte nicht gelscht werden.", e));
          return;
        }
      }

      Object ueo = _hierarchie.getUebergeordnetesObjekt(ho);

      // if (!(ueo instanceof HierarchieObjekt))
      {
        HierarchieObjekt vaterHo = (HierarchieObjekt)ueo;
        SystemObject vaterSo = vaterHo.getSystemObjekt(0);
        SystemObject so = ho.getSystemObjekt(0);

        AttributeGroup atg = _hierarchie.getDataModel().getAttributeGroup("atg.messStellenGruppe");

        ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(_hierarchie.getDataModel());
        Data data = ((ConfigConfigurationObject)vaterSo).getConfigurationData(atg, lookup);

        if (data != null)
        {
          ReferenceArray ra = data.getReferenceArray("MessStellen");

          List<SystemObject> delList = new ArrayList<SystemObject>();
          List<SystemObject> list = new ArrayList<SystemObject>();

          for (int i = 0; i < ra.getLength(); i++)
          {
            SystemObject ref = ra.getReferenceValue(i).getSystemObject();
            if (ref != null)
            {
              list.add(ref);
              if (ref.equals(so))
              {
                delList.add(ref);
              }
            }

          }

          if (delList.size() > 0)
          {
            list.removeAll(delList);

            ra.setLength(list.size());

            for (int i = 0; i < ra.getLength(); i++)
            {
              ra.getReferenceValue(i).setSystemObject(list.get(i));
            }

            try
            {
            	if (!HierarchieObjekt.isAttributgruppeAenderbar(vaterSo, atg))
              {  
                vaterSo = behandleSystemObjektAenderung(vaterHo, (ConfigurationObject)vaterSo, vaterSo.getPid());
                vaterHo.setSystemObjekt(0, vaterSo);
              }  
            	
              vaterSo.setConfigurationData(atg, data);
              status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Systemobjekt " + vaterSo.getPid()
                  + " als Referenz " + so.getPid() + " gesetzt"));
            }
            catch (ConfigurationChangeException e)
            {
              status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Systemobjekt " + vaterSo
                  + " konnt nicht als Refrenz " + so.getPid() + " gesetzt werden", e));
            }
          }
        }
      }
    }
    else if (hot.equals(HOT_MQGrp.MQAbfahrt))
    {
      // Entferne Referenz aus Menge des bergeordneten Objekts
      SystemObject anzeige = ((HierarchieObjekt)vater).getSystemObjekte().get(0);

      ObjectSet menge = ((ConfigurationObject)anzeige).getNonMutableSet("Abfahrten");
      if (menge != null)
      {
        try
        {
          menge.remove(ko);

          status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Konfigurationsobjekt " + ko.getPid() + " vom Typ "
              + ko.getType().getName() + " aus der Menge Abfahrten des Objekts " + anzeige.getPid() + " vom Typ "
              + anzeige.getType().getName() + " entfernt."));

        }
        catch (ConfigurationChangeException e)
        {
          status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Konfigurationsobjekt " + ko.getPid() + " vom Typ "
              + ko.getType().getName() + " konnte nicht aus der Menge Abfahrten des Objekts " + anzeige.getPid()
              + " vom Typ " + anzeige.getType().getName() + " entfernt werden.", e));
          return;
        }
        catch (RuntimeException e)
        {
          status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Konfigurationsobjekt " + ko.getPid() + " vom Typ "
              + ko.getType().getName() + " konnte nicht aus der Menge Abfahrten des Objekts " + anzeige.getPid()
              + " vom Typ " + anzeige.getType().getName() + " entfernt werden.", e));
          return;
        }
      }
    }
    else if (hot.equals(HOT_MQGrp.MQZufahrt))
    {
      // Entferne Referenz aus Menge des bergeordneten Objekts
      SystemObject anzeige = ((HierarchieObjekt)vater).getSystemObjekte().get(0);

      ObjectSet menge = ((ConfigurationObject)anzeige).getNonMutableSet("Zufahrten");
      if (menge != null)
      {
        try
        {
          menge.remove(ko);

          status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Konfigurationsobjekt " + ko.getPid() + " vom Typ "
              + ko.getType().getName() + " aus der Menge Zufahrten des Objekts " + anzeige.getPid() + " vom Typ "
              + anzeige.getType().getName() + " entfernt."));

        }
        catch (ConfigurationChangeException e)
        {
          status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Konfigurationsobjekt " + ko.getPid() + " vom Typ "
              + ko.getType().getName() + " konnte nicht aus der Menge Zufahrten des Objekts " + anzeige.getPid()
              + " vom Typ " + anzeige.getType().getName() + " entfernt werden.", e));
          return;
        }
        catch (RuntimeException e)
        {
          status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Konfigurationsobjekt " + ko.getPid() + " vom Typ "
              + ko.getType().getName() + " konnte nicht aus der Menge Zufahrten des Objekts " + anzeige.getPid()
              + " vom Typ " + anzeige.getType().getName() + " entfernt werden.", e));
          return;
        }
      }
    }
    else if (hot.equals(HOT_MQGrp.MQPruefling))
    {
      HierarchieObjekt vaterHo = (HierarchieObjekt)_hierarchie.getUebergeordnetesObjekt(ho);
      SystemObject vaterSo = vaterHo.getSystemObjekt(0);
      SystemObject so = ho.getSystemObjekt(0);

      AttributeGroup atg = _hierarchie.getDataModel().getAttributeGroup("atg.messStelle");

      ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(_hierarchie.getDataModel());
      Data data = ((ConfigConfigurationObject)vaterSo).getConfigurationData(atg, lookup);

      if (data != null)
      {
        SystemObject ref = data.getReferenceValue("Prfling").getSystemObject();

        if (ref != null)
        {
          if (ref.equals(so))
          {
            data.getReferenceValue("Prfling").setSystemObject(null);
            try
            {
            	if (!HierarchieObjekt.isAttributgruppeAenderbar(vaterSo, atg))
              {  
                vaterSo = behandleSystemObjektAenderung(vaterHo, (ConfigurationObject)vaterSo, vaterSo.getPid());
                vaterHo.setSystemObjekt(0, vaterSo);
              }  
            	
              vaterSo.setConfigurationData(atg, data);
              status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Systemobjekt " + vaterSo.getPid()
                  + " als Referenz " + so.getPid() + " gelscht"));
            }
            catch (ConfigurationChangeException e)
            {
              status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Systemobjekt " + vaterSo
                  + " konnt nicht als Refrenz " + so.getPid() + " gelscht werden", e));
            }
          }
        }

      }
    }

    if (!(hot.equals(HOT_MQGrp.MQPruefling) || hot.equals(HOT_MQGrp.MQZufahrt) || hot.equals(HOT_MQGrp.MQAbfahrt)))
    {
      // Lsche Objekt
      try
      {
        ko.invalidate();

        status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Konfigurationsobjekt " + ko.getPid() + " vom Typ "
            + ko.getType().getName() + " gelscht."));
      }
      catch (ConfigurationChangeException e)
      {
        status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Konfigurationsobjekt " + ko.getPid() + " vom Typ "
            + ko.getType().getName() + " konnte nicht gelscht werden.", e));
        return;
      }
      catch (RuntimeException e)
      {
        status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Konfigurationsobjekt " + ko.getPid() + " vom Typ "
            + ko.getType().getName() + " konnte nicht gelscht werden.", e));
        return;
      }
    }

    getHierarchie().objektEntfernen(vater, ho);
  }

  public IStatus pruefeDaten(HierarchieObjekt ho, HODaten daten)
  {
    IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
    IPruefungen pruefInstanz = DatenSpeicher.getInstanz().getPruefungen();
    MultiStatus result = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe Hierarchieobjekt " + ho, null);

    // Prfe Hierarchieobjekt
    result.merge(pruefInstanz.pruefeHierarchieObjekt(ho, daten));

    // Prfe auf offene Referenzen bei PID-nderungen
    LoeschenChecker loeschenChecker = new LoeschenChecker(this);
    List<SystemObject> liste = new Vector<SystemObject>();
    StringBuffer aend = new StringBuffer();
    StringBuffer loesch = new StringBuffer();

    for (SystemObject so : ho.getSystemObjekte())
    {
      SystemObjektDaten soDaten = daten.getSystemObjektDaten(so);

      if (!so.getPid().equals(soDaten.getPid()))
      {
        liste.add(so);
        if (aend.length() > 0)
        {
          aend.append(", ");
        }
        aend.append(soDaten.getPid());
        if (loesch.length() > 0)
        {
          loesch.append(", ");
        }
        loesch.append(so.getPid());
      }
    }

    if (liste.size() > 0)
    {
      MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Durch PID-nderung(en) auf " + aend
          + " werden folgende Objekte gelscht: " + loesch, null);

      Collection<SystemObjectType> typen = loeschenOffeneReferenzenVerboten(ho.getHierarchieObjektTyp());
      boolean flag = verwendeZuLoeschendeObjekteZurPruefungOffeneReferenzen();

      status.merge(loeschenChecker.bestimmeOffeneReferenzen(liste, typen, flag));
      result.merge(status);
    }

    // Prfe ATG-Daten
    for (SystemObject so : ho.getSystemObjekte())
    {
      SystemObjektDaten soDaten = daten.getSystemObjektDaten(so);
      if (soDaten != null)
      {
        MultiStatus objektStatus;
        if (ho.getSystemObjekte().size() > 1)
        {
          objektStatus = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe ATG-Daten fr Systemobjekt "
              + so.getPid() + " vom Typ " + so.getType().getPid(), null);
        }
        else
        {
          objektStatus = result;
        }

        // Prfe ATG-Daten fr das Systemobjekt
        TypInfo typInfo = hot.getTypInfo(so.getType().getPid());
        if (typInfo != null)
        {
          for (String atgPid : typInfo.getAtgs())
          {
            AtgInfo atgInfo = typInfo.getAtgInfo(atgPid);
            if (atgInfo != null)
            {
              Data data = soDaten.getAtgDaten(atgPid);
              if (data == null && (atgInfo.isMandatory() || !atgInfo.isOptional()))
              {
                objektStatus.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Fr die nicht-optionale ATG " + atgPid
                    + " sind keine Daten vorhanden"));
              }
              if (data != null)
              {
                MultiStatus atgStatus = new MultiStatus(Activator.PLUGIN_ID, Status.OK,
                    "Prfe Daten der ATG " + atgPid, null);
                atgStatus.merge(pruefInstanz.pruefeDaten(ho, so, atgInfo, data));

                objektStatus.add(atgStatus);
              }
            }
            else
            {
              // Kein AtgInfo fr den aktuellen Typ-Info-Typ im HOT fr aktuellen Systemobjekttyp
              throw new IllegalStateException("AtgInfo fr ATG " + atgPid + " fehlt in TypInfo fr "
                  + so.getType().getPid() + " im HierarchieObjektTyp " + hot);
            }
          }
        }
        else
        {
          // Keine Typ-Info im HOT fr aktuellen Systemobjekttyp
          throw new IllegalStateException("TypInfo fehlt fr  " + so.getType().getPid() + " im HierarchieObjektTyp "
              + hot);
        }
        if (result != objektStatus)
        {
          result.add(objektStatus);
        }

      }
      else
      {
        // Keine Daten fr Systemobjekt so bergeben
        throw new IllegalStateException("Daten fr Systemobjekt " + so.getPid() + " erforderlich, aber nicht bergeben");
      }
    }

    return result;
  }

  private ConfigurationObject dupliziereObjekt(ConfigurationObject ko, String neuePid)
      throws ConfigurationChangeException
  {
    List<ObjectSet> sets = new Vector<ObjectSet>();
    ConfigurationObject neu = null;
    ConfigurationObject result = null;

    try
    {
      for (ObjectSet set : ko.getObjectSets())
      {
        ObjectSetType setTyp = set.getObjectSetType();
        assert (setTyp.getReferenceType() != ReferenceType.COMPOSITION);
        ObjectSet menge = (ObjectSet)ko.getConfigurationArea().createConfigurationObject(set.getObjectSetType(), "",
            set.getName(), null);
        List<SystemObject> elemente = set instanceof NonMutableSet ? ((NonMutableSet)set)
            .getElementsInModifiableVersion() : set.getElements();
        menge.add(elemente.toArray(new SystemObject[elemente.size()]));
        sets.add(menge);
      }

      neu = ko.getConfigurationArea().createConfigurationObject((ConfigurationObjectType)ko.getType(),
          neuePid == null ? ko.getPid() : neuePid, ko.getName(), sets);

      ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion((ConfigDataModel)ko.getDataModel());

      for (AttributeGroupUsage atgu : ko.getUsedAttributeGroupUsages())
      {
        Data dataSrc = ((ConfigSystemObject)ko).getConfigurationData(atgu, lookup);
        if (dataSrc != null)
        {
          Data dataDest = DataFactory.createModifiableCopy(dataSrc, lookup);
          neu.setConfigurationData(atgu, dataDest);
        }
      }
      result = neu;
    }
    finally
    {
      // Rollback
      if (result == null)
      {
        for (ObjectSet set : sets)
        {
          set.invalidate();
        }
        if (neu != null)
        {
          neu.invalidate();
        }
      }
    }

    return result;
  }

  private ConfigurationObject behandleSystemObjektAenderung(HierarchieObjekt ho, ConfigurationObject ko, String neuePid)
      throws ConfigurationChangeException
  {
    ConfigurationObject result = dupliziereObjekt(ko, neuePid);

    Object vater = _hierarchie.getUebergeordnetesObjekt(ho);

    if (vater instanceof HierarchieObjekt)
    {
      HierarchieObjekt vaterHO = (HierarchieObjekt)vater;
      IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
      ConfigurationObject objekt = null;
      ObjectSet menge = null;

      if (hot.equals(HOT_MQGrp.MQPruefling))
      {
      	if (!ko.getPid().equals( result.getPid()))
      	{	
	        objekt = (ConfigurationObject)vaterHO.getSystemObjekt(0);
	        ConfigDataModel dataModel = _hierarchie.getDataModel();
	        AttributeGroup atg = dataModel.getAttributeGroup("atg.messStelle");
	        // SystemObject mq = ho.getSystemObjekt(0);
	        ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
	        Data data = ((ConfigSystemObject)objekt).getConfigurationData(atg, lookup);
	        Data neu = DataFactory.createModifiableCopy(data, lookup);
	        neu.getReferenceValue("Prfling").setSystemObject(result);
	        
	        if (!HierarchieObjekt.isAttributgruppeAenderbar(objekt, atg))
	        {  
	        	objekt = behandleSystemObjektAenderung(vaterHO, (ConfigurationObject)objekt, objekt.getPid());
	        	vaterHO.setSystemObjekt(0, objekt);
	        }  
	        
	        objekt.setConfigurationData(atg, neu);
      	}  
      }
      else if (hot.equals(HOT_MQGrp.MQZufahrt))
      {
        objekt = (ConfigurationObject)vaterHO.getSystemObjekt(0);
        menge = objekt.getNonMutableSet("Zufahrten");
      }
      else if (hot.equals(HOT_MQGrp.MQAbfahrt))
      {
        objekt = (ConfigurationObject)vaterHO.getSystemObjekt(0);
        menge = objekt.getNonMutableSet("Abfahrten");
      }
      else if (hot.equals(HOT_MQGrp.MessStelle))
      {
      	if (!ko.getPid().equals( result.getPid()))
      	{	
          objekt = (ConfigurationObject)vaterHO.getSystemObjekt(0);
	        ConfigDataModel dataModel = _hierarchie.getDataModel();
	        AttributeGroup atg = dataModel.getAttributeGroup("atg.messStellenGruppe");
	        // SystemObject ms = ho.getSystemObjekt(0);
	        ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
	        Data data = ((ConfigSystemObject)objekt).getConfigurationData(atg, lookup);
	
	        if (data != null)
	        {
	          Data neu = DataFactory.createModifiableCopy(data, lookup);
	          ReferenceArray ra = neu.getReferenceArray("MessStellen");
	
	          for (int i = 0; i < ra.getLength(); i++)
	          {
	            SystemObject ref = ra.getReferenceValue(i).getSystemObject();
	            if (ref != null)
	            {
	              if (ref.equals((SystemObject)ko))
	                ra.getReferenceValue(i).setSystemObject(result);
	            }
	
	          }
	          
	          if (!HierarchieObjekt.isAttributgruppeAenderbar(objekt, atg))
	          {  
	          	objekt = behandleSystemObjektAenderung(vaterHO, (ConfigurationObject)objekt, objekt.getPid());
	          	vaterHO.setSystemObjekt(0, objekt);
	          }  
	          
	          objekt.setConfigurationData(atg, neu);
	        }  
        }
      }

      if (menge != null)
      {
        try
        {
          menge.add(result);
        }
        catch (ConfigurationChangeException e)
        {
          for (ObjectSet set : result.getObjectSets())
          {
            set.invalidate();
          }
          result.invalidate();
          throw (e);
        }
        catch (RuntimeException e)
        {
          for (ObjectSet set : result.getObjectSets())
          {
            set.invalidate();
          }
          result.invalidate();
          throw (e);
        }
        menge.remove(ko);
      }
    }

    ko.invalidate();

    return result;
  }

  private void behandleMengenVerschieben(HierarchieObjekt ho, HierarchieObjekt alterVater, HierarchieObjekt neuerVater,
      String mengenTyp, String mengenName, int index, MultiStatus status)
  {
    ConfigurationObject koSrc = (ConfigurationObject)alterVater.getSystemObjekte().get(0);
    ConfigurationObject koDest = (ConfigurationObject)neuerVater.getSystemObjekte().get(0);

    ObjectSet mengeSrc = koSrc.getObjectSet(mengenName);
    Assert.isNotNull(mengeSrc);

    ObjectSet mengeDest = koDest.getObjectSet(mengenName);
    if (mengeDest == null)
    {
      ConfigurationArea kb = _hierarchie.getKonfigurationsBereich(neuerVater);
      Assert.isNotNull(kb);

      mengeDest = erzeugeBeliebigeMenge(mengenTyp, mengenName, koDest, status);
    }

    if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
    {
      SystemObject ko = ho.getSystemObjekte().get(index);

      try
      {
        mengeDest.add(ko);
        status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Systemobjekt " + ko.getPid() + " der Menge \""
            + mengenName + "\" des Systemobjekts " + koDest.getPid() + " hinzugefgt"));
      }
      catch (ConfigurationChangeException e)
      {
        status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Systemobjekt " + ko.getPid()
            + " konnte nicht der Menge \"" + mengenName + "\" des Systemobjekts " + koDest.getPid()
            + " hinzugefgt werden", e));
      }

      if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
      {
//        _hierarchie.objektHinzufuegen(neuerVater, ho);

        try
        {
          mengeSrc.remove(ko);
          status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Systemobjekt " + ko.getPid() + " aus der Menge \""
              + mengenName + "\" des Systemobjekts " + koSrc.getPid() + " entfernt"));
        }
        catch (ConfigurationChangeException e)
        {
          status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Systemobjekt " + ko.getPid()
              + " konnte nicht aus der Menge \"" + mengenName + "\" des Systemobjekts " + koSrc.getPid()
              + " entfernt werden", e));
        }

        if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
        {
//          _hierarchie.objektEntfernen(alterVater, ho);
          _hierarchie.objektVerschieben(alterVater, neuerVater, ho);
        }
      }
    }
  }

  private void behandleAtgVerschieben(HierarchieObjekt ho, HierarchieObjekt alterVater, HierarchieObjekt neuerVater,
      String pidAtg, String attributName, boolean isArray, MultiStatus status)
  {
    // Object ueo = _hierarchie.getUebergeordnetesObjekt(ho);
    //    
    // if (!(ueo instanceof ConfigurationArea))
    {
      // Referenz der ATG des bergeordneten Ziel-Objekts hinzufgen
      ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(_hierarchie.getDataModel());

      AttributeGroup atg = _hierarchie.getDataModel().getAttributeGroup(pidAtg);

      SystemObject soNeuerVater = neuerVater.getSystemObjekt(0);
      SystemObject soAlterVater = alterVater.getSystemObjekt(0);

      Data destData = ((ConfigConfigurationObject)soNeuerVater).getConfigurationData(atg, lookup);
      Data srcData = ((ConfigConfigurationObject)soAlterVater).getConfigurationData(atg, lookup);

      SystemObject so = ho.getSystemObjekt(0);

      if (destData == null)
      {
        destData = DataFactory.createData(atg);
        // Default und undefiniert Werte setzen
        destData.setToDefault();
      }

      if (isArray)
      {
        ReferenceArray ra = destData.getReferenceArray(attributName);
        int len = ra.getLength();
        ra.setLength(len + 1);
        ra.getReferenceValue(len).setSystemObject(so);
      }
      else
      {
        SystemObject replacement = destData.getReferenceValue(attributName).getSystemObject();

        if (replacement != null)
        {
          List<HierarchieObjekt> uos = _hierarchie.getUntergeordneteObjekte(neuerVater);

          HierarchieObjekt del = null;

          for (HierarchieObjekt lauf : uos)
          {
            SystemObject systemObjekt = lauf.getSystemObjekt(0);
            if (replacement.equals(systemObjekt))
            {
              del = lauf;
            }

          }
//          if (del != null)
//            _hierarchie.objektEntfernen(neuerVater, del);
        }

        destData.getReferenceValue(attributName).setSystemObject(so);
      }

      try
      {
      	 if (!HierarchieObjekt.isAttributgruppeAenderbar(soNeuerVater, atg))
         {  
      		 soNeuerVater = behandleSystemObjektAenderung(neuerVater, (ConfigurationObject)soNeuerVater, soNeuerVater.getPid());
      		 neuerVater.setSystemObjekt(0, soNeuerVater);
         }  
      	
        soNeuerVater.setConfigurationData(atg, destData);
        status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Systemobjekt " + soNeuerVater.getPid()
            + " als Referenz " + so.getPid() + " gesetzt"));
      }
      catch (ConfigurationChangeException e)
      {
        status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Systemobjekt " + soNeuerVater
            + " konnt nicht als Refrenz " + so.getPid() + " gesetzt werden", e));
      }

      if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
      {
//        _hierarchie.objektHinzufuegen(neuerVater, ho);
        _hierarchie.objektVerschieben(alterVater, neuerVater, ho);
      }

      // Entferne Referenz aus der ATG des bergeordneten Quell-Objekts
      if (srcData != null)
      {
        if (isArray)
        {
          ReferenceArray ra = srcData.getReferenceArray(attributName);

          List<SystemObject> delList = new ArrayList<SystemObject>();
          List<SystemObject> list = new ArrayList<SystemObject>();

          for (int i = 0; i < ra.getLength(); i++)
          {
            SystemObject ref = ra.getReferenceValue(i).getSystemObject();
            if (ref != null)
            {
              list.add(ref);
              if (ref.equals(so))
                delList.add(ref);
            }

          }

          if (delList.size() > 0)
          {
            list.removeAll(delList);

            ra.setLength(list.size());

            for (int i = 0; i < ra.getLength(); i++)
            {
              ra.getReferenceValue(i).setSystemObject(list.get(i));
            }
          }
        }
        else
        {
          srcData.getReferenceValue(attributName).setSystemObject(null);
        }

        try
        {
        	if (!HierarchieObjekt.isAttributgruppeAenderbar(soAlterVater, atg))
          {  
        		soAlterVater = behandleSystemObjektAenderung(alterVater, (ConfigurationObject)soAlterVater, soAlterVater.getPid());
        		alterVater.setSystemObjekt(0, soAlterVater);
          }  
        	
          soAlterVater.setConfigurationData(atg, srcData);
          status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Systemobjekt " + soAlterVater.getPid()
              + " als Referenz " + so.getPid() + " gesetzt"));
        }
        catch (ConfigurationChangeException e)
        {
          status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Systemobjekt " + soAlterVater
              + " konnt nicht als Refrenz " + so.getPid() + " gesetzt werden", e));
        }

        if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
        {
          _hierarchie.objektEntfernen(alterVater, ho);
//          _hierarchie.objektVerschieben(alterVater, neuerVater, ho);
        }

      }
    }
  }

  public IStatus uebernehmeDaten(HierarchieObjekt ho, HODaten daten)
  {
    MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK,
        "bernehme bearbeitete Daten fr Hierarchieobjekt " + ho, null);

    List<SystemObject> objektListe = daten.getSystemObjekte();
    

//    DatenModellUpdateListener.getInstanz().notifyDatenModellUpdateListener(objektListe);

    for (int i = 0; i < objektListe.size(); i += 1)
    {
      SystemObject so = objektListe.get(i);

      if (ho.getSystemObjekte().contains(so))
      {
        SystemObjektDaten soDaten = daten.getSystemObjektDaten(so);
        Object vater = getHierarchie().getUebergeordnetesObjekt(ho);

        // PID
        if (!so.getPid().equals(soDaten.getPid()))
        {
          try
          {
            // so = behandleSystemObjektAendeung(ho, (ConfigurationObject)so, soDaten.getPid());
            so = behandleSystemObjektAenderung(ho, (ConfigurationObject)so, soDaten.getPid());
            ho.setSystemObjekt(i, so);
            status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, "Neues Objekt mit PID " + so.getPid() + " vom Typ "
                + so.getType().getPid() + " erzeugt"));
          }
          catch (ConfigurationChangeException e)
          {
            status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Fehler beim behandeln der PID-nderung auf "
                + soDaten.getPid() + " fr Objekt " + so.getPid(), e));
          }
        }

        // Name
  			if (so.getName() == null || !so.getName().equals(soDaten.getName()))
  			{
	        try
	        {
	          so.setName(soDaten.getName());
	          status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, "Name \"" + soDaten.getName() + "\" bernommen"));
	        }
	        catch (ConfigurationChangeException e)
	        {
	          status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Name \"" + soDaten.getName()
	              + "\" konnte nicht bernommen werden", e));
	        }
  			}  

        // ATGs

        List<String> atgList = ho.getHierarchieObjektTyp().getTypInfo(so.getType().getPid()).getAtgs();

        for (String atgPid : atgList)
        {
          AttributeGroup atg = _hierarchie.getDataModel().getAttributeGroup(atgPid);
          if (atg != null)
          {
            /*
             * AttributeGroupUsage atgUsage = atg.getAttributeGroupUsage(_dataModel.getAspect("asp.eigenschaften")); if
             * (atgUsage != null) { ConfigurationObject ko = (ConfigurationObject)so; if (ko.getValidSince() <
             * ko.getConfigurationArea().getModifiableVersion()) // Objekt ist bereits aktiviert {
             * AttributeGroupUsage.Usage usage = atgUsage.getUsage(); if (usage ==
             * AttributeGroupUsage.Usage.OptionalConfigurationData || usage ==
             * AttributeGroupUsage.Usage.RequiredConfigurationData) { status.add(new Status(IStatus.INFO,
             * Activator.PLUGIN_ID, "Eventuelle nderungen an konfigurierenden Daten der ATG " + atgPid +
             * " werden IGNORIERT wegen ATG-Verwendung " + usage)); continue; } } }
             */

            ConfigDataModel dataModel = _hierarchie.getDataModel();
            ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);

            // Data ist = so.getConfigurationData(atg);
            Data ist = ((ConfigSystemObject)so).getConfigurationData(atg, lookup);

            Data soll = soDaten.getAtgDaten(atgPid);

            if (soll != null)
            {
            	if (ist == null || soDaten.isDatenGeaendert(atgPid))
	          	{	
	              if (!HierarchieObjekt.isAttributgruppeAenderbar(so, atg))
	              {
	              	try
		              {
	              		so = behandleSystemObjektAenderung(ho, (ConfigurationObject)so, so.getPid());
		     	  				ho.setSystemObjekt(i, so);
		     	  				status.add(new Status(IStatus.OK, Activator.PLUGIN_ID,
		     	  						"ATG-Verwendung " + AttributeGroupUsage.Usage.OptionalConfigurationData +
		     	  						" bedingt Erzeugung neues Objekt mit PID " + so.getPid() + " vom Typ " + 
		     	  						so.getType().getPid()));
		               }
		               catch (ConfigurationChangeException e)
		             	 {
		                 status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
		               			"Bei der durch ATG-Verwendung " + AttributeGroupUsage.Usage.OptionalConfigurationData +
		     	  						" bedingten Erzeugung neues Objekts mit PID " + so.getPid() + " vom Typ " +	
		     	  						so.getType().getPid() + " ist ein Fehler aufgetreten", e));
		               	 continue;
		             	 }
	              	
//	                status.add(new Status(IStatus.INFO, Activator.PLUGIN_ID,
//	                    "Eventuelle nderungen an konfigurierenden Daten der ATG " + atgPid
//	                        + " werden IGNORIERT, da wegen ATG-Verwendung ("
//	                        + AttributeGroupUsage.Usage.OptionalConfigurationData + ") nicht mglich"));
//	                continue;
	              }
	              try
	              {
	                so.setConfigurationData(atg, soll);
	               
	                if (atg.getPid().equals("atg.messStelle")){

	                  List<HierarchieObjekt> uos = _hierarchie.getUntergeordneteObjekte(ho);
	                  
	                  ArrayList<HierarchieObjekt> delList = new ArrayList<HierarchieObjekt>();
	                  
	                  for (HierarchieObjekt ho2 : uos)
	                  {
	                    delList.add(ho2);
	                  }
	                  
	                  for (HierarchieObjekt ho2 : delList)
	                  {
	                    _hierarchie.objektEntfernen(ho, ho2);                    
	                  }
	                  
	                  bestimmeUntergeordneteHOs(ho);	                  
	                }
	                
	                
	                status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, "Konfigurierende Daten der ATG " + atgPid
	                    + " bernommen"));
	              }
	              catch (ConfigurationChangeException e)
	              {
	                status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID,
	                    "Fehler beim bernehmen konfigurierender Daten der ATG " + atgPid, e));
	              }
	          	}  
            }
            else
            // Keine Daten fr diese ATG bergeben => vorhanden Daten lschen
            {
              if (ist != null)
              {
                if (!HierarchieObjekt.isAttributgruppeAenderbar(so, atg))
                {
                	try
		              {
	              		so = behandleSystemObjektAenderung(ho, (ConfigurationObject)so, so.getPid());
		     	  				ho.setSystemObjekt(i, so);
		     	  				status.add(new Status(IStatus.OK, Activator.PLUGIN_ID,
		     	  						"ATG-Verwendung " + AttributeGroupUsage.Usage.OptionalConfigurationData +
		     	  						" bedingt Erzeugung neues Objekt mit PID " + so.getPid() + " vom Typ " + 
		     	  						so.getType().getPid()));
		               }
		               catch (ConfigurationChangeException e)
		             	 {
		                 status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
		               			"Bei der durch ATG-Verwendung " + AttributeGroupUsage.Usage.OptionalConfigurationData +
		     	  						" bedingten Erzeugung neues Objekts mit PID " + so.getPid() + " vom Typ " +	
		     	  						so.getType().getPid() + " ist ein Fehler aufgetreten", e));
		               	 continue;
		             	 }
                	
//                   status.add(new Status(IStatus.INFO, Activator.PLUGIN_ID,
//                      "Eventuelle nderungen an konfigurierenden Daten der ATG " + atgPid
//                          + " werden IGNORIERT, da wegen ATG-Verwendung ("
//                          + AttributeGroupUsage.Usage.OptionalConfigurationData + ") nicht mglich"));
//                  continue;
                }
                try
                {
                  so.setConfigurationData(atg, null);
                  
                  status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, "Konfigurierende Daten der ATG " + atgPid
                      + " gelscht"));
                }
                catch (ConfigurationChangeException e)
                {
                  status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID,
                      "Fehler beim Lschen konfigurierender Daten der ATG " + atgPid, e));
                  return status;
                }
              }
            }
          }
          else
          {
            _debug.warning("Bearbeitung Objekt " + so.getPid() + ": Keine Attributgruppe im Datenmodell gefunden fr "
                + atgPid);
          }
        }
      }
    }

    return status;
  }

  @Override
  public boolean pruefeNeuanlageVerboten(Object objekt, IHierarchieObjektTyp hot)
  {

    // TODO Auto-generated method stub
    // if (hot.equals(HOT_AQ.Anzeige))
    // {
    // boolean b = bestimmeObObjekteInKonfigurationVorhanden("typ.anzeigeTyp");
    //
    // return b;
    // }
    // // else if (hot.equals(HOT_AQ.AnzeigeQuerschnitt) || hot.equals(HOT_AQ.AnzeigeQuerschnittSub))
    // else if (hot.equals(HOT_AQ.AnzeigeQuerschnitt))
    // {
    // boolean b = bestimmeObObjekteInKonfigurationVorhanden("typ.anzeigeQuerschnittTyp");
    //
    // return b;
    // }

    return false;
  }

  private SystemObject behandlePidAendeung(Object vater, SystemObject ko, String neuePid)
      throws ConfigurationChangeException
  {
    SystemObject result = null;

    Map<String, String> substitute = new HashMap<String, String>();
    substitute.put(ko.getPid(), neuePid);

    result = ((ConfigurationObject)ko).duplicate(substitute);
    ko.invalidate();

    return result;
  }

  private boolean bestimmeObObjekteInKonfigurationVorhanden(String pidTyp)
  {
    Collection<ConfigurationArea> areas = _hierarchie.getDataModel().getAllConfigurationAreas().values();

    SystemObjectType typ = _hierarchie.getDataModel().getType(pidTyp);
    Collection<SystemObjectType> typeSpez = new Vector<SystemObjectType>();

    typeSpez.add(typ);

    for (ConfigurationArea kb : areas)
    {

      if (kb != null)
      {
        Collection<SystemObject> elements = kb.getObjects(typeSpez, ObjectTimeSpecification.valid());
        if (elements != null && elements.size() > 0)
          return false;

        for (SystemObject so : kb.getNewObjects())
        {
          if (so.isOfType(pidTyp))
            return false;

        }
      }
    }
    return true;
  }

  @Override
  public Collection<SystemObjectType> loeschenOffeneReferenzenVerboten(IHierarchieObjektTyp hot)
  {
    Collection<SystemObjectType> returnList = null;

    if (hot.equals(HOT_MQGrp.MessStelle))
    {

      returnList = new ArrayList<SystemObjectType>();
      SystemObjectType type = _hierarchie.getDataModel().getType("typ.messStellenGruppe");
      returnList.add(type);

      return returnList;
    }
    else if (hot.equals(HOT_MQGrp.MQPruefling) || hot.equals(HOT_MQGrp.MQZufahrt) || hot.equals(HOT_MQGrp.MQZufahrt))
    {

      returnList = new ArrayList<SystemObjectType>();
      SystemObjectType type = _hierarchie.getDataModel().getType("typ.messStelle");
      returnList.add(type);

      return returnList;
    }

    return null;
  }

  private IHierarchieObjektTyp[] getTypSortierFolge(IHierarchieObjektTyp hot)
  {
    IHierarchieObjektTyp[] sortierFolge = null;

    if (hot.equals(HOT_MQGrp.MessStellenGruppe))
    {
      sortierFolge = SORTIER_FOLGE_1;
    }
    else if (hot.equals(HOT_MQGrp.MessStelle))
    {
      sortierFolge = SORTIER_FOLGE_2;
    }
    else if (hot.equals(HOT_MQGrp.MQPruefling) || hot.equals(HOT_MQGrp.MQZufahrt) || hot.equals(HOT_MQGrp.MQAbfahrt))
    {
      sortierFolge = SORTIER_FOLGE_3;
    }

    return sortierFolge;
  }

  @Override
  public boolean verwendeZuLoeschendeObjekteZurPruefungOffeneReferenzen()
  {
    // TODO Auto-generated method stub
    return false;
  }
  
  /**
   * Zeigt an, ob das bergebene Hierarchieobjekt beim rekursiven Kopieren als untergeordnetes 
   * Objekt mitkopiert werden soll oder nicht
   *  
   * @param ho das aktuell zu kopierende HO
   * @return <code>true</code>, wenn das aktuell zu kopierende Objekt nicht kopiert werden soll, 
   * <code>false</code> (default) sonst
   */
  public boolean kopierenNichtErlaubt(HierarchieObjekt ho)
  {
  	if (HOT_MQGrp.MQPruefling.equals(ho.getHierarchieObjektTyp()))
		{
  		return true;
		}
  	return false;
  }

}
