/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkanba;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;

import de.bsvrz.buv.plugin.tkabasis.Activator;
import de.bsvrz.buv.plugin.tkabasis.LookupFactory;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.AtgInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HODaten;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HierarchieObjekt;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IAttributInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchie;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.SystemObjektDaten;
import de.bsvrz.buv.plugin.tkabasis.pruefungen.IPruefungen;
import de.bsvrz.buv.plugin.tkabasis.pruefungen.PruefungenTools;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ObjectLookup;
import de.bsvrz.dav.daf.main.config.StringAttributeType;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;
import de.bsvrz.puk.config.configFile.datamodel.ConfigSystemObject;

/**
 * Prfungen von Attributwerten fr den TKA Netzbeeinflussungsanlagen
 * 
 * @author Krause
 *
 */
public class PruefungenNBA implements IPruefungen
{

	private IHierarchie _hierarchie;
	
	 /**
   * Bestimmt die Hierarchie, die fr die Prfungen benutzt werden kann
   * 
   * @return die Hierarchie, die fr die Prfungen benutzt werden kann
   */
	public IHierarchie getHierarchie()
	{
		return _hierarchie;
	}
  
	/**
   * Setzt die Hierarchie, die fr die Prfungen benutzt werden kann
   * @param hierachie die Hierarchie, die fr die Prfungen benutzt werden kann
   */
	public void setHierarchie(IHierarchie hierarchie)
	{
		_hierarchie = hierarchie;
	}
	
	public IStatus pruefeHierarchieObjekt(HierarchieObjekt ho, HODaten daten)
	{
		MultiStatus result = new MultiStatus(Activator.PLUGIN_ID, Status.OK, 
				"Prfe Hierarchieobjekt " + ho, null);
//	  IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
		
		for (SystemObject so: ho.getSystemObjekte())
  	{
			 SystemObjektDaten soDaten = daten.getSystemObjektDaten(so);
			 if (soDaten != null)
			 {
				 MultiStatus objektStatus;
				 if (ho.getSystemObjekte().size() > 1)
				 {
					 objektStatus = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe Systemobjekt " + 
							 so.getPid() + " vom Typ " + so.getType().getPid(), null);
				 }
				 else
				 {
					 objektStatus = result;
				 }
				 
				 //objektStatus.add(pruefePid(so, soDaten.getPid()));				
				 objektStatus.add(new PruefungenTools(getHierarchie().getDataModel()).pruefePid(soDaten.getPid(), so));
			 }	
  	}
		
		return result;
	}

  /**
   * Prft die Daten einer Attributgruppe, die fr ein Systemobjekt eines
   * Hierarchieobjekts bernommen werden sollen
   * 
   * @param ho das Hierarchieobjekt, zu dem das bergebene Systemobjekt gehrt
   * @param so das Systemobjekt, zu dem ATG-Daten bernommen werden sollen
   * @param atgInfo Info ber die ATG der zu bernehmenden Daten
   * @param daten Datensatz der Daten, die geprft werden sollen
   * @return eine Instanz vom Typ IStatus, die aussagt, ob es zulssig ist, die 
   * bergebenen ATG-Daten fr das Systemobjekt zu bernehmen, oder <code>null</code>, wenn
   * keine Prfung fr die bergebenen Parameter implementiert ist.
   */
	public IStatus pruefeDaten(HierarchieObjekt ho, SystemObject so, AtgInfo atgInfo, Data daten)
  {
    MultiStatus result = new MultiStatus( Activator.PLUGIN_ID, Status.OK, "Prfe Daten der ATG " + atgInfo.getAtg(),
        null );
    
    pruefeDatenDefinition( atgInfo, daten, result );
    
    if ( so.isOfType( "typ.masche" ) )
    {
      if ( "atg.routen".equals( atgInfo.getAtg() ) )
      {       
        Data item = daten.getItem("Route");

        if (item.isArray())
        {         
//          if ( item.asArray().getLength() > 0 && item.asArray().getLength() < 2 )
//          {
//            //mindestens 2 Routen mssen eingetragen sein
//            //dann funktioniert die Anlage der ersten Route jedoch nicht
//            result.add( new Status( IStatus.WARNING, Activator.PLUGIN_ID, so.getType() + " muss mindestens 2 " + atgInfo.getAtg() + " enthalten" ) );
//          }
          if ( item.asArray().getLength() >= 0 && item.asArray().getLength() < 2 )
          {
            //genau 2 Routen mssen eingetragen sein
            //dann funktioniert die Neu-Anlage der Routen jedoch nicht 0> deshalb als Warnung
            result.add( new Status( IStatus.WARNING, Activator.PLUGIN_ID, so.getType() + " muss genau 2 Routen enthalten" ) );
          }

          else if ( item.asArray().getLength() > 0 )
          {
            //nur unterschiedliche Routen sind erlaubt
            SystemObject[] routen = item.asReferenceArray().getSystemObjectArray();
            if ( hatGleicheObjekte(routen) )
            {
              result.add( new Status( IStatus.ERROR, Activator.PLUGIN_ID, "Alle " + item.getName() + "n von " + atgInfo.getAtg() + " mssen verschieden sein" ) );              
            } 

//            List<HierarchieObjekt> uos = getHierarchie().getUntergeordneteObjekte( ho );
//            for (HierarchieObjekt hoKind : uos)
//            {
//              SystemObject soKind = hoKind.getSystemObjekt(0);
//              if (soKind.getPid().equals( hr.getPid() ) && !hoKind.getHierarchieObjektTyp().equals( HOT_NBA.HauptRoute ))
//              {
//                result.add( new Status( IStatus.ERROR, Activator.PLUGIN_ID, "Die 1.Route ist keine Hauptroute" ) );                             
//              }
//              else if (soKind.getPid().equals( nr.getPid() ) && !hoKind.getHierarchieObjektTyp().equals( HOT_NBA.NebenRoute ))
//              {
//                result.add( new Status( IStatus.ERROR, Activator.PLUGIN_ID, "Die 2.Route ist keine Nebenroute" ) );                             
//              }
//            }
          }
        }
      }
    }
    else if ( so.isOfType( "typ.route" ) )
    {
//      System.out.println("Typ ist Route");
      pruefeLinienReferenz( so, atgInfo, daten, "typ.routenStck", result );
    }
    else if ( so.isOfType( "typ.routenStck" ) )
    {
//      System.out.println("Typ ist RoutenStck");
      pruefeLinienReferenz( so, atgInfo, daten, "typ.ueresStraenSegment", result );
    }
    
    return result;
  }

  /**
   * Prfen des Arrays LinienReferenz
   * 
   * @param so
   * @param atgInfo
   * @param daten
   * @param result
   * 
   * @return RoutenStck mit den Informationen zu vonKnoten, nachKnoten, tmcRichtung
   */
  private RoutenStueck pruefeLinienReferenz( SystemObject so, AtgInfo atgInfo, Data daten, String subTyp, MultiStatus result )
  {
//    System.out.println( "pruefeLinienReferenz" );

    if ( "atg.bestehtAusLinienObjekten".equals( atgInfo.getAtg() ) && daten != null )
    {
      Data item = daten.getItem( "LinienReferenz" );

      if ( item.isArray() && item.asArray().getLength() > 0 )
      {
        // nur unterschiedliche LinienReferenzen sind erlaubt
        SystemObject[] linien = item.asReferenceArray().getSystemObjectArray();
        if ( hatGleicheObjekte( linien ) )
        {
          result.add( new Status( IStatus.ERROR, Activator.PLUGIN_ID, "Alle " + item.getName() + "en von "
              + atgInfo.getAtg() + " mssen verschieden sein" ) );
        }

        String richtung = null;
        SystemObject startKnoten = null;
        SystemObject knoten = null;
        ConfigDataModel dataModel = getHierarchie().getDataModel();
        ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion( dataModel );

//        System.out.println( "Linien-Anzahl: " + linien.length );

        for (int i = 0; i < linien.length; i++)
        {
//          System.out.println( "Linie(" + i + "): " + linien[i] );
          // nur LinienReferenzen von einem bestimmten Typ sind erlaubt
          if ( !linien[i].isOfType( subTyp ) )
          {
            result.add( new Status( IStatus.ERROR, Activator.PLUGIN_ID, linien[i].getNameOrPidOrId() + " muss vom Typ "
                + subTyp + " sein" ) );
            break;
          }

          // nur hintereinander liegende LinienReferenzen in gleicher
          // Fahrtrichtung sind erlaubt
          String tmcRichtung = null;
          SystemObject vonKnoten = null;
          SystemObject nachKnoten = null;

          if ( linien[i].isOfType( "typ.ueresStraenSegment" ) )
          {
//            System.out.println("--> typ.ueresStraenSegment");
            AttributeGroup atg = dataModel.getAttributeGroup( "atg.ueresStraenSegment" );
            Data coDaten = ((ConfigSystemObject) linien[i]).getConfigurationData( atg, lookup );

            if ( coDaten != null)
            {
              tmcRichtung = coDaten.getItem( "TmcRichtung" ).asTextValue().getValueText();
              vonKnoten = coDaten.getItem( "vonKnoten" ).asReferenceValue().getSystemObject();
              nachKnoten = coDaten.getItem( "nachKnoten" ).asReferenceValue().getSystemObject();
            }

          }
          else if ( linien[i].isOfType( "typ.routenStck" ) )
          {
//            System.out.println("--> typ.routenStck");            
            AttributeGroup atg = dataModel.getAttributeGroup( "atg.bestehtAusLinienObjekten" );
            Data coDaten = ((ConfigSystemObject) linien[i]).getConfigurationData( atg, lookup );

            RoutenStueck rs = pruefeLinienReferenz( linien[i], atgInfo, coDaten, "typ.ueresStraenSegment", result );

            if (rs != null)
            {
              tmcRichtung = rs.getTmcRichtung();
              vonKnoten = rs.getVonKnoten();
              nachKnoten = rs.getNachKnoten();
            }
          }

          // Richtung
//          System.out.println("Richtung: " + tmcRichtung);
          if ( richtung == null && tmcRichtung != null )
            richtung = tmcRichtung;
          else if ( tmcRichtung == null )
          {
//            result.add( new Status( IStatus.ERROR, Activator.PLUGIN_ID,
//                "LinienReferenzen zu " + so.getPid() + " liegen nicht in einer Fahrtrichtung" ) );
//            richtung = null;
//            break;
          }
          else if ( !richtung.equals( tmcRichtung ) )
          {
//            System.out.println( "Linie " + linien[i] + " hat falsche Richtung: " + tmcRichtung + " <> " + richtung );
            result.add( new Status( IStatus.ERROR, Activator.PLUGIN_ID,
                "LinienReferenzen zu " + so.getPid() + " liegen nicht in einer Fahrtrichtung: " + linien[i].getPidOrNameOrId() + " hat andere Fahrtrichtung" ) );
            richtung = null;
            break;
          }

          // hintereinander liegende LinienReferenzen
//          System.out.println("Knoten von: " + vonKnoten + " Knoten nach: " + nachKnoten);
          if ( startKnoten == null )
            startKnoten = vonKnoten;

          if ( nachKnoten != null && ( knoten == null || knoten == vonKnoten ) )
            knoten = nachKnoten;
          else if ( vonKnoten == null || nachKnoten == null && i > 1 )
          {
//            result.add( new Status( IStatus.ERROR, Activator.PLUGIN_ID,
//                "LinienReferenzen zu " + so.getPid() + " liegen nicht hintereinander") );
//            knoten = null;
//            break;
          }
          else if ( !(knoten == vonKnoten) && i > 0 )
          {
//            System.out.println( "Linie " + linien[i] + " nicht hintereinander " + vonKnoten + " <> " + knoten );
            result.add( new Status( IStatus.ERROR, Activator.PLUGIN_ID,
                "LinienReferenzen zu " + so.getPid() + " liegen nicht hintereinander: " + linien[i].getPidOrNameOrId() + " folgt nicht auf " + linien[i-1].getPidOrNameOrId() ) );
            knoten = null;
            break;
          }

        }

        return new RoutenStueck( startKnoten, knoten, richtung );
      }
    }

    return null;

  }
	  
  private boolean hatGleicheObjekte( SystemObject[] objekte )
  {
//    System.out.println("hatGleicheObjekte: ");
    for (int i = 0; i < objekte.length-1; i++)
    {
      for (int j = i+1; j < objekte.length; j++)
      {
        if ( objekte[i] == objekte[j] )
        {
//          System.out.println(objekte[i].getPid() + " und " + objekte[j].getPid() + " sind gleich!");
          return true;
        }
      }                 
    }
    
    return false;
  }

  private boolean pruefeDatenDefinition(IAttributInfo attInfo, Data daten, MultiStatus status)
  {
		Data item = daten.getItem(attInfo.getName());	
		
		if (!item.isDefined())
		{
			status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Wert ist nicht korrekt definiert"));  
			return false;
		}
		else
		{
			status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, "Prfen Daten-Definition: OK"));
		}
		
		// Pflichtfeld-Prfung:
		if (item.isPlain())
		{	
			if (attInfo.isPflichtFeld())
			{	
				String wert = item.asTextValue().getText();
				if ("".equals(wert) || (!(item.getAttributeType() instanceof StringAttributeType) && 
						"undefiniert".equals(wert) || "_Undefiniert_".equals(wert)))
				{
					status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Wert fr Pflichtfeld nicht angegeben"));  
					return false;
				}
				else
				{
					status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, "Prfen Pflichtfeld-Eigenschaft: OK"));
				}
			}	
		}		
  	return true;
  }
  
  private void pruefeDatenDefinition(AtgInfo atgInfo, Data daten, MultiStatus status)
  {
  	for (String attribut: atgInfo.getAttribute())
  	{
  		MultiStatus	meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe Wert fr Attribut " +
    			attribut, null);
  		IAttributInfo attInfo = atgInfo.getAttributInfo(attribut);
  		pruefeDatenDefinition(attInfo, daten,  meldung);
  		status.add(meldung);
  	}
  }
 
}
  