/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkanetz;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

import de.bsvrz.buv.plugin.tkabasis.DataFactory;
import de.bsvrz.buv.plugin.tkabasis.LookupFactory;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HierarchieObjekt;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchie;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieObjektTyp;
import de.bsvrz.buv.plugin.tkabasis.regeln.BildungsRegelnBeobachter;
import de.bsvrz.buv.plugin.tkabasis.regeln.BildungsRegelnTools;
import de.bsvrz.buv.plugin.tkabasis.regeln.EingabeVorbelegung;
import de.bsvrz.buv.plugin.tkabasis.regeln.IBildungsRegeln;
import de.bsvrz.buv.plugin.tkanetz.preferences.DefsPara;
import de.bsvrz.dav.daf.communication.dataRepresentation.AttributeBaseValueDataFactory;
import de.bsvrz.dav.daf.communication.dataRepresentation.AttributeHelper;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationAuthority;
import de.bsvrz.dav.daf.main.config.MutableSet;
import de.bsvrz.dav.daf.main.config.ObjectLookup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;

/**
 * BildungsRegelnAllgemein fr den TKA Netz Netzbestandteile
 * 
 * @author Krause
 *
 */

public class BildungsRegelnAllgemeinNetz implements IBildungsRegeln
{
	private IHierarchie _hierarchie;
		
  protected Map<String, String> _praefixe = new HashMap<String, String>();
    
  protected Map<IHierarchieObjektTyp, String[]> _felder = new HashMap<IHierarchieObjektTyp, String[]>();
     
  protected Map<IHierarchieObjektTyp, Map<String, String>> _feldWerte = new HashMap<IHierarchieObjektTyp, Map<String, String>>();
  
  protected Map<String, Map<String, String>> _historie = new HashMap<String, Map<String, String>>();
  
  
  public IHierarchie getHierarchie()
	{
		return _hierarchie;
	}
  
	public void setHierarchie(IHierarchie hierarchie)
	{
		_hierarchie = hierarchie;
	}
  
  protected void initPraefixe()
  {
    _praefixe.put("typ.verkehrsModellNetz", "verkehrsModellNetz");
    _praefixe.put("typ.schwachStelle", "schwachStelle");
  }
  
  protected void initFelder()
  {
    _felder.put(HOT_Netz.VerkehrsModellNetz, new String[] { "Bezeichnung" });
    _felder.put(HOT_Netz.SchwachStelle, new String[] { "Bezeichnung" });
  }
  
  public BildungsRegelnAllgemeinNetz()
  {
    initPraefixe();
    initFelder();
  }
  
  public String getPraefix(String pidTypNeuesObjekt)
  {
    String result = _praefixe.get(pidTypNeuesObjekt);
    
    return result != null ? result : BildungsRegelnTools.LEERER_WERT;
  }
  
  public String[] getEingabeFelder(IHierarchieObjektTyp hotNeuesObjekt)
  {
    String[] result = _felder.get(hotNeuesObjekt);
    
    return result != null ? result : BildungsRegelnTools.LEERE_TEXT_LISTE;
  }
  
  private static EingabeVorbelegung STANDARD_VORBELEGUNG = new EingabeVorbelegung(null, false);
  
  public EingabeVorbelegung getEingabeVorbelegung(IHierarchieObjektTyp hotNeuesObjekt, String eingabeFeld, HierarchieObjekt verbundenesHO)
  {
  	if (Arrays.asList(_felder.get(hotNeuesObjekt)).contains(eingabeFeld))
    {
// Keine Vererbung der Feldinhalte  		
//  		if (verbundenesHO != null) 
//	    {
//	      Map<String, String> werte = _historie.get(verbundenesHO.getSystemObjekte().get(0).getPid());
//	      
//	      if (werte != null)
//	      {
//	        String text = werte.get(eingabeFeld);
//	        if (text != null)
//	        {
//	          return new EingabeVorbelegung(text, false);
//	        }
//	      }
//	    }
	    return STANDARD_VORBELEGUNG;
    }  
    
    return BildungsRegelnTools.LEERE_VORBELEGUNG;
  }
  
  public void setWert(IHierarchieObjektTyp hotNeuesObjekt, String eingabeFeld, String wert)
  {
    Map<String, String> werte = _feldWerte.get(hotNeuesObjekt);
    
    if (werte == null)
    {
      werte = new HashMap<String, String>();
      _feldWerte.put(hotNeuesObjekt, werte);
    }
    
    werte.put(eingabeFeld, wert);
  }
  
  public String getWert(IHierarchieObjektTyp hotNeuesObjekt, String eingabeFeld)
  {
    Map<String, String> werte = _feldWerte.get(hotNeuesObjekt);
    
    if (werte == null)
    {
      return BildungsRegelnTools.LEERER_WERT;
    }
    
    String result = werte.get(eingabeFeld);
    
    return result != null ? result : BildungsRegelnTools.LEERER_WERT;
  }
 
  public boolean setWerte (IHierarchieObjektTyp hotNeuesObjekt, HierarchieObjekt uebergeordnetesHO, 
      HierarchieObjekt vorlageHO)
  {
  
  	return false;
  }
  
  public String getPidRegelText(IHierarchieObjektTyp hot)
  {
   	String felder[] = _felder.get(hot);
  	if (felder != null)
  	{
  		String typPid = hot.getTypen().get(0);
  		String result = getPraefix(typPid) + ".Bezeichnung";
  		
  		return result;
  	}
  	return BildungsRegelnTools.LEERER_WERT;
  }
  
  public String getNameRegelText(IHierarchieObjektTyp hot)
  {
  	String felder[] = _felder.get(hot);
  	if (felder != null)
  	{
  		return "<PID Systemobjekt>";
  	}
  	return BildungsRegelnTools.LEERER_WERT;
  }
    
  public String[] getPids(IHierarchieObjektTyp hotNeuesObjekt, HierarchieObjekt uebergeordnetesHO)
  {
  	String[] result = new String[1];
  	String pidTypNeuesObjekt = hotNeuesObjekt.getTypen().get(0);
  	
    StringBuffer buffer = new StringBuffer();
    
    buffer.append(getPraefix(pidTypNeuesObjekt));
    buffer.append('.');
    
    String bezeichnung = getWert(hotNeuesObjekt, "Bezeichnung");
    buffer.append(BildungsRegelnTools.getPidText(bezeichnung, false, false));
    
    String pid = buffer.toString(); 
    Map<String, String> werte = _feldWerte.get(hotNeuesObjekt);
    if (werte != null && !werte.isEmpty())
    {
      _historie.put(pid, new HashMap<String, String>(werte));
    }
    
    result[0] = pid;
    
    return result; 
  }
  
  public String getName(SystemObject objekt)
  {
    return objekt.getPid();
  }
  
  public Data getDatensatz(HierarchieObjekt ho, SystemObject objekt, AttributeGroup atg, Data daten,
  		boolean nurPflichtFelder)
  {
  	Data data;
  	
  	if (daten == null)
  	{
	    data = DataFactory.createData(atg);
	    // Default und undefinierte Werte setzen
	    data.setToDefault();
  	}
  	else
  	{
  		ObjectLookup lookUp = LookupFactory.createLookupForModifiableVersion(_hierarchie.getDataModel());
  		data = DataFactory.createModifiableCopy(daten, lookUp);
  	}

  	String kvVorbelegung = Activator.getDefault().getPreferenceStore().getString(DefsPara.VorbelegungVerwaltung);
  	
  	if (kvVorbelegung == null || kvVorbelegung.isEmpty())
  	{	
	  	ConfigDataModel dataModel = _hierarchie.getDataModel();
	  	
	  	ConfigurationAuthority kv = dataModel.getConfigurationAuthority();
	  	
	  	if (kv.isOfType("typ.autarkeOrganisationsEinheit"))
	  	{
	  		kvVorbelegung = kv.getPid();
	  	}
	  	else
	  	{
//	  		boolean gefunden = false;
	  		for (ConfigurationArea kb: dataModel.getAllConfigurationAreas().values())
	  		{
	  			kv = kb.getConfigurationAuthority();
	  			if (kv.isOfType("typ.autarkeOrganisationsEinheit"))
	  			{
	  				kvVorbelegung = kv.getPid();
	  			}
	  		}
	  	}
  	}	
  	
  	if (objekt instanceof MutableSet && atg.getPid().equals( "atg.dynamischeMenge" ))
  	  data.getItem("verwaltung").asTextValue().setText(kvVorbelegung);
  	
    return data;
  }
  
  public Data getDatenArrayElement(HierarchieObjekt ho, SystemObject objekt, AttributeGroup atg, Data daten, String arrayName, int arrayIndex,
      boolean nurPflichtFelder)
  {
    return null;
  }
  
  public void beobachterAnmelden(BildungsRegelnBeobachter beobachter)
  {
  }
  
  public void alleBeobachterInformieren(String feldName, String wert)
  {
  }
  
  public void beobachterAbmelden(BildungsRegelnBeobachter beobachter)
  {
  }
  
  public void wertGeaendert(String feldName, String wert)
  {
  }
}
