/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkanetz;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;

import de.bsvrz.buv.plugin.tkabasis.Activator;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.AtgInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.AttributInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HODaten;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HierarchieObjekt;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchie;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieObjektTyp;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.SystemObjektDaten;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.TypInfo;
import de.bsvrz.buv.plugin.tkabasis.pruefungen.IPruefungen;
import de.bsvrz.buv.plugin.tkabasis.pruefungen.PruefungenTools;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.ObjectSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;

/**
 * Prfungen von Attributwerten fr den TKA Netz Netzbestandteile
 * 
 * @author Krause
 *
 */
public class PruefungenNetz implements IPruefungen
{

	private IHierarchie _hierarchie;
	
	 /**
   * Bestimmt die Hierarchie, die fr die Prfungen benutzt werden kann
   * 
   * @return die Hierarchie, die fr die Prfungen benutzt werden kann
   */
	public IHierarchie getHierarchie()
	{
		return _hierarchie;
	}
  
	/**
   * Setzt die Hierarchie, die fr die Prfungen benutzt werden kann
   * @param hierachie die Hierarchie, die fr die Prfungen benutzt werden kann
   */
	public void setHierarchie(IHierarchie hierarchie)
	{
		_hierarchie = hierarchie;
	}
	
	public IStatus pruefeHierarchieObjekt(HierarchieObjekt ho, HODaten daten)
	{
		MultiStatus result = new MultiStatus(Activator.PLUGIN_ID, Status.OK, 
				"Prfe Hierarchieobjekt " + ho, null);
	  IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
		
		for (SystemObject so: ho.getSystemObjekte())
  	{
			 SystemObjektDaten soDaten = daten.getSystemObjektDaten(so);
			 if (soDaten != null)
			 {
				 MultiStatus objektStatus;
				 if (ho.getSystemObjekte().size() > 1)
				 {
					 objektStatus = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe Systemobjekt " + 
							 so.getPid() + " vom Typ " + so.getType().getPid(), null);
				 }
				 else
				 {
					 objektStatus = result;
				 }
				 
				 //objektStatus.add(pruefePid(so, soDaten.getPid()));
				 ConfigDataModel dataModel = getHierarchie().getDataModel();
				 objektStatus.add(new PruefungenTools(dataModel).pruefePid(soDaten.getPid(), so));
				 TypInfo typInfo = hot.getTypInfo(so.getType().getPid());
	       
	       AttributeGroup atg = dataModel.getAttributeGroup("atg.dynamischeMenge");
	       AttributInfo attInfo = new AttributInfo("verwaltung", true, true, true, true, true, true);
	       
	       for (String mengenName: typInfo.getDynMengen())
	       {
	      	 ObjectSet menge = ((ConfigurationObject)so).getObjectSet(mengenName);
	      	 if (menge != null)
	      	 {
	      		 Data data = soDaten.getDynMengenDaten(mengenName);
	      		 if (data != null)
		         {
		           MultiStatus atgStatus = new MultiStatus(Activator.PLUGIN_ID, Status.OK, 
		               "Prfe Daten der ATG " + atg.getPid() + " der Menge " + mengenName, null);
		           atgStatus.add(PruefungenTools.pruefeDatenDefinition(attInfo, data));
		           objektStatus.add(atgStatus);
		         }
	      	 }
	       }
	       if (objektStatus != result)
				 {
					 result.add(objektStatus);
				 }
			 }	
  	}
		
		return result;
	}

  /**
   * Prft einen Attributwert auf Zulssigkeit
   * 
   * @param ho Hierarchieobjekt, dessen Daten geprft werden sollen
   * @param so Systemobjekt, gehrt zu ho
   * @param atgInfo Info ber das zu prfenden Attribut
   * @param daten Datensatz der Daten, die geprft werden sollen
   * @return eine Instanz vom Typ IStatus, die aussagt, ob es zulssig ist, die 
   * bergebenen Daten als Attributwert zu bernehmen, oder <code>null</code>, wenn
   * keine Prfung fr die bergebenen Parameter implementiert ist
   */
	public IStatus pruefeDaten(HierarchieObjekt ho, SystemObject so, AtgInfo atgInfo, Data daten)
  {
	  
    MultiStatus result = new MultiStatus( Activator.PLUGIN_ID, Status.OK, "Prfe Daten der ATG " + atgInfo.getAtg(),
        null );

     return result;
  }
	
}
  