/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkatls;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;

import org.eclipse.core.runtime.Assert;

import de.bsvrz.buv.plugin.tkabasis.DataFactory;
import de.bsvrz.buv.plugin.tkabasis.LookupFactory;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HierarchieObjekt;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchie;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieObjektTyp;
import de.bsvrz.buv.plugin.tkabasis.regeln.BildungsRegelnBeobachter;
import de.bsvrz.buv.plugin.tkabasis.regeln.BildungsRegelnTools;
import de.bsvrz.buv.plugin.tkabasis.regeln.EingabeVorbelegung;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.NonMutableSet;
import de.bsvrz.dav.daf.main.config.ObjectLookup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;
import de.bsvrz.puk.config.configFile.datamodel.ConfigSystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

public class BildungsRegelnAllgemeinTLS implements IBildungsRegelnTLS
{
	/**
   * Debug-Logger fr Logging-Ausgaben
   */
  private final static Debug _debug = Debug.getLogger();
	
	private IHierarchie _hierarchie;
		
  protected Map<String, String> _praefixe = new HashMap<String, String>();
    
  protected Map<IHierarchieObjektTyp, String[]> _felder = new HashMap<IHierarchieObjektTyp, String[]>();
     
  protected Map<IHierarchieObjektTyp, Map<String, String>> _feldWerte = new HashMap<IHierarchieObjektTyp, Map<String, String>>();
  
  protected Map<String, Map<String, String>> _historie = new HashMap<String, Map<String, String>>();
  
  protected final EingabeVorbelegung _belegungDEKanal = new EingabeVorbelegung(0, 254);
  
  protected Map<IHierarchieObjektTyp, EingabeVorbelegung> _belegungFg = new HashMap<IHierarchieObjektTyp, EingabeVorbelegung>();
  
  public IHierarchie getHierarchie()
	{
		return _hierarchie;
	}
  
	public void setHierarchie(IHierarchie hierarchie)
	{
		if (hierarchie != _hierarchie)
		{
			_hierarchie = hierarchie;
			
			_praefixe.clear();
			initPraefixe();
			
			_felder.clear();
	    initFelder();
	    
	    _feldWerte.clear();
	    
	    _historie.clear();
	    
	    _belegungFg.clear();
	    DeTypenManager deTypenManager = DatenSpeicher.getInstanz().getDeTypenManager();
	    for (DeUnterTyp unterTyp: deTypenManager.getDeEbene())
	    {
	    	_belegungFg.put(unterTyp, new EingabeVorbelegung("" + unterTyp.getFg(), null, false, false));
	    }
	    
	    _hoVRZUZ = null;
	    _vorherigeEakBezeichnung = BildungsRegelnTools.LEERER_WERT;
	    _vorherigerDeKanal = BildungsRegelnTools.LEERER_WERT;
		}
	}
  
  protected void initPraefixe()
  {
    _praefixe.put("typ.viz", "viz");
    _praefixe.put("typ.vrz", "vrz");
    _praefixe.put("typ.uz", "uz");
    _praefixe.put("typ.kri", "kri");
    _praefixe.put("typ.steuerModul", "sm");
    _praefixe.put("typ.eak", "eak");
    
    _praefixe.put("typ.anschlussPunkt", "ap");
    _praefixe.put("typ.anschlussPunktKommunikationsPartner", "apkp");
    
    DeTypenManager deTypenManager = DatenSpeicher.getInstanz().getDeTypenManager();
    for (String typ: deTypenManager.getSystemObjektTypen())
    {	
      _praefixe.put(typ, "de");
    }
  }
  
  
  protected void initFelder()
  {
    _felder.put(HOT_TLS.VIZ, new String[] { "Bezeichnung" });
    _felder.put(HOT_TLS.VRZ, new String[] { "Bezeichnung" });
    _felder.put(HOT_TLS.UZ, new String[] { "Bezeichnung" });
    _felder.put(HOT_TLS.KRI, new String[] { "Bezeichnung" });
    _felder.put(HOT_TLS.Inselbus, new String[] { "ZustzlicheBezeichnung" });
    _felder.put(HOT_TLS.SM, new String[] { "Bezeichnung" });
    _felder.put(HOT_TLS.EAK, new String[] { "Bezeichnung" });
    
    DeTypenManager deTypenManager = DatenSpeicher.getInstanz().getDeTypenManager();
    for (IHierarchieObjektTyp hot: deTypenManager.getDeEbene())
    {	
    	_felder.put(hot, new String[] { "DEKanal", "Funktionsgruppe" });
    }	
  }
    
  public BildungsRegelnAllgemeinTLS()
  {
//    initPraefixe();
//    initFelder();
//    
//    DeTypenManager deTypenManager = DatenSpeicher.getInstanz().getDeTypenManager();
//    
//    for (DeUnterTyp unterTyp: deTypenManager.getDeEbene())
//    {
//    	_belegungFg.put(unterTyp, new EingabeVorbelegung("" + unterTyp.getFg(), null, false, false));
//    }
  }
  
  public String getPraefix(String pidTypNeuesObjekt)
  {
    String result = _praefixe.get(pidTypNeuesObjekt);
    
    return result != null ? result : BildungsRegelnTools.LEERER_WERT;
  }
  
  public String[] getEingabeFelder(IHierarchieObjektTyp hotNeuesObjekt)
  {
    String[] result = _felder.get(hotNeuesObjekt);
    
    return result != null ? result : BildungsRegelnTools.LEERE_TEXT_LISTE;
  }
  
  private static EingabeVorbelegung STANDARD_VORBELEGUNG = new EingabeVorbelegung(null, false);
  
  public EingabeVorbelegung getEingabeVorbelegung(IHierarchieObjektTyp hotNeuesObjekt, String eingabeFeld, HierarchieObjekt verbundenesHO)
  {
  	if (Arrays.asList(_felder.get(hotNeuesObjekt)).contains(eingabeFeld))
    {
  		if (hotNeuesObjekt instanceof DeUnterTyp)
  		{
  			if ("DEKanal".equals(eingabeFeld))
  			{	
  				return _belegungDEKanal;
  			}
  			else if ("Funktionsgruppe".equals(eingabeFeld))
  			{
  				return _belegungFg.get(hotNeuesObjekt);
  			}
  		}
// Keine Vererbung der Feldinhalte  		  		
//  		else if (verbundenesHO != null) 
//	    {
//	      Map<String, String> werte = _historie.get(verbundenesHO.getSystemObjekte().get(0).getPid());
//	      
//	      if (werte != null)
//	      {
//	        String text = werte.get(eingabeFeld);
//	        if (text != null)
//	        {
//	        	if (HOT_TLS.KRI.equals(hotNeuesObjekt) && "Bezeichnung".equals(eingabeFeld))
//	        	{
//	        		return STANDARD_VORBELEGUNG;
//	        	}
//	        	else
//	        	{
//	        		return new EingabeVorbelegung(text, false);
//	        	}
//	        }
//	      }
//	    }
	    return STANDARD_VORBELEGUNG;
    }  
    
    return BildungsRegelnTools.LEERE_VORBELEGUNG;
  }
  
  public void setWert(IHierarchieObjektTyp hotNeuesObjekt, String eingabeFeld, String wert)
  {
    Map<String, String> werte = _feldWerte.get(hotNeuesObjekt);
    
    if (werte == null)
    {
      werte = new HashMap<String, String>();
      _feldWerte.put(hotNeuesObjekt, werte);
    }
    
    werte.put(eingabeFeld, wert);
  }
  
  public String getWert(IHierarchieObjektTyp hotNeuesObjekt, String eingabeFeld)
  {
    Map<String, String> werte = _feldWerte.get(hotNeuesObjekt);
    
    if (werte == null)
    {
      return BildungsRegelnTools.LEERER_WERT;
    }
    
    String result = werte.get(eingabeFeld);
    
    return result != null ? result : BildungsRegelnTools.LEERER_WERT;
  }
 
  public boolean setWerte (IHierarchieObjektTyp hotNeuesObjekt, HierarchieObjekt uebergeordnetesHO, 
      HierarchieObjekt vorlageHO)
  {
  	boolean result = false;
    
    if (HOT_TLS.VIZ.equals(hotNeuesObjekt) ||
    		HOT_TLS.VRZ.equals(hotNeuesObjekt) ||
    		HOT_TLS.UZ.equals(hotNeuesObjekt) ||
    		HOT_TLS.KRI.equals(hotNeuesObjekt) ||
    		HOT_TLS.EAK.equals(hotNeuesObjekt))
    {
      if (uebergeordnetesHO != null)
      {
        EingabeVorbelegung belegung = getEingabeVorbelegung(hotNeuesObjekt, "Bezeichnung", uebergeordnetesHO);
        String wert = belegung.getWert();
      
        Map<IHierarchieObjektTyp, Integer> typMap = new HashMap<IHierarchieObjektTyp, Integer>();
            
        for (HierarchieObjekt ho: getHierarchie().getUntergeordneteObjekte(uebergeordnetesHO))
        {
        	 if (HOT_TLS.EAK.equals(hotNeuesObjekt)) // Fr EAK keine rekursive Behandlung ntig
        	 {	 
	           IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
	           Integer anzahl = typMap.get(hot);
	           typMap.put(hot, anzahl == null ? 1 : anzahl.intValue() + 1);
        	 }
        	 else
        	 {
        		 bestimmeTypMapRekursiv(ho, typMap);
        	 }
        }
        Integer suffix = typMap.get(hotNeuesObjekt);
//        if (suffix != null)
//        {
//          wert += (suffix.intValue() + 1);
//        }
//        
//        setWert(hotNeuesObjekt, "Bezeichnung", wert);
//
//        if (suffix == null)
//        {
//          result = true;
//        }
       
        if (suffix == null)
        {
          setWert(hotNeuesObjekt, "Bezeichnung", wert);
          result = true;
        }
      } 
    }
    // else
    // HOT_TLS.Inselbus wird immer gefragt
    // HOT_TLS.DE wird immer gefragt
    
    if (uebergeordnetesHO == null)
    {	
    	//setVorlageHO(vorlageHO);
    }
    
    return result;
   
//  	String felder[] = _felder.get(hotNeuesObjekt);
//  	
//  	if (uebergeordnetesHO != null && felder != null)
//  	{	
//	  	for (String eingabeFeld: felder)
//	  	{
//	  		if (HOT_TLS.Inselbus.equals(hotNeuesObjekt) && "ZustzlicheBezeichnung".equals(eingabeFeld))
//	  		{
//	  			// Bei HOT Inselbus wird "ZustzlicheBezeichnung" mit ib<laufender Nr. des Inselbusses> vorbelegt 
//	    		int ibNr = 1;
//	    		
//	    		for (HierarchieObjekt ho: _hierarchie.getUntergeordneteObjekte(uebergeordnetesHO))
//	    		{
//	    			if (HOT_TLS.Inselbus.equals(ho.getHierarchieObjektTyp()))
//	  				{
//	    				ibNr++;
//	  				}
//	    		}
//	    		setWert(hotNeuesObjekt, eingabeFeld, "ib" + ibNr);
//	  		}
//	  		else
//	  		{	
//		  		// Es werden alle Felder aus uebergeordnetesHO bernommen, wenn bereits untergeordnete HO des
//	  			// HOT vorhanden sind, wir an die "Bezeichnung" eine laufende NR. beginnend mit 2 angehngt
//		  		EingabeVorbelegung belegung = getEingabeVorbelegung(hotNeuesObjekt, eingabeFeld, uebergeordnetesHO);
//		  		String wert = belegung.getWert();
//		  		if ("Bezeichnung".equals(eingabeFeld))
//		  		{
//		  			if (HOT_TLS.SM.equals(hotNeuesObjekt))
//  					{
//		  				HierarchieObjekt vaterHO = (HierarchieObjekt)_hierarchie.getUebergeordnetesObjekt(uebergeordnetesHO);
//		  				Object vater = _hierarchie.getUebergeordnetesObjekt(vaterHO);
//		  				belegung = getEingabeVorbelegung(vaterHO.getHierarchieObjektTyp(), eingabeFeld, 
//		  						vater instanceof HierarchieObjekt ? (HierarchieObjekt)vater : null);
//		  				wert = belegung.getWert();
//  					}
//		  			
//		  			Map<IHierarchieObjektTyp, Integer> typMap = new HashMap<IHierarchieObjektTyp, Integer>();
//		  			
//		  			for (HierarchieObjekt ho: _hierarchie.getUntergeordneteObjekte(uebergeordnetesHO))
//		    		{
//		  				 IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
//		  				 Integer anzahl = typMap.get(hot);
//		  				 typMap.put(hot, anzahl == null ? 1 : anzahl.intValue() + 1);
//		    		}
//		  			Integer suffix = typMap.get(hotNeuesObjekt);
//		  			if (suffix != null)
//		  			{
//		  				wert += (suffix.intValue() + 1);
//		  			}
//		  		}
//		  		setWert(hotNeuesObjekt, eingabeFeld, wert);
//	  		}	
//	  	}
//	  	
//  	}
// 
//  	setVorlageHO(vorlageHO);
//  	
//  	return false;
  }

  /**
   * Bestimmt die Anzahl des Vorkommens von HOT fr ein HO und seinen untergeordeten Objekten
   * 
   * @param ho das HO, das mit Unterobjekten betrachtet wird - nur fr HO vom Typ VIZ, VRZ, UZ und KRI
   * werden Infos in der typMap upgedatet
   * @param typMap die Map, in der die Information upgedatet wird
   */
  protected void bestimmeTypMapRekursiv(HierarchieObjekt ho, Map<IHierarchieObjektTyp, Integer> typMap)
  {
  	IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
  	
  	if (HOT_TLS.VIZ.equals(hot) ||
    		HOT_TLS.VRZ.equals(hot) ||
    		HOT_TLS.UZ.equals(hot) ||
    		HOT_TLS.KRI.equals(hot))
    { 
      Integer anzahl = typMap.get(hot);
      typMap.put(hot, anzahl == null ? 1 : anzahl.intValue() + 1);
    }
  	
     for (HierarchieObjekt kind: getHierarchie().getUntergeordneteObjekte(ho))
     {
    	 bestimmeTypMapRekursiv(kind, typMap);
     }
  }
  
  public String getPidRegelText(IHierarchieObjektTyp hot)
  {
   	String felder[] = _felder.get(hot);
  	if (felder != null)
  	{
  		String result;
  		if (hot.equals(HOT_TLS.Inselbus))
  		{
  			String typPid = hot.getTypen().get(0);
    		result = getPraefix(typPid) + ".<PID bergeordnetes Systemobjekt>";
    		result += ".ZustzlicheBezeichnung"; 
  		}
  		else
  		{	
	  		String typPid = hot.getTypen().get(0);
	  		if (typPid.startsWith("typ.de"))
	  		{
	  			result = getPraefix(typPid) + ".DEKanal.fg.Funktionsgruppe";
	  			result += ".<PID bergeordnetes Systemobjekt>"; 
	  		}
	  		else
	  		{	
	  			result = getPraefix(typPid) + ".Bezeichnung";
	  			if (!HOT_TLS.VIZ.equals(hot) && !HOT_TLS.VRZ.equals(hot) && !HOT_TLS.UZ.equals(hot))
	  			{
	  	  			result += ".<PID bergeordnetes Systemobjekt>"; 
	  			}
	  			else if (HOT_TLS.VRZ.equals(hot) || HOT_TLS.UZ.equals(hot))
	  			{
	  	  			result += "[.<PID bergeordnetes Systemobjekt>]"; 
	  			}
	  		}
  		}
  		return result;
  	}
  	return BildungsRegelnTools.LEERER_WERT;
  }
  
  public String getNameRegelText(IHierarchieObjektTyp hot)
  {
  	String felder[] = _felder.get(hot);
  	if (felder != null)
  	{
  		return "<PID Systemobjekt>";
  	}
  	return BildungsRegelnTools.LEERER_WERT;
  }
   
  protected String bestimmePidErsterKP(SystemObject ap)
  {
  	String result = null;
  	ConfigurationObject koAP = (ConfigurationObject)ap;
  	ConfigDataModel dataModel = _hierarchie.getDataModel();
  	ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
  	AttributeGroup atg = dataModel.getAttributeGroup( "atg.anschlussPunktKommunikationsPartner");
  	NonMutableSet mengeKP = koAP.getNonMutableSet( "AnschlussPunkteKommunikationsPartner" );
  	
    if (mengeKP != null)
    {
      for (SystemObject apKP: mengeKP.getElementsInModifiableVersion())
      {
      	if (apKP != null)
      	{
	        Data data = ((ConfigSystemObject)apKP).getConfigurationData(atg, lookup );
	        if (data != null)
	        {	
	          SystemObject ref = data.getReferenceValue( "KommunikationsPartner" ).getSystemObject();
	          if (ref != null)
	          {	
	          	result = ref.getPid();
	          	break;
	          }
	        }
      	}  
      }  
    }
  	return result;
  }	
  	
//  private HierarchieObjekt _vorlageHO;
//  
//  
//  public HierarchieObjekt getVorlageHO()
//	{
//		return _vorlageHO;
//	}
//
//	public void setVorlageHO(HierarchieObjekt vorlageHO)
//	{
//		_vorlageHO = vorlageHO;
//	}
	  
	public String[] getPids(IHierarchieObjektTyp hotNeuesObjekt, HierarchieObjekt uebergeordnetesHO)
  {
  	String[] result = BildungsRegelnTools.LEERE_TEXT_LISTE;
  	
  	if (hotNeuesObjekt.equals(HOT_TLS.VIZ) || 
  			hotNeuesObjekt.equals(HOT_TLS.VRZ) || 
  			hotNeuesObjekt.equals(HOT_TLS.UZ) ||
  			hotNeuesObjekt.equals(HOT_TLS.KRI))
  	{
  		if (!(uebergeordnetesHO instanceof HierarchieObjekt))
  		{		
	  		result = new String[1];
	  		
	  		StringBuffer buffer = new StringBuffer();
	  		String pidTypNeuesObjekt = hotNeuesObjekt.getTypen().get(0);
	  		buffer.append(getPraefix(pidTypNeuesObjekt));
	      buffer.append('.');
	      String bezeichnung = getWert(hotNeuesObjekt, "Bezeichnung");
	      buffer.append(BildungsRegelnTools.getPidText(bezeichnung, false, false));
	      
	      if (uebergeordnetesHO != null)
	      {
	      	buffer.append('.');
	      	buffer.append(uebergeordnetesHO.getSystemObjekt(0).getPid());
	      }
	      
	      result[0] = buffer.toString();
  		}
  		else
  		{
  			result = new String[3];
  			    		
    		StringBuffer buffer = new StringBuffer();
    		String pidTypNeuesObjekt = hotNeuesObjekt.getTypen().get(0);
    		buffer.append(getPraefix(pidTypNeuesObjekt));
        buffer.append('.');
        String bezeichnung = getWert(hotNeuesObjekt, "Bezeichnung");
        buffer.append(BildungsRegelnTools.getPidText(bezeichnung, false, false));
        
        if (uebergeordnetesHO != null)
        {
        	buffer.append('.');
        	buffer.append(uebergeordnetesHO.getSystemObjekt(0).getPid());
        }

        result[0] = buffer.toString();

//        // Fr ap und apkp wird die PID des KP vorrrangig aus dem vorlageHO geholt
//        HierarchieObjekt vorlageHO = getVorlageHO();
//        String kpPid = vorlageHO != null ? vorlageHO.getSystemObjekt(0).getPid() : result[0];
        
        // Fr ap und apkp wird die PID des KP vorrrangig aus _hoVRZUZ geholt
        String kpPid = _hoVRZUZ != null ? _hoVRZUZ.getSystemObjekt(0).getPid() : result[0];
        
        // ap
        buffer = new StringBuffer();
        buffer.append(getPraefix("typ.anschlussPunkt"));
        buffer.append('.');
        buffer.append(uebergeordnetesHO.getSystemObjekt(0).getPid());
        buffer.append('.');
        buffer.append(getPraefix("typ.anschlussPunktKommunikationsPartner"));
        buffer.append('.');
        buffer.append(kpPid);
        result[1] = buffer.toString();
        
        // apkp
        buffer = new StringBuffer();
        buffer.append(getPraefix("typ.anschlussPunktKommunikationsPartner"));
        buffer.append('.');
        buffer.append(kpPid);
        result[2] = buffer.toString();
  		}
  	}
  	
  	else if (hotNeuesObjekt.equals(HOT_TLS.Inselbus))
  	{
  		result = new String[1];
  		
  		StringBuffer buffer = new StringBuffer();
  		String pidTypNeuesObjekt = hotNeuesObjekt.getTypen().get(0);
  		buffer.append(getPraefix(pidTypNeuesObjekt));
      buffer.append('.');
      buffer.append(uebergeordnetesHO.getSystemObjekt(0).getPid());
      buffer.append('.');
      String bezeichnung = getWert(hotNeuesObjekt, "ZustzlicheBezeichnung");
      buffer.append(BildungsRegelnTools.getPidText(bezeichnung, false, false));
            
      result[0] = buffer.toString();

  	}
  	else if (hotNeuesObjekt.equals(HOT_TLS.SM))
  	{
  		result = new String[2];
  		
  		StringBuffer buffer = new StringBuffer();
  		String pidTypNeuesObjekt = hotNeuesObjekt.getTypen().get(0);
  		buffer.append(getPraefix(pidTypNeuesObjekt));
      buffer.append('.');
      String bezeichnung = getWert(hotNeuesObjekt, "Bezeichnung");
      buffer.append(BildungsRegelnTools.getPidText(bezeichnung, false, false));
      
      if (uebergeordnetesHO != null)
      {
      	buffer.append('.');
      	buffer.append(uebergeordnetesHO.getSystemObjekt(0).getPid());
      }
      
      result[0] = buffer.toString();
      
      // apkp
      buffer = new StringBuffer();
      buffer.append(getPraefix("typ.anschlussPunktKommunikationsPartner"));
      buffer.append('.');
      buffer.append(result[0]);
      result[1] = buffer.toString();
  	}
  	else if (hotNeuesObjekt.equals(HOT_TLS.EAK))
  	{
  		result = new String[1];
  		
  		StringBuffer buffer = new StringBuffer();
  		String pidTypNeuesObjekt = hotNeuesObjekt.getTypen().get(0);
  		buffer.append(getPraefix(pidTypNeuesObjekt));
      buffer.append('.');
      String bezeichnung = getWert(hotNeuesObjekt, "Bezeichnung");
      buffer.append(BildungsRegelnTools.getPidText(bezeichnung, false, false));
      
      if (uebergeordnetesHO != null)
      {
      	buffer.append('.');
      	buffer.append(uebergeordnetesHO.getSystemObjekt(0).getPid());
      }
      
      result[0] = buffer.toString();
  	}
  	else if (hotNeuesObjekt instanceof DeUnterTyp)
  	{
  		result = new String[1];
  		StringBuffer buffer = new StringBuffer();
  		String pidTypNeuesObjekt = hotNeuesObjekt.getTypen().get(0);
  		buffer.append(getPraefix(pidTypNeuesObjekt));
      buffer.append('.');
      String deKanal = getWert(hotNeuesObjekt, "DEKanal");
      buffer.append(BildungsRegelnTools.getPidText(deKanal, false, false));
      buffer.append('.');
      buffer.append("fg");
      buffer.append('.');
      buffer.append(((DeUnterTyp)hotNeuesObjekt).getFg());
      
      if (uebergeordnetesHO != null)
      {
      	buffer.append('.');
      	buffer.append(uebergeordnetesHO.getSystemObjekt(0).getPid());
      }
      
      result[0] = buffer.toString();
  	}
  	
  	if (result.length > 0)
  	{
	  	Map<String, String> werte = _feldWerte.get(hotNeuesObjekt);
	    if (werte != null && !werte.isEmpty())
	    {
	      _historie.put(result[0], new HashMap<String, String>(werte));
	    }
  	}
  
  	return result;
  }	
  	
  	
  
  
  public String getName(SystemObject objekt)
  {
  	String pid = objekt.getPid();
  	
  	if (true /*_historie.get(pid) != null*/)
  	{
  		return pid;
  	}
  	
  	return BildungsRegelnTools.LEERER_WERT;
  }
  
  protected long bestimmePortNummerAP(HierarchieObjekt ho)
  {
  	HashSet<Long> vorhandenePortNummern = new HashSet<Long>();
  	ConfigDataModel dataModel = _hierarchie.getDataModel();
  	AttributeGroup atg = dataModel.getAttributeGroup("atg.anschlussPunkt");
  	ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
  	
  	// Baue Menge der vorhandenen AP-Portnummern innerhalb des begeordneten Objekts
  	HierarchieObjekt hoVater = (HierarchieObjekt)_hierarchie.getUebergeordnetesObjekt(ho); 
  	
  	for (HierarchieObjekt hoKind: _hierarchie.getUntergeordneteObjekte(hoVater))
  	{
  		if (hoKind != ho)
  		{
  			SystemObject ap = null;
  			IHierarchieObjektTyp hot = hoKind.getHierarchieObjektTyp();
  			if (hot.equals(HOT_TLS.VRZ) || hot.equals(HOT_TLS.UZ) || hot.equals(HOT_TLS.KRI))
  			{
  				ap = hoKind.getSystemObjekt(1);
  			}
  			else if (hot.equals(HOT_TLS.Inselbus))
  			{
  				ap = hoKind.getSystemObjekt(0);
  			}
  				
  			if (ap != null)
  			{	
	  			Data data = ((ConfigSystemObject)ap).getConfigurationData(atg, lookup);
	  			if (data != null)
	  			{
						long portNummer = data.getItem("PortNummer").asUnscaledValue().longValue();
						vorhandenePortNummern.add(portNummer);
					}
  			}	
  		}	
  	}
  	
  	long result = 200;
  	
  	while (result > 0)
  	{
  		if (!vorhandenePortNummern.contains(result))
  		{
  			break;
  		}
  		result += 1;
  	}
  	
  	return result;
  }
  
  protected long bestimmePortNummerAPKP(HierarchieObjekt ho)
  {
  	HashSet<Long> vorhandenePortNummern = new HashSet<Long>();
//  	ConfigDataModel dataModel = _hierarchie.getDataModel();
//  	AttributeGroup atg = dataModel.getAttributeGroup("atg.anschlussPunktKommunikationsPartner");
//  	ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
  	
		IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
		SystemObject ap = null;
		SystemObject apkp = null;

		if (_hierarchie.getUebergeordnetesObjekt(ho) instanceof HierarchieObjekt) // Nicht fr HO unmittelbar unter KB
		{	
			if (hot.equals(HOT_TLS.VRZ) || hot.equals(HOT_TLS.UZ) || hot.equals(HOT_TLS.KRI))
			{
				ap = ho.getSystemObjekt(1);
				apkp = ho.getSystemObjekt(2);
			}	
			else if (HOT_TLS.SM.equals(hot))
			{
				ap = ((HierarchieObjekt)_hierarchie.getUebergeordnetesObjekt(ho)).getSystemObjekt(0);
				apkp = ho.getSystemObjekt(1);
			}
		}
		
		if (ap != null && apkp != null)
		{
			NonMutableSet apkpMenge = ((ConfigurationObject)ap).getNonMutableSet("AnschlussPunkteKommunikationsPartner");
			if (apkpMenge != null)
			{
				for (SystemObject so: apkpMenge.getElementsInModifiableVersion())
				{
					if (so == null)
          {
          	_debug.warning("null-Referenz in Menge " + apkpMenge.getName() + " von " + ap.getPid());
          	continue;
          }
					if (so != apkp)
					{
						ConfigDataModel dataModel = getHierarchie().getDataModel();
      			AttributeGroup atg = dataModel.getAttributeGroup("atg.anschlussPunktKommunikationsPartner");
      			ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);

            Data data = ((ConfigSystemObject)so).getConfigurationData( atg, lookup );
            if (data != null)
            {	
            	long nummer = data.getItem("PortNummer").asUnscaledValue().longValue();
							vorhandenePortNummern.add(nummer);
            }  
					}  
				}
			}
		}
  	
  	// Baue Menge der vorhandenen AP-Portnummern innerhalb des begeordneten Objekts
//  	HierarchieObjekt hoVater = (HierarchieObjekt)_hierarchie.getUebergeordnetesObjekt(ho); 
  	
//  	for (HierarchieObjekt hoKind: _hierarchie.getUntergeordneteObjekte(hoVater))
//  	{
//  		if (hoKind != ho)
//  		{
//  			SystemObject apkp = null;
//  			IHierarchieObjektTyp hot = hoKind.getHierarchieObjektTyp();
//  			if (hot.equals(HOT_TLS.VRZ) || hot.equals(HOT_TLS.UZ) || hot.equals(HOT_TLS.KRI))
//  			{
//  				apkp = hoKind.getSystemObjekt(2);
//  			}
//  			else if (hot.equals(HOT_TLS.SM))
//  			{
//  				apkp = hoKind.getSystemObjekt(1);
//  			}
//  				
//  			if (apkp != null)
//  			{	
//	  			Data data = ((ConfigSystemObject)apkp).getConfigurationData(atg, lookup);
//	  			if (data != null)
//	  			{
//						long portNummer = data.getItem("PortNummer").asUnscaledValue().longValue();
//						vorhandenePortNummern.add(portNummer);
//					}
//  			}	
//  		}	
//  	}
  	
  	long result = 1;
  	
  	while (result > 0)
  	{
  		if (!vorhandenePortNummern.contains(result))
  		{
  			break;
  		}
  		result += 1;
  	}
  	
  	return result;
  }
  
  protected int bestimmeOSI2Adresse(HierarchieObjekt ho, SystemObject apkp)
  {
  	Collection<Integer> vorhandeneAdressen = new HashSet<Integer>();
  	ConfigDataModel dataModel = _hierarchie.getDataModel();
  	AttributeGroup atg = dataModel.getAttributeGroup("atg.eak");
  	ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
  	
  	// Baue Menge der vorhandenen OSI2-Adressen innerhalb SM
  	HierarchieObjekt hoSM = (HierarchieObjekt)_hierarchie.getUebergeordnetesObjekt(ho); 
  	
  	for (HierarchieObjekt hoEAK: _hierarchie.getUntergeordneteObjekte(hoSM))
  	{
  		if (hoEAK != ho)
  		{
  			SystemObject eak = hoEAK.getSystemObjekt(0);
  			Data data = ((ConfigSystemObject)eak).getConfigurationData(atg, lookup);
  			if (data != null)
  			{
					int adresse = data.getItem("OSI2Adresse").asUnscaledValue().intValue();
					vorhandeneAdressen.add(adresse);
				}	
  		}	
  	}
  	
  	// Bestimme niedrigsten freien Kanal
  	int result;
  	
  	for (result = 1; result < 254; result += 1)
  	{
  		if (!vorhandeneAdressen.contains(result))
  		{
  			break;
  		}
  	}
  	
  	return result;
  }
  
  protected String bestimmeUmsetzungsModul(int fg)
  {
  	String result = "";
     	
  	switch (fg)
  	{
  		case 1:
  		case 2:
  		case 4:
  		case 5:
  			result = "";
  			break;
  			
  		case 3:
  			result = "de.bsvrz.kex.tls.osi7.conversion.Fg003TypXxx";
  			break;	
  				
  		case 6:
  			result = "de.bsvrz.kex.tls.osi7.conversion.Fg006TypXxx";
  			break;
  	}
  	
  	return result;
  }
 
  
  /**
   * Finde nchsten freien EAKanal/umsetzungsModul-Kombination innerhalb bergeordnetem SM
   * 
   * @param ho
   * @return
   */
  protected int bestimmeEAKanal(HierarchieObjekt ho)
	{
  	Assert.isTrue(ho.getHierarchieObjektTyp() instanceof DeUnterTyp);
  	int fg = ((DeUnterTyp)ho.getHierarchieObjektTyp()).getFg();
  	
  	Collection<Integer> vorhandenKanaele = new HashSet<Integer>();
  	ConfigDataModel dataModel = _hierarchie.getDataModel();
  	AttributeGroup atg = dataModel.getAttributeGroup("atg.de");
  	ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
  	
  	// Baue Menge der vorhandenen Kanaele fr FG
  	Object vater = _hierarchie.getUebergeordnetesObjekt(ho);
//  	HierarchieObjekt hoSM = (HierarchieObjekt)_hierarchie.getUebergeordnetesObjekt((HierarchieObjekt)vater); 
//  	
//  	for (HierarchieObjekt hoEAK: _hierarchie.getUntergeordneteObjekte(hoSM))
//  	{
//  		for (HierarchieObjekt hoDE: _hierarchie.getUntergeordneteObjekte(hoEAK))
    	for (HierarchieObjekt hoDE: _hierarchie.getUntergeordneteObjekte(vater))	
    	{
    		if (hoDE.getHierarchieObjektTyp() instanceof DeUnterTyp)
    		{
  				if (hoDE != ho && fg == ((DeUnterTyp)hoDE.getHierarchieObjektTyp()).getFg())
  				{	
	    			SystemObject de  = hoDE.getSystemObjekt(0);
	    			Data data = ((ConfigSystemObject)de).getConfigurationData(atg, lookup);
	    			if (data != null)
	    			{
    					int eaKanal = data.getItem("EAKanal").asUnscaledValue().intValue();
    					vorhandenKanaele.add(eaKanal);
    				}	
    			}
    		}
//    	}
  	}
  		
  	// Bestimme niedrigsten freien Kanal
  	int result;
  	
  	for (result = 1; result < 253; result += 1)
  	{
  		if (!vorhandenKanaele.contains(result))
  		{
  			break;
  		}
  	}
  	
		return result;
	}
	
	
  /**
   * Finde nchsten freien EAKanal/umsetzungsModul-Kombination innerhalb bergeordnetem SM
   * 
   * @param ho
   * @param de
   * @param umsetzungsModul
   * @return
   */
  protected int bestimmeDEKanal(HierarchieObjekt ho)
	{
  	Assert.isTrue(ho.getHierarchieObjektTyp() instanceof DeUnterTyp);
  	int fg = ((DeUnterTyp)ho.getHierarchieObjektTyp()).getFg();
  	
  	Collection<Integer> vorhandenKanaele = new HashSet<Integer>();
  	ConfigDataModel dataModel = _hierarchie.getDataModel();
  	AttributeGroup atg = dataModel.getAttributeGroup("atg.de");
  	ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
  	
  	// Baue Menge der vorhandenen Kanaele fr umsetzungsModul
  	Object vater = _hierarchie.getUebergeordnetesObjekt(ho);
  	HierarchieObjekt hoSM = (HierarchieObjekt)_hierarchie.getUebergeordnetesObjekt((HierarchieObjekt)vater);
  	
  	for (HierarchieObjekt hoEAK: _hierarchie.getUntergeordneteObjekte(hoSM))
  	{
  		for (HierarchieObjekt hoDE: _hierarchie.getUntergeordneteObjekte(hoEAK))
    	{
    		if (hoDE.getHierarchieObjektTyp() instanceof DeUnterTyp)
    		{
  				if (hoDE != ho && fg == ((DeUnterTyp)hoDE.getHierarchieObjektTyp()).getFg())
  				{	
	    			SystemObject de  = hoDE.getSystemObjekt(0);
	    			Data data = ((ConfigSystemObject)de).getConfigurationData(atg, lookup);
	    			if (data != null)
	    			{
    					int deKanal = data.getItem("DEKanal").asUnscaledValue().intValue();
    					vorhandenKanaele.add(deKanal);
    				}	
    			}
    		}
    	}
  	}
  		
  	// Bestimme niedrigsten freien Kanal
  	int result;
  	
  	for (result = 1; result < 253; result += 1)
  	{
  		if (!vorhandenKanaele.contains(result))
  		{
  			break;
  		}
  	}
  	
		return result;
	}
	  
  
  public Data getDatensatz(HierarchieObjekt ho, SystemObject objekt, AttributeGroup atg, Data daten,
  		boolean nurPflichtFelder)
  {
  	if (!ho.getSystemObjekte().contains(objekt))
  	{
  		throw new IllegalArgumentException("Systemobjekt " + objekt.getPid() + 
  				" ist nicht im Hierarchieobjet enthalten");
  	}
  	
  	if (!objekt.getType().getAttributeGroups().contains(atg))
  	{
  		throw new IllegalArgumentException("Attributgruppe " + atg.getName() +
  				" kann nicht mit Systemobjekt " + objekt.getPid() + "verwendet werden "); 
  	}
  	
  	//IHierarchie hierarchie = getHierarchie();
  	//ConfigDataModel dataModel = hierarchie.getDataModel();
  	
  	Data data;
  	
  	if (daten == null)
  	{
	    data = DataFactory.createData(atg);
	    // Default und undefiniert Werte setzen
	    data.setToDefault();
  	}
  	else
  	{
  		ObjectLookup lookUp = LookupFactory.createLookupForModifiableVersion(_hierarchie.getDataModel());
  		data = DataFactory.createModifiableCopy(daten, lookUp);
  	}
  	
  	IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
  	
  	if (hot.equals(HOT_TLS.VIZ) ||
  			hot.equals(HOT_TLS.VRZ) ||
  			hot.equals(HOT_TLS.UZ) ||
  			hot.equals(HOT_TLS.KRI))
  	{
  		if (objekt.isOfType("typ.viz") || objekt.isOfType("typ.vrz") || objekt.isOfType("typ.uz") || objekt.isOfType("typ.kri"))
  		{
  			if (atg.getPid().equals("atg.gert"))
  			{
//  				if (!nurPflichtFelder) // Bezeichnung ist kein Pflichtfeld
//    			{	
//    				data.getItem("Bezeichnung").asTextValue().setText(getName(objekt));
//    			}
  				// Bezeichnung soll Pflichtfeld sein
  				data.getItem("Bezeichnung").asTextValue().setText(getName(objekt));
  				
  				// Feld Knotennummer nicht automisch und nicht vorbelegt
  				
  				if (!nurPflichtFelder) // UmsetzungsModul  ist kein Pflichtfeld
    			{	
    				data.getItem("UmsetzungsModul").asTextValue().setText("");
    			}
  			}
  			if (atg.getPid().equals("atg.gertZustndigkeit"))
  			{
  				if (!nurPflichtFelder) // Gert ist kein Pflichtfeld
    			{	
    				data.getItem("Gert").asArray().setLength(0);
    			}	
  			}
  			if (atg.getPid().equals("atg.punkKoordinaten"))
  			{
  				// Feld x nicht vorbelegt
  				// Feld y nicht vorbelegt
  			}
  		}
  		else if (objekt.isOfType("typ.anschlussPunkt"))
  		{
  			if (atg.getPid().equals("atg.anschlussPunkt"))
  			{
//  				if (!nurPflichtFelder) // Bezeichnung ist kein Pflichtfeld
//    			{	
//    				data.getItem("Bezeichnung").asTextValue().setText(getName(objekt));
//    			}
  				// Bezeichnung soll Pflichtfeld sein
  				data.getItem("Bezeichnung").asTextValue().setText(getName(objekt));

  				if (!nurPflichtFelder) // ProtokollTyp soll kein Pflichtfeld sein
      		{	
  					data.getItem("ProtokollTyp").asTextValue().setText("kex.tls.osi2.tc57.Primary");
  		  	}
  				
  				long portNummer = bestimmePortNummerAP(ho);
  				data.getItem("PortNummer").asTextValue().setText("" + portNummer);
  			}
  		}
  		else if (objekt.isOfType("typ.anschlussPunktKommunikationsPartner"))
  		{
  			if (atg.getPid().equals("atg.anschlussPunktKommunikationsPartner"))
  			{
//  				if (!nurPflichtFelder) // Bezeichnung ist kein Pflichtfeld
//    			{	
//    				data.getItem("Bezeichnung").asTextValue().setText(getName(objekt));
//    			}
  				// Bezeichnung soll Pflichtfeld sein
  				data.getItem("Bezeichnung").asTextValue().setText(getName(objekt));

  				if (!nurPflichtFelder) // ProtokollTyp soll kein Pflichtfeld sein
      		{	
  					data.getItem("ProtokollTyp").asTextValue().setText("kex.tls.osi2.tc57.Secondary");
      		}	
  				
  				// KommunikationsPartner wird beim Bilden der Hierarchie gesetzt
  				
  				long portNummer = bestimmePortNummerAPKP(ho);
  				data.getItem("PortNummer").asTextValue().setText("" + portNummer);
  			}
  		}
  	}
  	else if (hot.equals(HOT_TLS.Inselbus))
 		{
  		if (objekt.isOfType("typ.anschlussPunkt"))
  		{
  			if (atg.getPid().equals("atg.anschlussPunkt"))
  			{
//  				if (!nurPflichtFelder) // Bezeichnung ist kein Pflichtfeld
//    			{	
//    				data.getItem("Bezeichnung").asTextValue().setText(getName(objekt));
//    			}
  				// Bezeichnung soll Pflichtfeld sein
  				data.getItem("Bezeichnung").asTextValue().setText(getName(objekt));
  				
  				if (!nurPflichtFelder) // ProtokollTyp soll kein Pflichtfeld sein
      		{	
  					data.getItem("ProtokollTyp").asTextValue().setText("kex.tls.osi2.tc57.Primary");
      		}	
  				
  				long portNummer = bestimmePortNummerAP(ho);
  				data.getItem("PortNummer").asTextValue().setText("" + portNummer);
  			}
  		}
 		}
  	else if (hot.equals(HOT_TLS.SM))
		{
  		if (objekt.isOfType("typ.steuerModul"))
  		{
	  		if (atg.getPid().equals("atg.gert"))
				{
//					if (!nurPflichtFelder) // Bezeichnung ist kein Pflichtfeld
//	  			{	
//	  				data.getItem("Bezeichnung").asTextValue().setText(getName(objekt));
//	  			}	
  				// Bezeichnung soll Pflichtfeld sein
  				data.getItem("Bezeichnung").asTextValue().setText(getName(objekt));

					
					// Feld Knotennummer nicht automisch und nicht vorbelegt
					if (!nurPflichtFelder) // UmsetzungsModul  ist kein Pflichtfeld
	  			{	
	  				data.getItem("UmsetzungsModul").asTextValue().setText("");
	  			}
				}
	  		else if (atg.getPid().equals("atg.gertZustndigkeit"))
				{
					if (!nurPflichtFelder) // Gert ist kein Pflichtfeld
	  			{	
	  				data.getItem("Gert").asArray().setLength(0);
	  			}	
				}
	  		else if (atg.getPid().equals("atg.punkKoordinaten"))
				{
					// Feld x nicht vorbelegt
					// Feld y nicht vorbelegt
				}
			}
			else if (objekt.isOfType("typ.anschlussPunktKommunikationsPartner"))
			{
				if (atg.getPid().equals("atg.anschlussPunktKommunikationsPartner"))
				{
//					if (!nurPflichtFelder) // Bezeichnung ist kein Pflichtfeld
//	  			{	
//	  				data.getItem("Bezeichnung").asTextValue().setText(getName(objekt));
//	  			}
					
  				// Bezeichnung soll Pflichtfeld sein
  				data.getItem("Bezeichnung").asTextValue().setText(getName(objekt));
					
  				if (!nurPflichtFelder) // ProtokollTyp soll kein Pflichtfeld sein
      		{	
  					data.getItem("ProtokollTyp").asTextValue().setText("kex.tls.osi2.tc57.Secondary");
      		}	
					
					// KommunikationsPartner wird beim Bilden der Hierarchie gesetzt
					
					long portNummer = bestimmePortNummerAPKP(ho);
					data.getItem("PortNummer").asTextValue().setText("" + portNummer);
				}
			}
		}
  	else if (hot.equals(HOT_TLS.EAK))
		{
  		if (objekt.isOfType("typ.eak"))
  		{
	  		if (atg.getPid().equals("atg.eak"))
				{
//					if (!nurPflichtFelder) // Bezeichnung ist kein Pflichtfeld
//	  			{	
//	  				data.getItem("Bezeichnung").asTextValue().setText(getName(objekt));
//	  			}
	  			
  				// Bezeichnung soll Pflichtfeld sein
  				data.getItem("Bezeichnung").asTextValue().setText(getName(objekt));
					
					int osi2Adresse = bestimmeOSI2Adresse(ho, objekt);
					data.getItem("OSI2Adresse").asTextValue().setText("" + osi2Adresse);
					
					if (!nurPflichtFelder) // UmsetzungsModul  ist kein Pflichtfeld
	  			{	
	  				data.getItem("UmsetzungsModul").asTextValue().setText("");
	  			}
				}
	  		else if (atg.getPid().equals("atg.punkKoordinaten"))
				{
					// Feld x nicht vorbelegt
					// Feld y nicht vorbelegt
				}
			}
		}
  	else if (hot instanceof DeUnterTyp)
  	{
  		if (objekt.isOfType("typ.de"))
  		{
	  		if (atg.getPid().equals("atg.de"))
				{
//					if (!nurPflichtFelder) // Bezeichnung ist kein Pflichtfeld
//	  			{	
//	  				data.getItem("Bezeichnung").asTextValue().setText(getName(objekt));
//	  			}	
	  			
  				// Bezeichnung soll Pflichtfeld sein
  				data.getItem("Bezeichnung").asTextValue().setText(getName(objekt));
					
					data.getItem("Cluster").asTextValue().setText("Nein");
					
//					if (!nurPflichtFelder) // EAKanal und DEKanal werden wie Nicht-Pflichtfelder behandelt
//					{	
						int eaKanal = bestimmeEAKanal(ho);
						data.getItem("EAKanal").asTextValue().setText("" + eaKanal);
						
						String textDeKanal = getWert(hot, "DEKanal");
						int deKanal;
						try
						{
							deKanal = Integer.parseInt(textDeKanal);
						}
						catch (NumberFormatException e)
						{
							deKanal = bestimmeDEKanal(ho);
						}
						 
						data.getItem("DEKanal").asTextValue().setText("" + deKanal);
//					}
										
					if (!nurPflichtFelder) // UmsetzungsModul  ist kein Pflichtfeld
	  			{	
						String text;
						
						if ("typ.deUfdMicKS".equals(objekt.getType().getPid()))
						{
							text = "de.bsvrz.kex.tls.osi7.conversion.mv.Fg003Typ132MvMicKS";
						}
						else
						{
							text = bestimmeUmsetzungsModul(((DeUnterTyp)hot).getFg());
						}
							
	  				data.getItem("UmsetzungsModul").asTextValue().setText(text);
	  			}
				}
	  		else if (atg.getPid().equals("atg.punkKoordinaten"))
				{
					// Feld x nicht vorbelegt
					// Feld y nicht vorbelegt
				}
			}
  	}
  	    
    return data;
  }
  
  public void beobachterAnmelden(BildungsRegelnBeobachter beobachter)
  {
  }
  
  public void alleBeobachterInformieren(String feldName, String wert)
  {
  }
  
  public void beobachterAbmelden(BildungsRegelnBeobachter beobachter)
  {
  }
  
  public void wertGeaendert(String feldName, String wert)
  {
  }

  @Override
  public Data getDatenArrayElement( HierarchieObjekt ho, SystemObject objekt, AttributeGroup atg, Data daten,
      String arrayName, int arrayIndex, boolean nurPflichtFelder )
  {
    // TODO Auto-generated method stub
    return null;
  }
  
  
  protected HierarchieObjekt _hoVRZUZ = null;
  private String _vorherigeEakBezeichnung = BildungsRegelnTools.LEERER_WERT;
  private String _vorherigerDeKanal = BildungsRegelnTools.LEERER_WERT;
  
  /**
	 * Setzen von Zusatzinformationen fr einen bestimmten HOT
	 *    
	 * @param hot der Hierarchieobjekttyp, fr den die Zusatzinfos gelten
	 * @param info - Zusatzinfo:<br>
	 * Fr die HOTs VRZ und UZ wird das jeweilige HO vom HO-Typ VRZ oder UZ bergeben<br>
	 * Fr den HOT EAK wird das Feld Bezeichnung korrekt gesetzt, info bleibt unbercksichtigt<br>
	 * Fr DE-Untertypen wird der Wert des Feldes DE-Kanal mit der String-Reprsentation von info gesetzt
	 */
	public void setZusatzInfo(IHierarchieObjektTyp hot, Object info)
	{
		if (HOT_TLS.VRZ.equals(hot) || HOT_TLS.UZ.equals(hot))
		{
			Assert.isLegal(info instanceof HierarchieObjekt);
			_hoVRZUZ = (HierarchieObjekt)info;
		}
		else if (HOT_TLS.EAK.equals(hot))
		{
			_vorherigeEakBezeichnung = getWert(hot, "Bezeichnung");
			setWert(hot, "Bezeichnung", info.toString());
		}
		else if (hot instanceof DeUnterTyp)
		{
			Assert.isNotNull(info);
			
			_vorherigerDeKanal = getWert(hot, "DEKanal");
			setWert(hot, "DEKanal", info.toString());
		}
		
	}
	
	/**
	 * Setzt die Zusatzinformationen fr einen bestimmeten HOT auf den Wert vor
	 * dem letzten <code>setZusatzInfo</code>-Aufruf zurck
	 * 
	 * @param hot der Hierarchieobjekttyp, fr den die Zusatzinfos zurckgestellt werden
	 */
	public void resetZusatzInfo(IHierarchieObjektTyp hot)
	{
		if (HOT_TLS.VRZ.equals(hot) || HOT_TLS.UZ.equals(hot))
		{
			_hoVRZUZ = null;
		}
		else if (HOT_TLS.EAK.equals(hot))
		{
			setWert(hot, "Bezeichnung", _vorherigeEakBezeichnung);
		}
		else if (hot instanceof DeUnterTyp)
		{
			setWert(hot, "DEKanal", _vorherigerDeKanal);
		}
		
	}
	
}
