/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkatls;

import java.util.Collection;
import java.util.Collections;
import java.util.Map;
import java.util.TreeMap;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;

import de.bsvrz.buv.plugin.konfigass.MeldungenSpeicher;
import de.bsvrz.buv.plugin.tkabasis.Activator;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieObjektTyp;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sys.funclib.debug.Debug;

public class DeTypenManager 
{
	/**
   * Debug-Logger fr Logging-Ausgaben
   */
  private final static Debug _debug = Debug.getLogger();
	
  // LinkedHashMap -> Sortierung wie im Modell
	// private Map<String, DeUnterTyp> _deUnterTypen = new LinkedHashMap<String, DeUnterTyp>();
  // TreeMap -> Sortierung lexikalisch
  private Map<String, DeUnterTyp> _deUnterTypen = new TreeMap<String, DeUnterTyp>();
  
	private DeUnterTyp[] _deEbene;
	
	private DataModel _dataModel;
	
	DeTypenManager(DataModel dataModel)
	{
		_dataModel = dataModel;
		
		bestimmeUnterTypen(DeUnterTyp.DE_OBER_TYP);
		
		_deEbene = _deUnterTypen.values().toArray(new DeUnterTyp[_deUnterTypen.size()]);
	}
	
	private void bestimmeUnterTypen(String typ)
	{
		SystemObjectType oberTyp = _dataModel.getType(typ);
				
		if (oberTyp != null)
		{
			MultiStatus meldung = new MultiStatus(Activator.PLUGIN_ID, IStatus.OK,
					"Es wurden ungltige DE-Typen gefunden", null);
			
			for (SystemObjectType unterTyp: oberTyp.getSubTypes())
			{
				int fg = bestimmeFg(unterTyp);
				String pid = unterTyp.getPid();
				DeUnterTyp deUnterTyp;
				
				if (!(fg < 0))
				{	
					deUnterTyp = new DeUnterTyp(pid, fg, null);
					_deUnterTypen.put(pid,deUnterTyp);
				}
				else
				{
					meldung.add(new Status(Status.WARNING, Activator.PLUGIN_ID,	"Fr den Typ " + pid  +
						" konnte keine FG bestimmt werden - Objekte von diesem Typ knnen nicht angezeigt oder bearbeitet werden"));
					continue;
				}	
				bestimmeUnterTypen(unterTyp, deUnterTyp);
			}
			if (meldung.getChildren().length > 0)
			{
				MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(meldung, System.currentTimeMillis(), false);
			}
		}
		else
		{
			_debug.warning("Typ " + typ + " ist im Datenmodell nicht vorhanden");
		}
	}
	
	private void bestimmeUnterTypen(SystemObjectType typ, DeUnterTyp deHOT)
	{
		for (SystemObjectType unterTyp: typ.getSubTypes())
		{
			String pid = unterTyp.getPid();
			_deUnterTypen.put(pid, new DeUnterTyp(pid, deHOT.getFg(), deHOT));				
			bestimmeUnterTypen(unterTyp, deHOT);
		}
	}
	
	public DeUnterTyp[] getDeEbene()
	{
		return _deEbene;
	}
	
	public Collection<String> getSystemObjektTypen()
	{
		return Collections.unmodifiableCollection(_deUnterTypen.keySet());
	}
	
	public DeUnterTyp getDeUnterTyp(String typ)
	{
		return _deUnterTypen.get(typ);
	}
	
	public boolean hasHOT(IHierarchieObjektTyp hot)
	{
		return _deUnterTypen.values().contains(hot);
	}
	
	private int bestimmeFg(SystemObjectType typ)
	{
		int result = -1;
		String typPid = typ.getPid();
		
		if (typPid.equals("typ.deLve"))
		{
			result = 1;
		}
		else if (typPid.equals("typ.deAxl"))
		{
			result = 2;
		}
		else if (typPid.equals("typ.deUfd"))
		{
			result = 3;
		}
//		else if (typPid.equals("typ.deUfdNW"))
//		{
//			result = 3;
//		}
//		else if (typPid.equals("typ.deUfdMicKS"))
//		{
//			result = 3;
//		}
		else if (typPid.equals("typ.deWzg"))
		{
			result = 4;
		}	
		else if (typPid.equals("typ.deWww"))
		{
			result = 5;		
		}
		else if (typPid.equals("typ.deVlt"))
		{
			result = 6;		
		}
		else if (typPid.equals("typ.deZr"))
		{
			result = 9;		
		}
		else if (typPid.equals("typ.deSys"))
		{
			result = 254;		
		}
		else if (typPid.startsWith("typ.de") && typPid.length() > 6)
		{
			String zahl = typPid.substring(6);
			try
			{
				result = Integer.parseInt(zahl);
			}
			catch (NumberFormatException e)
			{
				// Es wird result < 0 zurckgegeben
			}
			if (result > 255 || result < 0)
			{
				result = -1;
			}
		}
		
		return result;
	}
		
}
