/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkatls;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;
import java.util.Vector;

import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.preference.IPreferenceStore;

import de.bsvrz.buv.plugin.konfigass.MeldungenSpeicher;
import de.bsvrz.buv.plugin.tkabasis.DataFactory;
import de.bsvrz.buv.plugin.tkabasis.LookupFactory;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.AtgInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HODaten;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HOHierarchie;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HOMitStatus;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HierarchieObjekt;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchie;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieManager;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieObjektTyp;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.MengeInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.SystemObjektDaten;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.TypInfo;
import de.bsvrz.buv.plugin.tkabasis.pruefungen.IPruefungen;
import de.bsvrz.buv.plugin.tkabasis.pruefungen.LoeschenChecker;
import de.bsvrz.buv.plugin.tkabasis.pruefungen.PruefungenTools;
import de.bsvrz.buv.plugin.tkabasis.regeln.IBildungsRegeln;
import de.bsvrz.buv.plugin.tkatls.preferences.DefsPara;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.AttributeGroupUsage;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationChangeException;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.ConfigurationObjectType;
import de.bsvrz.dav.daf.main.config.NonMutableSet;
import de.bsvrz.dav.daf.main.config.ObjectLookup;
import de.bsvrz.dav.daf.main.config.ObjectSet;
import de.bsvrz.dav.daf.main.config.ObjectSetType;
import de.bsvrz.dav.daf.main.config.ObjectTimeSpecification;
import de.bsvrz.dav.daf.main.config.ReferenceType;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.puk.config.configFile.datamodel.ConfigConfigurationObject;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;
import de.bsvrz.puk.config.configFile.datamodel.ConfigSystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

public class HierachieManagerTLS implements IHierarchieManager
{
	
	/**
   * Debug-Logger fr Logging-Ausgaben
   */
  private final static Debug _debug = Debug.getLogger();

	private static IHierarchieObjektTyp[] EBENE1 = new IHierarchieObjektTyp[] {HOT_TLS.VIZ, HOT_TLS.VRZ, HOT_TLS.UZ};
	private static IHierarchieObjektTyp[] EBENE2 = new IHierarchieObjektTyp[] {HOT_TLS.VRZ, HOT_TLS.UZ, HOT_TLS.KRI, HOT_TLS.Inselbus};
	private static IHierarchieObjektTyp[] EBENE3 = new IHierarchieObjektTyp[] {HOT_TLS.UZ, HOT_TLS.KRI, HOT_TLS.Inselbus};
	private static IHierarchieObjektTyp[] EBENE4 = new IHierarchieObjektTyp[] {HOT_TLS.KRI, HOT_TLS.Inselbus};
	private static IHierarchieObjektTyp[] EBENE5 = new IHierarchieObjektTyp[] {HOT_TLS.Inselbus};
	private static IHierarchieObjektTyp[] EBENE6 = new IHierarchieObjektTyp[] {HOT_TLS.SM};
	private static IHierarchieObjektTyp[] EBENE7 = new IHierarchieObjektTyp[] {HOT_TLS.EAK};
	// EBENE8 wird mit DeTypenManager abgehandelt

	private static IHierarchieObjektTyp[] SORTIER_FOLGE_AP = new IHierarchieObjektTyp[] {HOT_TLS.VIZ, HOT_TLS.VRZ, HOT_TLS.UZ, HOT_TLS.KRI, HOT_TLS.Inselbus};
	private static IHierarchieObjektTyp[] SORTIER_FOLGE_SM = new IHierarchieObjektTyp[] {HOT_TLS.SM};
	private static IHierarchieObjektTyp[] SORTIER_FOLGE_EAK = new IHierarchieObjektTyp[] {HOT_TLS.EAK};
  // Sortierung DE wird mit DeTypenManager abgehandelt
	
	private IHierarchieObjektTyp[] getTypSortierFolge(IHierarchieObjektTyp hot)
	{
		IHierarchieObjektTyp[] sortierFolge = null;
		
		if (hot.equals(HOT_TLS.VIZ) ||
			  hot.equals(HOT_TLS.VRZ) ||
			  hot.equals(HOT_TLS.UZ) ||
		    hot.equals(HOT_TLS.KRI) ||
		    hot.equals(HOT_TLS.Inselbus))
		{
			sortierFolge = SORTIER_FOLGE_AP;
		}
		else if (hot.equals(HOT_TLS.SM))
		{
			sortierFolge = SORTIER_FOLGE_SM;
		}
		else if (hot.equals(HOT_TLS.EAK))
		{
			sortierFolge = SORTIER_FOLGE_EAK;
		}
		else
		{
			DeTypenManager deTypenManager = DatenSpeicher.getInstanz().getDeTypenManager();
			return deTypenManager.getDeEbene();
		}
		
		return sortierFolge;
	}
	
  public IHierarchieObjektTyp[] getUntergeordneteHOTs(Object objekt)
  {
  	if (objekt instanceof ConfigurationArea)
  	{
  		return EBENE1;
  	}
  	else if (objekt instanceof HierarchieObjekt)
  	{
  		HierarchieObjekt ho = (HierarchieObjekt)objekt;
  	
  		if (ho.getHierarchieObjektTyp().equals(HOT_TLS.VIZ))
  		{
  			return EBENE2;
  		}
  		else if (ho.getHierarchieObjektTyp().equals(HOT_TLS.VRZ))
  		{
  			return EBENE3;
  		}
  		else if (ho.getHierarchieObjektTyp().equals(HOT_TLS.UZ))
  		{
  			return EBENE4;
  		}
  		else if (ho.getHierarchieObjektTyp().equals(HOT_TLS.KRI))
  		{
  			return EBENE5;
  		}
  		else if (ho.getHierarchieObjektTyp().equals(HOT_TLS.Inselbus))
  		{
  			return EBENE6;
  		}
  		else if (ho.getHierarchieObjektTyp().equals(HOT_TLS.SM))
  		{
  			return EBENE7;
  		}
  		else if (ho.getHierarchieObjektTyp().equals(HOT_TLS.EAK))
  		{
  			DeTypenManager deTypenManager = DatenSpeicher.getInstanz().getDeTypenManager();
  			return deTypenManager.getDeEbene();
  		}
   	}
  	
  	return null;
  }
	
	private IHierarchie _hierarchie;
	
	public IHierarchie getHierarchie()
	{
		return _hierarchie;
	}
	
	public void setHierarchie(IHierarchie hierarchie)
	{
		_hierarchie = hierarchie;
	}
		
		
	public void bestimmeUntergeordneteHOs(Object objekt)
	{
		ConfigDataModel dataModel = _hierarchie.getDataModel();
		ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(_hierarchie.getDataModel());
		
		if (objekt instanceof ConfigurationArea)	
    {
			ConfigurationArea kb = (ConfigurationArea)objekt;
			Collection<SystemObjectType> typeSpez = new Vector<SystemObjectType>();
			
			for (IHierarchieObjektTyp hot: EBENE1)
			{
				String typPid = hot.getTypen().get(0);
			  SystemObjectType  typ = _hierarchie.getDataModel().getType(typPid);
			  
			  if (typ == null)
			  {
			  	throw new RuntimeException("Typ " + typPid + " wurde im aktuellen Datemmodell nicht gefunden");
			  }
			  
  	  	typeSpez.add(typ);
			}
			
			// Liste der in Frage kommenenden Wurzelobjekte in allen kb
			Collection<SystemObject> objekte = new LinkedList<SystemObject>();
			
			for (ConfigurationArea lauf: dataModel.getAllConfigurationAreas().values())
			{	
				for (SystemObject so : lauf.getObjects(typeSpez, ObjectTimeSpecification.valid()))
				{
					ConfigurationObject ko = (ConfigurationObject)so;
		    	if (!(ko.getNotValidSince() >= ko.getConfigurationArea().getModifiableVersion()))
		    	// Objekt wird in zuknftiger Version nicht ungltig	
		    	{		
		    		objekte.add(so);
		    	}  
				}
				
				for (SystemObject so : lauf.getNewObjects())
				{
					for (SystemObjectType typ: typeSpez)
					{
						if (so.isOfType(typ))
						{
							objekte.add(so);
						}
					}
				}
			}
			
			// Liste der Objekte, die nicht in der ersten Ebene der Hierarchie sein knnen
			// => Objekte des entsprechenden Typs, die ber einen APKP refernziert werden
			Collection<SystemObject> nichtWurzelObjekte = new HashSet<SystemObject>();
			
			// Liste der Objekte, die fr diesen KB als Wurzelobjekte in Frage kommen
			Collection<SystemObject> objekteInKB = new LinkedList<SystemObject>();
			
			for (SystemObject so: objekte)
			{	
				if (so.getConfigurationArea().equals(kb))
				{
					objekteInKB.add(so);
				}
				
				NonMutableSet mengeAP = ((ConfigurationObject)so).getNonMutableSet( "AnschlussPunkteGert" );
				
				if (mengeAP != null)
				{	
	        for (SystemObject ap: mengeAP.getElementsInModifiableVersion())
	        {
	          ConfigurationObject koAP = (ConfigurationObject)ap;
	          if (koAP == null)
	          {
	          	_debug.warning("null-Referenz in Menge " + mengeAP.getName() + " von " + so.getPid());
	          	continue;
	          }
	          NonMutableSet mengeKP = koAP.getNonMutableSet( "AnschlussPunkteKommunikationsPartner" );
	          if (mengeKP != null)
	          {
	            for (SystemObject apKP: mengeKP.getElementsInModifiableVersion())
	            {
	            	if (apKP == null)
	  	          {
	  	          	_debug.warning("null-Referenz in Menge " + mengeKP.getName() + " von " + koAP.getPid());
	  	          	continue;
	  	          }
	              Data data = ((ConfigSystemObject)apKP).getConfigurationData( dataModel
	                  .getAttributeGroup( "atg.anschlussPunktKommunikationsPartner"), lookup );
	              if (data != null)
	              {	
		              SystemObject ref = data.getReferenceValue( "KommunikationsPartner" ).getSystemObject();
		              if (ref != null && ref.getConfigurationArea().equals(kb))
		              {	
		              	nichtWurzelObjekte.add(ref);
		              }
	              }  
	            }  
	          }
	        }
				}	
			}
									
			for (SystemObject so: objekteInKB)
			{	
				if (nichtWurzelObjekte.contains(so))
				//if (nichtWurzelObjekteGlobal.contains(so))
				{
					// Objekte, die hier enthalten sind, werden als Kommunikationspartner referenziert 
					// und knnen daher nicht auf oberster Ebene liegen
					continue;
				}
				
				IHierarchieObjektTyp hot = null;
				if (so.isOfType("typ.uz"))
				{
					hot = HOT_TLS.UZ;
				}
				else if (so.isOfType("typ.vrz"))
				{
					hot = HOT_TLS.VRZ;
				}
				else if (so.isOfType("typ.viz"))
				{
					hot = HOT_TLS.VIZ;
				}
				
				if (hot != null)
				{
// Zum provisorischen Anzeigen der kompletten Konfiguration KoKo					
//					if (so.getPid().equals("vrz.neueVrz.Leverkusen") && hot.equals(HOT_TLS.VRZ))
//					{
//						hot = HOT_TLS.VIZ;
//					}
					
				  HierarchieObjekt kind  = new HierarchieObjekt(hot, new SystemObject[] { so }, getTypSortierFolge(hot));
				  _hierarchie.objektHinzufuegen(objekt, kind);
				  bestimmeUntergeordneteHOs(kind, lookup);
				}
			}		
    }		
    else if (objekt instanceof HierarchieObjekt)
    {
    	bestimmeUntergeordneteHOs((HierarchieObjekt)objekt, lookup);
    }
	  else
	  {
	  	_debug.warning("bergebenes Objekt " + objekt + " ist von unzulssigem Typ");
	  
		}
	}	
	
  private void bestimmeUntergeordneteHOs(HierarchieObjekt ho, ObjectLookup lookup)	
  {	
  	ConfigDataModel dataModel = _hierarchie.getDataModel();
    IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp(); 
    
    if (hot.equals(HOT_TLS.VIZ) ||
        hot.equals(HOT_TLS.VRZ) ||
        hot.equals(HOT_TLS.UZ) ||
        hot.equals(HOT_TLS.KRI))
    {
      ConfigurationObject koVater = (ConfigurationObject)ho.getSystemObjekt(0);
      NonMutableSet mengeAP = koVater.getNonMutableSet( "AnschlussPunkteGert" );
     	      
      if (mengeAP != null)
      {
      	MultiStatus meldung = new MultiStatus(Activator.PLUGIN_ID, IStatus.OK,
						"Es gibt als Hierarchieobjekte unterhalb von " + ho + " ungltige Objekte", null);
      	
      	IHierarchieObjektTyp[] hotSortierFolge = getUntergeordneteHOTs(ho);
      	List<IHierarchieObjektTyp> zulaessigeHOT = Arrays.asList(hotSortierFolge);
      	
        for (SystemObject ap: mengeAP.getElementsInModifiableVersion())
        {
        	if (ap == null)
          {
          	_debug.warning("null-Referenz in Menge " + mengeAP.getName() + " von " + koVater.getPid());
          	continue;
          }
        	
        	if (ap.isOfType("typ.anschlussPunkt"))
        	{	
	          ConfigurationObject koAP = (ConfigurationObject)ap;
	          NonMutableSet mengeKP = koAP.getNonMutableSet( "AnschlussPunkteKommunikationsPartner" );
	          HierarchieObjekt ib = null;
	          	          
	          if (mengeKP == null || mengeKP.getElementsInModifiableVersion().size() == 0)
	          //AP ohne KP-Menge oder mit leerer KP-Menge wird als Inselbus angesehen	
	  	      {
	          	if (zulaessigeHOT.contains(HOT_TLS.Inselbus))
          		{
		          	ib = new HierarchieObjekt(HOT_TLS.Inselbus, new SystemObject[] { ap }, getTypSortierFolge(HOT_TLS.Inselbus));
	              _hierarchie.objektHinzufuegen(ho, ib); 
          		}
	          	else
	          	{
	          		meldung.add(new Status(Status.WARNING, Activator.PLUGIN_ID,	HOT_TLS.Inselbus + " ist als HO-Typ unterhalb von " +	
	          				ho.getHierarchieObjektTyp() +	" nicht zulssig - " + ap.getPid() + " ist als untergeordnetes Objekt von " +	
	          				ho.getSystemObjekt(0).getPid() + " nicht akzeptabel und wird in dieser Hierarchie ignoriert"));
          		}
	  	      }
	          else if (mengeKP != null)
	          {
	          	List<SystemObject> listeKP = mengeKP.getElementsInModifiableVersion();
	          	
	            for (SystemObject apKP: listeKP)
	            {
	            	if (apKP == null)
	  	          {
	  	          	_debug.warning("null-Referenz in Menge " + mengeKP.getName() + " von " + koAP.getPid());
	  	          	continue;
	  	          }
	            	
	            	if (apKP.isOfType("typ.anschlussPunktKommunikationsPartner"))
	            	{	
		              Data data = ((ConfigSystemObject)apKP).getConfigurationData( dataModel
		                  .getAttributeGroup( "atg.anschlussPunktKommunikationsPartner" ), lookup );
		              if (data != null)
		              {	
		              	SystemObject ref = data.getReferenceValue( "KommunikationsPartner" ).getSystemObject();
		              
			              if (ref != null)
			              {	
				              if ( ref.isOfType( "typ.steuerModul" ) )
				              {
				                // AP ist Inselbus mit Partner SM
				              	if (ib == null)
				              	{
				              		if (zulaessigeHOT.contains(HOT_TLS.Inselbus))
				              		{
					              		ib = new HierarchieObjekt(HOT_TLS.Inselbus, new SystemObject[] { ap }, getTypSortierFolge(HOT_TLS.Inselbus));
					                  _hierarchie.objektHinzufuegen(ho, ib);
				              		}
				              		else
				              		{
				              			meldung.add(new Status(Status.WARNING, Activator.PLUGIN_ID,	HOT_TLS.Inselbus + " ist als HO-Typ unterhalb von " +	
				    	          				ho.getHierarchieObjektTyp() +	" nicht zulssig - " + ap.getPid() + " ist als untergeordnetes Objekt von " +	
				    	          				ho.getSystemObjekt(0).getPid() + " nicht akzeptabel und wird in dieser Hierarchie ignoriert"));
				              		}
				              	}
				              	if (ib != null)
				              	{	
			  	              	HierarchieObjekt kind  = new HierarchieObjekt(HOT_TLS.SM, new SystemObject[] { ref, apKP }, getTypSortierFolge(HOT_TLS.SM));
					                _hierarchie.objektHinzufuegen(ib, kind);
					                bestimmeUntergeordneteHOs(kind, lookup);
				              	}  
				              }
			  	            else
			  	            {	
			  	            	SystemObject[] objektArray = null;
					              IHierarchieObjektTyp hierachieObjektTyp = null;
					              
				  	            if ( ref.isOfType( "typ.kri" ) )
			 		              {
					                // VIZ, VRZ oder UZ mit Partner KRI
					                objektArray = new SystemObject[] { ref, ap, apKP };
					                hierachieObjektTyp = HOT_TLS.KRI;
					              }
					              else if ( ref.isOfType( "typ.uz" ) )
					              {
					                // VIZ, VRZ t Partner UZ
					                objektArray = new SystemObject[] { ref, ap, apKP };
					                hierachieObjektTyp = HOT_TLS.UZ;
					              }
					              else if ( ref.isOfType( "typ.vrz" ) )
					              {
					                  // VIZ mit Partner VRZ
					                  objektArray = new SystemObject[] { ref, ap, apKP };
					                  hierachieObjektTyp = HOT_TLS.VRZ;
					              }
					              
					              if (objektArray != null && hierachieObjektTyp != null)
					              {
				              	  if (zulaessigeHOT.contains(hierachieObjektTyp))
				              	  {	
					                  HierarchieObjekt kind  = new HierarchieObjekt(hierachieObjektTyp, objektArray, getTypSortierFolge(hierachieObjektTyp));
					                  _hierarchie.objektHinzufuegen(ho, kind);
					                  bestimmeUntergeordneteHOs(kind, lookup);
				              	  }
				              	  else
				              	  {
				              	  	meldung.add(new Status(Status.WARNING, Activator.PLUGIN_ID,	hierachieObjektTyp + " ist als HO-Typ unterhalb von " +	
				    	          				ho.getHierarchieObjektTyp() +	" nicht zulssig - " + objektArray[0].getPid() + " ist als untergeordnetes Objekt von " +	
				    	          				ho.getSystemObjekt(0).getPid() + " nicht akzeptabel und wird in dieser Hierarchie ignoriert"));
				              	  }
					              }
			  	            }  
		  	            }
		              }  
	              }
	            	else
	            	{
	            		_debug.warning(apKP.getPid() + " ist nicht vom Typ typ.anschlussPunktKommunikationsPartner und wird ignoriert");
	            	}
	            }
	          }
	          else
	  	      {
	  	      	_debug.warning("Menge AnschlussPunkteKommunikationsPartner ist am Systemobjekt " + koAP.getPid() + " nicht vorhanden");
	  	      }
	        }
        	else
        	{
        		_debug.warning(ap.getPid() + " ist nicht vom Typ typ.anschlussPunkt und wird ignoriert");
        	}
        	
	        if (meldung.getChildren().length > 0)
	        {
	        	MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(meldung, System.currentTimeMillis(), false);
	        }
        }  
      }
      else
      {
      	_debug.warning("Menge AnschlussPunkteGert ist am Systemobjekt " + koVater.getPid() + " nicht vorhanden");
      }
    }
    else if (hot.equals(HOT_TLS.SM))
    {
      // SM mit Partner EAK
      ConfigurationObject ko = (ConfigurationObject)ho.getSystemObjekt(0);
      NonMutableSet mengeEAK = ko.getNonMutableSet( "Eak" );

      if (mengeEAK != null)
      {	
      	List<SystemObject> listeEAK = mengeEAK.getElementsInModifiableVersion();

      	for (SystemObject eak : listeEAK)
        {
      		if (eak == null)
          {
          	_debug.warning("null-Referenz in Menge " + mengeEAK.getName() + " von " + ko.getPid());
          	continue;
          }
      		
      		if (eak.isOfType("typ.eak"))
      		{	
	        	HierarchieObjekt kind  = new HierarchieObjekt(HOT_TLS.EAK, new SystemObject[] { eak }, getTypSortierFolge(HOT_TLS.EAK));
	        	_hierarchie.objektHinzufuegen(ho, kind);
	          bestimmeUntergeordneteHOs(kind, lookup);
      		}
      		else
        	{
        		_debug.warning(eak.getPid() + " ist nicht vom Typ typ.eak und wird ignoriert");
        	}
        }
      }
      else
      {
      	_debug.warning("Menge Eak ist am Systemobjekt " + ko.getPid() + " nicht vorhanden");
      }
    }
    else if (hot.equals(HOT_TLS.EAK))
    {
      // EAK mit Partner DE
      ConfigurationObject ko = (ConfigurationObject)ho.getSystemObjekt(0);
      NonMutableSet mengeDE = ko.getNonMutableSet( "De" );

      if (mengeDE != null)
      {	
      	List<SystemObject> listeDE = mengeDE.getElementsInModifiableVersion();

        for (SystemObject de: listeDE)
        {
        	if (de == null)
          {
          	_debug.warning("null-Referenz in Menge " + mengeDE.getName() + " von " + ko.getPid());
          	continue;
          }
        	
        	if (de.isOfType("typ.de"))
        	{	
//	          Data data = ((ConfigSystemObject)de).getConfigurationData( dataModel.getAttributeGroup( "atg.de" ), lookup );
//	          if ( data != null && data.getUnscaledValue( "DEKanal" ).intValue() == 255 ) // Sammelkanal herausfiltern
//	          {
//	            continue;
//	          }  
	          
	          DeTypenManager deTypenManager = DatenSpeicher.getInstanz().getDeTypenManager();
	          IHierarchieObjektTyp hotDE = deTypenManager.getDeUnterTyp(de.getType().getPid());
	          	
	          if (hotDE != null)
	          {	
	            HierarchieObjekt kind  = new HierarchieObjekt(hotDE, new SystemObject[] { de }, getTypSortierFolge(hotDE));
	            _hierarchie.objektHinzufuegen(ho, kind);
	            // Rekursionsende
	          }
	          else
	          {
	          	_debug.warning("Es gibt keinen Hierarchieobjekttyp zum Typ " + de.getType().getPid() + 
	          			" des Systemobjekts " + de.getPid());
	          }
        	}
        	else
        	{
        		_debug.warning(de.getPid() + " ist nicht vom Typ typ.de und wird ignoriert");
        	}
        }
      }
      else
      {
      	_debug.warning("Menge De ist am Systemobjekt " + ko.getPid() + " nicht vorhanden");
      }
    }
    else
    {
    	_debug.warning("bergebenes HO " + ho + " ist vom unzulssigen (HO-)Typ + " +
    			ho.getHierarchieObjektTyp());
    }
	}		
	
 
	 private SystemObject erzeugeSystemObjekt(String pid, String typ, ConfigurationArea kb,
	  		List<ObjectSet> mengen, MultiStatus status, List<SystemObject> undoListe)
	  {
		  SystemObject result = null;
		  ConfigurationObjectType koTyp = (ConfigurationObjectType)kb.getDataModel().getType(typ);
		  if (koTyp != null)
		  {	
				try
				{
					result = kb.createConfigurationObject(koTyp, pid, "", mengen);
						
					undoListe.add(0, result);
					MultiStatus meldung = new MultiStatus(Activator.PLUGIN_ID, IStatus.OK,
							"Systemobjekt " +	pid + " vom Typ " + typ + " erzeugt", null);
					
					if (mengen != null && !mengen.isEmpty())
					{
						for (ObjectSet set: mengen)
						{
							meldung.add(new Status(Status.OK, Activator.PLUGIN_ID,	"Menge " + set.getName() + 
									" dem Systemobjekt " + pid + " vom Typ " + typ + " zugewiesen"));
						}
					}
					
					status.add(meldung);
				}
				catch (ConfigurationChangeException e)
				{
					status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, 
							"Systemobjekt " + pid + " vom Typ " + typ + " konnte nicht erzeugt werden", e));
				}
		  }
		  else
		  {
		  	status.add(new Status(Status.ERROR, Activator.PLUGIN_ID,	"Fr " + typ + 
						" kann kein Typ im Datenmodell bestimmt werden", null));
		  }
			return result;
	  }
	
	private List<ObjectSet> erzeugeMengenFuerTyp(IHierarchieObjektTyp hot, String typ, 
  		ConfigurationArea kb, MultiStatus status, List<SystemObject> undoListe)
  {
  	List<ObjectSet> result = new Vector<ObjectSet>();
		
		for (String mengenName: hot.getTypInfo(typ).getMengen())
		{
			MengeInfo info = hot.getTypInfo(typ).getMengeInfo(mengenName);
			String mengenTyp = info.getTyp();
			ObjectSetType setTyp = kb.getDataModel().getObjectSetType(mengenTyp);
			ObjectSet menge;
			try
			{
				menge = (ObjectSet)kb.createConfigurationObject(setTyp, "", mengenName, null);
				
				undoListe.add(0, menge);
				status.add(new Status(Status.OK, Activator.PLUGIN_ID, 
						"Menge " + mengenName + " vom Typ " + setTyp.getPid() + " erzeugt"));
			}
			catch (ConfigurationChangeException e)
			{
				status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Fehler beim Erzeugen der Menge " 
						+ mengenName + " vom Typ " + setTyp.getName() + " erzeugt", e));
				return result;
			}
			
			result.add(menge);
		}
		
		return result;
  }
	
	 private ObjectSet erzeugeMengeAmObjekt(ConfigurationObject ko,	String mengenTyp, 
			 String mengenName, MultiStatus status)
	  {
	  	ObjectSet result = null;
	  	
			ObjectSetType setTyp = DatenSpeicher.getInstanz().holeConfigDataModel().getObjectSetType(mengenTyp);
			try
			{
				result = (ObjectSet)ko.getConfigurationArea().createConfigurationObject(setTyp, null, mengenName, null);
				ko.addSet(result);
				
				status.add(new Status(Status.OK, Activator.PLUGIN_ID,	"Menge " + mengenName+ 
						" vom Typ " +  setTyp.getPid() +	" fr Systemobjekt " + ko.getPid() + " erzeugt"));
				
			}
			catch (ConfigurationChangeException e)
			{
				status.add(new Status(Status.ERROR, Activator.PLUGIN_ID,	"Menge " + mengenName + " vom Typ " +	
						setTyp.getPid() +	" konnte fr das Systemobjekt " +	ko.getPid() + " nicht erstellt werden", e));
			}
			
			return result;
	  }
	
	private void behandleMengenZuordung(HierarchieObjekt vater, IHierarchieObjektTyp hotNeuesHO, 
			SystemObject so, SystemObject[] neueObjekte, MultiStatus status, Map<ObjectSet, SystemObject> undoMengenListe, List<SystemObject> undoObjektListe)
	{
		if (hotNeuesHO.equals(HOT_TLS.VRZ) ||
		     hotNeuesHO.equals(HOT_TLS.UZ) ||
		     hotNeuesHO.equals(HOT_TLS.KRI) ||
		     hotNeuesHO.equals(HOT_TLS.Inselbus) ||
		     hotNeuesHO.equals(HOT_TLS.SM))
		{
			if (so.isOfType("typ.anschlussPunkt"))
			{
				ConfigurationObject ko = (ConfigurationObject)vater.getSystemObjekt(0);
				ObjectSet menge = ko.getNonMutableSet("AnschlussPunkteGert");
				if (menge == null)
				{
					menge = erzeugeMengeAmObjekt(ko, "menge.anschlussPunkteGert", "AnschlussPunkteGert", status);
				}
				if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
	  		{
					try
					{
						menge.add(so);
						
						undoMengenListe.put(menge, so);
						status.add(new Status(Status.OK, Activator.PLUGIN_ID,	"Systemobjekt " + so.getPid() + 
								" der Menge AnschlussPunkteGert des Systemobjekts " +	ko.getPid() + " hinzugefgt"));		
					}
					catch (ConfigurationChangeException e)
					{
						status.add(new Status(Status.ERROR, Activator.PLUGIN_ID,	"Systemobjekt " + so.getPid() + 
								" konnte nicht der Menge AnschlussPunkteGert des Systemobjekts " +	ko.getPid() + " hinzugefgt werden", e));
					}
	  		}
			}
			else if (so.isOfType("typ.anschlussPunktKommunikationsPartner"))
			{
				ConfigurationObject ko = hotNeuesHO.equals(HOT_TLS.SM) ?
						(ConfigurationObject)vater.getSystemObjekt(0) : 
						(ConfigurationObject)neueObjekte[1]; // Benutze ap-Eintrag der neuen Objekte!
									
				AttributeGroup atg = _hierarchie.getDataModel().getAttributeGroup("atg.anschlussPunktKommunikationsPartner");
				Data data = DataFactory.createData(atg);
		    // Default und undefiniert Werte setzen
		    data.setToDefault();
		    // Setze KP
		    data.getReferenceValue("KommunikationsPartner").setSystemObject(neueObjekte[0]);
		    try
		    {
		    	so.setConfigurationData(atg, data);
					status.add(new Status(Status.OK, Activator.PLUGIN_ID,	"Systemobjekt " + neueObjekte[0].getPid() + 
							" als KommunikationsPartner in APKP " +	ko.getPid() + " gesetzt"));		
		    }
				catch (ConfigurationChangeException e)
				{
					status.add(new Status(Status.ERROR, Activator.PLUGIN_ID,	"Systemobjekt " + neueObjekte[0].getPid() + 
							" konnt nicht als KommunikationsPartner in APKP " +	ko.getPid() + " gesetzt werden", e));		
				}
				if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
	  		{
					ObjectSet menge = ko.getNonMutableSet("AnschlussPunkteKommunikationsPartner");
					if (menge == null)
					{
						menge = erzeugeMengeAmObjekt(ko, "menge.anschlussPunkteKommunikationsPartner", "AnschlussPunkteKommunikationsPartner", 
								status);
					}
					if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
		  		{
						try
						{
							menge.add(so);
							
							undoMengenListe.put(menge, so);
							status.add(new Status(Status.OK, Activator.PLUGIN_ID,	"Systemobjekt " + so.getPid() + 
									" der Menge AnschlussPunkteKommunikationsPartner des Systemobjekts " +	ko.getPid() + " hinzugefgt"));		
						}
						catch (ConfigurationChangeException e)
						{
							status.add(new Status(Status.ERROR, Activator.PLUGIN_ID,	"Systemobjekt " + so.getPid() + 
									" konnte nicht der Menge AnschlussPunkteKommunikationsPartner des Systemobjekts " +	ko.getPid() + " hinzugefgt werden", e));
						}
		  		}
	  		}	
			}	
		}
		else if (hotNeuesHO.equals(HOT_TLS.EAK))
		{
			if (so.isOfType("typ.eak"))
			{
				ConfigurationObject ko = (ConfigurationObject)vater.getSystemObjekt(0);
				ObjectSet menge = ko.getNonMutableSet("Eak");
				if (menge == null)
				{
					menge = erzeugeMengeAmObjekt(ko, "menge.eak", "Eak", status);
				}
				if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
	  		{
					try
					{
						menge.add(so);
						
						undoMengenListe.put(menge, so);
						status.add(new Status(Status.OK, Activator.PLUGIN_ID,	"Systemobjekt " + so.getPid() + 
								" der Menge Eak des Systemobjekts " +	ko.getPid() + " hinzugefgt"));		
					}
					catch (ConfigurationChangeException e)
					{
						status.add(new Status(Status.ERROR, Activator.PLUGIN_ID,	"Systemobjekt " + so.getPid() + 
								" konnte nicht der Menge Eak des Systemobjekts " +	ko.getPid() + " hinzugefgt werden", e));
					}
	  		}
			}
		}
	  else if (DatenSpeicher.getInstanz().getDeTypenManager().hasHOT(hotNeuesHO))
		{
			if (so.isOfType("typ.de"))
			{
				ConfigurationObject ko = (ConfigurationObject)vater.getSystemObjekt(0);
				ObjectSet menge = ko.getNonMutableSet("De");
				if (menge == null)
				{
					menge = erzeugeMengeAmObjekt(ko, "menge.de", "De", status);
				}
				if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
	  		{
					try
					{
						menge.add(so);
						
						undoMengenListe.put(menge, so);
						status.add(new Status(Status.OK, Activator.PLUGIN_ID,	"Systemobjekt " + so.getPid() + 
								" der Menge De des Systemobjekts " +	ko.getPid() + " hinzugefgt"));		
					}
					catch (ConfigurationChangeException e)
					{
						status.add(new Status(Status.ERROR, Activator.PLUGIN_ID,	"Systemobjekt " + so.getPid() + 
								" konnte nicht der Menge De des Systemobjekts " +	ko.getPid() + " hinzugefgt werden", e));
					}
	  		}
			}
		}
	}
	
	public HOMitStatus erzeugeNeuesHO(ConfigurationArea kb, Object vater, String pid[], IHierarchieObjektTyp hot,
			boolean pruefePid)
	{
		HOMitStatus result = erzeugeNeuesHO(kb, vater, pid, hot, pruefePid, true);
		return result;
	}
	
	private HOMitStatus erzeugeNeuesHO(ConfigurationArea kb, Object vater, String pid[], IHierarchieObjektTyp hot, boolean pruefePid, 
			boolean automatischeObjekte)
	{
		MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK,
				"Erzeuge neues Hierarchieobjekt vom Typ " + hot, null);
		
		HOMitStatus result = erzeugeNeuesHOIntern(kb, vater, pid, hot, pruefePid);
		HierarchieObjekt neu = result.getHO();
		
		status.merge(result.getStatus());
		
		if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
		{
			if (automatischeObjekte)
			{
				if (HOT_TLS.SM.equals(hot)) // SM-Typ
				{
					status.add(erzeugeEak254Automatisch(neu, kb));
				}
				else if (hot instanceof DeUnterTyp) // DE-Typ
				{
					Assert.isTrue(vater instanceof HierarchieObjekt);
					behandleErzeugenDEMitDEKanal255((HierarchieObjekt)vater, kb, hot, pruefePid, status);
				}
			}
		}
		
		return new HOMitStatus(neu, status);
	}
	
	private Collection<HierarchieObjekt> getPufferDe(HierarchieObjekt ho, IHierarchieObjektTyp hot)
	{
		Assert.isLegal(HOT_TLS.EAK.equals(ho.getHierarchieObjektTyp()));
		Assert.isLegal(hot instanceof DeUnterTyp);
		
		Collection<HierarchieObjekt> result = new Vector<HierarchieObjekt>();
	
		int fg = ((DeUnterTyp)hot).getFg();
		ConfigDataModel dataModel = _hierarchie.getDataModel();
		AttributeGroup atg = dataModel.getAttributeGroup("atg.de");
		ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
		HierarchieObjekt hoSM = (HierarchieObjekt)_hierarchie.getUebergeordnetesObjekt(ho);
		
		for (HierarchieObjekt hoEAK: _hierarchie.getUntergeordneteObjekte(hoSM))
		{	
			for (HierarchieObjekt hoDE: _hierarchie.getUntergeordneteObjekte(hoEAK))
			{
				IHierarchieObjektTyp deHot = hoDE.getHierarchieObjektTyp();
				
				if (deHot instanceof DeUnterTyp && fg == ((DeUnterTyp)deHot).getFg())
				{
					SystemObject de = hoDE.getSystemObjekt(0);
					Assert.isTrue(de.isOfType("typ.de"));
					
					Data data = ((ConfigSystemObject)de).getConfigurationData(atg, lookup);
					if (data != null)
					{
						int deKanal = data.getItem("DEKanal").asUnscaledValue().intValue();
						if (deKanal == 255)
						{
							result.add(hoDE);
						}
					}	
				}
			}	
		}	
		
		return result;
	}
	
//	private boolean isPufferDe(HierarchieObjekt ho)
//	{
//		boolean result = false;
//		Assert.isTrue(DatenSpeicher.getInstanz().getDeTypenManager().hasHOT(ho.getHierarchieObjektTyp()));
//		
//		ConfigDataModel dataModel = _hierarchie.getDataModel();
//		AttributeGroup atg = dataModel.getAttributeGroup("atg.de");
//		ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
//		
//		SystemObject de = ho.getSystemObjekt(0);
//		Assert.isTrue(de.isOfType("typ.de"));
//		
//		Data data = ((ConfigSystemObject)de).getConfigurationData(atg, lookup);
//		if (data != null)
//		{
//			int deKanal = data.getItem("DEKanal").asUnscaledValue().intValue();
//			if (deKanal == 255)
//			{
//				result = true;
//			}
//		}	
//		
//		return result;
//	}
	
	private IStatus erzeugeDeAutomatisch(HierarchieObjekt vater, ConfigurationArea kb, IHierarchieObjektTyp hot, 
		int deKanal, boolean pruefePid, boolean automatischeObjekte)
	{
		Assert.isLegal(HOT_TLS.EAK.equals(vater.getHierarchieObjektTyp()));
		Assert.isLegal(hot instanceof DeUnterTyp);
		
		DeUnterTyp deHOT = ((DeUnterTyp)hot).getFrgRelevantenHOT();
				
		MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK,
				"Erzeuge neues Hierarchieobjekt vom Typ " + deHOT + " mit DE-Kanal " + deKanal, null);
		
		IBildungsRegelnTLS bildungsRegeln = DatenSpeicher.getInstanz().getBildungsRegeln();
		bildungsRegeln.setZusatzInfo(deHOT, "" + deKanal);
		
		try
		{
			String[] pid = bildungsRegeln.getPids(deHOT, vater);
		
			HOMitStatus result = erzeugeNeuesHO(kb, vater, pid, deHOT, pruefePid, automatischeObjekte);
			IStatus meldung = result.getStatus(); 
			status.merge(meldung);
			
			if (!meldung.matches(IStatus.ERROR | IStatus.CANCEL))
			{
				SystemObject de = result.getHO().getSystemObjekt(0);
				
				// Setze Name
				String name = de.getPid();
				try
				{
					de.setName(name);
					status.add(new Status(Status.OK, Activator.PLUGIN_ID,	"Setze Name \"" + name + 
							"\" fr Systemobjekt " +	de.getPid()));
				}
				catch (ConfigurationChangeException e)
				{
					status.add(new Status(Status.WARNING, Activator.PLUGIN_ID,	"Name \"" + name + 
							"\" fr Systemobjekt " +	de.getPid() + " konnte nicht gesetzt werden", e));
				}
				
				// Erzeuge Datensatz der atg.de
				AttributeGroup atg = _hierarchie.getDataModel().getAttributeGroup("atg.de");
				Data data = DataFactory.createData(atg);
		    // Default und undefiniert Werte setzen
		    data.setToDefault();
		    data.getTextValue("Bezeichnung").setText(deKanal == 255 ? "Sammelkanal" : "DE " + deKanal);
		    data.getTextValue("Cluster").setText("Nein");
		    data.getUnscaledValue("EAKanal").set(deKanal);
		    data.getUnscaledValue("DEKanal").set(deKanal);
		    data.getTextValue("UmsetzungsModul").setText("");
		    
		    try
				{
					de.setConfigurationData(atg, data);
					status.add(new Status(Status.OK, Activator.PLUGIN_ID,	"Daten der ATG " + atg.getPid() + 
							" mit DEKanal=" + deKanal + " fr " + de.getPid() + " bernommen"));
				}
				catch (ConfigurationChangeException e)
				{
					status.add(new Status(Status.ERROR, Activator.PLUGIN_ID,	"Daten der ATG " + atg.getPid() + 
							" mit DEKanal=" + deKanal + " fr " + de.getPid() + " konnten nicht bernommen werden"));
				}
			}
		}
		finally
		{
			bildungsRegeln.resetZusatzInfo(deHOT);
		}
		
		return status;
	}
	
	private IStatus erzeugeEak254Automatisch(HierarchieObjekt vater, ConfigurationArea kb)
	{
		Assert.isTrue(HOT_TLS.SM.equals(vater.getHierarchieObjektTyp()));
				
		MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK,
				"Erzeuge neues Hierarchieobjekt vom Typ " + HOT_TLS.EAK + " fr die FG 254", null);
		
		IBildungsRegelnTLS bildungsRegeln = DatenSpeicher.getInstanz().getBildungsRegeln();
		bildungsRegeln.setZusatzInfo(HOT_TLS.EAK, "254");
		
		try
		{
			String[] pid = bildungsRegeln.getPids(HOT_TLS.EAK, vater);
					
			HOMitStatus result = erzeugeNeuesHO(kb, vater, pid, HOT_TLS.EAK, false, false);
			IStatus meldung = result.getStatus(); 
			status.merge(meldung);
			
			if (!meldung.matches(IStatus.ERROR | IStatus.CANCEL))
			{
				HierarchieObjekt neu = result.getHO();
				SystemObject eak = neu.getSystemObjekt(0);
				
				// Setze Name
				String name = eak.getPid();
				try
				{
					eak.setName(name);
					status.add(new Status(Status.OK, Activator.PLUGIN_ID,	"Setze Name \"" + name + 
							"\" fr Systemobjekt " +	eak.getPid()));
				}
				catch (ConfigurationChangeException e)
				{
					status.add(new Status(Status.WARNING, Activator.PLUGIN_ID,	"Name \"" + name + 
							"\" fr Systemobjekt " +	eak.getPid() + " konnte nicht gesetzt werden", e));
				}
				
				// Erzeuge Datensatz der atg.eak
				AttributeGroup atg = _hierarchie.getDataModel().getAttributeGroup("atg.eak");
				Data data = DataFactory.createData(atg);
		    // Default und undefiniert Werte setzen
		    data.setToDefault();
		    data.getTextValue("Bezeichnung").setText("EAK FG 254");
		    data.getTextValue("OSI2Adresse").setText("254");
		    data.getTextValue("UmsetzungsModul").setText("");
		    
		    try
				{
					eak.setConfigurationData(atg, data);
					status.add(new Status(Status.OK, Activator.PLUGIN_ID,	"Daten der ATG " + atg.getPid() + 
							" mit OSI2Adresse=254 fr " + eak.getPid() + " bernommen"));
				}
				catch (ConfigurationChangeException e)
				{
					status.add(new Status(Status.ERROR, Activator.PLUGIN_ID,	"Daten der ATG " + atg.getPid() + 
							" mit OSI2Adresse=254 fr " + eak.getPid() + " konnten nicht bernommen werden"));
				}
				
				if (!meldung.matches(IStatus.ERROR | IStatus.CANCEL))
				{
					IHierarchieObjektTyp deTyp = DatenSpeicher.getInstanz().getDeTypenManager().getDeUnterTyp("typ.deSys");
					if (deTyp != null)
					{	
						status.add(erzeugeDeAutomatisch(neu, kb, deTyp,	0, false, false)); // ohne automat. DE 255
						// status.add(erzeugeDeAutomatisch(neu, deTyp,	0, false, true)); // mit automat. DE 255
					}
					else
					{
						status.add(new Status(Status.ERROR, Activator.PLUGIN_ID,	"DE-Untertyp " + "typ.deSys" +
								" wurde nicht gefunden - DE der FG 254 kann nicht angelegt werden"));
					}
				}
			}
		}
		finally
		{
			bildungsRegeln.resetZusatzInfo(HOT_TLS.EAK);
		}
		
		return status;
	}
	
	private HierarchieObjekt holeEak255(HierarchieObjekt vater, ConfigurationArea kb, MultiStatus status)
	{
		Assert.isTrue(HOT_TLS.SM.equals(vater.getHierarchieObjektTyp()));
		
		IHierarchie hierarchie = getHierarchie();
		
		for (HierarchieObjekt ho: hierarchie.getUntergeordneteObjekte(vater))
		{
			if (HOT_TLS.EAK.equals(ho.getHierarchieObjektTyp()))
			{
				SystemObject eak = ho.getSystemObjekt(0);
				ConfigDataModel dataModel = hierarchie.getDataModel();
				AttributeGroup atg = dataModel.getAttributeGroup("atg.eak");
				ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
				Data data = ((ConfigSystemObject)eak).getConfigurationData(atg, lookup);
				
				if (data != null)
				{
					int osi2Adresse = data.getUnscaledValue("OSI2Adresse").intValue();
					if (osi2Adresse == 255)
					{
						return ho;
					}
				}
			}
		}
		
		MultiStatus meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK,
				"Erzeuge neues Hierarchieobjekt vom Typ " + HOT_TLS.EAK + " fr die FG 255", null);
		
		IBildungsRegelnTLS bildungsRegeln = DatenSpeicher.getInstanz().getBildungsRegeln();
		bildungsRegeln.setZusatzInfo(HOT_TLS.EAK, "255");
		HOMitStatus result = null; 
		
		try
		{
			String[] pid = bildungsRegeln.getPids(HOT_TLS.EAK, vater);
					
			result = erzeugeNeuesHO(kb, vater, pid, HOT_TLS.EAK, false, false);
			meldung.merge(meldung);
			
			if (!meldung.matches(IStatus.ERROR | IStatus.CANCEL))
			{
				HierarchieObjekt neu = result.getHO();
				SystemObject eak = neu.getSystemObjekt(0);
				
				// Setze Name
				String name = eak.getPid();
				try
				{
					eak.setName(name);
					status.add(new Status(Status.OK, Activator.PLUGIN_ID,	"Setze Name \"" + name + 
							"\" fr Systemobjekt " +	eak.getPid()));
				}
				catch (ConfigurationChangeException e)
				{
					status.add(new Status(Status.WARNING, Activator.PLUGIN_ID,	"Name \"" + name + 
							"\" fr Systemobjekt " +	eak.getPid() + " konnte nicht gesetzt werden", e));
				}
				
				// Erzeuge Datensatz der atg.eak
				AttributeGroup atg = _hierarchie.getDataModel().getAttributeGroup("atg.eak");
				Data data = DataFactory.createData(atg);
		    // Default und undefiniert Werte setzen
		    data.setToDefault();
		    data.getTextValue("Bezeichnung").setText("EAK FG 255");
		    data.getTextValue("OSI2Adresse").setText("255");
		    data.getTextValue("UmsetzungsModul").setText("");
		    
		    try
				{
					eak.setConfigurationData(atg, data);
					status.add(new Status(Status.OK, Activator.PLUGIN_ID,	"Daten der ATG " + atg.getPid() + 
							" mit OSI2Adresse=255 fr " + eak.getPid() + " bernommen"));
				}
				catch (ConfigurationChangeException e)
				{
					status.add(new Status(Status.ERROR, Activator.PLUGIN_ID,	"Daten der ATG " + atg.getPid() + 
							" mit OSI2Adresse=255 fr " + eak.getPid() + " konnten nicht bernommen werden"));
				}
			}
		}
		finally
		{
			bildungsRegeln.resetZusatzInfo(HOT_TLS.EAK);
		}
		
		return result != null ? result.getHO() : null;
	}
	
	
	private IStatus loeschePufferDe(HierarchieObjekt ho)
	{
		MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK,
				"Lsche Hierarchieobjekt vom Typ " + ho.getHierarchieObjektTyp() + " mit DE-Kanal 255", null);
		
		status.merge(loescheHO(ho, false));
		
		return status;
	}
	
  private void behandleLoeschenDEMitDEKanal255(HierarchieObjekt ho, IHierarchieObjektTyp hot, 
  		MultiStatus status)
  {
  	Assert.isLegal(HOT_TLS.EAK.equals(ho.getHierarchieObjektTyp()));
  	Assert.isLegal(hot instanceof DeUnterTyp);
  	
  	int fg = ((DeUnterTyp)hot).getFg();
  	int anzKinder = 0;
  	HierarchieObjekt hoSM = (HierarchieObjekt)_hierarchie.getUebergeordnetesObjekt(ho);
  	
  	for (HierarchieObjekt hoEAK: _hierarchie.getUntergeordneteObjekte(hoSM))
  	{	
	  	for (HierarchieObjekt hoDE: _hierarchie.getUntergeordneteObjekte(hoEAK))
	  	{
	  		IHierarchieObjektTyp deHot = hoDE.getHierarchieObjektTyp();
	  		
	  		if (deHot instanceof DeUnterTyp && fg == ((DeUnterTyp)deHot).getFg())
	  		{
	  			anzKinder += 1;
	  		}
	  	}
  	}
  	
	  Collection<HierarchieObjekt> pufferDe = getPufferDe(ho, hot);

	  if (!(anzKinder > pufferDe.size()))
	  {	
	  	
			for (HierarchieObjekt kind: pufferDe)
			{
				status.add(loeschePufferDe(kind));
				
				if (status.matches(IStatus.CANCEL | IStatus.ERROR))
				{	
					return;
				}
			}
			
			IPreferenceStore preferences = Activator.getDefault().getPreferenceStore();
    	boolean selectedEAK255 = preferences.getBoolean(DefsPara.BenutzeEAK255FuerDE255);
	
    	if (selectedEAK255)
    	{
    		// Lsche EAK255, wenn vorhanden und leer
    		HierarchieObjekt eak255 = null;
    			
  			for (HierarchieObjekt lauf: _hierarchie.getUntergeordneteObjekte(hoSM))
  			{
  				if (HOT_TLS.EAK.equals(lauf.getHierarchieObjektTyp()))
  				{
  					SystemObject eak = lauf.getSystemObjekt(0);
  					ConfigDataModel dataModel = _hierarchie.getDataModel();
  					AttributeGroup atg = dataModel.getAttributeGroup("atg.eak");
  					ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
  					Data data = ((ConfigSystemObject)eak).getConfigurationData(atg, lookup);
  					
  					if (data != null)
  					{
  						int osi2Adresse = data.getUnscaledValue("OSI2Adresse").intValue();
  						if (osi2Adresse == 255)
  						{
  							eak255 = lauf;
  							break;
  						}
  					}
  				}
  			}
  			
  			if (eak255 != null)
  			{
  				if (_hierarchie.getUntergeordneteObjekte(eak255).isEmpty())
  				{
  					status.add(loescheHO(eak255, false));
  				}
  			}
    	}
	  }	
	}
  
  private boolean behandleErzeugenDEMitDEKanal255(HierarchieObjekt ho, ConfigurationArea kb,
  		IHierarchieObjektTyp hot,	boolean pruefePid, MultiStatus status)
  {
  	Assert.isTrue(HOT_TLS.EAK.equals(ho.getHierarchieObjektTyp()));
  	
  	Collection<HierarchieObjekt> pufferDe = getPufferDe(ho, hot);
		if (pufferDe.isEmpty()) // Es gibt noch kein Puffer-DE
		{
			IPreferenceStore preferences = Activator.getDefault().getPreferenceStore();
    	boolean selectedEAK255 = preferences.getBoolean(DefsPara.BenutzeEAK255FuerDE255);
	
    	if (selectedEAK255)
    	{
    		Object vater = getHierarchie().getUebergeordnetesObjekt(ho);
    		HierarchieObjekt eak = holeEak255((HierarchieObjekt)vater, kb, status);
    		status.add(erzeugeDeAutomatisch(eak, kb, hot, 255, pruefePid, false));
    	}
    	else
    	{	
    		status.add(erzeugeDeAutomatisch(ho, kb, hot, 255, pruefePid, false));
    	}
  		return true;
		}
		else
		{
			return false;
		}
  }

	
  private HOMitStatus erzeugeNeuesHOIntern(ConfigurationArea kb, Object vater, String pid[], IHierarchieObjektTyp hot, boolean pruefePid)
  {
  	if (!Arrays.asList(getUntergeordneteHOTs(vater)).contains(hot))
  	{
  		throw new IllegalArgumentException("Unzulssiger HOT " + hot + 
  				" fr bergeordnetes Objekt " + vater);
  	}
  	
  	List<SystemObject> undoObjektListe = new LinkedList<SystemObject>();
  	Map<ObjectSet, SystemObject> undoMengenListe = new LinkedHashMap<ObjectSet, SystemObject>();
  	
  	MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK,
				"Erzeuge neues Hierarchieobjekt vom Typ " + hot, null);
  	  	
  	HierarchieObjekt vaterHO = null;
  	SystemObject[] neueObjekte = null;
  	
  	if (vater instanceof ConfigurationArea)
  	{
  		neueObjekte = new SystemObject[1];
  	}
  	else if (vater instanceof HierarchieObjekt)
  	{
  		neueObjekte = new SystemObject[hot.getTypen().size()];
  		vaterHO = (HierarchieObjekt)vater;
  	}
  	else
  	{
  		throw new IllegalArgumentException("bergebenes Objekt " + vater + 
  				" ist von unzulssigem Typ");
  	}
  	
  	HierarchieObjekt neuesHO = null;
  	  	
  	// PID-Eindeutigkeitsprfung
  	if (pruefePid)
  	{	
	  	PruefungenTools pruefungenTools = new PruefungenTools(_hierarchie.getDataModel());
	  	for (int i = 0; i < pid.length; i += 1)
	  	{	
	  		status.add(pruefungenTools.pruefePid(pid[i]));
	  	}
  	}	
  	
  	if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
  	{	
	  	try
	  	{
		  	int index = 0;
		  	
		  	for (String typ: hot.getTypen())
		  	{
		  		List<ObjectSet> mengen = erzeugeMengenFuerTyp(hot, typ, kb, status, undoObjektListe);
		  		if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
		  		{
		  			neueObjekte[index] = erzeugeSystemObjekt(pid[index], typ, kb, mengen, status, undoObjektListe);
		  		}
		  		if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
		  		{
		  			if (vaterHO != null) // Keine Mengenzuordnung auf der 1. Ebene
		    		{
		  				behandleMengenZuordung(vaterHO, hot, neueObjekte[index], neueObjekte, status, 
		  						undoMengenListe, undoObjektListe);
		    		}
		  		}
		  			  		
		  		if ((HOT_TLS.VRZ.equals(hot) || HOT_TLS.UZ.equals(hot)) && !(vater instanceof HierarchieObjekt))
					{
		  			// Nur 1 Durchlauf fr VRZ, UZ auf Ebene 1
		  			break;
					}
		  		
		  		index += 1;
		  	}
	  	}
	  	finally
	  	{
	  		if (status.matches(IStatus.ERROR | IStatus.CANCEL))
		  	{
		  	  // Alle Aktionen rckgngig
		  		MultiStatus meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK,
		  				"Rollback Erzeugen Hierarchieobjekt", null);
		  		meldung.add(new Status(Status.INFO, Activator.PLUGIN_ID, "Rollback Erzeugen Hierarchieobjekt", null));
		  		
		  		for (ObjectSet menge: undoMengenListe.keySet())
		  		{
		  			SystemObject so = undoMengenListe.get(menge);
		  			try
		  			{
		  				menge.remove(so);
		  				meldung.add(new Status(Status.INFO, Activator.PLUGIN_ID, "Systemobjekt " + so.getPid() + 
		  						" aus der Menge " +	menge.getName() +	" vom Typ " + menge.getType().getPid() + 
		  						" entfernt", null));
		  			}
		  			catch (ConfigurationChangeException e)
		  			{
		  				meldung.add(new Status(Status.WARNING, Activator.PLUGIN_ID, "Systemobjekt " + so.getPid() + " " +
		  						"konnte nicht aus der Menge " + menge.getName() +	" vom Typ " + menge.getType().getPid() +
		  						" entfernt werden", e));
		  			}
		  			catch (RuntimeException e)
		  			{
		  				meldung.add(new Status(Status.WARNING, Activator.PLUGIN_ID, "Systemobjekt " + so.getPid() + " " +
		  						"konnte nicht aus der Menge " + menge.getName() +	" vom Typ " + menge.getType().getPid() +
		  						" entfernt werden", e));
		  			}
		  		}
		  		for (SystemObject so: undoObjektListe)
		  		{
		  			try
		  			{
		  				so.invalidate();
		  				meldung.add(new Status(Status.INFO, Activator.PLUGIN_ID, "Systemobjekt " + so.getPidOrNameOrId() + 
		  						" vom Typ " +	so.getType().getPid() + " gelscht", null));
		  			}
		  			catch (ConfigurationChangeException e)
		  			{
		  				meldung.add(new Status(Status.WARNING, Activator.PLUGIN_ID, "Systemobjekt " + so.getPidOrNameOrId() + 
		  						" vom Typ " + so.getType().getPid() +	" konnte nicht gelscht werden", e));
		  			}
		  			catch (RuntimeException e)
		  			{
		  				meldung.add(new Status(Status.WARNING, Activator.PLUGIN_ID, "Systemobjekt " + so.getPidOrNameOrId() + 
		  						" vom Typ " + so.getType().getPid() +	" konnte nicht gelscht werden", e));
		  			}
		  		}
		  		status.add(meldung);
		  	}
	  	}	
	  	
	  	if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
			{
	  		IHierarchieObjektTyp[] sortierFolge = getTypSortierFolge(hot);
				neuesHO = new HierarchieObjekt(hot, neueObjekte, sortierFolge);
				getHierarchie().objektHinzufuegen(vater, neuesHO);
			}
  	}	
  	
  	return new HOMitStatus(neuesHO, status);
  }
  
 
  private MultiStatus kopiereDaten(HierarchieObjekt srcHO, HierarchieObjekt destHO, 
  		boolean bildungsRegelnFuerName, boolean bildungsRegelnFuerAtg, boolean automatischeObjekte)
  {
  	if (!srcHO.getHierarchieObjektTyp().equals(destHO.getHierarchieObjektTyp()))
  	{
  		throw new IllegalArgumentException("Beide bergebenen Hierarchieobjekte mssen vom gleichen Hierarchieobjekttyp sein");
  	}
  	
		int index = 0;
		MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Kopiere Daten von " + srcHO +	" nach " + destHO, null);
		IBildungsRegeln br = DatenSpeicher.getInstanz().getBildungsRegeln();
		ConfigDataModel dataModel = _hierarchie.getDataModel();
		Object neuerVater = _hierarchie.getUebergeordnetesObjekt(destHO);
		IHierarchieObjektTyp destHOT = destHO.getHierarchieObjektTyp();		
		ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
		boolean apKopiert = false;
		boolean apkpKopiert = false;
		
		for (SystemObject src: srcHO.getSystemObjekte())
		{
			SystemObject dest = destHO.getSystemObjekte().get(index);
			String name;
			if (bildungsRegelnFuerName)
			{
				name = br.getName(dest);
			}
			else
			{
				name = src.getName();
			}
			
			try
			{
				dest.setName(name);
				status.add(new Status(Status.OK, Activator.PLUGIN_ID,	"Setze Name \"" + name + 
						"\" fr Systemobjekt " +	dest.getPid()));
			}
			catch (ConfigurationChangeException e)
			{
				status.add(new Status(Status.ERROR, Activator.PLUGIN_ID,	"Name \"" + name + 
						"\" fr Systemobjekt " +	dest.getPid() + " konnte nicht gesetzt werden", e));
				return status;
			}
			
			for (AttributeGroupUsage atgu: src.getUsedAttributeGroupUsages())
			{
				Data dataSrc = ((ConfigSystemObject)src).getConfigurationData(atgu, lookup);
				Data dataDest = null;
				if (dataSrc != null)
				{
					try
					{
						dataDest = DataFactory.createModifiableCopy(dataSrc, lookup);;
						if (bildungsRegelnFuerAtg)
						{
							dataDest = br.getDatensatz(destHO, dest, atgu.getAttributeGroup(), dataDest, true);
						}
						
						// TLS spezifisch
						if (src.isOfType("typ.anschlussPunkt"))
						{
							if (atgu.getAttributeGroup().getPid().equals("atg.anschlussPunkt"))
							{
								apKopiert = true;
							}
						}	
						if (src.isOfType("typ.anschlussPunktKommunikationsPartner"))
						{
							if (atgu.getAttributeGroup().getPid().equals("atg.anschlussPunktKommunikationsPartner"))
							{
								if (destHO.getHierarchieObjektTyp().equals(HOT_TLS.VRZ) ||
										destHO.getHierarchieObjektTyp().equals(HOT_TLS.UZ) ||
										destHO.getHierarchieObjektTyp().equals(HOT_TLS.KRI) ||
										destHO.getHierarchieObjektTyp().equals(HOT_TLS.SM))
								{	
									dataDest.getReferenceValue( "KommunikationsPartner" ).setSystemObject(destHO.getSystemObjekt(0));
								}	
								apkpKopiert = true;
							}
						}
					  // Ende TLS spezifisch
						
						dest.setConfigurationData(atgu, dataDest);
	  				status.add(new Status(Status.OK, Activator.PLUGIN_ID,	"Kopiere Daten der ATG-Verwendung " +
	  						atgu.getAttributeGroup().getPid() + "/" + atgu.getAspect().getPid() + 
	  						" fr Systemobjekt " +	dest.getPid())); 
					}
					catch (ConfigurationChangeException e)
					{
						status.add(new Status(Status.ERROR, Activator.PLUGIN_ID,	"Daten der ATG-Verwendung " +
								atgu.getAttributeGroup().getPid() + "/" + atgu.getAspect().getPid() + 
								" fr Systemobjekt " +	dest.getPid() + " konnten nicht kopiert werden", e));
						return status;
					}
					if (automatischeObjekte)
					{	
						if (HOT_TLS.EAK.equals(destHOT) && "atg.eak".equals(atgu.getAttributeGroup().getPid()))
	       		{
							behandleErzeugenDEAnEAK254(destHO, dataDest, null, status);
	       		}
					}	
				}
			}
			
			if ((HOT_TLS.VRZ.equals(destHOT) || HOT_TLS.UZ.equals(destHOT)) && !(neuerVater instanceof HierarchieObjekt))
			{
  			// Nur 1 Durchlauf fr VRZ, UZ auf Ebene 1
  			break;
			}
  		
  		index += 1;
		}
		
		if ((HOT_TLS.VRZ.equals(destHOT) || HOT_TLS.UZ.equals(destHOT)) && neuerVater instanceof HierarchieObjekt)
		{
			if (!(_hierarchie.getUebergeordnetesObjekt(srcHO) instanceof HierarchieObjekt))
			{
				// VRZ oder UZ wird evtl. von Ebene 1 auf untere Ebene kopiert => ergnze Daten
				
				if (!apKopiert) // falls ap nicht schon kopiert wurde 
				{	
				  // AP
					SystemObject ap = destHO.getSystemObjekt(1);
				  // ATG
					AttributeGroup atgAP = dataModel.getAttributeGroup("atg.anschlussPunkt");
					Data dataAP = DataFactory.createData(atgAP);
			    // Default und undefiniert Werte setzen
			    dataAP.setToDefault();
			    dataAP = br.getDatensatz(destHO, ap, atgAP, dataAP, false);
			    
			    try
			    {
				  	// APName
						ap.setName(br.getName(ap));
						ap.setConfigurationData(atgAP, dataAP);
						status.add(new Status(Status.OK, Activator.PLUGIN_ID,	"Fr  neues Objekt " + ap.getPid() +
								" vom Typ " + ap.getType().getPid() + " Daten erzeugt"));
			    }
			    catch (ConfigurationChangeException e)
			    {
						status.add(new Status(Status.ERROR, Activator.PLUGIN_ID,	"Fr  neues Objekt " + ap.getPid() +
								" vom Typ " + ap.getType().getPid() + " konnten keine Daten erzeugt werden"));
			    	return status;
			    }
				}  
		    
			  // APKP
				if (!apkpKopiert) // falls apkp nicht schon kopiert wurde
				{
					SystemObject apkp = destHO.getSystemObjekt(2);
					// ATG
					AttributeGroup atgAPKP = dataModel.getAttributeGroup("atg.anschlussPunktKommunikationsPartner");
					Data dataAPKP = DataFactory.createData(atgAPKP);
			    // Default und undefiniert Werte setzen
			    dataAPKP.setToDefault();
			    dataAPKP = br.getDatensatz(destHO, apkp, atgAPKP, dataAPKP, false);
			    dataAPKP.getReferenceValue( "KommunikationsPartner" ).setSystemObject(destHO.getSystemObjekt(0));
			    
			    try
			    {
						// APName
						apkp.setName(br.getName(apkp));
						apkp.setConfigurationData(atgAPKP, dataAPKP);
						status.add(new Status(Status.OK, Activator.PLUGIN_ID,	"Fr  neues Objekt " + apkp.getPid() +
								" vom Typ " + apkp.getType().getPid() + " Daten erzeugt"));
			    }
			    catch (ConfigurationChangeException e)
			    {
			    	status.add(new Status(Status.ERROR, Activator.PLUGIN_ID,	"Fr  neues Objekt " + apkp.getPid() +
								" vom Typ " + apkp.getType().getPid() + " konnten keine Daten erzeugt werden"));
			    	return status;
			    }
				}  
			}
		}
		
		return status;
  }
  
  public HOMitStatus kopiereHO(HierarchieObjekt ho,  ConfigurationArea kb, Object neuerVater, 
  		String neuePid[],	boolean rekursiv, boolean isWurzel)
  {
  	return kopiereHO(ho, kb, neuerVater, neuePid, rekursiv, true, true, isWurzel, true);
  }
  
  private HOMitStatus kopiereHO(HierarchieObjekt ho, ConfigurationArea kb, Object neuerVater, 
  		String neuePid[],	boolean rekursiv, boolean anwendenBR, boolean pruefePid, boolean isWurzel,
  		boolean automatischeObjekte)
  {
//  	if (neuePid == null && !(neuerVater instanceof HierarchieObjekt))
//  	{
//  		throw new IllegalArgumentException("Wenn fr die PID null bergeben wird"
//  				+ ", muss das neue bergeordnete Objekt vom Typ " + HierarchieObjekt.class + " sein"); 
//  	}
  	
  	MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Kopiere Hierarchieobjekt " + ho, null);
  
  	String[] pid = neuePid;
  	HOMitStatus result = null;
  	HierarchieObjekt neu = null;
  	boolean werteGesetzt = false;
  	IBildungsRegelnTLS bildungsRegeln = DatenSpeicher.getInstanz().getBildungsRegeln();
  	IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
  	
//  	boolean automatischeObjekte = isWurzel && (HOT_TLS.EAK.equals(hot) || hot instanceof DeUnterTyp);
  	
  	try
  	{
	  	if (pid == null)
	  	{
	//  		IBildungsRegeln bildungsRegeln = DatenSpeicher.getInstanz().getBildungsRegeln();
	//  		bildungsRegeln.setWerte(ho.getHierarchieObjektTyp(), (HierarchieObjekt)neuerVater, null);
	//  		pid = bildungsRegeln.getPids(ho.getHierarchieObjektTyp(), (HierarchieObjekt)neuerVater);
	  		
	  		int pidLaenge = ho.getSystemObjekte().size();
	  			  		
	  		if (HOT_TLS.VRZ.equals(hot) || HOT_TLS.UZ.equals(hot))
	  		{
	  			if (neuerVater instanceof HierarchieObjekt && pidLaenge < 3)
	  			{
	  			// Objekt wird von unterer Hierachieebene in obere kopiert
	  				
		    		//bildungsRegeln.setWerte(hot, (HierarchieObjekt)neuerVater, null);
	  				//bildungsRegeln.setWerte(hot, null, ho);
	  				bildungsRegeln.setZusatzInfo(hot, ho);
	  				werteGesetzt = true;
		    		
		    		pid = bildungsRegeln.getPids(hot, (HierarchieObjekt)neuerVater);
	  				
	  				pid[0] = ho.getSystemObjekt(0).getPid();
	  			}
	  			else if (!(neuerVater instanceof HierarchieObjekt) && pidLaenge > 1)
	  			{
	  				// Objekt wird von oberer Hierachieebene in untere kopiert
	  				pid = new String[3];
	  				pid[0] = ho.getSystemObjekt(0).getPid();
	  			}
	  		}
	  		
	  		if (pid == null)
	  		{	
	  			pid = new String[pidLaenge];
				
					for (int i = 0; i < pid.length; i += 1)
					{
						pid[i] = ho.getSystemObjekt(i).getPid();
					}
	  		}	
	  	}
	  	
	  	result = erzeugeNeuesHO(kb, neuerVater, pid, ho.getHierarchieObjektTyp(), pruefePid, automatischeObjekte);
	  	neu = result.getHO();
	  	status.add(result.getStatus());
  	}
  	finally
  	{
  		if (werteGesetzt)
  		{
  			bildungsRegeln.resetZusatzInfo(hot);
  		}
  	}
  	 
  	if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
		{
  		IStatus meldung = kopiereDaten(ho, neu, anwendenBR, anwendenBR, automatischeObjekte);
  	  		
  		status.merge(meldung);
		}
  	
  	if (!status.matches(IStatus.ERROR | IStatus.CANCEL) && rekursiv)
		{
  		for (HierarchieObjekt lauf: _hierarchie.getUntergeordneteObjekte(ho))
  		{
  			MultiStatus meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK, 
  					"Kopiere untergeordnetes Hierarchieobjekt " + lauf + " zu " + ho, null);
  			HOMitStatus ret = kopiereHO(lauf, kb, neu, null, true, anwendenBR, pruefePid, false, automatischeObjekte);
  			meldung.merge(ret.getStatus());
  			status.add(meldung);
  			if (status.matches(IStatus.ERROR | IStatus.CANCEL))
				{
  				break;
				}
  		}
		}
  	
  	return new HOMitStatus(neu, status);
  }
    
  
  private void behandleMengenVerschieben(HierarchieObjekt ho, HierarchieObjekt alterVater, 
  		HierarchieObjekt neuerVater, String mengenTyp, String mengenName, int index, MultiStatus status)
  {
		ConfigurationObject koSrc = (ConfigurationObject)alterVater.getSystemObjekte().get(0);
		ConfigurationObject koDest = (ConfigurationObject)neuerVater.getSystemObjekte().get(0);
		
		ObjectSet mengeSrc = koSrc.getObjectSet(mengenName);
		Assert.isNotNull(mengeSrc);

		ObjectSet mengeDest = koDest.getObjectSet(mengenName);
		if (mengeDest == null)
		{
			mengeDest = erzeugeMengeAmObjekt(koDest, mengenTyp, mengenName, status);
		}	
		
		if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
		{
			SystemObject ko = ho.getSystemObjekte().get(index);
			
			try
			{
  			mengeDest.add(ko);
  			status.add(new Status(Status.OK, Activator.PLUGIN_ID,	"Systemobjekt " + ko.getPid() +
				" der Menge \"" + mengenName + "\" des Systemobjekts " + koDest.getPid() + " hinzugefgt"));
			}
			catch (ConfigurationChangeException e)
			{
  			status.add(new Status(Status.ERROR, Activator.PLUGIN_ID,	"Systemobjekt " + ko.getPid() +
  					" konnte nicht der Menge \"" + mengenName + "\" des Systemobjekts " + koDest.getPid() + 
  					" hinzugefgt werden", e));
			}
			
			if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
			{
				//_hierarchie.objektHinzufuegen(neuerVater, ho);
				
				try
				{
	  			mengeSrc.remove(ko);
	  			status.add(new Status(Status.OK, Activator.PLUGIN_ID,	"Systemobjekt " + ko.getPid() +
					" aus der Menge \"" + mengenName +  "\" des Systemobjekts " + koSrc.getPid() + " entfernt"));
				}
				catch (ConfigurationChangeException e)
				{
	  			status.add(new Status(Status.ERROR, Activator.PLUGIN_ID,	"Systemobjekt " + ko.getPid() +
	  					" konnte nicht aus der Menge \"" + mengenName + "\" des Systemobjekts " + koSrc.getPid() + 
	  					" entfernt werden", e));
				}
				
				if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
				{
					//_hierarchie.objektEntfernen(alterVater, ho);
					_hierarchie.objektVerschieben(alterVater, neuerVater, ho);
				}	
			}
		}
  }
  
  private void behandleVerschiebenVRZUZNachUnten(HierarchieObjekt ho, Object alterVater, 
  		HierarchieObjekt neuerVater, MultiStatus status)
  {
  	Assert.isLegal(HOT_TLS.VRZ.equals(ho.getHierarchieObjektTyp()) || 
  			HOT_TLS.UZ.equals(ho.getHierarchieObjektTyp()));
  	
  	IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
		SystemObject[] neueObjekte = new SystemObject[3];
  	IBildungsRegelnTLS bildungsRegeln = DatenSpeicher.getInstanz().getBildungsRegeln();
		bildungsRegeln.setZusatzInfo(hot, ho);
		String[] pid = bildungsRegeln.getPids(ho.getHierarchieObjektTyp(), (HierarchieObjekt)neuerVater);
  	ConfigurationArea kb = _hierarchie.getKonfigurationsBereich(ho);
  	List<SystemObject> undoObjektListe = new LinkedList<SystemObject>();
  	Map<ObjectSet, SystemObject> undoMengenListe = new LinkedHashMap<ObjectSet, SystemObject>();

  	neueObjekte[0] = ho.getSystemObjekt(0);
  	
  	try
  	{
	  	List<ObjectSet> mengen = erzeugeMengenFuerTyp(hot, "typ.anschlussPunkt", kb, status, undoObjektListe);
			if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
			{
				neueObjekte[1] = erzeugeSystemObjekt(pid[1], "typ.anschlussPunkt", kb, mengen, status, undoObjektListe);
			}

//			if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
//			{
//				String name = bildungsRegeln.getName(neueObjekte[1]);
//				try
//				{
//					neueObjekte[1].setName(name);
//					status.add(new Status(Status.OK, Activator.PLUGIN_ID,	"Setze Name \"" + name + 
//							"\" fr Systemobjekt " +	neueObjekte[1].getPid()));
//				}
//				catch (ConfigurationChangeException e)
//				{
//					status.add(new Status(Status.ERROR, Activator.PLUGIN_ID,	"Name \"" + name + 
//							"\" fr Systemobjekt " +	neueObjekte[1].getPid() + " konnte nicht gesetzt werden", e));
//				}
//			}
			
			if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
			{
					behandleMengenZuordung(neuerVater, hot, neueObjekte[1], neueObjekte, status, 
							undoMengenListe, undoObjektListe);
			}
	  	
			if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
			{
				neueObjekte[2] = erzeugeSystemObjekt(pid[2], "typ.anschlussPunktKommunikationsPartner", kb, null, 
						status, undoObjektListe);
			}
			
//			if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
//			{
//				String name = bildungsRegeln.getName(neueObjekte[2]);
//				try
//				{
//					neueObjekte[2].setName(name);
//					status.add(new Status(Status.OK, Activator.PLUGIN_ID,	"Setze Name \"" + name + 
//							"\" fr Systemobjekt " +	neueObjekte[2].getPid()));
//				}
//				catch (ConfigurationChangeException e)
//				{
//					status.add(new Status(Status.ERROR, Activator.PLUGIN_ID,	"Name \"" + name + 
//							"\" fr Systemobjekt " +	neueObjekte[2].getPid() + " konnte nicht gesetzt werden", e));
//				}
//			}
			
			if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
			{
					behandleMengenZuordung(neuerVater, hot, neueObjekte[2], neueObjekte, status, 
							undoMengenListe, undoObjektListe);
			}
  	}
  	finally
  	{
  		bildungsRegeln.resetZusatzInfo(hot);
  		if (status.matches(IStatus.ERROR | IStatus.CANCEL))
	  	{
	  	  // Alle Aktionen rckgngig
	  		MultiStatus meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK,
	  				"Rollback Erzeugen neue Systemobjekte", null);
	  		
	  		for (ObjectSet menge: undoMengenListe.keySet())
	  		{
	  			SystemObject so = undoMengenListe.get(menge);
	  			try
	  			{
	  				menge.remove(so);
	  				meldung.add(new Status(Status.INFO, Activator.PLUGIN_ID, "Systemobjekt " + so.getPid() + 
	  						" aus der Menge " +	menge.getName() +	" vom Typ " + menge.getType().getPid() + 
	  						" entfernt", null));
	  			}
	  			catch (ConfigurationChangeException e)
	  			{
	  				meldung.add(new Status(Status.WARNING, Activator.PLUGIN_ID, "Systemobjekt " + so.getPid() + " " +
	  						"konnte nicht aus der Menge " + menge.getName() +	" vom Typ " + menge.getType().getPid() +
	  						" entfernt werden", e));
	  			}
	  			catch (RuntimeException e)
	  			{
	  				meldung.add(new Status(Status.WARNING, Activator.PLUGIN_ID, "Systemobjekt " + so.getPid() + " " +
	  						"konnte nicht aus der Menge " + menge.getName() +	" vom Typ " + menge.getType().getPid() +
	  						" entfernt werden", e));
	  			}
	  		}
	  		for (SystemObject so: undoObjektListe)
	  		{
	  			try
	  			{
	  				so.invalidate();
	  				meldung.add(new Status(Status.INFO, Activator.PLUGIN_ID, "Systemobjekt " + so.getPidOrNameOrId() + 
	  						" vom Typ " +	so.getType().getPid() + " gelscht", null));
	  			}
	  			catch (ConfigurationChangeException e)
	  			{
	  				meldung.add(new Status(Status.WARNING, Activator.PLUGIN_ID, "Systemobjekt " + so.getPidOrNameOrId() + 
	  						" vom Typ " + so.getType().getPid() +	" konnte nicht gelscht werden", e));
	  			}
	  			catch (RuntimeException e)
	  			{
	  				meldung.add(new Status(Status.WARNING, Activator.PLUGIN_ID, "Systemobjekt " + so.getPidOrNameOrId() + 
	  						" vom Typ " + so.getType().getPid() +	" konnte nicht gelscht werden", e));
	  			}
	  		}
	  		status.add(meldung);
	  	}
  	}
		
//		ConfigurationObject koDest = (ConfigurationObject)neuerVater.getSystemObjekte().get(0);
		
//		ObjectSet mengeDest = koDest.getObjectSet("AnschlussPunkteGert");
//		if (mengeDest == null)
//		{
//			mengeDest = erzeugeMengeAmObjekt(kb, koDest, "menge.anschlussPunkteGert", "AnschlussPunkteGert", status);
//		}	
//		
//		if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
//		{
//			SystemObject ko = neueObjekte[1];
//			
//			try
//			{
//  			mengeDest.add(ko);
//  			status.add(new Status(Status.OK, Activator.PLUGIN_ID,	"Systemobjekt " + ko.getPid() +
//				" der Menge \"AnschlussPunkteGert\" des Systemobjekts " + koDest.getPid() + " hinzugefgt"));
//			}
//			catch (ConfigurationChangeException e)
//			{
//  			status.add(new Status(Status.ERROR, Activator.PLUGIN_ID,	"Systemobjekt " + ko.getPid() +
//  					" konnte nicht der Menge \"AnschlussPunkteGert\" des Systemobjekts " + koDest.getPid() + 
//  					" hinzugefgt werden", e));
//			}
//			
//			if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
//			{
//				_hierarchie.objektHinzufuegen(neuerVater, ho);
//				_hierarchie.objektEntfernen(alterVater, ho);
//			}				
//		}
		
		if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
		{
			ho.setSystemObjekte(neueObjekte);
//			_hierarchie.objektHinzufuegen(neuerVater, ho);
//			_hierarchie.objektEntfernen(alterVater, ho);
			_hierarchie.objektVerschieben(alterVater, neuerVater, ho);
		}				
  }
  
  private void behandleVerschiebenVRZUZNachOben(HierarchieObjekt ho, HierarchieObjekt alterVater, 
  		Object neuerVater, MultiStatus status)
  {
  	Assert.isLegal(HOT_TLS.VRZ.equals(ho.getHierarchieObjektTyp()) || 
  			HOT_TLS.UZ.equals(ho.getHierarchieObjektTyp()));
  	
		ConfigurationObject koSrc = (ConfigurationObject)alterVater.getSystemObjekte().get(0);
				
		ObjectSet mengeSrc = koSrc.getObjectSet("AnschlussPunkteGert");
		
		if (mengeSrc != null)
		{
			SystemObject ko = ho.getSystemObjekte().get(1);
			
			try
			{
  			mengeSrc.remove(ko);
  			status.add(new Status(Status.OK, Activator.PLUGIN_ID,	"Systemobjekt " + ko.getPid() +
				" aus der Menge \"AnschlussPunkteGert\" des Systemobjekts " + koSrc.getPid() + " entfernt"));
			}
			catch (ConfigurationChangeException e)
			{
  			status.add(new Status(Status.ERROR, Activator.PLUGIN_ID,	"Systemobjekt " + ko.getPid() +
  					" konnte nicht aus der Menge \"AnschlussPunkteGert\" des Systemobjekts " + koSrc.getPid() + 
  					" entfernt werden", e));
			}
			
			if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
			{
				try
				{
					ho.getSystemObjekt(1).invalidate();
					status.add(new Status(Status.OK, Activator.PLUGIN_ID,	"Systemobjekt " + ho.getSystemObjekt(1).getPid() +
					" gelscht"));
				}
				catch (ConfigurationChangeException e)
				{
					status.add(new Status(Status.WARNING, Activator.PLUGIN_ID,	"Systemobjekt " + ho.getSystemObjekt(1).getPid() +
					" konnte nicht gelscht werden"));
				}
				
				try
				{
					ho.getSystemObjekt(2).invalidate();
					status.add(new Status(Status.OK, Activator.PLUGIN_ID,	"Systemobjekt " + ho.getSystemObjekt(2).getPid() +
							" gelscht"));
				}
				catch (ConfigurationChangeException e)
				{
					status.add(new Status(Status.WARNING, Activator.PLUGIN_ID,	"Systemobjekt " + ho.getSystemObjekt(2).getPid() +
					" konnte nicht gelscht werden"));
				}

				ho.setSystemObjekte(new SystemObject[] { ho.getSystemObjekt(0)});
//				_hierarchie.objektHinzufuegen(neuerVater, ho);
//				_hierarchie.objektEntfernen(alterVater, ho);
				_hierarchie.objektVerschieben(alterVater, neuerVater, ho);
			}	
		}
//		try
//		{
//			ho.getSystemObjekt(1).invalidate();
//			status.add(new Status(Status.OK, Activator.PLUGIN_ID,	"Systemobjekt " + ho.getSystemObjekt(1).getPid() +
//			" gelscht"));
//		}
//		catch (ConfigurationChangeException e)
//		{
//			status.add(new Status(Status.WARNING, Activator.PLUGIN_ID,	"Systemobjekt " + ho.getSystemObjekt(1).getPid() +
//			" konnte nicht gelscht werden"));
//		}
//		
//		try
//		{
//			ho.getSystemObjekt(2).invalidate();
//			status.add(new Status(Status.OK, Activator.PLUGIN_ID,	"Systemobjekt " + ho.getSystemObjekt(2).getPid() +
//					" gelscht"));
//		}
//		catch (ConfigurationChangeException e)
//		{
//			status.add(new Status(Status.WARNING, Activator.PLUGIN_ID,	"Systemobjekt " + ho.getSystemObjekt(2).getPid() +
//			" konnte nicht gelscht werden"));
//		}
//	
//		ho.setSystemObjekte(new SystemObject[] { ho.getSystemObjekt(0)});
//		_hierarchie.objektHinzufuegen(neuerVater, ho);
//		_hierarchie.objektEntfernen(alterVater, ho);
	}	

  
  private void behandleVerschieben(HierarchieObjekt ho, Object neuerVater, MultiStatus status)
  {
  	IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
  	Object alterVater = _hierarchie.getUebergeordnetesObjekt(ho);
  	
  	if (DatenSpeicher.getInstanz().getDeTypenManager().hasHOT(hot))
  	{
  		// DE
  		Assert.isLegal(alterVater instanceof HierarchieObjekt && 
  				((HierarchieObjekt)alterVater).getHierarchieObjektTyp().equals(HOT_TLS.EAK));
  		Assert.isLegal(neuerVater instanceof HierarchieObjekt && 
  				((HierarchieObjekt)neuerVater).getHierarchieObjektTyp().equals(HOT_TLS.EAK));
  		
  		behandleMengenVerschieben(ho, (HierarchieObjekt)alterVater, (HierarchieObjekt)neuerVater, 
  				"menge.de", "De", 0, status);
  	}
  	else if (hot.equals(HOT_TLS.EAK))
		{
  		// EAK
  		Assert.isLegal(alterVater instanceof HierarchieObjekt && 
  				((HierarchieObjekt)alterVater).getHierarchieObjektTyp().equals(HOT_TLS.SM));
  		Assert.isLegal(neuerVater instanceof HierarchieObjekt && 
  				((HierarchieObjekt)neuerVater).getHierarchieObjektTyp().equals(HOT_TLS.SM));
  		 
  		behandleMengenVerschieben(ho, (HierarchieObjekt)alterVater, (HierarchieObjekt)neuerVater, 
  				"menge.eak", "Eak", 0, status);
		}
  	else if (hot.equals(HOT_TLS.SM))
  	{
  		// SM
    	Assert.isLegal(alterVater instanceof HierarchieObjekt && 
  				((HierarchieObjekt)alterVater).getHierarchieObjektTyp().equals(HOT_TLS.Inselbus));
  		Assert.isLegal(neuerVater instanceof HierarchieObjekt && 
  				((HierarchieObjekt)neuerVater).getHierarchieObjektTyp().equals(HOT_TLS.Inselbus));
  		
  		behandleMengenVerschieben(ho, (HierarchieObjekt)alterVater, (HierarchieObjekt)neuerVater, 
  				"menge.anschlussPunkteKommunikationsPartner", "AnschlussPunkteKommunikationsPartner", 1, status);
  		
  	}
  	else if (hot.equals(HOT_TLS.Inselbus))
  	{
  	  // Inselbus
  		HOT_TLS[] erlaubteHOT = new HOT_TLS[] {HOT_TLS.KRI, HOT_TLS.UZ, HOT_TLS.VRZ, HOT_TLS.VIZ};
    	Assert.isLegal(alterVater instanceof HierarchieObjekt && 
  				Arrays.asList(erlaubteHOT).contains(((HierarchieObjekt)alterVater).getHierarchieObjektTyp()));
  		Assert.isLegal(neuerVater instanceof HierarchieObjekt && 
  				Arrays.asList(erlaubteHOT).contains(((HierarchieObjekt)neuerVater).getHierarchieObjektTyp()));
  		
  		behandleMengenVerschieben(ho, (HierarchieObjekt)alterVater, (HierarchieObjekt)neuerVater, 
  				"menge.anschlussPunkteGert", "AnschlussPunkteGert", 0, status);
  	}
  	else if (hot.equals(HOT_TLS.KRI))
  	{
  	  // Inselbus
  		HOT_TLS[] erlaubteHOT = new HOT_TLS[] {HOT_TLS.UZ, HOT_TLS.VRZ, HOT_TLS.VIZ};
    	Assert.isLegal(alterVater instanceof HierarchieObjekt && 
  				Arrays.asList(erlaubteHOT).contains(((HierarchieObjekt)alterVater).getHierarchieObjektTyp()));
  		Assert.isLegal(neuerVater instanceof HierarchieObjekt && 
  				Arrays.asList(erlaubteHOT).contains(((HierarchieObjekt)neuerVater).getHierarchieObjektTyp()));
  		
  		behandleMengenVerschieben(ho, (HierarchieObjekt)alterVater, (HierarchieObjekt)neuerVater, 
  				"menge.anschlussPunkteGert", "AnschlussPunkteGert", 1, status);
  	}
  	else if (hot.equals(HOT_TLS.UZ))
  	{
  	  //UZ
  		HOT_TLS[] erlaubteHOT = new HOT_TLS[] {HOT_TLS.VRZ, HOT_TLS.VIZ};
  		
  		if (alterVater instanceof HierarchieObjekt && neuerVater instanceof HierarchieObjekt)
  		{
  			Assert.isLegal(Arrays.asList(erlaubteHOT).contains(((HierarchieObjekt)alterVater).getHierarchieObjektTyp()));
  			Assert.isLegal(Arrays.asList(erlaubteHOT).contains(((HierarchieObjekt)neuerVater).getHierarchieObjektTyp()));
  			
    		behandleMengenVerschieben(ho, (HierarchieObjekt)alterVater, (HierarchieObjekt)neuerVater, 
    				"menge.anschlussPunkteGert", "AnschlussPunkteGert", 1, status);
  		}
  		else if (alterVater instanceof HierarchieObjekt && neuerVater instanceof ConfigurationArea)
  		{
  			// Verschieben in 1. Ebene
  			Assert.isLegal(Arrays.asList(erlaubteHOT).contains(((HierarchieObjekt)alterVater).getHierarchieObjektTyp()));
    		behandleVerschiebenVRZUZNachOben(ho, (HierarchieObjekt)alterVater, neuerVater, status); 
  		}
  		else if (alterVater instanceof ConfigurationArea && neuerVater instanceof HierarchieObjekt)
  		{
  			// Verschieben von 1. Ebene
  			Assert.isLegal(Arrays.asList(erlaubteHOT).contains(((HierarchieObjekt)neuerVater).getHierarchieObjektTyp()));
    		behandleVerschiebenVRZUZNachUnten(ho, alterVater, (HierarchieObjekt)neuerVater, status); 
  		}
  	}
  	else if (hot.equals(HOT_TLS.VRZ))
  	{
  	  //VRZ
  		HOT_TLS[] erlaubteHOT = new HOT_TLS[] {HOT_TLS.VIZ};
  		
  		if (alterVater instanceof HierarchieObjekt && neuerVater instanceof HierarchieObjekt)
  		{
  			Assert.isLegal(Arrays.asList(erlaubteHOT).contains(((HierarchieObjekt)alterVater).getHierarchieObjektTyp()));
  			Assert.isLegal(Arrays.asList(erlaubteHOT).contains(((HierarchieObjekt)neuerVater).getHierarchieObjektTyp()));
  			
    		behandleMengenVerschieben(ho, (HierarchieObjekt)alterVater, (HierarchieObjekt)neuerVater, 
    				"menge.anschlussPunkteGert", "AnschlussPunkteGert", 1, status);
  		}
  		else if (alterVater instanceof HierarchieObjekt && neuerVater instanceof ConfigurationArea)
  		{
  			// Verschieben in 1. Ebene
  			Assert.isLegal(Arrays.asList(erlaubteHOT).contains(((HierarchieObjekt)alterVater).getHierarchieObjektTyp()));
    		behandleVerschiebenVRZUZNachOben(ho, (HierarchieObjekt)alterVater, neuerVater, status); 
  		}
  		else if (alterVater instanceof ConfigurationArea && neuerVater instanceof HierarchieObjekt)
  		{
  			// Verschieben von 1. Ebene
  			Assert.isLegal(Arrays.asList(erlaubteHOT).contains(((HierarchieObjekt)neuerVater).getHierarchieObjektTyp()));
    		behandleVerschiebenVRZUZNachUnten(ho, alterVater, (HierarchieObjekt)neuerVater, status); 
  		}
  	}
  }
  
  public HOMitStatus verschiebeHO(HierarchieObjekt ho, Object neuerVater)
  {
  	Object alterVater = _hierarchie.getUebergeordnetesObjekt(ho);
  	ConfigurationArea srcKB = _hierarchie.getKonfigurationsBereich(alterVater);
  	Assert.isNotNull(srcKB, "Der Konfigurationsbereich zu " + alterVater + " darf nicht null sein");
  	ConfigurationArea destKB = _hierarchie.getKonfigurationsBereich(neuerVater);
  	Assert.isNotNull(destKB, "Der Konfigurationsbereich zu " + neuerVater + " darf nicht null sein");
  	
  	MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Verschiebe Hierarchieobjekt " + ho, null);
  	
  	HierarchieObjekt neu = null;
  	
  	if (srcKB.equals(destKB)) // HO wird im gleichen kb verschoben
  	{
  		if (neuerVater.equals(alterVater))
  		{
  			status.add(new Status(Status.INFO, Activator.PLUGIN_ID,	"Objekt " + ho +
				" kann nicht in sich selbst verschoben werden"));
  		}
  		else
  		{
  			behandleVerschieben(ho, neuerVater, status);
  			
  			if (!status.matches(IStatus.ERROR | IStatus.CANCEL ))
  			{
  				neu = ho; // HO verschoben
  			}
  		}
  		
//  		if (!status.matches(IStatus.ERROR | IStatus.CANCEL | IStatus.INFO))
  		if (neu != null) // Falls das Objekt verschoben wurde
  		{
  			IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
  			
  			if (HOT_TLS.EAK.equals(hot)) // EAK-Typ
				{
					Assert.isTrue(alterVater instanceof HierarchieObjekt);

					ConfigDataModel dataModel = getHierarchie().getDataModel();
					ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
					AttributeGroup atgEak = dataModel.getAttributeGroup("atg.eak");
					
					Data data = ((ConfigSystemObject)ho.getSystemObjekt(0)).getConfigurationData(atgEak, lookup);
					
					if (data != null)
					{	
						behandleErzeugenDEAnEAK254(ho, data, null, status);
						behandleLoeschenDEAnEAK254((HierarchieObjekt)alterVater, ho, status);
					}	
				}
				else if (DatenSpeicher.getInstanz().getDeTypenManager().hasHOT(hot)) // DE-Typ
				{
					Assert.isTrue(neuerVater instanceof HierarchieObjekt);
					Assert.isTrue(alterVater instanceof HierarchieObjekt);
					ConfigurationArea kb = ho.getSystemObjekt(0).getConfigurationArea(); 
					behandleErzeugenDEMitDEKanal255((HierarchieObjekt)neuerVater, kb, hot, false, status);
					behandleLoeschenDEMitDEKanal255((HierarchieObjekt)alterVater, hot, status);
				}
  		}
  	}
  	else // HO wird in einen anderen kb verschoben
  	{
  		HOHierarchie hoHierarchie = new HOHierarchie(ho, _hierarchie);
    	  
  	  status.add(loescheHO(ho));
  	  
  	  if (!status.matches( IStatus.ERROR | IStatus.CANCEL ))
  	  {
  	    HOMitStatus result = kopiereHORekursiv(hoHierarchie.getWurzel(), destKB, neuerVater,
  	        hoHierarchie, true);
  	    status.add( result.getStatus() );
  	    neu = result.getHO();
  	  }
  	}
  	
  	return new HOMitStatus( neu, status );
  }
  
  private HOMitStatus kopiereHORekursiv(HierarchieObjekt ho, ConfigurationArea kb, Object vater,
      HOHierarchie hierarchie, boolean isWurzel)
  {
    MultiStatus meldung;
    
    if (ho == hierarchie.getWurzel())
    {  
      meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Kopiere Hierarchieobjekt " + ho, null);
    }
    else
    {
      meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Kopiere untergeordnetes Hierarchieobjekt " + 
          ho + " zu " + vater, null);
    }
    
    IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
    boolean automatischeObjekte = isWurzel && (HOT_TLS.EAK.equals(hot) || hot instanceof DeUnterTyp);
    
    HOMitStatus result = kopiereHO(ho, kb, vater, null, false, false, false, isWurzel, automatischeObjekte);
    meldung.merge( result.getStatus() );
    HierarchieObjekt neu = result.getHO();

    if (neu != null)
    {  
      for (HierarchieObjekt kind: hierarchie.getUntergeordneteObjekte( ho ))
      {
        result = kopiereHORekursiv(kind, kb, neu, hierarchie, false);
        meldung.add( result.getStatus() );
      }
    }
    
    return new HOMitStatus(neu, meldung);
  }
  
  private void behandleZuordnungLoeschen(HierarchieObjekt ho, SystemObject so, MultiStatus status)
  {
  	Object vater = _hierarchie.getUebergeordnetesObjekt(ho);
  	if (vater instanceof HierarchieObjekt) // Nur Zuordnungen tiefer als Ebene 1
  	{
  		ConfigurationObject ko = null;
  		ObjectSet menge = null;
  		
	  	IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
	  	
		  if (DatenSpeicher.getInstanz().getDeTypenManager().hasHOT(hot))
			{
		  	HierarchieObjekt vaterHO = (HierarchieObjekt)vater;
		  	if (!(HOT_TLS.EAK.equals(vaterHO.getHierarchieObjektTyp())))
		  	{
		  		new IllegalStateException("bergeordnetes HO von " + ho + " muss vom (HO-)Typ " + 
		  				HOT_TLS.EAK + " sein ");
		  	}
		  	ko = (ConfigurationObject)vaterHO.getSystemObjekt(0);
		  	menge = ko.getNonMutableSet("De");
		  	
			}
		  else if (hot.equals(HOT_TLS.EAK))
			{
		  	HierarchieObjekt vaterHO = (HierarchieObjekt)vater;
		  	if (!(HOT_TLS.SM.equals(vaterHO.getHierarchieObjektTyp())))
		  	{
		  		throw new IllegalStateException("bergeordnetes HO von " + ho + " muss vom (HO-)Typ " + 
		  				HOT_TLS.SM + " sein ");
		  	}
		  	ko = (ConfigurationObject)vaterHO.getSystemObjekt(0);
		  	menge = ko.getNonMutableSet("Eak");
			}
		  else if (hot.equals(HOT_TLS.SM))
		  {
		  	HierarchieObjekt vaterHO = (HierarchieObjekt)vater;
		  	if (!(HOT_TLS.Inselbus.equals(vaterHO.getHierarchieObjektTyp())))
		  	{
		  		throw new IllegalStateException("bergeordnetes HO von " + ho + " muss vom (HO-)Typ " + 
		  				HOT_TLS.Inselbus + " sein ");
		  	}
		  	if (so.isOfType("typ.anschlussPunktKommunikationsPartner"))
		  	{
		  		ko = (ConfigurationObject)vaterHO.getSystemObjekt(0);
			  	menge = ko.getNonMutableSet("AnschlussPunkteKommunikationsPartner");
		  	}
		  }
		  else if (hot.equals(HOT_TLS.Inselbus))
		  {
		  	HierarchieObjekt vaterHO = (HierarchieObjekt)vater;
		  	if (!(HOT_TLS.VIZ.equals(vaterHO.getHierarchieObjektTyp())) &&
		  			!(HOT_TLS.VRZ.equals(vaterHO.getHierarchieObjektTyp())) &&
		  			!(HOT_TLS.UZ.equals(vaterHO.getHierarchieObjektTyp())) &&
		  			!(HOT_TLS.KRI.equals(vaterHO.getHierarchieObjektTyp())))
		  	{
		  		throw new IllegalStateException("bergeordnetes HO von " + ho + " muss vom (HO-)Typ " + 
		  				HOT_TLS.VIZ + ", " + HOT_TLS.VRZ + ", " + HOT_TLS.UZ + " oder " + HOT_TLS.KRI + " sein ");
  	  	}
	    	ko = (ConfigurationObject)vaterHO.getSystemObjekt(0);
		  	menge = ko.getNonMutableSet("AnschlussPunkteGert");
		  }
		  else if (hot.equals(HOT_TLS.VRZ) ||
	  			hot.equals(HOT_TLS.UZ) ||
	  			hot.equals(HOT_TLS.KRI))
		  {
		  	if (so.isOfType("typ.anschlussPunktKommunikationsPartner"))
		  	{
		  		ko = (ConfigurationObject)ho.getSystemObjekt(1);
			  	menge = ko.getNonMutableSet("AnschlussPunkteKommunikationsPartner");
		  	}
		  	if (so.isOfType("typ.anschlussPunkt"))
		  	{
		  		ko = (ConfigurationObject)((HierarchieObjekt)vater).getSystemObjekt(0);
			  	menge = ko.getNonMutableSet("AnschlussPunkteGert");
		  	}
		  }
		  
			if (ko != null && menge != null)
	  	{
				try
	    	{
	  			menge.remove(so);
	  			
//	  			_debug.warning("Systemobjekt " +	so.getPid() + 
//	  					" vom Typ " + so.getType().getPid() + " aus der Menge " + menge.getName() + 
//	  					" des Objekts " + ko.getPid() + " vom Typ " + ko.getType().getPid() + " entfernt.");
	  			
	  			status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Systemobjekt " +	so.getPid() + 
	  					" vom Typ " + so.getType().getPid() + " aus der Menge " + menge.getName() + 
	  					" des Objekts " + ko.getPid() + " vom Typ " + ko.getType().getPid() + " entfernt"));
	  			
	    	}
	    	catch (ConfigurationChangeException e)
	    	{
	    		status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Systemobjekt " + so.getPid() + 
	    				" vom Typ " + so.getType().getPid() + " konnte nicht aus der Menge " + menge.getName() +
	    				" des Objekts " +	ko.getPid() + " vom Typ " + ko.getType().getPid() + " entfernt werden", e));
	    	}
	  	}
		}
  }
  
  public IStatus loescheHO(HierarchieObjekt ho)
	{
  	return loescheHO(ho, true);
	}
  
  public IStatus loescheHO(HierarchieObjekt ho, boolean automatischeObjekte)
	{
  	// berprfung Plausibilitt der Parameter
		IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
	  Object vater = _hierarchie.getUebergeordnetesObjekt(ho); 
	  
	  if (!Arrays.asList(getUntergeordneteHOTs(vater)).contains(hot))
	  {
	  	throw new IllegalArgumentException("Unzulssiges bergeordnetes Objekt " + vater +
	  			" fr Hierarchieobjekt " + ho);
	  }
	  
		MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Lsche Hierarchieobjekt " + ho +
				" vom Typ " + hot, null);
		
		// Rekursiver Aufruf: Lschen der untergeordneten Objekte
		while (!getHierarchie().getUntergeordneteObjekte(ho).isEmpty())
  	{
  		HierarchieObjekt objekt = getHierarchie().getUntergeordneteObjekte(ho).get(0);
  		
  		status.add(loescheHO(objekt));
  		
  		if (status.matches(IStatus.CANCEL | IStatus.ERROR))
  		{
  			break;
   		}
  	}
		
		if (!status.matches(IStatus.CANCEL | IStatus.ERROR))
		{
			// Lschen der Systemobjekte - in umgekehrter Reihenfolge
			ListIterator<SystemObject> it = ho.getSystemObjekte().listIterator(ho.getSystemObjekte().size());
			
			while (it.hasPrevious())
			{
				SystemObject so = it.previous();
				String typ = so.getType().getPid();
				TypInfo typInfo = hot.getTypInfo(typ);
				if (typInfo == null)
				{
					throw new IllegalStateException("Systemobjekttyp " + typ + " ist nicht im Hierarchieobjekttyp " +
							hot + " enthalten");
				}
				// Entferne Hierarchiebeziehung
				behandleZuordnungLoeschen(ho, so, status);
				
				// Lsche Menge(n)
//				for (String mengenName: typInfo.getMengen())
//				{
//					if (status.matches(IStatus.CANCEL | IStatus.ERROR))
//					{
//						break;
//					}
//					ObjectSet menge = ((ConfigurationObject)so).getNonMutableSet(mengenName);
//					if (menge != null)
//					{
//						try
//			    	{
////		    			((ConfigurationObject)so).removeSet(menge);
//			  			menge.invalidate();
//			  			
////			  			_debug.warning("Menge " + mengenName + " des Objekts " +	
////		    					so.getPid() + " vom Typ " + so.getType().getName() + " gelscht");
//			  			
//		    			status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Menge " + mengenName + " des Objekts " +	
//		    					so.getPid() + " vom Typ " + so.getType().getName() + " gelscht"));
//			    	}
//			    	catch (ConfigurationChangeException e)
//			    	{
//			    		status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Menge " + mengenName + " des Objekts " + 
//			    				so.getPid() + " vom Typ " + so.getType().getName() + " konnte nicht gelscht werden", e));
//			    	}
//					}
//					else
//					{
//						_debug.warning("Menge " + mengenName + " ist am Systemobjekt " + so + " nicht vorhanden");
//					}
//				}
				if (!status.matches(IStatus.CANCEL | IStatus.ERROR))
				// Lsche Systemobjekt	
				{
					try
			  	{
			  		so.invalidate();
			  		
//			  		_debug.warning("Systemobjekt " + 
//			  				so.getPid() + " vom Typ " + so.getType().getName() + " gelscht");
			  		
			  		status.add(new Status(Status.OK, Activator.PLUGIN_ID, "Systemobjekt " + 
			  				so.getPid() + " vom Typ " + so.getType().getName() + " gelscht"));
			  	}
			  	catch (ConfigurationChangeException e)
			  	{
			  		status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Systemobjekt " + 
			  				so.getPid() + " vom Typ " + so.getType().getName() + " konnte nicht gelscht werden", e));
			  		break;
			  	}
				}
				else
				{
					break;
				}
			}
			
			if (!status.matches(IStatus.CANCEL | IStatus.ERROR))
			{	
				_hierarchie.objektEntfernen(vater, ho);
				
				if (automatischeObjekte)
				{
					if (HOT_TLS.EAK.equals(hot)) // EAK-Typ
					{
						Assert.isTrue(vater instanceof HierarchieObjekt);
						behandleLoeschenDEAnEAK254((HierarchieObjekt)vater, ho, status);
					}
					else if (DatenSpeicher.getInstanz().getDeTypenManager().hasHOT(hot)) // DE-Typ
					{
						Assert.isTrue(vater instanceof HierarchieObjekt);
						behandleLoeschenDEMitDEKanal255((HierarchieObjekt)vater, hot, status);
					}
				}
			}	
		}
		
		return status;
	}

  
  public IStatus pruefeDaten(HierarchieObjekt ho, HODaten daten)
	{
		IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
		IPruefungen pruefInstanz = DatenSpeicher.getInstanz().getPruefungen();
		MultiStatus result = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe Hierarchieobjekt " + ho, null);
		
		// Prfe Hierarchieobjekt
		result.merge(pruefInstanz.pruefeHierarchieObjekt(ho, daten));
		
		// Prfe auf offene Referenzen bei PID-nderungen
		LoeschenChecker loeschenChecker = new LoeschenChecker(this);
		List<SystemObject> liste = new Vector<SystemObject>();
    StringBuffer aend = new StringBuffer();
		StringBuffer loesch = new StringBuffer();
		
		for (SystemObject so: ho.getSystemObjekte())
		{
		 SystemObjektDaten soDaten = daten.getSystemObjektDaten(so);
		 if (soDaten == null)
		 {
			 throw new IllegalStateException("Zu dem Systemobjekt " + so.getPid() + 
					 " wurden keine Daten vom Typ SODaten bergeben");
		 }
		 
		 if (!so.getPid().equals(soDaten.getPid()))
		 {
			 liste.add(so);
			 if (aend.length() > 0)
			 {
				 aend.append(", ");
			 }
			 aend.append(soDaten.getPid());
			 if (loesch.length() > 0)
			 {
				 loesch.append(", ");
			 }
			 loesch.append(so.getPid());
		 }
		}
		
		if (liste.size() > 0)
		{
			 Vector<SystemObject> ausschlussListe  = new Vector<SystemObject>();
			 for (SystemObject so: ho.getSystemObjekte())
			 {
				 if (!liste.contains(so))
				 {
					 ausschlussListe.add(so);
				 }
			 }
			 
			 MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK,  "Durch PID-nderung(en) auf " + 
					 aend + " werden folgende Objekte gelscht: " + loesch, null);
			 
			 Collection<SystemObjectType> typen = loeschenOffeneReferenzenVerboten(ho.getHierarchieObjektTyp());
	     boolean flag = verwendeZuLoeschendeObjekteZurPruefungOffeneReferenzen();
	     
			 status.add(loeschenChecker.bestimmeOffeneReferenzen(liste, typen, ausschlussListe, flag));
			 result.add(status);
		 }
		
		// Prfe ATG-Daten
		for (SystemObject so: ho.getSystemObjekte())
  	{
			 SystemObjektDaten soDaten = daten.getSystemObjektDaten(so);
			 if (soDaten != null)
			 {
				 MultiStatus objektStatus;
				 if (ho.getSystemObjekte().size() > 1)
				 {
					 objektStatus = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe ATG-Daten fr Systemobjekt " + 
							 so.getPid() + " vom Typ " + so.getType().getPid(), null);
				 }
				 else
				 {
					 objektStatus = result;
				 }
				 
				 // Prfe ATG-Daten  fr das Systemobjekt
				 TypInfo typInfo = hot.getTypInfo(so.getType().getPid());
				 if (typInfo != null)
				 {
					 for (String atgPid: typInfo.getAtgs())
					 {
						 AtgInfo atgInfo = typInfo.getAtgInfo(atgPid);
						 if (atgInfo != null)
						 {
							 Data data = soDaten.getAtgDaten(atgPid);
							 if (data == null && (atgInfo.isMandatory() || !atgInfo.isOptional()))
							 {
								 objektStatus.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, "Fr die nicht-optionale ATG " +
										 atgPid + " sind keine Daten vorhanden"));					 
							 }
							 if (data != null)
							 {
								 MultiStatus atgStatus = new MultiStatus(Activator.PLUGIN_ID, Status.OK, 
										 "Prfe Daten der ATG " + atgPid, null);
								 atgStatus.merge(pruefInstanz.pruefeDaten(ho, so, atgInfo, data));
							 
								 objektStatus.add(atgStatus);
							 }
						 }
						 else
						 {
								// Kein AtgInfo fr den aktuellen Typ-Info-Typ im HOT fr aktuellen Systemobjekttyp
							 throw new IllegalStateException("AtgInfo fr ATG " + atgPid + " fehlt in TypInfo fr " + 
									 so.getType().getPid() + " im HierarchieObjektTyp " + hot);
						 }
					 }
				 }
				 else
				 {
				// Keine Typ-Info im HOT fr aktuellen Systemobjekttyp
					 throw new IllegalStateException("TypInfo fehlt fr  " + so.getType().getPid() + 
							 " im HierarchieObjektTyp " + hot);
				 }
				 if (result != objektStatus)
				 {
					 result.add(objektStatus);
				 }
				 
			 }
			 else
			 {
				 // Keine Daten fr Systemobjekt so bergeben
				 throw new IllegalStateException("Daten fr Systemobjekt " + so.getPid() +
						 " erforderlich, aber nicht bergeben");
			 }
  	}
  	
  	return result;
	}  
  
  private ConfigurationObject dupliziereObjekt(ConfigurationObject ko, String neuePid)
  	throws ConfigurationChangeException
  {
  	List<ObjectSet> sets = new Vector<ObjectSet>();
  	ConfigurationObject neu = null;
  	ConfigurationObject result = null;
  	
  	try
  	{
	  	for (ObjectSet set: ko.getObjectSets())
	  	{
	  		ObjectSetType setTyp = set.getObjectSetType();
	  		Assert.isTrue(setTyp.getReferenceType() != ReferenceType.COMPOSITION);
	  		ObjectSet menge = (ObjectSet)ko.getConfigurationArea().createConfigurationObject(set.getObjectSetType(), 
	  				"", set.getName(), null);
	  		List<SystemObject> elemente = set instanceof NonMutableSet ? 
	  				((NonMutableSet)set).getElementsInModifiableVersion() : set.getElements();
	  		menge.add(elemente.toArray(new SystemObject[elemente.size()]));
	  		sets.add(menge);
	  	}
	  	
	  	neu = ko.getConfigurationArea().createConfigurationObject((ConfigurationObjectType)ko.getType(),
	  			neuePid == null ? ko.getPid() : neuePid, ko.getName(), sets);
	  	
	  	ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion((ConfigDataModel)ko.getDataModel());
	  	
	  	for (AttributeGroupUsage atgu: ko.getUsedAttributeGroupUsages())
			{
				Data dataSrc = ((ConfigSystemObject)ko).getConfigurationData(atgu, lookup);
				if (dataSrc != null)
				{
					Data dataDest = DataFactory.createModifiableCopy(dataSrc, lookup);;
					neu.setConfigurationData(atgu, dataDest);
				}
			}
	  	result = neu;
  	}
  	finally
  	{
  		// Rollback
  		if (result == null)
  		{	
	  		for (ObjectSet set: sets)
	  		{
	  			set.invalidate();
	  		}
	  		if (neu != null)
	  		{
	  			neu.invalidate();
	  		}
  		}	
  	}
  	
  	return result;
  }
  
  private ConfigurationObject behandleSystemObjektAendeung(HierarchieObjekt ho, ConfigurationObject ko, 
  		String neuePid)	throws ConfigurationChangeException
	{
  	ConfigurationObject result = dupliziereObjekt(ko, neuePid);
  	
  	Object vater = _hierarchie.getUebergeordnetesObjekt(ho);
  	
		if (vater instanceof HierarchieObjekt)
		{
			HierarchieObjekt vaterHO = (HierarchieObjekt)vater;
			IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
			ConfigurationObject objekt = null;
			ObjectSet menge = null;
			
			if (DatenSpeicher.getInstanz().getDeTypenManager().hasHOT(hot))
			{
				objekt = (ConfigurationObject)vaterHO.getSystemObjekt(0);
				menge	= objekt.getNonMutableSet("De");
			}
			else if (hot.equals(HOT_TLS.EAK))
			{
				objekt = (ConfigurationObject)vaterHO.getSystemObjekt(0);
				menge	= objekt.getNonMutableSet("Eak");
			}
			else if (hot.equals(HOT_TLS.SM))
			{
				if (result.isOfType("typ.steuerModul"))
				{
					if (!ko.getPid().equals(result.getPid()))
					{	
						ConfigDataModel dataModel = _hierarchie.getDataModel();
						AttributeGroup atg = dataModel.getAttributeGroup("atg.anschlussPunktKommunikationsPartner");
						ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
						SystemObject apkp = ho.getSystemObjekt(1);
						Data data = ((ConfigSystemObject)apkp).getConfigurationData(atg, lookup);
						Data neu = DataFactory.createModifiableCopy(data, lookup);;
						neu.getReferenceValue("KommunikationsPartner").setSystemObject(result);
						try
						{
							if (!HierarchieObjekt.isAttributgruppeAenderbar(apkp, atg))
			    		{
           			apkp = behandleSystemObjektAendeung(ho, (ConfigurationObject)apkp, apkp.getPid());
	     	  		  ho.setSystemObjekt(1, apkp);
			    		}
							apkp.setConfigurationData(atg, neu);
						}
						catch (ConfigurationChangeException e)
						{
							result.invalidate();
							throw(e);
						}
						catch (RuntimeException e)
						{
							result.invalidate();
							throw(e);
						}
					}	
				}
				else if (result.isOfType("typ.anschlussPunktKommunikationsPartner"))
				{
					objekt = (ConfigurationObject)vaterHO.getSystemObjekt(0);
					menge	= objekt.getNonMutableSet("AnschlussPunkteKommunikationsPartner");
				}
			}
			else if (hot.equals(HOT_TLS.Inselbus))
			{
				objekt = (ConfigurationObject)vaterHO.getSystemObjekt(0);
				menge	= objekt.getNonMutableSet("AnschlussPunkteGert");
			}
			else if (hot.equals(HOT_TLS.KRI) ||
					hot.equals(HOT_TLS.UZ) ||
					hot.equals(HOT_TLS.VRZ))
			{
				if (result.isOfType("typ.kri") || result.isOfType("typ.uz") || result.isOfType("typ.vrz"))
				{
					if (_hierarchie.getUebergeordnetesObjekt(ho) instanceof HierarchieObjekt)
					{	
						if (!ko.getPid().equals(result.getPid()))
						{
							ConfigDataModel dataModel = _hierarchie.getDataModel();
							AttributeGroup atg = dataModel.getAttributeGroup("atg.anschlussPunktKommunikationsPartner");
							ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
							SystemObject apkp = ho.getSystemObjekt(2);
							Data data = ((ConfigSystemObject)apkp).getConfigurationData(atg, lookup);
							Data neu = DataFactory.createModifiableCopy(data, lookup);
							neu.getReferenceValue("KommunikationsPartner").setSystemObject(result);
							try
							{
								if (!HierarchieObjekt.isAttributgruppeAenderbar(apkp, atg))
				    		{
	           			apkp = behandleSystemObjektAendeung(ho, (ConfigurationObject)apkp, apkp.getPid());
		     	  		  ho.setSystemObjekt(2, apkp);
				    		}
								apkp.setConfigurationData(atg, neu);
							}
							catch (ConfigurationChangeException e)
							{
								result.invalidate();
								throw(e);
							}
							catch (RuntimeException e)
							{
								result.invalidate();
								throw(e);
							}
						}	
					}	
				}
				else if (result.isOfType("typ.anschlussPunkt"))
				{
					objekt = (ConfigurationObject)vaterHO.getSystemObjekt(0);
					menge	= objekt.getNonMutableSet("AnschlussPunkteGert");
				}
				else if (result.isOfType("typ.anschlussPunktKommunikationsPartner"))
				{
					objekt = (ConfigurationObject)ho.getSystemObjekt(1);
					menge	= objekt.getNonMutableSet("AnschlussPunkteKommunikationsPartner");
				}
			}
			
			if (menge != null)
			{	
				try
				{
					menge.add(result);
				}
				catch (ConfigurationChangeException e)
				{
					for (ObjectSet set : result.getObjectSets())
					{
						set.invalidate();
					}
					result.invalidate();
					throw(e);
				}
				catch (RuntimeException e)
				{
					for (ObjectSet set : result.getObjectSets())
					{
						set.invalidate();
					}
					result.invalidate();
					throw(e);
				}
				menge.remove(ko);
			}	
		}
		
		ko.invalidate();
  	
  	return result;
	}
  
  public IStatus uebernehmeDaten(HierarchieObjekt ho, HODaten daten)
  {
  	MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, 
  			"bernehme bearbeitete Daten fr Hierarchieobjekt " + ho, null);
  	
  	List<SystemObject> objektListe = new ArrayList<SystemObject>(ho.getSystemObjekte());
  	IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
  	
//  	DatenModellUpdateListener.getInstanz().notifyDatenModellUpdateListener(objektListe);
  	
  	// Durchlaufen der Systemobjekte in umgekehrter Reihenfolge
  	int index = objektListe.size();
  	ListIterator<SystemObject> it = objektListe.listIterator(index);
		while (it.hasPrevious())
		{
			index -= 1;
			SystemObject so = it.previous();
  		
  		if (ho.getSystemObjekte().contains(so))
			{
  			SystemObjektDaten soDaten = daten.getSystemObjektDaten(so);
  		  	
  			// PID
  			if (!so.getPid().equals(soDaten.getPid()))
  			{ 
  				boolean apkpWarnung = false;
//  				if (i == 0 && objektListe.size() > 1)
//  				{
//  					int apkpIndex = 0;
//  					ConfigDataModel dm = getHierarchie().getDataModel();
//    				AttributeGroup atg = dm.getAttributeGroup("atg.anschlussPunktKommunikationsPartner"); 
//  					
//  					if (HOT_TLS.VRZ.equals(hot) || HOT_TLS.UZ.equals(hot) || HOT_TLS.KRI.equals(hot))
//  					{
//  						Assert.isTrue(objektListe.size() >= 3);
//  						apkpIndex = 2;
//  					}
//  					else if (HOT_TLS.SM.equals(hot))
//  					{
//  						Assert.isTrue(objektListe.size() >= 2);
//  						apkpIndex = 1;
//  					}
//  					
//  					if (apkpIndex > 0 && !HierarchieObjekt.isAttributgruppeAenderbar(ho.getSystemObjekt(apkpIndex), atg))
//  					{
//  						// Wenn versucht wird, die PID des SO mit Index 0 zu ndern, aber ein 
//  	  				// APKP-Objekt mit Index 2 existiert, dessen massgebliche ATG nicht mehr
//  	  				// gendert werden darf
//    				 	status.add(new Status(IStatus.WARNING, Activator.PLUGIN_ID, 
//        					"PID kann nicht auf " + soDaten.getPid() + " gendert werden, da wegen ATG-Verwendung (" +
//        					AttributeGroupUsage.Usage.OptionalConfigurationData + ") der ATG " + atg.getPid() +
//        					" die Daten des Objekts " + ho.getSystemObjekt(2).getPid() + " nicht gendert werden knnen"));
//    				 	apkpWarnung = true;
//  					}
//  				}				
//  				
  				if (!apkpWarnung)
  				{	
		        try
		        {
		  				so = behandleSystemObjektAendeung(ho, (ConfigurationObject)so, soDaten.getPid());
		  				ho.setSystemObjekt(index, so);
		  				status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, 
		  						"Neues Objekt mit PID " + so.getPid() + 
		  						" vom Typ " + so.getType().getPid() + " erzeugt"));
		        }
		        catch (ConfigurationChangeException e)
		      	{
		        	status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
		    					"Fehler beim behandeln der PID-nderung auf " + soDaten.getPid() +
		    					" fr Objekt " + so.getPid(), e));
		        	return status;
		      	}
  				}  
  			}
  			
  			// Name
  			if (so.getName() == null || !so.getName().equals(soDaten.getName()))
  			{	
	  			try
	      	{
	  				so.setName(soDaten.getName());
	  				status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, 
	   						"Name \"" + soDaten.getName() + "\" bernommen"));
	      	}
	  	   	catch (ConfigurationChangeException e)
	      	{
	  	   		status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
	   						"Name \"" + soDaten.getName() + "\" konnte nicht bernommen werden" , e));
	      	}
  			}
  			
  			// ATGs
  			List<String> atgList = ho.getHierarchieObjektTyp().getTypInfo(so.getType().getPid()).getAtgs();
  			
		    for (String atgPid: atgList)
    		{
		    	AttributeGroup atg = _hierarchie.getDataModel().getAttributeGroup(atgPid);
		    	if (atg != null)
		    	{
		    		ConfigDataModel dataModel = _hierarchie.getDataModel();
		    		ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
		    		
		    		//Data ist = so.getConfigurationData(atg);
		        Data ist = ((ConfigSystemObject)so).getConfigurationData(atg, lookup);
		        		        
		    		Data soll = soDaten.getAtgDaten(atgPid);
		  
	          if (soll != null)
	          {
	          	if (ist == null || soDaten.isDatenGeaendert(atgPid))
	          	{	
		          	if (!HierarchieObjekt.isAttributgruppeAenderbar(so, atg))
				    		{
		          		 try
		               {
		     	  				so = behandleSystemObjektAendeung(ho, (ConfigurationObject)so, so.getPid());
		     	  				ho.setSystemObjekt(index, so);
		     	  				status.add(new Status(IStatus.OK, Activator.PLUGIN_ID,
		     	  						"ATG-Verwendung " + AttributeGroupUsage.Usage.OptionalConfigurationData +
		     	  						" bedingt Erzeugung neues Objekt mit PID " + so.getPid() + " vom Typ " + 
		     	  						so.getType().getPid()));
		               }
		               catch (ConfigurationChangeException e)
		             	{
		               	status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
		               			"Bei der durch ATG-Verwendung " + AttributeGroupUsage.Usage.OptionalConfigurationData +
		     	  						" bedingten Erzeugung neues Objekts mit PID " + so.getPid() + " vom Typ " +	
		     	  						so.getType().getPid() + " ist ein Fehler aufgetreten", e));
		               	continue;
		             	}
		          		
//		          		status.add(new Status(IStatus.INFO, Activator.PLUGIN_ID, 
//				   						"Eventuelle nderungen an konfigurierenden Daten der ATG " + atgPid +
//				   						" werden IGNORIERT, da wegen ATG-Verwendung (" + 
//				   						AttributeGroupUsage.Usage.OptionalConfigurationData + ") nicht mglich"));
//				    			continue;
		               
				    		}
		          	try
		          	{
		          		// TLS spezifisch
		          		//if (!so.getPid().equals(soDaten.getPid()))
//		          		{	
//			  						if (so.isOfType("typ.anschlussPunktKommunikationsPartner"))
//			  						{
//			  							if (atgPid.equals("atg.anschlussPunktKommunikationsPartner"))
//			  							{
//			  								if (ho.getHierarchieObjektTyp().equals(HOT_TLS.VRZ) ||
//			  										ho.getHierarchieObjektTyp().equals(HOT_TLS.UZ) ||
//			  										ho.getHierarchieObjektTyp().equals(HOT_TLS.KRI) ||
//			  										ho.getHierarchieObjektTyp().equals(HOT_TLS.SM))
//			  								{	
//			  									soll.getReferenceValue( "KommunikationsPartner" ).setSystemObject(ho.getSystemObjekt(0));
//			  								}	
//			  							}
//			  						}
//		          		}	
		  					  // Ende TLS spezifisch
		          		
		          		so.setConfigurationData(atg, soll);
		          		status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, 
		       						"Konfigurierende Daten der ATG " + atgPid + " bernommen"));
		          	}
		          	catch (ConfigurationChangeException e)
		          	{
		          		status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
		       						"Fehler beim bernehmen konfigurierender Daten der ATG " + atgPid, e));
		          		continue;
		          	}
		          	if (HOT_TLS.EAK.equals(hot) && "atg.eak".equals(atg.getPid()))
		         		{
		          		behandleErzeugenDEAnEAK254(ho, soll, ist, status);
		         		}
	          	}	
	          }
	          else // Keine Daten fr diese ATG bergeben => vorhanden Daten lschen
	          {
	          	if (ist != null)
	          	{
	          		if (!HierarchieObjekt.isAttributgruppeAenderbar(so, atg))
				    		{
	          			 try
		               {
		     	  				so = behandleSystemObjektAendeung(ho, (ConfigurationObject)so, so.getPid());
		     	  				ho.setSystemObjekt(index, so);
		     	  				status.add(new Status(IStatus.OK, Activator.PLUGIN_ID,
		     	  						"ATG-Verwendung " + AttributeGroupUsage.Usage.OptionalConfigurationData +
		     	  						" bedingt Erzeugung neues Objekt mit PID " + so.getPid() + " vom Typ " + 
		     	  						so.getType().getPid()));
		               }
		               catch (ConfigurationChangeException e)
		             	{
		               	status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
		               			"Bei der durch ATG-Verwendung " + AttributeGroupUsage.Usage.OptionalConfigurationData +
		     	  						" bedingten Erzeugung neues Objekts mit PID " + so.getPid() + " vom Typ " +	
		     	  						so.getType().getPid() + " ist ein Fehler aufgetreten", e));
		               	continue;
		             	}
	          			
//				    			status.add(new Status(IStatus.INFO, Activator.PLUGIN_ID, 
//				   						"Eventuelle nderungen an konfigurierenden Daten der ATG " + atgPid +
//				   						" werden IGNORIERT, da wegen ATG-Verwendung (" + 
//				   						AttributeGroupUsage.Usage.OptionalConfigurationData + ") nicht mglich"));
//				    			continue;
		               
				    		}
	          		try
		          	{
		          		so.setConfigurationData(atg, null);
		          		status.add(new Status(IStatus.OK, Activator.PLUGIN_ID, 
		       						"Konfigurierende Daten der ATG " + atgPid + " gelscht"));
		          	}
		          	catch (ConfigurationChangeException e)
		          	{
		          		status.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
		       						"Fehler beim Lschen konfigurierender Daten der ATG " + atgPid, e));
		          	}
	          	}
	          }
		    	}
		    	else
		    	{
		    		_debug.warning("Bearbeitung Objekt " + so.getPid() + 
		    				": Keine Attributgruppe im Datenmodell gefunden fr " + atgPid);
		    	}
    		}	
			}   	
  	}    
  	
  	return status;
  }
  
  
  private boolean behandleLoeschenDEAnEAK254(HierarchieObjekt vater, HierarchieObjekt ho, MultiStatus status)
  {
  	Assert.isLegal(HOT_TLS.SM.equals(vater.getHierarchieObjektTyp()));
   	Assert.isLegal(HOT_TLS.EAK.equals(ho.getHierarchieObjektTyp()));
  	Assert.isNotNull(status);
  	  
  	boolean result = false;
  	
  	ConfigDataModel dataModel = getHierarchie().getDataModel();
		ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
		AttributeGroup atgEak = dataModel.getAttributeGroup("atg.eak");
		
		Data dataEak = ((ConfigSystemObject)ho.getSystemObjekt(0)).getConfigurationData(atgEak, lookup);

		if (dataEak != null)
		{	
	  	HierarchieObjekt hoEak254 = null;
	  
	  	for (HierarchieObjekt kind: _hierarchie.getUntergeordneteObjekte(vater))
	  	{
	  		if (HOT_TLS.EAK.equals(kind.getHierarchieObjektTyp()) && kind != ho)
	  		{
	  			SystemObject eak = kind.getSystemObjekt(0);
	  			Data data = ((ConfigSystemObject)eak).getConfigurationData(atgEak, lookup);
	  			if (data != null)
	  			{
	  				int osi2Adr = data.getUnscaledValue("OSI2Adresse").intValue();
	  				if (osi2Adr == 254) // Kriterium fr EAK 254 ist OSI2Adresse == 254
	  				{
	  					hoEak254 = kind;
	  					break;
	  				}
	  			}
	  		}
	  	}
  	
	  	if (hoEak254 != null)
	  	{
	  		AttributeGroup atgDe = dataModel.getAttributeGroup("atg.de");
	  		int osi2Adr = dataEak.getUnscaledValue("OSI2Adresse").intValue();
	  		
	  		for (HierarchieObjekt kind: _hierarchie.getUntergeordneteObjekte(hoEak254))
		  	{
		  		if (kind.getHierarchieObjektTyp() instanceof DeUnterTyp)
		  		{
		  			SystemObject de = kind.getSystemObjekt(0);
		  			Data data = ((ConfigSystemObject)de).getConfigurationData(atgDe, lookup);
		  			if (data != null)
		  			{
		  				int deKanal = data.getUnscaledValue("DEKanal").intValue();
		  				if (deKanal == osi2Adr)
		  				{	
		  					if (osi2Adr != 0) // vorhandene OSI2Adresse 0 wird nie gelscht, da fr SM
      	      	{
			  					status.add(loescheHO(kind));
			  				  result = true;
      	      	}  
			  				break;
		  				}
		  			}
		  		}
		  	}
	  	}
		}
		
		return result;
  }
    
  
  private boolean behandleErzeugenDEAnEAK254(HierarchieObjekt ho, Data soll, Data ist, MultiStatus status)
  {
  	boolean result = false;
  	
  	Assert.isLegal(HOT_TLS.EAK.equals(ho.getHierarchieObjektTyp()));
  	Assert.isNotNull(soll);
  	Assert.isNotNull(status);
  	
  	int osi2AdrNeu = soll.getUnscaledValue("OSI2Adresse").intValue();
  	
  	int osi2AdrAlt = -1;
  	
  	if (ist != null)
  	{
  		osi2AdrAlt = ist.getUnscaledValue("OSI2Adresse").intValue();
  	}
  	
  	if (osi2AdrNeu != osi2AdrAlt)
  	{
  		HierarchieObjekt hoEak254 = getEak254(ho);
      if (hoEak254 != null)
      {
      	boolean vorhanden = false;
    		ConfigDataModel dataModel = getHierarchie().getDataModel();
    		AttributeGroup atg = dataModel.getAttributeGroup("atg.de");
    		ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
      	List<HierarchieObjekt> listeDe = new LinkedList<HierarchieObjekt>(_hierarchie.getUntergeordneteObjekte(hoEak254));
      	
      	for (HierarchieObjekt kind: listeDe)
      	{
      		if (kind.getHierarchieObjektTyp() instanceof DeUnterTyp)
      		{
      			SystemObject de = kind.getSystemObjekt(0);
      			Data data = ((ConfigSystemObject)de).getConfigurationData(atg, lookup);
      			if (data != null)
      			{
      				int deKanal = data.getUnscaledValue("DEKanal").intValue();
      				if (deKanal == osi2AdrAlt) 
      				{
      					if (ist != null && osi2AdrAlt != 0) // vorhandene OSI2Adresse 0 wird nie gelscht, da fr SM
      	      	{
        					status.add(loescheHO(kind));
        					result |= true;
      	      	}	
      				}
      				if (deKanal == osi2AdrNeu)
      				{
      					vorhanden = true;
      				}
      			}
      		}
      	}	
      	
      	if (osi2AdrNeu == 0)
      	{	
      		status.add(new Status(Status.OK, Activator.PLUGIN_ID,	
      				"Fr EAK mit OSI2-Adresse 0 wird kein DE der FG 254 angelegt"));
      	}
      	else
      	{	
	      	if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
	      	{
	      		IHierarchieObjektTyp deTyp = DatenSpeicher.getInstanz().getDeTypenManager().getDeUnterTyp("typ.deSys");
						if (deTyp != null)
						{	
							if (vorhanden)
							{
								status.add(new Status(Status.OK, Activator.PLUGIN_ID,	"Hierarchieobjekt vom Typ "
										+ deTyp + " ist bereits mit DE-Kanal " + osi2AdrNeu + " (OSI2Adresse) vorhanden"));
							}
							else
							{
								ConfigurationArea kb = ho.getSystemObjekt(0).getConfigurationArea();
								status.add(erzeugeDeAnEak254(hoEak254, kb, deTyp,	osi2AdrNeu));
							}
							result |= true;
						}
						else
						{
							status.add(new Status(Status.ERROR, Activator.PLUGIN_ID,	"DE-Untertyp " + "typ.deSys" +
									" wurde nicht gefunden - DE der FG 254 kann nicht angelegt werden"));
						}
	      	}	
      	}	
      }	
  	}
  	
  	return result;
  }
  
  private IStatus erzeugeDeAnEak254(HierarchieObjekt hoEak254, ConfigurationArea kb, IHierarchieObjektTyp deTyp, int osi2Adr)
  {
  	return erzeugeDeAutomatisch(hoEak254, kb, deTyp,	osi2Adr, true, false);
  }
  
  private HierarchieObjekt getEak254(HierarchieObjekt ho)
  {
  	Assert.isLegal(HOT_TLS.EAK.equals(ho.getHierarchieObjektTyp()));
  	
  	Object vater = _hierarchie.getUebergeordnetesObjekt(ho);
  	Assert.isTrue(vater instanceof HierarchieObjekt);
  	Assert.isTrue(HOT_TLS.SM.equals(((HierarchieObjekt)vater).getHierarchieObjektTyp()));
  	
  	ConfigDataModel dataModel = getHierarchie().getDataModel();
		AttributeGroup atg = dataModel.getAttributeGroup("atg.eak");
		ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
  	
  	for (HierarchieObjekt kind: _hierarchie.getUntergeordneteObjekte(vater))
  	{
  		if (HOT_TLS.EAK.equals(kind.getHierarchieObjektTyp()) && kind != ho)
  		{
  			SystemObject eak = kind.getSystemObjekt(0);
  			Data data = ((ConfigSystemObject)eak).getConfigurationData(atg, lookup);
  			if (data != null)
  			{
  				int osi2Adr = data.getUnscaledValue("OSI2Adresse").intValue();
  				if (osi2Adr == 254) // Kriterium fr EAK 254 ist OSI2Adresse == 254
  				{
  					return kind;
  				}
  			}
  		}
  	}
  	
  	return null;
  }
  
  @Override
  public boolean pruefeNeuanlageVerboten( Object objekt, IHierarchieObjektTyp hot)
  {
    // TODO Auto-generated method stub
    return false;
  }

  @Override
  public Collection<SystemObjectType> loeschenOffeneReferenzenVerboten(IHierarchieObjektTyp hot)
  {
    // TODO Auto-generated method stub
    return null;
  }

  @Override
  public boolean verwendeZuLoeschendeObjekteZurPruefungOffeneReferenzen()
  {
    // TODO Auto-generated method stub
    return false;
  }

  /**
   * Zeigt an, ob das bergebene Hierarchieobjekt beim rekursiven Kopieren als untergeordnetes 
   * Objekt mitkopiert werden soll oder nicht
   *  
   * @param ho das aktuell zu kopierende HO
   * @return <code>true</code>, wenn das aktuell zu kopierende Objekt nicht kopiert werden soll, 
   * <code>false</code> (default) sonst
   */
  public boolean kopierenNichtErlaubt(HierarchieObjekt ho)
  {
  	IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
  	
  	if (hot instanceof DeUnterTyp)
  	{
  		ConfigDataModel dataModel = _hierarchie.getDataModel();
  		AttributeGroup atg = dataModel.getAttributeGroup("atg.de");
  		
  		if (atg != null)
  		{	
  			ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
  			SystemObject de = ho.getSystemObjekt(0);
  			Data data = ((ConfigConfigurationObject)de).getConfigurationData(atg, lookup);
  			
  			if (data != null)
  			{
  				int deKanal = data.getUnscaledValue("DEKanal").intValue();
  				
  				if (deKanal == 255)
  				{
  					//DEs mit deKanal 255 sollen nicht kopiert werden
  					return true;
  				}
  			}
  		}	
  	}
  	else if (HOT_TLS.EAK.equals(hot))
  	{
  		ConfigDataModel dataModel = _hierarchie.getDataModel();
  		AttributeGroup atg = dataModel.getAttributeGroup("atg.eak");
  		
  		if (atg != null)
  		{	
  			ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
  			SystemObject eak = ho.getSystemObjekt(0);
  			Data data = ((ConfigConfigurationObject)eak).getConfigurationData(atg, lookup);
  			
  			if (data != null)
  			{
  				int osi2Adresse = data.getUnscaledValue("OSI2Adresse").intValue();
  				
  				if (osi2Adresse == 254)
  				{
  					//EAKs mit Adresse 254 sollen nicht kopiert werden
  					return true;
  				}
  			}
  		}
  	}
  		
  	return false;
  		
  }
  
}
