/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkaufd;

import java.util.Collection;
import java.util.Collections;
import java.util.Map;
import java.util.TreeMap;

import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieObjektTyp;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Dynamisched Verwalten der Untertypen des HOT UFDS
 * 
 * @author Sans
 *
 */
public class UFDSTypenManager 
{
	/**
   * Debug-Logger fr Logging-Ausgaben
   */
  private final static Debug _debug = Debug.getLogger();
	
  /**
   * 
   */
	private Map<String, UFDSUnterTyp> _ufdsUnterTypen = new TreeMap<String, UFDSUnterTyp>();
	
	/**
	 * 
	 */
	private UFDSUnterTyp[] _ufdsEbene;
	
	/**
	 * 
	 */
	private DataModel _dataModel;
	
	/**
	 * 
	 * @param dataModel
	 */
	UFDSTypenManager(DataModel dataModel)
	{
		_dataModel = dataModel;
		
		bestimmeUnterTypen(UFDSUnterTyp.UFDS_OBER_TYP);
		
		_ufdsEbene = _ufdsUnterTypen.values().toArray(new UFDSUnterTyp[_ufdsUnterTypen.size()]);
	}
	
	/**
	 * 
	 * @param typ
	 */
	private void bestimmeUnterTypen(String typ)
	{
		SystemObjectType oberTyp = _dataModel.getType(typ);
				
		if (oberTyp != null)
		{
			for (SystemObjectType unterTyp: oberTyp.getSubTypes())
			{
				String pid = unterTyp.getPid();
			
					_ufdsUnterTypen.put(pid, new UFDSUnterTyp(pid));
				
				bestimmeUnterTypen(pid);
			}
		}
		else
		{
			_debug.warning("Typ " + typ + " ist im Datenmodell nicht vorhanden");
		}
	}
	
	/**
	 * 
	 * @return
	 */
	public UFDSUnterTyp[] getUFDSEbene()
	{
		return _ufdsEbene;
	}
	
	/**
	 * 
	 * @return
	 */
	public Collection<String> getSystemObjektTypen()
	{
		return Collections.unmodifiableCollection(_ufdsUnterTypen.keySet());
	}
	
	/**
	 * 
	 * @param typ
	 * @return
	 */
	public UFDSUnterTyp getUFDSUnterTyp(String typ)
	{
		return _ufdsUnterTypen.get(typ);
	}
	
	/**
	 * 
	 * @param hot
	 * @return
	 */
	public boolean hasHOT(IHierarchieObjektTyp hot)
	{
		return _ufdsUnterTypen.values().contains(hot);
	}

}
